<?php
 
# returns true if alert exists, false if not.  Notify_id is optional.
function alert_exists($source, $target, $notify_id = 0) {
	if (get_alert($source, $target, $notify_id) === false) {
		return false;
	} else {
		return true;
	}
}

# returns alert details for an open alert, notify_id is optional; returns false if the alert doesn't exist
function get_alert($source, $target, $notify_id = 0) {
	global $dbconn;

	# get source id
	$query = pg_query($dbconn, "SELECT source_id FROM bp.alert_source WHERE source_name = '$source'");
	if (!$query or pg_num_rows($query) == 0) {
		return false;
	}
	$row = pg_fetch_row($query);
	$source_id = $row[0];

	# create optional filter on notify_id
	if ($notify_id > 0) {
		$and_notify = "AND notification_id = $notify_id";
	} else {
		$and_notify = '';
	}

	# look for the alert
	$query = pg_query($dbconn, "SELECT * FROM bp.alerts WHERE source_id = $source_id AND target = '$target' AND closed = FALSE $and_notify");
	if (!$query or pg_num_rows($query) == 0) {
		return false;
	} else {
		return pg_fetch_assoc($query, 0);
	}
}

/*
 * Name: AgentAlerts
 *
 * Description: Process alerts for Agent Push.
 *
 * Note: ClientID=0 means process alerts for all qualifying clients.
 *       Non-zero means process alerts for that clientID only.
 * 
 * Returns: nothing
 */
function AgentAlerts($clientID=0)
{
	global $dbconn;

	$ueb_type = bp_get_capabilities("CE");
	#if ($ueb_type['CE']) {
		#return;
	#}
	
	#echo "AgentAlerts: ClientID: $clientID\n";
	
	# Collect data needed for alert processing; don't get grand-clients in this query
	if($clientID == 0)
	{
		$query = "SELECT node_no, node_name, os, type, version, (properties & 16) = 16 FROM bp.nodes where (properties & 32)=0";
	}
	else
	{
		$query = "SELECT node_no, node_name, os, type, version, (properties & 16) = 16 FROM bp.nodes where node_no=$clientID AND (properties & 32)=0";
	}
	$res = pg_query($dbconn, $query);
	if ($res == false)
	{
		return;
	}
	
	# Win32 local client version
	exec ("/usr/bp/bin/cmc_AgentPush.sh getLocalClientVersion win32", $outputArray32, $retval);
	if($retval == 0)
	{
		$localWin32Version = $outputArray32[0];
	}
	else
	{
		$localWin32Version = "0.0.0-0";
	}
	
	#echo "AgentAlerts: Found local Win32 agent version $localWin32Version\n";
	
	#Win 64 local client version
	exec ("/usr/bp/bin/cmc_AgentPush.sh getLocalClientVersion win64", $outputArray64, $retval);
	if($retval == 0)
	{
		$localWin64Version = $outputArray64[0];
	}
	else
	{
		$localWin64Version = "0.0.0-0";
	}
	
	#echo "AgentAlerts: Found local Win64 agent version $localWin64Version\n";

	# Loop through the list and set or cancel alerts as needed
	while ($row = pg_fetch_row($res))
	{
		# Skip Hyper-V cluster virtual clients
		if($row[4] == "t")
		{
			continue;
		}
		
		#echo "AgentAlerts: Processing client ID=$row[0], Name=$row[1], OS=$row[2], type=$row[3], version=$row[4]\n";

		# If the OS is windows
		switch ($row[2])
		{
			case 0x001E:
			case 0x001F:
			case 0x0020:
			case 0x0023:
			case 0x0024:
			case 0x0025:
			case 0x0026:
			case 0x0027:
			case 0x0029:
			case 0x002A:
			
				$alertTarget = "BP Version $row[1]";
			
				# If the architecture is 32 bit. (Values from nodetype.h)
				if( $row[3] == 0x0001 || $row[3] == 0x0020 )
				{
					# If the local DPU version is newer than the client version
					if( $row[4] < $localWin32Version)
					{
						# Send
						#echo "AgentAlerts: New 32bit version available for client ID=$row[0], Name=$row[1], sending notification.\n";
						if ($ueb_type['CE']) {
							bp_send_notification(190, $alertTarget, $localWin32Version, $row[1]);
						} else {
							bp_send_notification(141, $alertTarget, $localWin32Version, $row[1]);
						}
					}
					else
					{
						# Close
						if(alert_exists('Clients', $alertTarget, 141) || alert_exists('Clients', $alertTarget, 190))
						{
							#echo "AgentAlerts: Canceling 32bit alert for client ID=$row[0], Name=$row[1].\n";
							if ($ueb_type['CE']) {
								bp_send_notification(191, $alertTarget, $row[1], $localWin32Version);
								bp_send_notification(8, $alertTarget);
							} else {
								bp_send_notification(144, $alertTarget, $row[1], $localWin32Version);
								bp_send_notification(8, $alertTarget);
							}
						}
					}
				}
				# If the architecture is 64 bit. Value from nodetype.h
				else if ( $row[3] == 0x0108 )
				{
					# If the local DPU version is newer than the client version
					if( $row[4] < $localWin64Version )
					{
						# Send
						#echo "AgentAlerts: New 64bit version available for client ID=$row[0], Name=$row[1], sending notification.\n";
						if ($ueb_type['CE']) {
							bp_send_notification(190, $alertTarget, (string)$localWin64Version, $row[1]);
						} else {
							bp_send_notification(141, $alertTarget, (string)$localWin64Version, $row[1]);
						}
					}
					else
					{
						# Close
						if(alert_exists('Clients', $alertTarget, 141) || alert_exists('Clients', $alertTarget, 190))
						{
							#echo "AgentAlerts: Canceling 64bit alert for client ID=$row[0], Name=$row[1].\n";
							if ($ueb_type['CE']) {
								bp_send_notification(191, $alertTarget, $row[1], $localWin64Version);
								bp_send_notification(8, $alertTarget);
							} else {
								bp_send_notification(144, $alertTarget, $row[1], $localWin64Version);
								bp_send_notification(8, $alertTarget);
							}
						}
					}
				}
				break;
			default:
				#echo "AgentAlerts: Agent Upgrade not supported for client ID=$row[0], Name=$row[1], OS=$row[2]\n";
				break;
		}
	}
}

?>
