# This script displays the number of Master and Differential backups for Store level backups.
# DO NOT CHANGE THE OUTPUT OF THIS FILE
# The output of this script is as follows:
# NumberOfMasters,numberOfDifferentials,sizeOfLargestMaster,sizeOfLastMaster,clientName1
# NumberOfMasters,numberOfDifferentials,sizeOfLargestMaster,sizeOfLastMaster,clientName2

SMB_CONF="/etc/samba/smb.conf"
SMBSHARE_PATTERN='^[[:space:]]*path[[:space:]]*=[[:space:]]*'


MAGIC_FILE="UniExch.magic"
INI_SECTION="Unitrends"
INI_MAGIC_KEY="Magic"
INI_MAGIC_VAL="Unitrends_Exchange"
STATUS_SECTION="Status"

# Determine where BP is installed
eval `grep -v "^[;#]" /etc/default/bp.ini | grep '=[^	]'` > /dev/null 2>&1

# Set basic environment variables
eval `${BPDIR}/bin/bputil -e -c`

INIGETTER="$BP_BINDIR/bputil -g "

CheckStatus() {	
	# 5543 - also make sure Status.ini has Status=success
	status=`$INIGETTER $STATUS_SECTION Status "" $PWD/$1/Status.ini`
	if [ "$status" != "Success" ]
	then
		return 1
	fi

	# 3642 - make sure log file indicates success before creating
	#	symlink to LAST directory.
	JobStatus=`grep BPEXCH_Job_Status $1/log.txt 2> /dev/null | awk '{print $2}' | awk -F"
" '{print $1}'`

	for stat in $JobStatus
	do
		if [ "$stat" != "Success" ]
		then
			return 1
		fi 
	done
	return 0
}

# process single samba share
ProcessShare() {
    for fname in `find $1 -name $MAGIC_FILE` 
    do
	magic=`$INIGETTER $INI_SECTION $INI_MAGIC_KEY "" $fname`
	if [ "Z$magic" = "Z$INI_MAGIC_VAL" ]; then 
	    clientroot=`dirname $fname`
	    cd $clientroot
	    numberOfMasters=0
	    numberOfDiffs=0
	    masterMegs=0
	    lastMasterMegs=0
	    for dirs in `find . -name "*Master" -print`	    
	    do
	    	CheckStatus $dirs
		if [ $? -eq 0 ]; then
			numberOfMasters=`expr $numberOfMasters + 1`
                        size=`du -m $dirs | cut -f 1`
                        if [ $size -gt $masterMegs ]; then
                                masterMegs=$size
                        fi
		fi
	    done

            for dirs in `find . -name "*Master" -print | sort -t/ -k3` # sort by date
            do
                CheckStatus $dirs
                if [ $? -eq 0 ]; then
                        lastMasterMegs=`du -m $dirs | cut -f 1`
                fi
            done

	    for dirs in `find . -name "*Differential" -print`	    
	    do
	    	CheckStatus $dirs
		if [ $? -eq 0 ]; then
			numberOfDiffs=`expr $numberOfDiffs + 1`
		fi
	    done

	    echo "$numberOfMasters,$numberOfDiffs,$masterMegs,$lastMasterMegs,`basename $clientroot`"

	else 
	    echo $fname " has wrong magic, skipping"	    
	fi
    done
}

# Loop through samba shares
for sharepath in `grep -i $SMBSHARE_PATTERN $SMB_CONF | grep -E -v "/backups/vmware_flr/|/backups/backups_restore_share/" | awk -F = '{ print $2 }'`
do
    if [ -d "${sharepath}" ]; then
	ProcessShare $sharepath
    fi
done

