#!/bin/sh

# Change log entries
# Bug# Date    Who Description
# ---- ------- --- ------------------------------------------------------
# 6395 18Nov08 LCG Initial implementation
# 7515 05Jun09 RJM Cleanup cores
#
# This script cleans up old data on the $BPDIR partition 

# set $BPDIR
eval `grep BPDIR /etc/default/bp.ini`
LISTS_DIR=$BPDIR/lists.dir
LOGS_DIR=$BPDIR/logs.dir
LOG=$LOGS_DIR/disk_cleanup.log

function finish_and_exit() {
	echo "_________________________________________________________________">> $LOG
	echo "Ending disk space usage:" >> $LOG
	df -h /backups $BPDIR >> $LOG
	exit $1
}

trap " echo $0 interrupted! | tee -a $LOG ; finish_and_exit 1 " 1 2 3 15

# Don't run if another disk_cleanup is running
mypid=$$
for pid in `pgrep disk_cleanup.sh`
do
	if [ $pid -ne $mypid ]
	then
		exit 0
	fi
done

echo "Disk cleanup on `date`" >$LOG
echo >> $LOG
echo "Beginning disk space usage:" >>$LOG
df -h /backups $BPDIR >> $LOG
echo "_________________________________________________________________">> $LOG

# Purge cores
$BPDIR/bin/dumps_utility.sh -c >> $LOG 2>&1
echo >>$LOG

# Clean up profile files & ulf*.not_posted that are more than 10 days old
# .SYNTH ulf files will be removed after 20 days.
size=`du -sh $LISTS_DIR | awk '{print $1}'`
nfiles=`ls $LISTS_DIR | wc -l`
echo "$LISTS_DIR before cleanup: $size, $nfiles files." >>$LOG

find $LISTS_DIR -name "profile_submit*" -daystart -mtime +10 -exec rm -f '{}' \;
find $LISTS_DIR -name "profile_media*" -daystart -mtime +10 -exec rm -f '{}' \;
find $LISTS_DIR -name "profile_client*" -daystart -mtime +10 -exec rm -f '{}' \;
find $LISTS_DIR -name "ulf*.not_posted" -daystart -mtime +10 -exec rm -f '{}' \;
find $LISTS_DIR -name "*.SYNTH" -daystart -mtime +20 -exec rm -f '{}' \;

size=`du -sh $LISTS_DIR | awk '{print $1}'`
nfiles=`ls $LISTS_DIR | wc -l`
echo "$LISTS_DIR after cleanup:  $size, $nfiles files." >>$LOG
echo >> $LOG


# Clean up log files in logs.dir that are more than 10 days old.
# Do not remove purger logs (purge_<D2Ddevice>.log).
# If /usr/bp is more than 95% full, keep removing files up to 1 day old.
size=`du -sh $LOGS_DIR | awk '{print $1}'`
nfiles=`ls $LOGS_DIR | wc -l`
echo "$LOGS_DIR before cleanup: $size, $nfiles files." >>$LOG


# Clean-up old logs managed by the logger API, based on 
# the logger settings in master.ini

echo "Calling bputil to Purge old logs">> $LOG
$BPDIR/bin/bputil --purgeoldlogs


# Don't remove any devmonitor logs.

for i in 10 9 8 7 6 5 4 3 2 1
do

	echo " Removing log files more than $i days old..." >>$LOG
	# Delete only logs not created by the logger api, since those logs
	# are purged by the bputil --purgeoldlogs

      	find $LOGS_DIR -name "*.log" ! -regex '.*-[0-9]+_[0-9]+.log' ! -regex '.*-[0-9]+.log' \
                ! -name "purge_*.log" ! -name "devmonitor*.log" \
                -daystart -mtime +$i -exec rm -f '{}' 2>>$LOG \;


	pct_full=`df -P $BPDIR | tail -1 | awk '{print $5}' | sed 's/%//'`
	if [ $pct_full -ge 95 ]
	then
		echo " $BPDIR is 95% or more full (${pct_full}% used)." >>$LOG
	
		# BPDIR is >95% full, so delete logs managed by logger api too to free up space
		find $LOGS_DIR -name "*.log" ! -name "purge_*.log" ! -name "devmonitor*.log" \
			-daystart -mtime +$i -exec rm -f '{}' 2>>$LOG \;

	else
		echo " $BPDIR is less than 95% full (${pct_full}% used)." >>$LOG
		break
	fi
done

size=`du -sh $LOGS_DIR | awk '{print $1}'`
nfiles=`ls $LOGS_DIR | wc -l`
echo "$LOGS_DIR after cleanup:  $size, $nfiles files." >>$LOG
echo >> $LOG
finish_and_exit 0

