# Script to start data_tunnel and run SQL file on remote DPV
# usage: dpv_bp_sql_run.sh sqlfile.sql
# ERROR cases:
# 1 - tunnel not up
# 2 - SQL file had error

. /etc/default/bp.ini 2> /dev/null
LOCAL_ERR_FILE=$BPDIR/logs.dir/transfer_exchdump.log

if [ $# -eq 0 ]
then
	echo "Usage: $0  dumpfile.sql"
	exit 1
fi

if [ ! -r $1 ]
then
	echo "$0: $1 is not readable!"
	exit 1
fi

echo "=== Transferring $1 workspace dump to vault on `date`" >>$LOCAL_ERR_FILE

$BPDIR/bin/data_tunnel_up.sh >>$LOCAL_ERR_FILE 2>&1
loc_ret=$?
if [ $loc_ret -ne 0 ]
then
	echo "data_tunnel_up.sh: ERROR: data_tunnel_up failed with error $loc_ret." >>$LOCAL_ERR_FILE
	exit 1
fi

psql -U postgres -v ON_ERROR_STOP=TRUE -f $1 -h localhost -p 5555 bpdb 2>>$LOCAL_ERR_FILE

loc_ret=$?
if [ $loc_ret -ne 0 ]
then
	echo "Transfer failed on `date`" >>$LOCAL_ERR_FILE
	echo "dpv_bp_sql_run.sh: ERROR: running $1 on the database failed with error $loc_ret."
	echo "Details are in $LOCAL_ERR_FILE"
	exit 2
else
	echo "Transfer completed successfully on `date`" | tee -a $LOCAL_ERR_FILE
fi

exit $loc_ret
