#!/bin/sh
#
# backup_iSeries.sh
#
# ME    4623    040208  Main script which performs the iSeries backup either via a given
#                       profile which was created by config_iseries.sh or on demand.
# ME    5103    052008  Give better error message is no default profile exists.
#

. /etc/default/bp.ini 2> /dev/null
if [ "$BPDIR" = "" ]
then
	export BPDIR="/usr/bp"
fi
BPBIN=$BPDIR/bin
BPLOG=$BPDIR/logs.dir
iSERIESLOG=$BPLOG/iSeries_backup$$.log
BPPRO=$BPDIR/profiles.dir
i5PRO=$BPPRO/iSeries
mkdir -p $i5PRO

ECHO="echo -e"


#########################################################################
#
# cleanup
#
# End state routine to clean up any temporary files and exit.
#
#########################################################################
cleanup()
{
	Status=$1
	# keep the number of iSeries backup logs from growing exponetially
	NumiSeriesLogs=`ls $BPLOG/iSeries_backup* 2> /dev/null | wc -l`
	while [ $NumiSeriesLogs -gt 20 ]
	do
		ls -tr $BPLOG/iSeries_backup* > /tmp/blogs$$ 2> /dev/null
		thisLog=`head -1 /tmp/blogs$$`
		rm -f $thisLog 2> /dev/null
		NumiSeriesLogs=`ls $BPLOG/iSeries_backup* 2> /dev/null | wc -l`
	done
	rm -f /tmp/blogs$$
	rm -f /tmp/bpt$$
	rm -f /tmp/err$$
	rm -f /tmp/.config_profile_name
	rm -f $iSERIESLOCK
	# 5119
	# 5156
	if [ "$OnDemand" = "true" -a "$RemoveProfile" != "no" ]
	then
		rm -f $iSERIESPROFILE
	fi
	stty sane
	exit $Status
}
trap 'cleanup $?' EXIT

#########################################################################
#
# profile_read
#
# Read the profile entry requested and echo it for assignment in calling
# routine
#
#########################################################################
profile_read()
{
	Section="$1"
	Keyword="$2"
	$BPBIN/bputil -g "$Section" "$Keyword" "xxx" $iSERIESPROFILE
}

#
# MAIN PROGRAM
#
# iSeries backups will be placed in the Default backup device folder.
# iSeries_util.php -d returns the pathname for the D2D device
# directory.

iSERIESPROFILE=""
if [ $# -gt 1 ]
then
	while getopts 'p:' i; do
		case "$i" in
		p )	iSERIESPROFILE="$OPTARG"
			if [ ! -f "$iSERIESPROFILE" ]
			then
				if [ ! -f $i5PRO/$iSERIESPROFILE ]
				then
					$ECHO "Cannot locate profile $iSERIESPROFILE"
					$ECHO "In order to use this feature of the User Interface, you must have previously created a default profile for iSeries backups on this System."
					exit 1
				else
					iSERIESPROFILE="$i5PRO/$iSERIESPROFILE"
				fi
			fi
		;;
		* )	$ECHO "${0}: Usage - $0 [-p <profile_name>]"
			exit 1
		;;
		esac
	done
fi

# If no profile is supplied on the command line we must be running an
# on-demand backup. Call config_iSeries.sh so a profile can be built.

$ECHO "`date`: iSeries backup started." >> $iSERIESLOG

if [ "$iSERIESPROFILE" = "" ]
then
	OnDemand="true"
	$ECHO "`date`: On demand iSeries backup -- calling config_iSeries.sh." >> $iSERIESLOG
	$BPBIN/config_iSeries.sh -b
	RC=$?
	if [ $RC -ne 0 ]
	then
		$ECHO "Cannot perform backup because backup configuration has failed."
		$ECHO "Please address the reported problems and try again."
		$ECHO "`date`: Cannot perform backup because configuration failed." >> $iSERIESLOG
		cleanup 1
	fi
	iSERIESPROFILE=`cat /tmp/.config_profile_name`
	if [ ! -s $iSERIESPROFILE ]
	then
		$ECHO "Cannot perform backup because no profile has been found."
		$ECHO "`date`: Cannot perform backup because no profile has been found." >> $iSERIESLOG
		cleanup 1
	fi
	$ECHO "`date`: On demand iSeries backup -- Using profile $iSERIESPROFILE." >> $iSERIESLOG
fi

iSeriesClient=`profile_read "iSeriesBackup" "ClientName"`
iSeriesBackupPath=`profile_read "iSeriesBackup" "DevicePath"`
iSeriesBackupName=`profile_read "iSeriesBackup" "DeviceName"`
iSeriesBackupStrategy=`profile_read "iSeriesBackup" "BackupStrategy"`
iSeriesExclList=`profile_read "iSeriesBackup" "ExcludeListFile"`
iSeriesInclList=`profile_read "iSeriesBackup" "IncludeListFile"`
iSeriesUserName=`profile_read "iSeriesBackup" "UserName"`
export IPW=`cat $i5PRO/.${iSeriesClient}_IPW`

$ECHO "`date`: Backup for iSeries client $iSeriesClient has been queued."
$ECHO "You may check the Job Status section for progress."

nohup $BPBIN/iSeries/backup_iseries.sh -p $iSERIESPROFILE &

