#!/bin/sh
#
# 3641
# Filename: iSeriesCron.sh
# v4.0
#
## Change log
## Bug  Inititals       Description
## ---  ---------       -----------
## 4623 ME              Initial submit
## 5110 ME              wording change for View option
## 5111 ME              bug in remove loop. replace return with call to calling routine.

. /etc/default/bp.ini 2> /dev/null
if [ "$BPDIR" = "" ]
then
	export BPDIR="/usr/bp"
fi
BPLOG="$BPDIR/logs.dir"
iSERIESLOG=$BPLOG/iSeries_sched_$$.log
BPRPT="$BPDIR/reports.dir"
BPSPN="$BPDIR/span.dir"
BPBIN="$BPDIR/bin"
i5BIN="$BPDIR/bin/iSeries"
i5PRO="$BPDIR/profiles.dir/iSeries"
BPARC="$BPBIN/archive"
ECHO="echo -e"
TmpCrontabFile="/tmp/crontab$$"
NewCrontabFile="/tmp/newcrontab$$"
SavCrontabFile="/tmp/savcrontab$$"
iSERIESBACKUP="$i5BIN/backup_iseries.sh -p"
stty sane



# Need to setup trap function in case user hits delete.
mkdir -p $i5PRO > /dev/null 2>&1

#########################################################################
#
# cleanup
# 
# End state routine to clean up any temporary files and exit.
#
#########################################################################
cleanup()
{
	Status=$1
	rm -f $TmpCrontabFile
	rm -f $NewCrontabFile
	rm -f /tmp/proflist$$
	rm -f /tmp/string$$
	rm -f /tmp/chklist$$
        # keep the number of iSeries backup logs from growing exponetially
        NumiSeriesLogs=`ls $BPLOG/iSeries_sched_* | wc -l`
        if [ $NumiSeriesLogs -gt 20 ]
        then
                LogCount=0
                for log in `ls -tr $BPLOG/iSeries_sched_*`
                do
                        rm -f $log
                        LogCount=`expr $LogCount + 1`
                        if [ $LogCount -gt 20 ]
                        then
                                break
                        fi
                done
        fi
	stty sane
        exit $Status
}


#########################################################################
#
# CronDaily
#
# Request from user which day of the week and what time of day to set the
# archiving schedule for.
#
#########################################################################
CronDaily()
{
        $ECHO "Enter the hour of the day when you would like the iSeries backups"
        $ECHO "to run (0-23, where 0 = midnight, 12 = noon): \c"
        read HOUR
	$ECHO "User has scheduled iSeries backup for ${HOUR}:15 daily." >> $iSERIESLOG
        case "$HOUR" in
                0|1|2|3|4|5|6|7|8|9|10|11|12|13|14|15|16|17|18|19|20|21|22|23)
                        # 3552
                        CronString="15 $HOUR \* \* \*"
                        return 0
                        ;;
                *)
                        $ECHO "Invalid hour."
                        CronDaily
                        return $?
                        ;;
        esac

}


#########################################################################
#
# WriteNewSched
# 
# Replace existing call to $BPDIR/bin/archive/archive in crontab file with
# newly built string.
#
#########################################################################
WriteNewSched()
{
	# 3296 3297 - resets the temp crontab files
	rm -f $TmpCrontabFile
	rm -f $NewCrontabFile
	crontab -l | grep -v "^#" > $TmpCrontabFile
	# 3097
	cp -f $TmpCrontabFile $NewCrontabFile

	# Test for duplicate scheduled archive jobs in cron and allow for user
	# input decision prior to writing the cronstring to crontab.
	grep -q "$CronString" $NewCrontabFile 
	if [ $? -eq 0 ]
	then
		$ECHO "This exact schedule already exists. Overwrite?  <y or n> default n: \c"
		read ANS
		case $ANS in
			Y|y|Yes|yes)
				crontab -l | grep -v "$CronString" > $TmpCrontabFile
				cp -f $TmpCrontabFile $NewCrontabFile
				;;
			*)
				return 0
				;;
		esac	
	fi
	eval echo $CronString >> $NewCrontabFile
	crontab $NewCrontabFile
	$ECHO "The following iSeries backup job has been scheduled:" >> $iSERIESLOG
	$ECHO $CronString >> $iSERIESLOG
	# 2061
	$ECHO "New crontab file registered."
	$ECHO "Press <Enter> to return... \c"
	read ans
}

##########################################################################
##
## 3550
## cron_list 
##
##########################################################################
cron_list()
{
	crontab -l | grep -v "$i5PRO" > $SavCrontabFile
	crontab -l | grep -v "^#" | grep "$i5PRO" > $TmpCrontabFile
	> /tmp/string$$
	> /tmp/chklist$$
	awk '{ print $1"|" $2"|" $3"|" $4"|" $5 $8 }' $TmpCrontabFile > /tmp/string$$
	strng=`cat /tmp/string$$`
	count=0
	for ea in $strng
	do
		count=`expr $count + 1`
		$ECHO "$count. $ea" >> /tmp/chklist$$
	done
	clear
	$ECHO "Scheduled iSeries backups:"
	$ECHO ""
	$ECHO "Profile Name\t\tType\t\tHour"
	$ECHO "------------\t\t----\t\t----"
	> /tmp/prosched$$
	count=0
	names=`cat /tmp/string$$ | awk -F"|" '{ print $NF }' | awk -F"/" '{print $NF}'`
	for each in $names
	do
		type=`$BPBIN/bputil -g iSeriesBackup BackupStrategy xxx $i5PRO/$each`
		if [ $type = "DailySelective" ]
		then
			TYPETABS="\t\t"
		else
			TYPETABS="\t"
		fi
		pnameLength=`$ECHO $each | wc -c`
		if [ $pnameLength -lt 9 ]
		then
			PNAMETABS="\t\t"
		else
			PNAMETABS="\t"
		fi
		count=`expr $count + 1`
		selectstring=`grep "^${count}." /tmp/chklist$$`
		selecthour=`cat /tmp/chklist$$ | awk -v THISREC=$count -F"|" '{if(NR == THISREC) print $2}'`
		$ECHO "$count. ${each}\t${type}${TYPETABS}${selecthour}"
		$ECHO $each >> /tmp/prosched$$
	done
}

#########################################################################
#
# 2051
# 3550
# RemoveCron
# 
# Remove to archive schedule previously created.
#
#########################################################################
RemoveCron()
{
	# 3014
	schCheck="`crontab -l | grep "$i5PRO"`"
	if [ "$schCheck" = "" ]
	then
		$ECHO "There is no iSeries backup schedule to delete."
		$ECHO "Press <Enter> to return... \c"
		read ans
	else
		${ECHO} "Delete iSeries Schedules:"
		${ECHO} "\t1.  Select Schedule"
		${ECHO} "\t2.  Delete all Schedules"
		${ECHO} "\t3.  Return"
		${ECHO} "Enter selection: \c"
		read pick
		$ECHO "Pick is $pick"
		case ${pick} in
			1|s|S|"Select"|"select")
				cron_list
				$ECHO ""
				$ECHO "Please select the number of the schedule to delete: \c"
				read choice
				case ${choice} in
				[0-9]*)
					DeleteProfile=`cat /tmp/prosched$$ |  awk -vthisfile="$choice" '{if(NR == thisfile) print $1}'`
					# 5111
					if [ ! -r "$i5PRO/$DeleteProfile" -o "$DeleteProfile" = "" ]
					then
						$ECHO "Invalid Selection. Please try again."
						sleep 2
						clear
						RemoveCron
					fi
				;;
				*)
					$ECHO "Invalid Selection. Please try again."
					sleep 2
					RemoveCron
				;;
				esac
				crontab -l | grep -v "$DeleteProfile" > $SavCrontabFile
				crontab -l | grep -v "^#" | grep "$DeleteProfile" > $TmpCrontabFile
				# find choice in /tmp/chklist$$ and build string for fgrep -v command 
				schedstr=`cat /tmp/chklist$$ | grep "${choice}. " | awk '{ print $2 }' | awk -F"|" '{print $1" " $2" " $3" " $4" " $5" "}' | awk -F"/" '{print $1}'` 
				fgrep -v "$schedstr $iSERIESBACKUP $i5PRO/$DeleteProfile" $TmpCrontabFile > $NewCrontabFile
				# Test for legacy archive cron schedules
				if [ -f $NewCrontabFile ]
				then
					cat $NewCrontabFile >> $SavCrontabFile
					crontab $SavCrontabFile
					$ECHO "The following iSeries backup job schedule has been removed:" >> $iSERIESLOG
					$ECHO "$schedstr $iSERIESBACKUP $i5PRO/$DeleteProfile" >> $iSERIESLOG
				fi
				$ECHO "iSeries schedule removed."
				$ECHO "Press <Enter> to return... \c"
				read ans
			;;

			2|d|D|"Delete"|"delete") 
				$ECHO "Are you sure you want to delete all archive schedules? <y or n> default n: \c"	
				read verify
				if [ $verify = "Y" -o $verify = "y" -o $verify = "Yes" -o $verify = "yes" ]
				then
					crontab -l | grep -v "$i5PRO" > $SavCrontabFile
					crontab $SavCrontabFile
					$ECHO "All iSeries schedules deleted."
					$ECHO "All iSeries backup jobs schedule have been removed." >> $iSERIESLOG
				else
					$ECHO "All iSeries schedules retained."
				fi
				sleep 2
				return 0
			;;
			3|e|R|"r"|"Return") 
				# 5111
				GetProfileToSched
			;;
			*)	
				RemoveCron
				;;

			"")	
				RemoveCron
				;;
		esac
	fi
	rm -f $SavCrontabFile $NewCrontabFile 2> /dev/null
}
		
#########################################################################
#
# 2784
# CronView
# 
# Display current archive schedule through cron.
#
#########################################################################

# 3297 - Allows proper formatting and adds record counts to select and display
# 	 cron archive schedules

CronView()
{
	# 3014
	schCheck=`crontab -l | grep "$i5PRO"`
	if [ "$schCheck" = "" ]
	then
		$ECHO "There is no iSeries profile scheduled to run."
	else
		cron_list
	fi
	$ECHO ""
	$ECHO "Press <Enter> to return... \c"
	read ans
}
	
#########################################################################
#
# 3297  Build CronString from partial strings and commit to cron.
#
#########################################################################
commit()
{
	CronString="$CronString $iSERIESBACKUP $iSeriesProfile"
	WriteNewSched
}
#
#########################################################################
#
# choose_profile
# 
# Allow user to select the client profile to schedule.
#
#########################################################################
choose_profile()
{
	clear
	$ECHO "iSeries Profile Selection:"
	> /tmp/proflist$$
	ProfileList=`ls $i5PRO/*.spr`
	listItem=1
	for profile in $ProfileList
	do
		$ECHO "$listItem.  $profile" 
		$ECHO "$profile" >> /tmp/proflist$$
		listItem=`expr $listItem + 1`
	done
	${ECHO} "Enter selection for profile: \c" 
	read ANS
	case ${ANS} in
	[0-9]*)
		SelectedProfile=`cat /tmp/proflist$$ |  awk -vthisfile="$ANS" '{if(NR == thisfile) print $1}'`
		# 5111
		if [ ! -r "$SelectedProfile" -o "$SelectedProfile" = "" ]
		then
			$ECHO "Invalid Selection. Please try again."
			sleep 2
			clear
			choose_profile
		else
			iSeriesProfile=$SelectedProfile
			$ECHO "You have selected profile $iSeriesProfile."
			$ECHO "Contents of this profile are:"
			more $iSeriesProfile
			CronDaily 
			return 0
		fi
		;;
	*)
		$ECHO "Invalid Selection. Please try again."
		sleep 2
		choose_profile
		;;
	esac
	return
}

##########################################################################
##
## GetProfileToSched
## 
## Allow user to select the client profile to schedule.
##
##########################################################################
GetProfileToSched()
{
	clear
	$ECHO "iSeries Backup Scheduling:"
	${ECHO} "\t1. Select Profile"
	${ECHO} "\t2. Remove iSeries Schedule"
	# 5110
	${ECHO} "\t3. View List of Scheduled Backups"
	${ECHO} "\t4. Exit"
	${ECHO} "Enter selection for schedule action: \c"
	read ANS
	case ${ANS} in
		1|p|P|"Profile"|"profile") 
			clear
			choose_profile
			commit
			GetProfileToSched
			;;
		# 2051
		2|R|r|"remove"|"Remove") 
			clear
			# 3550
			procall=0
			RemoveCron
			GetProfileToSched
			;;
		3|V|v) 
			clear
			CronView
			GetProfileToSched
			;;
		4|E|e) 
			return 1
			;;
		*)	
			GetProfileToSched
			;;
		"")	
			GetProfileToSched
			;;
	esac
}


# 1925 Need to setup trap function in case user hits delete.
trap 'cleanup $?' ERR EXIT

GetProfileToSched
cleanup $?
