<?php
require_once('/usr/bp/bin/psa/ConnectWiseDS.php');
require_once('/usr/bp/bin/psa/util.php');

/**
 * Autoload for local classes generated by the WSDL2PHP process.
 * @param string $className The class to be loaded.
 */
function autoloaderForWSDL($className)
{
    global $logHandle;
    global $debugxml;
    $className = ltrim($className, '\\');
    $fileName  = '';
    $namespace = '';
    if ($lastNsPos = strrpos($className, '\\')) {
        $namespace = substr($className, 0, $lastNsPos);
        $className = substr($className, $lastNsPos + 1);
        $fileName  = str_replace('\\', DIRECTORY_SEPARATOR, $namespace) . DIRECTORY_SEPARATOR;
    }
    $fileName .= str_replace('_', DIRECTORY_SEPARATOR, $className) . '.php';
    $fileName = realpath('/usr/bp/bin/psa/schema/' . $fileName);

    if ( $debugxml ) {
       fwrite($logHandle, "Loading ".$fileName." for class ".$className."\n");
    }
    require $fileName;
}

/**
 * Create a ticket based on the values passed.
 * @param $psa_tool_name : PSA Tool name
 * @param url : url of the PSA
 * @param $companyIntegration : Integration company name
 * @param $username : username
 * @param $password : password
 * @param $summary : summary of the ticket
 * @param $board : hardcoded to "Professional Services"
 * @param $status: hardcoded to "N" that represents New
 * @param $companyId : Company to which the service ticket should be created
 * @param $severity : severity of the notification
 * @return ticket_id : Response object which represents the service ticket if the call was successful
 * Otherwise, exception is thrown
 */
function createTicket($psa_tool_name, $url, $companyIntegration, $username, $password, $summary, $detailNotes, $companyId, $severity) {
	global $logHandle;
	global $client;
        global $debugxml;
	$doupdate = 0;
	$debugxml = get_master_ini_setting("PSA", "DebugTrace", 0);
        $now = date("Y-m-d H:i:s"); 
        if ( $psa_tool_name == "ConnectWise" ){
            $arr = getConnectWiseConfiguration($url, $companyId); 
            $wsdl = $arr['wsdl'];
            $board = $arr['board']; 
            $status = $arr['status'];
            $companyIntegration = $arr['companyLogin'];
            $companyId = $arr['companyId'];
            $apiver = $arr['apiVer'];
	    $excludefile =  $arr['exclusionFile'];
	    if ( $arr['notif'] == "0" ) $notif = FALSE;
	    else $notif = TRUE;
	    $pri = $arr['pri4'];
	    if ($severity == "1") {
	       $pri = $arr['pri2'];
	    } else if ($severity == "2") {
	       $pri = $arr['pri3'];
	    } else if ($severity == "3") {
	       $pri = $arr['pri4'];
	    }
	    if ( isExcluded($excludefile, $summary) ) {
		$emsg = sprintf("Excluding ticket with summary: %s\n",$summary);
		writeErrorAndExit($emsg);
	    }
	    fwrite($logHandle, $now. " createTicket: wsdl=".$wsdl." companyId=". $companyId . " board=" . $board .  " status=". $status. " sev=".  $severity.  " notif=".  $notif.  " debug=" .$debugxml. " summary=" . $summary . "\n");

            if ( $debugxml ) {
                $a_SoapOptions = array (
                        'encoding'  => 'UTF-8', 'exception' => 1,
                        'trace'     => 1, );
                // error_reporting(-1);
                // ini_set('display_errors', 1);
            } else {
                $a_SoapOptions = array (
                        'encoding'  => 'UTF-8', 'exception' => 1 );
            }
            spl_autoload_register('autoloaderForWSDL');

	    if ( $apiver == "1.5" ) {
	    $o_Params   = new ServiceTicketApi_AddServiceTicketViaCompanyId;
	    } else {
	    $o_Params   = new ServiceTicketApi_AddServiceTicketViaCompanyIdentifier;
	    }
	    $o_Params->credentials   = new ServiceTicketApi_ApiCredentials;
	    $o_Params->serviceTicket = new ServiceTicketApi_ServiceTicket;

	    // password from database is base64 encoded. so, decoding the password below
	    $o_Params->credentials->IntegratorPassword = base64_decode($password);
	    $o_Params->credentials->IntegratorLoginId  = $username;
	    $o_Params->credentials->CompanyId          = $companyIntegration;
	    $o_Params->companyId                       = $companyId;
	    $o_Params->serviceTicket->Summary          = $summary;
	    $o_Params->serviceTicket->Board            = $board;
	    $o_Params->serviceTicket->Status           = $status;
	    $o_Params->serviceTicket->Priority         = $pri;
	    $o_Params->serviceTicket->ProblemDescription    = $detailNotes;
	    $o_Params->serviceTicket->ProcessNotifications  = $notif;
	    // fill in required static elements
	    $o_Params->serviceTicket->TicketNumber     = 0;
    $o_Params->serviceTicket->DateReq               = '0001-01-01T00:00:00';
    $o_Params->serviceTicket->SendingSrServiceRecid = 0;
    $o_Params->serviceTicket->SubBillingMethodId    = 'Actual';
    $o_Params->serviceTicket->SubBillingAmount      = 0.00;
    $o_Params->serviceTicket->SubDateAccepted       = '0001-01-01T00:00:00';
    $o_Params->serviceTicket->SubDateAcceptedUtc    = '0001-01-01T00:00:00';
    $o_Params->serviceTicket->BudgetHours           = 0.00; // decimal
    $o_Params->serviceTicket->SkipCallback          = FALSE; // boolean
    $o_Params->serviceTicket->Approved              = FALSE; // boolean
    $o_Params->serviceTicket->ClosedFlag            = FALSE; // boolean

	if ( $apiver == "1.5" ) {
    $o_Params->serviceTicket->CustomerUpdatedFlag   = FALSE; // boolean
	} else {
    fwrite ($logHandle, "setting API 2.0 values\n");
    // New values in API 2.0 below:
    $o_Params->serviceTicket->CustomerUpdatedFlag   = FALSE; // boolean
    $o_Params->serviceTicket->EmergencyFlag         = FALSE; // boolean
    $o_Params->serviceTicket->Id  = 0;
    $o_Params->serviceTicket->SubBillingMethod      = 'Actual';
    $o_Params->serviceTicket->DateReqUtc            = '0001-01-01T00:00:00';
    $o_Params->serviceTicket->IsInSla               = FALSE; // boolean
    $o_Params->serviceTicket->OpportunityId 	    = 0;
    $o_Params->serviceTicket->AgreementId 	    = 0;
    $o_Params->serviceTicket->ServiceTeamId 	    = 0;
    $o_Params->serviceTicket->DateResolvedUTC       = '0001-01-01T00:00:00';
    $o_Params->serviceTicket->DateRespondedUTC      = '0001-01-01T00:00:00';
    $o_Params->serviceTicket->ResolveMinutes        = 0;
    $o_Params->serviceTicket->ResPlanMinutes        = 0;
    $o_Params->serviceTicket->PortalTypeId   	    = 0;
	}

	    if ( $debugxml ) {
		$a_SoapOptions = array (
			'encoding'  => 'UTF-8', 'exception' => 1, 
			'trace'     => 1, );
	    } else {
		$a_SoapOptions = array (
			'encoding'  => 'UTF-8', 'exception' => 1 );
	    }

	    try {
	       $o_client = new ServiceTicketApi($wsdl, $a_SoapOptions);
            } catch (Exception $e){
		fwrite ($logHandle, "getSoapClient(".$wsdl.") error,\n". $e->getMessage() ."\n");
		throw new Exception ("Soap error with WSDL file\n");
	    }

            $o_SoapFault = Null;
            $o_Exception = Null;
	    $response    = Null;
	    try{
		if ( $apiver == "1.5" ) {
		$response = $o_client->AddServiceTicketViaCompanyId($o_Params);
		} else {
		$response = $o_client->AddServiceTicketViaCompanyIdentifier($o_Params);
		}
	    }
	    catch(SoapFault $o_SoapFault){}
	    catch(Exception $o_Exception){}

	    if ( $debugxml ) {
              if (isset($o_client)) {
	       fwrite($logHandle, "AddServiceTicket Headers:\n". 
			$o_client->__getLastRequestHeaders()."\n");
	       fwrite($logHandle, "AddServiceTicket Request:\n". 
			$o_client->__getLastRequest()."\n");
	      } else {
	       fwrite($logHandle, "AddServiceTicket, empty client\n");
	      }
	    }

	    if (isset($o_SoapFault)) {
	        fwrite($logHandle, "AddServiceTicket Fault: ". $o_SoapFault->getMessage() ."\n");
	        throw new Exception ( $o_SoapFault->getMessage() );
	    }
	    if (isset($o_Exception)) {
	        fwrite($logHandle, "AddServiceTicket Exception: ". $o_Exception->getMessage() ."\n");
	        throw new Exception ( $o_Exception->getMessage() );
	    }

            $ticket_num = 0;
	    if (isset($response)) {
	      if ( $apiver == "1.5" ) {
                $ticket_num = $response->AddServiceTicketViaCompanyIdResult->TicketNumber;
	      } else {
                $ticket_num = $response->AddServiceTicketViaCompanyIdentifierResult->TicketNumber;
	      }
	    }
            return $ticket_num; 
        } else{
            throw new Exception ("Unsupported PSA Tool\n");
        }	    		
}

    function getConnectWiseConfiguration ($url, $companyId){
	global $logHandle;
        $arr = array();;
        $arr['wsdl'] = get_master_ini_setting("PSA", "WSDLLocation","https://".$url."/v4_6_release/apis/1.5/ServiceTicketApi.asmx?wsdl");
        $arr['board'] = get_master_ini_setting("PSA", "Board","Professional Services");
        $arr['status'] = get_master_ini_setting("PSA", "Status", "N");
        $arr['companyId'] = get_master_ini_setting("PSA", "ClientCompanyId", $companyId);
        $arr['companyLogin'] = get_master_ini_setting("PSA", "LoginCompanyId", $companyId);
        $arr['apiVer'] = get_master_ini_setting("PSA", "CWApiVersion", "1.5");
        $arr['exclusionFile'] = get_master_ini_setting("PSA", "ExclusionListFile", "");
        $arr['pri1'] = get_master_ini_setting("PSA", "Priority1", "Priority 1 - Emergency Response");
        $arr['pri2'] = get_master_ini_setting("PSA", "Priority2", "Priority 2 - Quick Response");
        $arr['pri3'] = get_master_ini_setting("PSA", "Priority3", "Priority 3 - Normal Response");
        $arr['pri4'] = get_master_ini_setting("PSA", "Priority4", "Priority 4 - Scheduled Maintenance");
        $arr['notif'] = get_master_ini_setting("PSA", "ProcessNotifications", "0");
	fwrite($logHandle, "getConfig: board=" . $arr['board'] . " status=". $arr['status'] . " notif=". $arr['notif'] . " pri4=". $arr['pri4'] . "\n");
        return $arr;        
    }
    
    // scan file and match with summary
    function isExcluded ($efile, $summary){
	global $logHandle;
	$match = false;
	if (empty($efile) || empty($summary)) return $match;
	fprintf($logHandle,"excludefile=%s\n",$efile);
	// generates warning if file not found 
	$fh = fopen($efile, "r");
	$done = 0; 
	if ( $fh == FALSE ) $done = 1;
	while ( $done <= 0 ) {
 	   $line = fgets( $fh );
	   if ( $line == FALSE ) $done = 1;
	   else {
	      $str = strstr($line, $summary);
	      if ( $str != FALSE ) {
	 	 $done = 1;
		 $match = true;
	      }
	   }
        }
	return $match;
    }
    
    function getSoapClient ( $wsdl ){
	global $logHandle;
	$cookie = 'unitrends';
        $retryCount = get_master_ini_setting("PSA", "RetryCount", 2);
        $trace = get_master_ini_setting("PSA", "DebugTrace", 0);
        $waitTime = NULL;
        while ( $retryCount > 0 ) {
            try {
                // if there is an issue, it throws an exception. 
		if ($trace == 0) {
		   $newclient = @new SoapClient($wsdl);
		} else {
		   // $newclient = @new SoapClient($wsdl, array('trace' => 1));
		   $newclient = @new SoapClient($wsdl, array( 'encoding'  => 'UTF-8', 'exception' => True, 'trace' => True, 'display_errors' => 1 ));
		   error_reporting(-1);
		   ini_set('display_errors', 1);
		}
		$newclient->__setCookie('cw-app-id', $cookie);
                return $newclient;
            } catch (Exception $e){
                global $type; 
                $retryCount--;  
                // if the retry count is zero or if the type of request is test,
		// then throw the exception right away
                if ( $retryCount == 0 || $type == "test" ){
                    // throw $e;
		    fwrite ($logHandle, "getSoapClient(".$wsdl.") error,\n". $e->getMessage() ."\n");
		    throw new Exception ("Soap error with WSDL file\n");
                }
                
                if ( $waitTime == NULL){
                    $waitTime = get_master_ini_setting("PSA","BaseDelay",5);
                }
                
                sleep($waitTime);                                                            
            }
        }
    }    
?>
