#!/usr/bin/php
<?php

// This program can be run standalone -- it generates the report to the
// file $BPDIR/reports.dir/schedule_report.html, and sends the email to
// recipients configured in the ScheduleSummaryMailto list in master.ini.
// It is normally run by alertman at the same time as the daily System/Vault
// reports are sent (ReportHourMin setting in the Alertman section).

$debug=false; // set this to true to get debug printout
$dbconn_string = "port=5432 dbname=bpdb user=postgres";
$bpdir = trim(shell_exec('eval `grep BPDIR /etc/default/bp.ini`; echo $BPDIR'));

// The mail_report function
require("$bpdir/bin/mail_report.php");

// Make an array of strings with boolean values of "true"
$true_value_array = array(0 => "true", 1 => "True", 2 => "TRue", 3=> "TRUe", 4=> "TRUE", 5=>"1", 
					6 => "On", 7=> "ON", 7=> "on", 9=> "Yes", 10=> "YEs", 11=> "YES", 
					12=> "yes", 13=> "y", 14=> "Y");

// These are app-specific column headings for the object names
$objcol = array(
	"Exchange" => "Storage Group or Database Name",
	"SQL Server" => "SQL Server Instance/Database Name",
	"VMware" => "ESX Server/Virtual Machine",
	"Hyper-V" => "Virtual Machine",
	"file-level" => "Client Name",
	"Oracle" => "Oracle Instance",
	"SharePoint" => "SharePoint Farm",
	"UCS Service Profile" => "UCS Service Profile",
	"Volume" => "Volume");

$dbconn = pg_connect($dbconn_string);
if($dbconn === FALSE) {
	print "ERROR: Failed to connect to the database.\n";
	die(1);
}

// Get the schedules that should be in the report
$query = "SELECT schedule_id, a.type AS app_type ";
$query .= "FROM bp.schedules AS s ";
$query .= "JOIN bp.application_lookup AS a USING(app_id) ";
$query .= "WHERE enabled=true AND email_report=true ";
$query .= "ORDER BY s.name";
if ($debug) print $query."\n";
$res = pg_query($dbconn, $query);
if ($res === false) {
	print "ERROR: failed to access schedules table:\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}

// Don't generate the report unless there is at least one schedule with
// reporting enabled.
if (pg_num_rows($res) == 0) {
	print "There are no schedules with the Schedule Report enabled.\n";
	pg_free_result($res);
	exit(0);
}

// Get the System name and version for the general info section.
$query = "SELECT name, version FROM bp.systems WHERE is_local_system=true";
$vres = pg_query($dbconn, $query);
if ($vres === false) {
	print "ERROR: failed to retrieve version from systems table:\n";
	print pg_last_error($dbconn)."\n";
	die(1);
}
$vers = pg_fetch_result($vres, 0, "version");
$dpu = pg_fetch_result($vres, 0, "name");
pg_free_result($vres);

$tmpfile = tempnam($bpdir."/reports.dir", "schedrpt");
if (!$fp = fopen($tmpfile, "w")) {
	echo "An error occurred opening $tmpfile for writing.\n";
	die(1);
}
if ($debug) print "temporary file name is: ".$tmpfile."\n";

$nschedules = 0;
$nfailed = 0;

$now = time();
// 24 hours ago;
$begin = $now - (24 * 3600);
// strip the minutes/seconds to get the hour (if the report is running some
// minutes after the hour, we want to start reporting at the top of the
// hour ((24 hours ago)) e.g., 8:12=>since 8:00 yesterday; 9:57=>since 9:00
$begin = round($begin/3600) * 3600 ;
if ($debug) {
	printf("=== now=%d (%s)\n=== begin=%d (%s)\n",
			$now, date("r", $now), $begin, date("r", $begin));
}
$input = array("start_time" => $begin);
bp_bypass_cookie(3, 'schedule_report');


$useHtml = bp_get_ini_value('Unattended Backup Information',
		'NotificationsUseHtml');

$useHtml = in_array($useHtml, $true_value_array);

if($useHtml == true){
	/*------------------------------------------------------
	 * Define the CSS styles, which are strings inserted into tags below.
	 * The php variables help clean up the style stuff for the tags below
	-------------------------------------------------------- */

	$idFooter = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;"'; 
	$classCell = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' .
			' border:1px solid #bbbbbb; font-size:.8em;padding:.2em 0" ';
	$idMain = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' .
			' margin:0 auto 0;" ';
	$tagHR = ' style="margin:3px 10px; border-color:#cccccc;border-style:solid;border-width:1px 0 0; clear:both; height:0;" ';
	$classBody = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' . 
			'font-size:1em;letter-spacing:1px;background-color:#bbbbbb; margin:.3em;" align="left"';
	$classColhead = ' style=" font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' .
			' letter-spacing: 1px; background-color: #bbbbbb; margin: .2em; font-size: 1em;" ';
	$classSubtabletitle = ' style="padding:.3em 1.5em;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial,' . 
			' sans-serif;letter-spacing:1px;background-color:#bbbbbb;" ';
	$tagH3 = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; font-size: 1.2em;" ';
	$classFooterp = ' style="text-align:left;font:.9em;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;" ';
	$classTitlecell = ' style="font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;' .
			' letter-spacing:1px; background-color:#999999; 720px; padding:0 3em 1.5em;text-align: center" ';
	$tagTable = ' style="border-collapse:collapse;" ';
	$superTableWidth = 'width="720px" style="width:720px"';
	$superStyleTableWidth = ' width="720px"';
	$bgColorGray = 'bgcolor="#bbbbbb" ';
	$colSpan2 = 'colspan="2"';
	$emptyCell = 'bgcolor=\"#ffffff\" style=\"background-color:#fff;padding:.25em;font-size:1px;\"';
} else {
	/*-----------------------------------------------------
	 * Remove CSS styles and replace values with simple HTML mode
	-------------------------------------------------------*/
	$idFooter = '';
	$classCell = '';
	$idMain = '';
	$tagHR = ' ';
	$classBody = ' ';
	$classColhead = ' ';
	$classSubtabletitle = ' ';
	$tagH3 = ' ';
	$classFooterp = '';
	$classTitlecell = ' ';
	$tagTable = '';
	$superTableWidth = ' width="100%" ';
	$superStyleTableWidth = ' width="100%"';
	$bgColorGray = ' ';
	$colSpan2 = 'colspan="2"';
	$emptyCell = '';
}


// Loop through the schedules and get event details from schedule_history
while ($item = pg_fetch_assoc($res)) {
	if ($debug) {
		printf("In schedules loop, the sched variable is %s\n", 
				isset($sched)? "set" :"unset");
		printf("Processing schedule_id %d, app_type: %s\n", 
				(int)$item["schedule_id"], $item["app_type"]);
	}

	if ($item["app_type"] == "Archive") {
		// We are not including these in this report, archive schedules
		// send their own email report when "email_report" is enabled.
		continue;
	}
	// Get the schedule history
	$input["schedule_id"] = (int)$item["schedule_id"];
	$ret = bp_get_schedule_history($input);
	if ($ret === false) {
		print "bp_get_schedule_history returned FALSE:\n";
		print "Error is: ".bp_error()."\n";
		die(1);
	}
	if (empty($ret[0]["backups"])) {
		if ($debug) {
			printf("Schedule %d did not have any events.\n",
					(int)$item["schedule_id"]);
		}
		continue;
	}

	$nschedules++;
	$nfailed += $ret[0]["failures"];
	$sched = $ret[0];

	// Construct the schedule information section.
	$str ="	  <table $superTableWidth style=\"$superStyleTableWidth\">\n";
	$str .="		<tr>\n";
	$str .="		  <td>\n";
	$str .= "     		 <div $tagH3><strong>Schedule events for schedule '".$sched["schedule_name"]."'</strong></div>\n";
	$str .= "		  </td>\n";
	$str .= "		</tr>\n";
	
	if ($sched["app_type"] != "file-level") {
		$str .="        <tr>\n";
		$str .="          <td><strong>Client name:  </strong>".$sched["client_name"]."</td>\n";
		$str .="        </tr>\n";
	}
	$str .="        <tr>\n";
	$str .="          <td $classBody $bgColorGray><strong>Application name:  </strong>".$sched["application_name"]."</td>\n";
	$str .="        </tr>\n";
	$str .="        <tr>\n";
	$str .="          <td $classBody $bgColorGray><strong>Description:  </strong>".$sched["schedule_description"]."</td>\n";
	$str .="        </tr>\n";
	$str .="        <tr>\n";
	$str .="          <td $classBody $bgColorGray><strong>Failures:  </strong>".$sched["failures"]."</td>\n";
	$str .="        </tr>\n";
	$str .="      </table>\n";

	fprintf($fp, "%s", $str);

	// These are the column headings for the table of events that
	// occurred during the reporting period.
//	$colhead ="      <thead>\n";
	$colhead ="        <tr>\n";
	$colhead .="          <td width=\"22%\" " . $classColhead . " ><strong>".$objcol[$sched["app_type"]]."</strong></td>\n";
	$colhead .="          <td width=\"7%\" " . $classColhead . " ><strong>Id</strong></td>\n";
	$colhead .="          <td width=\"11%\" " . $classColhead . " ><strong>Backup Type</strong></td>\n";
	$colhead .="          <td width=\"4%\" " . $classColhead . " ><strong>V</strong></td>\n";
	$colhead .="          <td width=\"7%\" " . $classColhead . " ><strong>Files</strong></td>\n";
	$colhead .="          <td width=\"9%\" " . $classColhead . " ><strong>MBs</strong></td>\n";
	$colhead .="          <td width=\"40%\" " . $classColhead . " ><strong>Status</strong></td>\n";
	$colhead .="        </tr>\n";
//	$colhead .="      </thead>\n";

	// The rows correspond to each backup.
	foreach($sched["backups"] as $event => $bkups) {

		$tableHead ="		  <table $superTableWidth style=\"$superStyleTableWidth border-collapse:collapse;\" > \n";
		$tableHead .="			<tr>\n";
		$tableHead .="			  <td $tagH3  colspan=\"7\">\n";
		$tableHead .="				 " . date("g:i A", $event) . " Backups \n";
		$tableHead .="			  </td>\n";
		$tableHead .="			</tr>\n";
		// The new event title (level 4 header)
		fprintf($fp,"%s",$tableHead);
		// The column titles
		fprintf($fp, "%s", $colhead);

		foreach($bkups as $row) {
			// Construct the row values for each backup.
			// Add code here to handle new application-specific name types.
			$name = $row["primary_name"];
	
			if ($sched["app_type"] == "SQL Server") {
				$name .= "/".$row["secondary_name"];
			}
			if ($sched["app_type"] == "VMware") {
				$name .= ", VM ".$row["secondary_name"];
			}
			if ($sched["app_type"] == "UCS Service Profile") {
				$name = $sched["client_name"];
				$name .= "/".$row["primary_name"];
			}
			if ($sched["app_type"] == "Volume") {
				$name = $sched["client_name"];
				$name .= "/".$row["primary_name"];
			}
		
			// Backup types - these are the strings defined by
			// bp_get_backup_status
			switch ($row["type"]) {
				case "master":
					$bkup_type = "Master";
					break;
				case "exchange full":
				case "hyperv full":
				case "mssql full":
				case "vmware full":
				case "oracle full":
				case "sharepoint full":
				case "ucs service profile full":
				case "ndmp full":
					$bkup_type = "Full";
					break;
				case "differential":
				case "exchange differential":
				case "hyperv differential":
				case "mssql differential":
				case "vmware differential":
				case "sharepoint diff":
				case "ndmp diff":
					$bkup_type = "Differential";
					break;
				case "exchange incremental":
				case "hyperv incremental":
				case "vmware incremental":
				case "oracle incr":
				case "ndmp incr":
				case "incremental":
					$bkup_type = "Incremental";
					break;
				case "mssql transaction":
					$bkup_type = "Transaction Log";
					break;
				case "selective":
					$bkup_type = "Selective";
					break;
				case "baremetal":
					$bkup_type = "BareMetal";
					break;
				default:  //just in case
					$bkup_type = "Unknown";
					break;
			}

			// In case there are no keys, use job type as the name
			if (!isset($name)) {
				$name = $bkup_type;
			}

			// Set the Backup Id.
			if (isset($row["backup_id"])) {
				$bkup_no = $row["backup_id"];
				if (isset($row["files"]) && $sched["app_type"] == "file-level"){
					$files = $row["files"];
				}
				else {
					$files = "N/A";
				}
				if (isset($row["size"])) {
					$size = $row["size"];
				}
				else {
					$size = "N/A";
				}
			}
			else {
				// It's possible there isn't a backup number,
				// for example, if the job failed to queue.
				$bkup_no = $files = $size = "N/A";
			}

			// Set the verify status letter
			switch($row["verify_status"]) {
				case 0: // not applicable
					$vf = '-';
					break;
				case 1: // verify successful
					$vf = 'S';
					break;
				case 2: // verify failed
					$vf = 'F';
					break;
				case 4: // verify in progress
					$vf = 'A';
					break;
				default:
					$vf = '-';
					break;
			}

		
			$status = $row["description"];
			if ($row["failed"] == true) {
				$status="<span style = \"color:#c00000\"><strong>".$status."</strong></span>";
			}

			// Compose the row with the values
			$str = "        <tr>\n";
			$str .="          <td $classCell>".$name."</td>\n";
			$str .="          <td $classCell>".$bkup_no."</td>\n";
			$str .="          <td $classCell>".$bkup_type."</td>\n";
			$str .="          <td $classCell>".$vf."</td>\n";
			$str .="          <td $classCell>".$files."</td>\n";
			$str .="          <td $classCell>".$size."</td>\n";
			$str .="          <td $classCell>".$status."</td>\n";
			$str .="        </tr>\n";

			// print the row for this backup
			fprintf($fp, "%s", $str);
		}
		// close the table for this event
		
		fprintf($fp, "      </table>\n");

	}
	// print the divider
	if($useHtml == true){
		$temp = "	<table $superTableWidth style=\"$superStyleTableWidth\">\n";
		$temp .= "		<tr>\n";
		$temp .= "			<td style=\"padding:.5em;\" colspan=\"7\">\n";
		$temp .= "				<div align=\"center\"><img src=\"http://www.unitrends.com/reports/notifications/hr.png\"></div>\n";
		$temp .="			</td>\n";
		$temp .= "		</tr>\n";
		$temp .= "	</table>";
	} else {
		$temp = "	<hr />";
	}
		fprintf($fp, "%s", $temp);
	unset($sched);
}
		
pg_free_result($res);

if ($nschedules == 0) {
	// There were no events processed
	echo "No schedule events occurred during the report period.\n";
	unlink($tmpfile);
	die(0);
}



// Standard footer text
if($useHtml == true){
	$str = "	<table $superTableWidth $idFooter>\n";
		$str .= '			<tr style="border:0;">' . "\n";
		$str .= '				<td style ="padding:.4em;"> ' . "\n";
	$asset = trim(shell_exec($bpdir . "/bin/cmc_command run 'Asset Tag'"));
	if (isset($asset) && !empty($asset)){
		$str .= '					<p style="color:#555555; font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;word-spacing:1px;"> ' . "\n";
		$str .= '						Asset Tag: ' . $asset . '</p>' . "\n";
	}
		$str .= '					<p style="color:#555555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif; text-align:left;font-size:80%;word-spacing:1px;"> ' . "\n";
		$str .= '						Schedule Report generated: ' . date("F j, Y, g:i a") . "\n";
		$str .= '					<p style="color:#555555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;text-align:left;font-size:80%;word-spacing:1px;"> ' . "\n";
		$str .= '						Archive schedules are not included in this report.  The archiving subsystem delivers a separate report upon job completion.' . "\n";
		$str .= '					</p>' . "\n";
		$str .= '					<p style="color:#555555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;text-align:left;font-size:80%;word-spacing:1px;"> ';
		$str .= '						Software version: ' . $vers . " </p>\n";
		$str .= '					<p style="color:#555555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;text-align:left;font-size:80%;word-spacing:1px;"> ' . "\n";
		$str .= '						Report preferences can be set through the user interface via <strong>Settings -> Clients, Networking and Notifications</strong>' . "\n";
		$str .= '					</p>' . "\n";
		$str .= '					<p style="color:#555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;text-align:left;font-size:80%;word-spacing:1px;"> ' . "\n";
		$str .= '						Additional resources are available at the following:' . "\n";
		$str .= '					</p>';
		$str .= '						<ul style="display:block;">' . "\n";
		$str .= '							<li style="color:#555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;text-align:left;font-size:80%;word-spacing:1px;"><a href="http://www.unitrends.com/documents/appliance-agent-user-manual.pdf">Manual</a></li>' . "\n";
		$str .= '							<li style="color:#555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;text-align:left;font-size:80%;word-spacing:1px;"><a href="http://support.unitrends.com/ikm/">Knowledge Base</a></li>' . "\n";
		$str .= '							<li style="color:#555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;text-align:left;font-size:80%;word-spacing:1px;"><a href="http://www.unitrends.com/support/customer-care-center/video-tutorials.html">Training Videos</a></li>' . "\n";
		$str .= '						</ul>' . "\n";
		$str .= '				</td>' . "\n";
		$str .= '				<td style = "padding:.2em;"> ' . "\n";
		$str .= '					<p style="color:#555;font-family: Calibri, Candara, Segoe, \'Segoe UI\', Optima, Arial, sans-serif;text-align:left;font-size:80%;"> Connect with us. <br />' . "\n";
		$str .= '						<a href="http://community.spiceworks.com/pages/unitrends"><img float="right"src="http://www.unitrends.com/reports/spiceworks.jpg"';
		$str .= '						border="0" alt="Spiceworks" width="42" height="42" style="margin: 3px 3px 0px 3px;" /></a><a href="http://www.facebook.com/unitrends">';
		$str .= '						<img float="right"src="http://www.unitrends.com/reports/facebook.jpg" border="0"alt="Be our friends on Facebok" width="42" height="42" style="margin:';
		$str .= ' 					3px 3px 0px 3px;" /></a><br /><a href="http://www.linkedin.com/company/25406?trk=tyah"><img float="right"src="http://www.unitrends.com/reports/linkd-in.jpg" border="0" alt="Linked In"width="42" height="42" style="margin: 3px 3px 0px 3px;" /></a><a href="http://twitter.com/#!/Unitrends"><img float="right"src="http://www.unitrends.com/reports/twitter.jpg" border="0" alt="Twitter" width="42" height="42" ';
		$str .= '						style="margin: 3px 3px 0px 3px;" /></a><a href="http://www.youtube.com/user/UnitrendsInc"><img float="right"src="http://www.unitrends.com/reports/you-tube.jpg" border="0"alt="You Tube" width="42" height="42" style="margin: 3px 3px 0px 3px;" /></a>' . "\n";
		$str .= '					</p>';
		$str .= '			</tr>' . "\n";
	$str .= '		</table>';
} else {
	$asset = trim(shell_exec($bpdir . "/bin/cmc_command run 'Asset Tag'"));
	if (isset($asset) && !empty($asset)){
		$str = '					<p align="left"> System Version '. $vers . ', Asset Tag: ' . $asset . '</p>' . "\n";
	}
}
fprintf($fp, "%s", $str);

fprintf($fp, " </div> </body>\n</html>\n");
fclose($fp);

/////////////////////////////////////////

$rptfile = $bpdir . "/reports.dir/schedule_report.html";
if (!$rptfp = fopen($rptfile, "w")) {
	echo "An error occurred opening $rptfile for writing.\n";
	unlink($tmpfile);
	die(1);
}

// Construct the report general info section.
if ($nfailed == 0) {
	$fstr = "<span style = \"color:green\"><strong><em>There were no failures.</em></strong></span>";
}
else {
	$fstr = "<span style = \"color:#c00000\"><strong><em>There ";
	if ($nfailed == 1) {
		$fstr .= "was one failure.</em></strong></span>";
	}
	else {
		$fstr .= "were ".$nfailed." failures.</em></strong></span>";
	} 
}

//Generates HTML header, logo, title, number of failures across schedules and prepends to rest of contents generated above
$str ='<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"><html xmlns="http://www.w3.org/1999/xhtml"> ' . "\n" . '<head><meta type="text/css"></meta>' . "\n";
$str .="    <title>SCHEDULE REPORT</title>\n";
$str .="  </head>\n";
$str .="  <body>\n";
if($useHtml == true){

	$str .="    <div align=\"center\" >";

	$str .= '		<table style="border:0; border-collapse: collapse; border-spacing: 0" ' . 
						$superStyleTableWidth . '" ' . $superTableWidth . ' cellpadding="0" ' .  
						$colSpan2 . ' cellspacing="0">'. "\n";
	$str .= '			<tr style="border:0;"> ' . "\n";
	$str .= '				<td style="border:0;padding:1.5em 2em;"> ' . "\n";
	$str .= '						<img src="http://www.unitrends.com/reports/color/logo.png" align="left" width="209px" height="37px">' . "\n";
	$str .= ' 				</td>' . "\n";
	
	$str .= '			</tr>';
	$str .= '	  <tr >' . "\n";
	$str .= '		<td $colSpan2 style="margin:0px; border:0px;"> <img src="http://www.unitrends.com/reports/notifications/roundedrectangletop.png" width="720px" height="26px" style="vertical-align:bottom;margin:0px 0px -2px;padding:0px;float:left;"></td>';
	$str .="      </tr>\n";
	$str .="      <tr>\n";
	$str .= "		 <td $colSpan2 $classTitlecell align=\"center\" bgcolor=\"#999999\">";
	$str .="          <strong>SCHEDULE REPORT</strong>";
	$str .="        </td>\n";
	$str .="      </tr>\n";
} else {
	$str .='    <table style="border:0; border-collapse: collapse; border-spacing: 0" ' .
                                                $superStyleTableWidth . '" ' . $superTableWidth . ' cellpadding="0" ' .
                                                $colSpan2 . ' cellspacing="0">'. "\n";
	$str .="      <tr>\n";
	$str .= "		 <td $colSpan2 $classTitlecell align=\"center\">";
	$str .="          <strong>SCHEDULE REPORT</strong>";
	$str .="        </td>\n";
	$str .="      </tr>\n";
}
$str .="      <tr>\n";
$str .="        <td $classBody $bgColorGray>Schedule activity for 24 hours prior to:  ";
$str .="          <strong>".date(DATE_RSS)."</strong></td>\n";
$str .="      </tr>\n";
$str .="      <tr>\n";
$str .="        <td $classBody $bgColorGray>System name:  <strong>".$dpu."</strong></td>\n";
$str .="      </tr>\n";
$str .="      <tr>\n";
$str .="        <td $classBody $bgColorGray>".$fstr."</td>\n";
$str .="      </tr>\n";
$str .="      <tr>\n";
$str .="        <td $classBody $bgColorGray>Verify status (V column): S=Success, F=Fail, A=Active, -=Not Applicable</td>\n";
$str .="      </tr>\n";
if($useHtml == true){
	$str .="	  <tr > \n";
	$str .='		 <td $colSpan2 style="padding:.5em 0;"><div align="center"> <img src="http://www.unitrends.com/reports/notifications/hr.png"> </div></td>' . "\n";
	$str .="	  </tr>\n";
	$str .="    </table>\n";
} else {
	$str .="    </table>\n";
	$str .="	<hr /> \n";
}


fprintf($rptfp, "%s", $str);
fclose($rptfp);

// read the temporary file and append to the report
$fret = file_put_contents($rptfile, file_get_contents($tmpfile), FILE_APPEND);

// clean up the temporary file
unlink($tmpfile);

if ($fret === false) {
	printf("Schedule report failed to create output file %s\n", $rptfile);
	die(1);
}

$report_addr = bp_get_ini_value('Unattended Backup Information',
		'ScheduleSummaryMailto');
if ($report_addr !== false && !empty($report_addr)) {
	$report = file_get_contents($rptfile);
	if (isset($report)) {
		$subject = $dpu." Schedule Report ";
		$from = "Schedule Report@".$dpu;
		
		// create PDF if needed
		$systemID = bp_get_local_system_id();
		$pdfFilename = $dpu."-ScheduleReport.pdf";
		$pdfFile = createPDF($systemID, $rptfile, $pdfFilename);
		
		// empty array of images
		$img = array();
		mail_report($from, $report_addr, $subject, $report, $img, $pdfFile);
		
		clearTempReports($pdfFile);
	}
}
else {
	printf("No email recipients are configured (ScheduleSummaryMailto in master.ini).\n");
}

bp_destroy_cookie();
printf ("Schedule Report processed %d schedules and found %d failure(s).\n", 
	$nschedules, $nfailed);
exit(0);

// Returns full path to PDF file upon success
// Returns false upon failure or if PDF is not to be created
function createPDF($systemID, $htmlFilePath, $pdfFilename) {
	// load pdf conversion class
	$rootdir = "/var/www/html/recoveryconsole";
	$pdfScript = $rootdir . '/system/pdf_conversion.php';
	// if this method is called more than one time - it will fail without require_once
	require_once $pdfScript;
	$pdfConversion = new PDFConversion();
	
	// create PDF file if necessary
	$result = $pdfConversion->createPDF($systemID, $htmlFilePath, $pdfFilename);
	return $result;
}

// Deletes PDF reports created
function clearTempReports($pdfFile) {
	if ($pdfFile !== false && file_exists($pdfFile)) {
		unlink($pdfFile);
	}	
}

?>
