#!/usr/bin/php
<?php

function exit_func($io) {
	echo "Exiting...\n";
	fclose($io);
	$res = bp_destroy_cookie();
	exit;
}

function get_media_selection($media, $fp) {
	echo "Select the media by entering the number corresponding to the name\n.";
	echo "Or, enter 'q' to exit.\n\n";
	foreach($media as $key => $name) {
		echo $key, " [", $name['name'], "]\n";
	}
	echo "Selection: ";
	$line = fgets($fp);
	if (trim($line) === "q") {
		return "";
	} else {
		$lineVal = intval($line);
		if ($lineVal >=0 && $lineVal < count($media)) {
			$requiredName = $media[$lineVal]['name'];
			return $requiredName;
		} else {
			echo "Invalid selection. No media available for selection:", $lineVal,  "\n";
			return "";
		}
	}
}


/* Main program starts here */

$res = bp_bypass_cookie(3);
if ($res === false) {
	echo "Bypass cookie failed\n";
	var_dump(bp_error());
	exit;
}

$io = fopen('php://stdin', 'r');

echo "Beginning selection process for transfering off-premise data to archive drive\n";
$res = bp_get_vaulted_dpus();
foreach ($res as $key => $dpu) {
	echo $key, " [", $dpu, "]\n";
}

echo "Select the appliance you want to pick clients from. Enter number corresponding to the system name.\n";
echo "Selection: ";
$selection =  fgets($io);
$selVal = intval($selection);
if ($selVal < 0 || $selVal >= count($res)) {
	Echo "Invalid selection. Relaunch to start over\n";
	exit_func($io);
}
$syncUser = $res[$selVal];
echo "Selected appliance:", $syncUser, "\n";

$dpu = bp_get_dpu_name($syncUser);
if ($dpu === false) {
	var_dump(bp_error());
	exit_func($io);
}

$transformList = array();

/* Need to check if DR is active. dpu_clients.rpt required for DR can 
 * be overwritten otherwise */
$res = bp_dr_in_progress($syncUser);
if ($res === false) {
	var_dump(bp_error());
	exit_func($io);
}
if ($res === "yes") {
	echo "Disaster recovery is in progress for $syncUser. Cannot continue at this time. Please wait for the recovery process to complete.\n";
	exit_func($io);
}
$clientList = bp_get_dpu_clients($syncUser, $dpu);
if ($clientList === false) {
	var_dump(bp_error());
	exit_func($io);
}
if (count($clientList) == 0) {
	echo "No client data is available for the selected appliance.\n";
	exit_func($io);
}

echo "Number of clients available (including Localdir) ", count($clientList), "\n";

echo "\n\nClient List:\n";
foreach($clientList as $key => $client) {
	if (trim($client) === "LocalDir") {
		unset($clientList[$key]);
		continue;
	}
	echo $key, " [", $client, "]\n";
}
echo "\n";
echo "Select clients to transform. [Enter 'a' for selecting all clients or 'i' to select individual clients']\n";
echo "Select 'a' if client list is empty and local dir is available\n";
echo "Selection (Enter 'a' or 'i'): ";
$selection = fgets($io);
if(trim($selection) === "a") {
	$transformList = $clientList;
	foreach ($clientList as $key => $client) {
		$transformListIndex[] = $key;
	}
} else if(trim($selection) === "i") {
	echo "Select individual clients by entering the number corresponding to each name.\n";
	echo "When selection of clients is complete, enter 'q'.\n";
	$number = 0;
	echo "Enter client number: ";
	while($selection = fgets($io)) {
		if (trim($selection) === "q") {
			break;
		}
		$selVal = intval($selection);
		if (trim($selection) !== "0" && $selVal == 0) {
			echo "Invalid selection. Enter a number corresponding to a client name.\n";

		} else {
			if ($selVal >= 0 && $selVal < count($clientList)) {
				if (!in_array($clientList[$selVal], $transformList)) {
					$number++;
					$transformList[] = $clientList[$selVal];
					$transformListIndex[] = $selVal;
				} else {
					echo "Client already selected\n";
				}

			} else {
				echo "Invalid selection. Enter a number corresponding to a client name.\n";
			}
		}
		if ($number < count($clientList)) {
			echo "Enter client number (or enter 'q' when selection is complete): ";
		} else if ($number == count($clientList)){
			echo "All clients selected. Enter 'q': ";
		}
	}
} else {
	echo "Incorrect option chosen. Either 'a' or 'i' has to be entered to select clients.\n";
	echo "Restart the application to start over\n";
	exit_func($io);
}

#var_dump($transformList);
#var_dump($transformListIndex);

/* Clients selected by now */

/* Check for local dirs */
$localdirs = array();
if (file_exists("/backups/$syncUser/syncd/$dpu/archives/lastLocalDir-v2")) {
	$localdirs[] = "/backups/$syncUser/syncd/$dpu/archives/lastLocalDir-v2";
}if (file_exists("/backups/$syncUser/syncd/$dpu/archives/lastLocalDir")) {
	$localdirs[] = "/backups/$syncUser/syncd/$dpu/archives/lastLocalDir";
}
if (file_exists("/backups/$syncUser/syncd/$dpu/localDir")) {
	$localdirs[] = "/backups/$syncUser/syncd/$dpu/localDir";
}

if (!count($transformList) && !count($localdirs)) {
	echo "No clients selected. Nothing to do.\n\n";
	exit_func($io);
}

/* Select Archive drive */
$res = bp_get_uarchive_status();
if ($res === false) {
	echo "Error retrieving status of uarchive daemon.\n";
	var_dump(bp_error());
	exit_func($io);
} else if ($res === "off") {
	/* start uarchive daemon */
	$res = bp_start_uarchive_daemon();
	if ($res == false) {
		echo "uarchive daemon is not running. Attempt to start failed";
		var_dump(bp_error());
		exit_func($io);
	}
}

$res = bp_get_connected_archive_media();
if ($res === false) {
	echo "No archive media is connected. Cannot proceed.\n";
	var_dump(bp_error());
	exit_func($io);
}

$selection = get_media_selection($res, $io);
if (!(strlen($selection))) {
	echo "A valid media should be selected to transfer vaulted data.\n";
	echo "Relauch to start over\n";
	exit_func($io);
}

/* build an archive_profile */
$profile = array();
$profile['description'] = "Export Vault Data to Archive";
$profile['target'] = $selection;
$profile['range_end'] = 0;
$profile['range_size'] = 0;
$profile['clients'] = $transformListIndex;
$profile['localdirs'] = $localdirs;
$types = array();
$types[] = "master";
$types[] = "differential";
$types[] = "baremetal";
$types[] = "mssql full";
$types[] = "mssql differential";
$types[] = "mssql transaction";
$types[] = "exchange full";
$types[] = "exchange differential";
$types[] = "vmware full";
$types[] = "vmware differential";
$types[] = "vmware incremental";
$types[] = "hyperv full";
$types[] = "hyperv incremental";
$profile['types'] = $types;

$options = array();
$options['append'] = true;
$options['false'] = false;
$options['email_report'] = true;

$profile['options'] = $options;


#var_dump($profile);
$res = bp_transfer_vault_data_to_archive($profile, $transformList, $syncUser, $dpu);
if ($res === false) {
	var_dump(bp_error());
	exit_func($io);
}
echo "Archive jobs are queued. See Job Queue for details\n\n";

/* cookie destroyed in exit_func() */
exit_func($io);
