#!/usr/bin/php
<?php

/////////////////////////////////////
//// GLOBALS
/////////////////////////////////////
$LOG_NAME         = "cloudMigration";
$io               = "";
$DBG              = true;
$CONT_ON_ERROR    = false; // cont after all the vault checks
$TRUE_VALUE_ARRAY = array(0 => "true", 1 => "True", 2 => "TRue", 3=> "TRUe",
  4=> "TRUE", 5=>"1", 6 => "On", 7=> "ON", 7=> "on", 9=> "Yes", 10=> "YEs",
  11=> "YES", 12=> "yes", 13=> "y", 14=> "Y");
$DEF_MAXCONCURRENCY = 2; // TODO SET THIS BASED ON HW TYPE, being used for concurrent replications
$MAXNUMOFSESSIONS   = 5; // TODO SET THIS BASED ON HW TYPE
$ST_SYNC_NEEDED = 1;
$P_ENCRYPTED    = 0x00000100;
$CP_CRYPT       = 0x00000004;
$COMPLETED      = "Completed";
$NOT_COMPLETED  = "Not Completed";
$TARGET_DB_CONN = "";
$MIGRATE        = false;
$STATUS         = false;
$SWITCH         = false;
$CLEANUP        = false;
$CANCEL         = false;
$REMOVE_OPENVPN = false;
$QUEUE_DETAILS  = false; // give rep queue details (backup_no, rep_start) during monitoring
$MONITOR_SLEEP  = 30;    // how frequently to display results, in seconds
$BYPASS_SRC_SEL = "";
$log_max_number = 4; // TODO is 4 ENOUGH
$log_level      = 0;
$log_filename   = "";
$log_handle;
$log_is_open    = false;;
$INTR_MSG       = "goodbye!\n";


// The VAULT_MIG_SUFFIX is needed to differentiate replicated backups for migrations
// (this tool), DR, and CrossVaults. For migrations (this tool) the replication queue
// vault value contains the $VAULT_MIG_SUFFIX. The /etc/hosts file on this target does as well.
$VAULT_MIG_SUFFIX ='-MIG7';

///////////////////////
//// START OF PROGRAM
///////////////////////
/// Have to be root, otherwise all hell will break loose later...
/// We'll inform the user if they have valid permissions in the prereq phase.
if (0 != posix_getuid()) {
  echo("Invalid user permissions. Must have root access privileges.\n");
  exit;
}

////////////////////////////////////////////////////////////////////////////////////
/* Process any args, e.g. -migrate, -status, -switch, -cleanup, -cancel, -dbg ... */
////////////////////////////////////////////////////////////////////////////////////
processArgs($argv);

//////////////////////////////////////////////////////////////////////
/* Logger Stuff */
//////////////////////////////////////////////////////////////////////
// LOG FILES:, one for initial processing, one per migrating source
$rc = initialize_log($LOG_NAME);
if ($rc !== true) {
  echo "Error:".var_dump($rc)."\n";
  exit;
}

/// Add signal handlers, useful to convey a message during --migrate phase
declare(ticks = 1);
function sig_handler($signo)
{
  global $INTR_MSG;

  // just in case we're at the PP prompt...enable echo again.
  system('stty echo');
  //echo "\nCaught signal $signo, cleaning up, exiting...\n";
  echo "$INTR_MSG";
  exit;
}

// setup signal handlers, once we have some things needed cleanup
pcntl_signal(SIGTERM, "sig_handler");
pcntl_signal(SIGHUP,  "sig_handler");
pcntl_signal(SIGUSR1, "sig_handler");
pcntl_signal(SIGINT,  "sig_handler");

//////////////////////////////////////////////////////////////////////
/* Main program starts here */
//////////////////////////////////////////////////////////////////////

$res = bp_bypass_cookie(3);
if ($res === false) {
  echo "Bypass cookie failed\n";
  var_dump(bp_error());
  exit;
}
$io = fopen('php://stdin', 'r');

if ($MIGRATE === true) {
  migrateMain();
}
else if ($STATUS === true) {
  statusMain();
} 
else if ($SWITCH === true) {
  switchMain();
} 
else if ($CLEANUP === true) {
  cleanupMain();
} 
else if ($CANCEL === true) {
  cancelMain();
} 
else if ($REMOVE_OPENVPN === true) {
  remove_openvpn();
}

exit(0);

///////////////////////////////////////////
///// SUBROUTINES   ///////////////////////
///////////////////////////////////////////

///////////////////////////////
//// MIGRATION MAIN ///////////
///////////////////////////////
function migrateMain()
{
  enterFunction("migrateMain");

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $INTR_MSG;
  global $io;

  // setup the existing target DB connection
  setupTargetDBConn();

  ////////////////////
  // Greeting
  ////////////////////

  echo "Welcome to the Unitrends Replication Migration Utility. We're running checks and gathering a list of source systems to MIGRATE to a new replication target...\n\n";

  //////////// PREREQUISITE PHASE ///////////////////
  // 1. Prereq Checks, ...verify we're a target replicating
  // Return Vault encryption. Needed for time and space estimates.
  $isVaultEncryptionEnabled = false;
  if (prerequisitePhase($isVaultEncryptionEnabled) != 0){
    echoLog("\nFound some problems, please correct, then try again.\n");
    if (!$CONT_ON_ERROR) exit_func($io);
  }

  // 2. Locate Sources (DPUs) and their migration state
  $foundErrors = false;
  // The returned dpuArray contains system_id / name pairs
  $dpuArray = displayAvailableSourcesAndStates(&$foundErrors);
  if (count($dpuArray) == 0){
    if ($foundErrors === true) {
      echoLog("\nDid not identify any systems ready for migration. Please correct conditions before retrying.\n");
    }
    else {
      echoLog("\nDid not discover any systems requiring migration. Please check the Vault ensuring all configured systems are replicating.\n");
    }

    exit_func($io);
  }

  // 3. Get Source Selection
  $selectedDPU = selectSource($io, $dpuArray);
  logger(2, "Selected DPU: ", $selectedDPU);

  //Switch logs to the src selected log
  close_log($LOG_NAME);
  $LOG_NAME = $LOG_NAME."_".$selectedDPU;
  $rc = initialize_log($LOG_NAME);
  if ($rc !== true) {
    echo "Error:".var_dump($rc)."\n";
    if (!$CONT_ON_ERROR) exit_func($io);
  }
  logger(0, "========= migrate request =========\n");

  // get the sources IP address
  $sourceIP = getSourceIP($selectedDPU);
  // Determine if src has openvpn configured
  $openvpnSource = isOpenVPNConfigured($sourceIP);


  // get old target network info
  $oldTargetHostname  = getHostNameLoc();
  $oldTargetIP        = getHostIP();
  $oldTargetOpenvpnIP = "";
  $openvpnOldTarget   = isOpenVPNConfigured($oldTargetIP);
  $oldTargetIPForSrc  = $oldTargetIP;
  if ($openvpnSource && $openvpnOldTarget) {
    $oldTargetOpenvpnIP = getOpenVPNIPaddr($oldTargetIP);
    $oldTargetIPForSrc  = $oldTargetOpenvpnIP;
  }

  // 4.Warn if replication is active for the source and if something is in the queue
  $rc = isSourceRepActive($selectedDPU);
  if ($rc < 0) { if (!$CONT_ON_ERROR) exit_func($io); }
  if ($rc > 0) {
    echo "\nWARNING, replication is currently enabled on $selectedDPU. Subsequent migrations may be needed.\n";
  }
  // Note: this routine print out any warnings messages directly, silent = false
  $noOfBackups = getSourceBackupsInTheQueue($selectedDPU, $oldTargetIPForSrc, false);

  // 5. locate the backups to be replicated per client and provide an estimate on how
  //     long migration will take
  //  a.provide the list for last group (get_all_last_backups $client)
  //  b.provide the list for all valid active backups per client (psql bp.backups)
  //  c.prompt to migrate last or all
  // all = 1, last = 0, anything else...
  $totalAllBUsize  = 0;
  $totalLastBUsize = 0;
  $totalBUsize     = 0;
  $allOrLast = displayBackupsToMigrate($selectedDPU, &$totalAllBUsize, &$totalLastBUsize);
  if ($allOrLast == 1) {
    $allOrLastTag  = "all backups";
    $totalBUsizeGB = $totalAllBUsize/1024;
  }
  else {
    $allOrLastTag  = "the last backup group";
    $totalBUsizeGB = $totalLastBUsize/1024;
  }

  // 7.prompt for the new target
  //   a. need to check the version of the target! (TBD)
  //   b. need to validate new target is replication capable
  //   c. need to check the new target is not the old target
  $newTargetNameAndIP = getNewTargetFromRepQueue($selectedDPU);
  validateNewTarget($newTargetNameAndIP);

  $newTargetNameExt = $newTargetNameAndIP["nameExt"];
  $newTargetNameInt = $newTargetNameAndIP["nameInt"];
  $newTargetIP      = $newTargetNameAndIP["ip"];

  // 8.Determine if new target's encryption is enabled and active
  // This will prevent issues from occurring if current backups are encrypted.
  // Determining backup encryptions based on the current encryption setting of existing target.
  // We also care if the existing Target is encrypted and enabled as this will cause decryption errors.
  $isSystemEncryptionSet = encryptionSetPerSystem($selectedDPU);
  if ($isSystemEncryptionSet == true || $isVaultEncryptionEnabled == true) {
    // this routine prints a message and exits if unencrypted new target
    checkEncryptionOnNewTarget($newTargetNameAndIP);
  }

  // 9.Update new targets systems table with old target info:
  //   a.temporarily add original Target to the host file
  //   b.temporarily add original Target to the pg_hba.conf, bp.managers so we can issue sql from the Target
  updateHostFile($newTargetIP, $oldTargetIP, $oldTargetHostname);
  grantRemoteAccess($newTargetIP, $oldTargetHostname);

  // 10.Determine if new target has adequate space for replication and for this migration
  $defaultDevice="";
  $defaultDevCap=0;
  $rc = getNewTargetDefaultDevice($newTargetIP, &$defaultDevice, &$defaultDevCap);
  if ($rc > 0) {
    echo "Fix new target device settings, then retry.\n";
    exit($io);
  }
  else if ($rc == 0) {
    $freeSpaceGB = round((getDefaultDeviceFreeSpace($newTargetIP, $defaultDevice, $defaultDevCap) / 1024),2);
    if ($freeSpaceGB >= $totalBUsize) {
      echoLog("Free space ($freeSpaceGB GB) available on the new Target's default device is adequate for this migration. Check with allocations being used by any other active migrations.\n");
    }
    else {
      echoLog("Free space ($freeSpaceGB GB) available on the new Target's default device is in adequate for this migration.\n");
      exit_func($io);
    }
  }

  // 11. Determine if new target has openvpn configured
  $openvpnNewTarget = isOpenVPNConfigured($newTargetIP);

  // 11a. Inform about any openvpn inconsisencies
  if ($openvpnSource == true && $openvpnNewTarget == false) {
    echo "WARNING, the source to be migrated has a secure tunnel configured but the new target does not. Remember to do this before switching.\n";//Is this ok? ('q' to quit.)\n";
  }

  // 12.Summarize and prompt to start migration..
  echo "Ready to migrate $allOrLastTag for Source $selectedDPU to the new Target $newTargetNameExt?\nEnter to start, 'q' to quit.";
  $selVal = trim(fgets($io));
  if ($selVal == 'q') exit_func($io);

  ////////////////////////////////////////////
  ////////////////////////////////////////////
  // Start of migration process.....
  ////////////////////////////////////////////
  ////////////////////////////////////////////

  // 10.Update local /etc/hosts file with new target (cmc_network save name ip)
  updateHostFile("", $newTargetIP, $newTargetNameInt);

  // 11.Update orig Target replication settings (e.g. master.ini)
  setOldTargetRepSettings();

  // 12.Update new targets /etc/hosts with source info
  updateHostFile($newTargetIP, $sourceIP, $selectedDPU);

  // 13.Update new targets systems table with src entry from target.systems table 
  addSrcToNewTarget($newTargetIP, $selectedDPU);

  // 14. add backups to the migration (replication) queue using vcd util enqueue
  //   a. Note: directly adding to the rep queue makes parallel migrations possible
  addBackupsToTheQueue($selectedDPU, $newTargetNameAndIP, $allOrLast);
  wakeUpVCD();

  // update our signal message
  $INTR_MSG = "\nMigration may still be running. Monitor it with --status, kill it with --cancel.\n";

  // 15. Monitor progress 1...of N complete....
  $startTime = time();
  monitorMigrationProgress($selectedDPU, $allOrLast, $newTargetNameExt);

  // Display some openvpn notes
  if ($openvpnSource || $openvpnNewTarget) {
    openvpnSequence();
  }

  // dump out actual time it took so we can adjust our estimates...
  $t = time() - $startTime;
  $s = sprintf("%02d%s%02d%s%02d", floor($t/3600), ":", ($t/60)%60, ":", $t%60);
  logger(0,"migration done, elapse time:", $s);
  exit(0);
}

///////////////////////////////
//// SWITCH SOURCE MAIN ///////
///////////////////////////////
function switchMain()
{
  enterFunction("switchMain");

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // setup the existing target DB connection
  setupTargetDBConn();

  ////////////////////
  // Greeting
  ////////////////////

  echo "Welcome to the Unitrends Replication Migration Utility. We're running checks and gathering a list of source systems ready to SWITCH from one replication target to another...\n\n";

  // 1. Locate Sources (DPUs) and their migration state
  $foundActiveMigrations = false;
  // The returned dpuArray contains system_id / name pairs
  $dpuArray = displayAvailableSourcesAndStates(&$foundActiveMigrations);
  if (count($dpuArray) == 0){
    if ($foundActiveMigrations === true) {
      echoLog("\nDid not identify any systems ready for switching. Please allow active migrations to complete (or cancel them) before retrying.\n");
    }
    else {
      echoLog("\nDid not discover any systems ready for switching. Please check the Vault ensuring all configured systems are replicating.\n");
    }

    exit_func($io);
  }

  // 2. Get Source Selection for Switch action
  $selectedDPU = selectSource($io, $dpuArray);
  logger(2, "Selected DPU: ", $selectedDPU);

  // Switch logs to the src selected log
  close_log($LOG_NAME);
  $LOG_NAME = $LOG_NAME."_".$selectedDPU;
  $rc = initialize_log($LOG_NAME);
  if ($rc !== true) {
    echo "Error:".var_dump($rc)."\n";
    if (!$CONT_ON_ERROR) exit_func($io);
  }
  logger(0, "========= switch request =========\n");

  // 3. Get the Source's IP address from the existing targets host file
  $sourceIP = getIP($selectedDPU);

  // 3a. Get the existing targets host and ip
  $oldTargetHostname  = getHostNameLoc();
  $oldTargetIP        = getIP($oldTargetHostname);

  // 4. Get New Target Name and IP
  $newTargetNameAndIP = getNewTargetFromRepQueue($selectedDPU);
  $newTargetNameExt   = $newTargetNameAndIP["nameExt"];
  $newTargetNameInt   = $newTargetNameAndIP["nameInt"];
  $newTargetIP        = $newTargetNameAndIP["ip"];
  $newTargetOpenVPNIP = "";

  // 5. Determine if src and targets have openvpn configured
  $openvpnSource    = isOpenVPNConfigured($sourceIP);
  $openvpnOldTarget = isOpenVPNConfigured($oldTargetIP);
  $openvpnNewTarget = isOpenVPNConfigured($newTargetIP);

  // 5a. Inform about any openvpn inconsisencies
  if ($openvpnSource != $openvpnNewTarget) {
    if ($openvpnNewTarget == false) {
      echo "WARNING, the migrated Source has a secure tunnel configured but the new Target does not. Is this ok? (Enter to continue, 'q' to quit.)";
    }
    else {
      echo "WARNING, the migrated Source does not have a secure tunnel configured but the new Target does. Is this ok? (Enter to continue, 'q' to quit.)";
    }
    $selVal = trim(fgets($io));
    if ($selVal == 'q') exit_func($io);
  }
  else {
    if ($openvpnNewTarget) {
      echo "The migrated Source and new Target have secure tunnels configured. Before switching, it is best to create a secure tunnel certificate on the new Target for Source '$selectedDPU'. Enter to continue if/when completed?\n";
      fgets($io);
    }
  }

  // 5b. Acquire openvpn IP
  if ( $openvpnSource && $openvpnNewTarget) {
    // Both configured, get the targets IP as this will be used in a few source settings
    // It must be different then the original targets openvpn value as it is messy to
    // try an get things switched properly.
    $oldTargetOpenvpnIP = getOpenVPNIPaddr($oldTargetIP);
    $newTargetOpenvpnIP = getOpenVPNIPaddr($newTargetIP);
    if ($oldTargetOpenvpnIP == $newTargetOpenvpnIP) {
      echo "The new target must not be the same as the existing target.\n";
      exit_func($io);
    }
    $newTargetNameAndIP["openvpn_ip"] = $newTargetOpenvpnIP;
    echo "Using the new Target's openvpn IP '$newTargetOpenvpnIP' for remote access from the source. Remember this address as we'll need to update the host file later with the updated source address. (Enter to continue, 'q' to quit.)";
     $selVal = trim(fgets($io));
     if ($selVal == 'q') exit_func($io);
     $newTargetOpenVPNIP = $newTargetNameAndIP["openvpn_ip"];
  }

  // 6. Check and clear if replication is active. Also, inform if there are entries
  //  in the replication queue on the Source.
  echo "Checking Source Replication state....";
  $sourceRepRC = isSourceRepActive($selectedDPU);
  if ($sourceRepRC < 0) { if (!$CONT_ON_ERROR) exit_func($io); }

  // Replication is ok, get items in the queue.
  $oldTargetIPForSrc  = $oldTargetIP;
  if ($openvpnSource && $openvpnOldTarget) {
    $oldTargetOpenvpnIP = getOpenVPNIPaddr($oldTargetIP);
    $oldTargetIPForSrc  = $oldTargetOpenvpnIP;
  }
  $noOfBackups = getSourceBackupsInTheQueue($selectedDPU, $oldTargetIPForSrc, true); // silent == true
  if ($sourceRepRC > 0) {
    // Replication is enabled
    echoLog("\nReplication is currently enabled for $selectedDPU. It must be suspended during switch sequence.");
    echo " (There are $noOfBackups backups in the queue.)";
    echo " To let us suspend replication and continue, hit enter, 'q' to quit.";
    $selVal = trim(fgets($io));
    if ($selVal == 'q') exit_func($io);
    suspendReplication($selectedDPU);
    echo "Replication suspended, continuing\n";
  }
  else {
    // Replication is suspended
    if ($noOfBackups == 0) {
      echo "\nReplication is currently suspended for $selectedDPU. There are no backups in the queue.\n";
    }
    else {
      echo "\nReplication is currently suspended for $selectedDPU, however there are backups ($noOfBackups) in the queue.\n";
    }
    echo "Continue, hit enter, 'q' to quit.";
    $selVal = trim(fgets($io));
    if ($selVal == 'q') exit_func($io);
  }

  // 7. Add new target (EXTERNAL NAME and OPEN_VPN) to source /etc/hosts
  // e.g. openvpn entries:
  //  - 172.17.3.1 VMware_UEB_237
  //  - 192.168.237.247 VMware_UEB_237-nonVPN
  if ($openvpnSource && $openvpnNewTarget) {
    // Must first check/remove the old Target's openvpn from the Source as it could be the same
    // as the new Target.
    // NA given we are validating the old and new VPN IP's are different!
    //if ($oldTargetOpenVPNIP == $newTargetOpenVPNIP) {
    //  echo "Removing old Target openvVPN from the Source host file (same as new Target)...";
    //  removeFromHostfile($sourceIP, $oldTargetHostname);
    //  echo "Done!\n";
    //}

    echo "Adding new Target '$newTargetNameExt / $newTargetOpenVPNIP' to /etc/hosts/on the Source...";
    updateHostFile($sourceIP, $newTargetOpenVPNIP, $newTargetNameExt);
    echo "Done!\n";

    ///* DON'T do these as they will get configured when we redo the openvpn on the src
    //$nonVPN_Host = $newTargetNameExt."-nonVPN";
    //echo "Adding new Target '$nonVPN_Host / $newTargetIP' to /etc/hosts/on the Source...";
    //updateHostFile($sourceIP, $newTargetIP, $nonVPN_Host);
    //echo "Done!\n";

    ///* DON'T do these as they will get configured when we redo the openvpn on the src
    //echo "Adding new Target '$newTargetNameExt / $newTargetIP' to /etc/hosts/on the Source...";
    //updateHostFile($sourceIP, $newTargetIP, $newTargetNameExt);
    //echo "Done!\n";
  }
  else {
    echo "Adding new Target '$newTargetNameExt / $newTargetIP ' to /etc/hosts/on the Source...";
    updateHostFile($sourceIP, $newTargetIP, $newTargetNameExt);
    echo "Done!\n";
  }

  // 8. Add Source to the new target pg_service.conf
  echo "Granting access for DB svc for the Source in the new Target '$newTargetNameExt'...";
  grantDBSvcAccess($newTargetIP, $selectedDPU);
  echo "Done!\n";

  // 9. Add new target to source management (bp.managers, pg_hba.conf)
  echo "Granting access for the new Target '$newTargetNameExt' for remote management of the Source...";
  grantRemoteAccess($sourceIP, $newTargetNameExt);
  echo "Done!\n";

  // 9a. Add old target non-vpn to source management (bp.managers, pg_hba.conf) as this will be
  // needed after the source tears down the openvpn session between source and target
  if ($openvpnSource && $openvpnOldTarget) {
    $nonVPN_Host = $oldTargetHostname."-nonVPN";
    echo "Granting access for the old Target '$nonVPN_Host' for temporary remote management of the Source...";
    grantRemoteAccess($sourceIP, $nonVPN_Host);
    echo "Done!\n";
  }

  // 10. Add vault user for source on the new Target
  echo "Adding a vault user for the Source on the new Target...";
  addSrcUserToNewTarget($selectedDPU, $newTargetIP);
  echo "Done!\n";

  // 11. Set replication queuing strategy and SyncTo on the source
  echo "Setting replication target to new Target within the Source...";
  updateSourceReplicationConf($selectedDPU, $sourceIP, $newTargetNameAndIP);
  echo "Done!\n";

  // 12. Resume replication for the source (if we disabled it)
  if ($sourceRepRC > 0) {
    echo "Resuming replication on the Source...";
    resumeReplication($selectedDPU);
    echo "Done!\n";
  }

  // Assuming no errors...
  // Display some openvpn notes
  if ($openvpnSource && $openvpnNewTarget) {
    echo "After the openvpn procedures are completed (see below), allow systems to sync, ~10 mins, then refresh any active browser sessions. Also, don't forget to remove (--cleanup) the source from the existing target once the new pairs are confirmed.\n";
    openvpnSequence();
  }
  else {
    echo "Allow systems to sync, ~10 mins, then refresh any active browser sessions. Also, don't forget to cleanup (-cleanup) the source from the existing target once the new pairs are confirmed.\n";
  }
}

///////////////////////////////
//// CLEANUP MAIN /////////////
///////////////////////////////
function cleanupMain()
{
  enterFunction("cleanupMain");

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // setup the existing target DB connection
  setupTargetDBConn();

  ////////////////////
  // Greeting
  ////////////////////

  echo "Welcome to the Unitrends Replication Migration Utility. We're gathering a list of source systems ready to CLEANUP from one this replication target...\n\n";

  // 1. Locate Sources (DPUs) and their migration state
  $foundActiveMigrations = false;
  // The returned dpuArray contains system_id / name pairs
  $dpuArray = displayAvailableSourcesAndStates(&$foundActiveMigrations);
  if (count($dpuArray) == 0){
    if ($foundActiveMigrations === true) {
      echoLog("\nDid not identify any systems ready for cleanup. Please allow active migrations to complete (or cancel them) before retrying.\n");
    }
    else {
      echoLog("\nDid not discover any systems requiring cleanup. Please check the Vault ensuring all configured systems are replicating.\n");
    }

    if (!$CONT_ON_ERROR) exit_func($io);
  }

  // 3. Get Source Selection for Cleanup action
  $selectedDPU = selectSource($io, $dpuArray);
  logger(2, "Selected DPU: ", $selectedDPU);

  //Switch logs to the src selected log
  close_log($LOG_NAME);
  $LOG_NAME = $LOG_NAME."_".$selectedDPU;
  $rc = initialize_log($LOG_NAME);
  if ($rc !== true) {
    echo "Error:".var_dump($rc)."\n";
    if (!$CONT_ON_ERROR) exit_func($io);
  }
  logger(0, "========= cleanup request =========\n");

  // Get the Source's IP address from the existing targets host file
  $sourceIP = getIP($selectedDPU);
  $openvpnSource = isOpenVPNConfigured($sourceIP);

  // Get the Source's DPU ID from the DB systems table
  $dpu_id = getSystemIdForDPU($selectedDPU);

  // 4. Get our local network info
  $oldTargetHostname = getHostNameLoc();
  $oldTargetIP       = getHostIP();

  // 4a Check the replication queue on the source to make sure no old target items are pending
  $oldTargetIPForSrc = $oldTargetIP;
  $openvpnOldTarget  = isOpenVPNConfigured($oldTargetIP);
  if ($openvpnSource && $openvpnOldTarget) {
    $oldTargetOpenvpnIP = getOpenVPNIPaddr($oldTargetIP);
    $oldTargetIPForSrc  = $oldTargetOpenvpnIP;
  }
  $noOfBackups = getSourceBackupsInTheQueue($selectedDPU, $oldTargetIPForSrc, true); // silent == true
  if ($noOfBackups > 0) {
    echo "\nWarning! There are backups ($noOfBackups) in the replication queue on the Source ($selectedDPU) for the old Target ($oldTargetHostname). These should be cleared prior to cleanup as they will retry indefinitely otherwise.\n";
    echo "Continue, hit enter, 'q' to quit.";
    $selVal = trim(fgets($io));
    if ($selVal == 'q') exit_func($io);
  }

  // 5. Get New Target Name and IP
  $newTargetNameAndIP = getNewTargetFromRepQueue($selectedDPU);
  $newTargetNameExt   = $newTargetNameAndIP["nameExt"];
  $newTargetNameInt   = $newTargetNameAndIP["nameInt"];
  $newTargetIP        = $newTargetNameAndIP["ip"];

  // 5a. Find if there are any active migrations in progress to new Target being cleaned up.
  //     If so, skip removal of some temp settings.
  $migrationsInProgress = anyActiveMigrations($newTargetNameInt);

  // New Target Things:
  if (!$migrationsInProgress) {
    // 6a. Remove old target from new target DB management
    echo "Revoking remote management access for the old Target from the new Target (a temp setting)...";
    revokeRemoteAccess($newTargetIP, $oldTargetHostname);
    echo "Done!\n";

    // 6b. Remove old target from new target host file
    echo "Removing old Target from the new Target host file (a temp setting)...";
    removeFromHostfile($newTargetIP, $oldTargetHostname);
    echo "Done!\n";
  }

  // Source Things:
  // 7a. Remove old target from source DB management
  echo "Revoking remote management access for the old Target from the Source...";
  revokeRemoteAccess($sourceIP, $oldTargetHostname);
  echo "Done!\n";

  // 7b. Remove old target from source host file
  echo "Removing old Target from the source host file...";
  removeFromHostfile($sourceIP, $oldTargetHostname);
  echo "Done!\n";

  // 7c. Remove old target nonVPN from mgmt and source host file
  if ($openvpnSource && $openvpnOldTarget) {
    $nonVPN_Host = $oldTargetHostname."-nonVPN";

    echo "Revoking remote management access for the old Target from the Source...";
    revokeRemoteAccess($sourceIP, $nonVPN_Host);
    echo "Done!\n";

    echo "Removing old Target nonVPN from the source host file...";
    removeFromHostfile($sourceIP, $nonVPN_Host);
    echo "Done!\n";
  }

  // Old Target Things:

  if (!$migrationsInProgress) {
    // 8a. Remove new target from old target host file
    echo "Removing new Target from the old Target host file (a temp setting)...";
    removeFromHostfile("", $newTargetNameInt);
    echo "Done!\n";
  }

  // 8b. Remove source from old Targets pg_system.conf
  echo "Revoking access for DB svc for the Source in the old Target...";
  revokeDBSvcAccess("", $selectedDPU);
  echo "Done!\n";

  // 8c. Disable our replication chaining setting for the old target.
  //    Only unset it if this system is not a CV (SyncTo is set)
  //    As other migrations take place, they will reset it.
  //    If we have an active migration, leave things asis.
  
  if (syncToSet() == false) {
    if (howManyDestinationsForActiveMigrations() > 0) {
      echo "There are active migrations in progress therefore not disabling replication (mirroring) for the old Target yet!...";
    }
    else {
      echo "Turning off replication (mirroring) for the old Target...";
      unsetOldTargetRepSettings();
      restartReplication("");
      echo "Done!\n";
    }
  }

  // 8d. Mark all backups deleted for this source
  echo "Marking all Source replicated backups for deletion...";
  // Note: these are tagged during the removeSrcFromDB() call next.
  //markAllSrcBackupForDeletion($dpu_id);
  //echo "Done!\n";
  echo "\n";

  // 8e. Remove Source from Systems, nodes, etc....
  echo "Removing the Source from the old Target DB tables (nodes, systems)...";
  removeSrcFromDB($selectedDPU);
  echo "Done!\n";

  // 8f. Remove source from old target host file
  echo "Removing Source from old Target host file...";
  removeFromHostfile("", $selectedDPU);
  echo "Done!\n";

  // 8g. Remove vault user for source on the old Target
  echo "Removing a vault user for the Source on the old Target...";
  removeSrcUserFromOldTarget($selectedDPU);
  echo "Done!\n";

  echo "Proceed with the next Source to migrate!\n";
}

///////////////////////////////
//// CANCEL MAIN //////////////
///////////////////////////////
function cancelMain()
{
  enterFunction("cancelMain");

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // setup the existing target DB connection
  setupTargetDBConn();

  ////////////////////
  // Greeting
  ////////////////////

  echo "Welcome to the Unitrends Replication Migration Utility. We're gathering a list of source systems with active migrations that can be CANCELED...\n\n";

  // 1. Locate Sources (DPUs) and their migration state
  $foundActiveMigrations = false;
  // The returned dpuArray contains system_id / name pairs
  $dpuArray = displayAvailableSourcesAndStates(&$foundActiveMigrations);
  if (count($dpuArray) == 0){
    if ($foundActiveMigrations === false) {
      echoLog("\nDid not identify any systems with active migrations.\n");
    }
    else {
      echoLog("\nDid not discover any systems requiring cleanup. Please check the Vault ensuring all configured systems are replicating.\n");
    }

    exit_func($io);
  }

  // 2. Get Source Selection for Cleanup action
  $selectedDPU = selectSource($io, $dpuArray);
  logger(2, "Selected DPU: ", $selectedDPU);

  //Switch logs to the src selected log
  close_log($LOG_NAME);
  $LOG_NAME = $LOG_NAME."_".$selectedDPU;
  $rc = initialize_log($LOG_NAME);
  if ($rc !== true) {
    echo "Error:".var_dump($rc)."\n";
    if (!$CONT_ON_ERROR) exit_func($io);
  }
  logger(0, "========= cancel request =========\n");

  // 3. Get New Target Name and IP
  $newTargetNameAndIP = getNewTargetFromRepQueue($selectedDPU);

  // 4. Delete incomplete migrations from the replication queue
  removeActiveMigrationsFromRepQueue($selectedDPU, $newTargetNameAndIP);

}

///////////////////////////////
//// STATUS MAIN //////////////
///////////////////////////////
function statusMain()
{
  enterFunction("statusMain");

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // setup the existing target DB connection
  setupTargetDBConn();

  $selectedDPU = ""; // get all DPU's
  $allOrLast   = 1;  // ALL 
  monitorMigrationProgress($selectedDPU, $allOrLast);
}

function encryptionSetPerSystem($selectedDPU)
{
  enterFunction("encryptionSetPerSystem",$selectedDPU);

  global $TARGET_DB_CONN;
  global $CP_CRYPT;

  $encryptionSet = false;

  $dpu_id   = getSystemIdForDPU($selectedDPU);
  $q = "SELECT properties from nodes where system_id = '$dpu_id'";
  logger(0, "system properties:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred obtaining properties from nodes.\n");
    echoLog($res);
    return $encryptionSet;
  }
  while ($row = pg_fetch_row($res)) {
    if (($row[0] & $CP_CRYPT) > 0) {
      $encryptionSet = true;
      break;
    }
  }

  return $encryptionSet;
}

function removeActiveMigrationsFromRepQueue($selectedDPU, $newTargetNameAndIP)
{
  enterFunction("removeActiveMigrationsFromRepQueue",$selectedDPU, $newTargetNameAndIP);

  global $TARGET_DB_CONN;
  global $LOG_NAME;
  global $ST_SYNC_NEEDED;

  // Use internal name as this is what the rep queue contains
  $nameInt = $newTargetNameAndIP["nameInt"];

  $dpu_id = getSystemIdForDPU($selectedDPU);

  // clear sync needed bit
  $q = "UPDATE backups SET sync_status = sync_status & ~$ST_SYNC_NEEDED where backup_no in (SELECT backup_no from replication_queue where backup_no in (SELECT backup_no from backups where node_no in (SELECT node_no from nodes where system_id = $dpu_id) and tape_no != 4294967295 ORDER by backup_no) and status != 2 and vault = '$nameInt')";
  logger(0, "clear sync needed bit:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred clearing ST_SYNC_NEEDED from backups.\n");
    echoLog($res);
  }

  // delete item from the queue
  $q = "DELETE from replication_queue where backup_no in (SELECT backup_no from backups where node_no in (SELECT node_no from nodes where system_id = $dpu_id) and tape_no != 4294967295 ORDER by backup_no) and status != 2 and vault = '$nameInt'";
  logger(0, "delete item from the queue:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred deleting backups from the replication queue.\n");
    echoLog($res);
  }

  // delete all the tasks from operations_progress
  $theNodes = getNodesNameAndNoPerSystemIdOnTarget($dpu_id);
  if (count($theNodes) == 0) return;
  $first = 1;
  $node_q = "";
  foreach ($theNodes as $key => $nodeEntry) {
    $gcname = $nodeEntry["gcname"];
    if ($first == 1) {
      $node_q = " on_behalf_of = '$gcname' ";
      $first = 0;
    }
    else {
      $node_q = $node_q." or on_behalf_of = '$gcname' ";
    }
  }
  $q = "SELECT pid_to_kill from operation_progress where $node_q and destination = '$nameInt'";
  logger(0, "pid_to_kill:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred deleting mirroring jobs from the operation_progress table.\n");
    echoLog($res);
  }
  // for each pid, issue a kill...
  while ($row = pg_fetch_row($res)) {
    $pid = (int)$row[0];
     logger(2,"killing replication-mirroring task, pid:", $pid);
    `kill $pid`;
  }

  // TODO DOES ORDER MATTER, clear, then delete from queue or vice versa

}

function wakeUpVCD()
{
  enterFunction("wakeUpVCD");

  // find the vcd process and wake it up by issuing a kill -n SIGCHLD(17) pid

  $output=array();

  // the process
  $VCD_PROCESS="/usr/bp/bin/vcd";
  $pid = -1;
  $pidCmd="ps -elf | grep -v grep |grep ".escapeshellarg($VCD_PROCESS)." | awk {'print $4'}";
  $pid = (int)`$pidCmd`;
  if ($pid > 0) {
    $pidCmd="kill -n 17 ".escapeshellarg($pid)."";
    `$pidCmd`;
  }
}

function markAllSrcBackupForDeletion($dpu_id)
{
  enterFunction("markAllSrcBackupForDeletion", $dpu_id);

  global $TARGET_DB_CONN;
  global $LOG_NAME;

  $q = "UPDATE backups SET tape_no = 4294967295 where backup_no in (SELECT backup_no from backups where node_no in (SELECT node_no from nodes where system_id = $dpu_id))";
  logger(0, "mark backups for deletion:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred marking backups for deletion.\n");
    echoLog($res);
  }
}

function updateSourceReplicationConf($selectedDPU, $sourceIP, $newTargetNameAndIP)
{
  enterFunction("updateSourceReplicationConf", $selectedDPU, $sourceIP, $newTargetNameAndIP);

  // sequence
  // - get current queuescheme on the Source
  // - set replication SyncTo to the new Target and define the queuing strategy e.g. 2 = recency) 
  //   using cmc_replication source configure new_target_name 2

  global $LOG_NAME;

  $nameExt = $newTargetNameAndIP["nameExt"];

  // Get my ID
  $dpu_id = getSystemIdForDPU($selectedDPU);
  if (empty($dpu_id)) {
    logger(0,"dpu_id not found, replication suspension unchecked, for:", $sourceIP);
    return -2; // indicates dpu not found error (bp_error() is not set!)
  }

  // queue scheme
  $queueScheme = bp_get_ini_value('Replication', 'QueueScheme', $dpu_id);
  logger(0,"queueScheme:",$queueScheme);

  // remote cmd for setting replication...
  $out = `/usr/bp/bin/bputil --remotecmd '$sourceIP' '/usr/bp/bin/cmc_replication source configure $nameExt $queueScheme'`;
  if (trim($out) != "SUCCESS.") {
    echoLog("WARNING, we were not able to update replication settings on the source.\n");
    echoLog($out);
  }
}

function grantDBSvcAccess($remoteSysIP, $addHost)
{
  enterFunction("grantDBSvcAccess", $remoteSysIP, $addHost);

  global $LOG_NAME;

  if ($remoteSysIP == "") {
    // cmc_dbservice add source_name
    $out = `/usr/bp/bin/cmc_dbservice add $addHost`;
  }
  else {
    // cmc_dbservice add source_name
    $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' '/usr/bp/bin/cmc_dbservice add $addHost'`;
    if (trim($out) != "SUCCESS.") {
      echoLog("WARNING, we were not able to update '$remoteSysIP DB service (pg_service.conf) file.\n");
      echoLog($out);
    }
  }
}

function revokeDBSvcAccess($remoteSysIP, $rmHost)
{
  enterFunction("revokeDBSvcAccess", $remoteSysIP, $rmHost);

  global $LOG_NAME;

  if ($remoteSysIP == "") {
    // cmc_dbservice add source_name
    $out = `/usr/bp/bin/cmc_dbservice remove $rmHost`;
  }
  else {
    // cmc_dbservice remove source_name
    $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' '/usr/bp/bin/cmc_dbservice remove $rmHost'`;
    if (trim($out) != "SUCCESS.") {
      echoLog("WARNING, we were not able to update '$remoteSysIP DB service (pg_service.conf) file.\n");
      echoLog($out);
    }
  }
}

function updateHostFile($remoteSysIP, $addIP, $addName)
{
  enterFunction("updateHostFile", $remoteSysIP, $addIP, $addName);

  global $LOG_NAME;
  global $io;

/* TODO This could be cleaner. The 1st 2 errors are misleading. The real error is line 3
System with name abc_MIG7 not found in hosts file.
WARNING, we were not able to resolve the IP address for the provided name.
Hosts table already has an entry for "192.168.237.247".
WARNING, this error occurred on the existing target.
*/
  //(cmc_network host save - new_target_name, new_target_ip
  $okFailure = "Hosts table already has an entry for";
  if ($remoteSysIP == "") {
    // local host file
    // NOTE: errors are written to the screen directly...
    $out="";
    $err="";
    exec("/usr/bp/bin/cmc_network host save - $addIP $addName 2> /dev/null",$out, $err);
    if ($err == 1) {
      // got an error, see if this is just a duplicate entry...
      $gettheip = getIP($addName);
      if ($gettheip != $addIP) {
        // run the cmd again, let the output print to the screen.
        exec("/usr/bp/bin/cmc_network host save - $addIP $addName ",$out, $err);
        echoLog("Note: Most likely, the LAST error is the most significant. This error occurred on the existing target.\n");
        exit_func($io);
      }
    }
    /* $out = `/usr/bp/bin/cmc_network host save - $addIP $addName 2> /dev/null`;
    if (trim($out) != "") {
      $pos = strpos($out, $okFailure);
      if ($pos === false) {
        echoLog("WARNING, we were not able to update the host information in the local host file.\n");
        echoLog($out);
      }
    }
    */
  }
  else {
    $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' '/usr/bp/bin/cmc_network host save - $addIP $addName'`;
    if (trim($out) != "SUCCESS.") {
      $pos = strpos($out, $okFailure);
      if ($pos === false) {
        echoLog("WARNING, we were not able to update the host information in the remote host file.\n");
        echoLog($out);
      }
    }
  }
}

function removeFromHostfile($remoteSysIP, $rmName)
{
  enterFunction("removeFromHostfile", $remoteSysIP, $rmName);

  global $LOG_NAME;

  $out = "";
  //(cmc_network host remove new_target_name
  if ($remoteSysIP == "") {
    // local host file
    $out = `/usr/bp/bin/cmc_network host remove $rmName`;
  }
  else {
    $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' '/usr/bp/bin/cmc_network host remove $rmName'`;
  }

  logger(1,"out:",$out);
}

function getNewTargetFromRepQueue($selectedDPU)
{
  enterFunction("getNewTargetFromRepQueue", $selectedDPU);

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;
  global $VAULT_MIG_SUFFIX;
  global $MIGRATE;

  $dpu_id             = getSystemIdForDPU($selectedDPU);
  $destination        = array();
  $newTargetNameAndIP = array();

  // from the replication queue, find the new target from a completed migration
  $compMigrated = getBackupsMigrated($dpu_id, $destination);
  if (count($destination) == 1) {
    $nameVal = current($destination);
    $newTargetNameAndIP["nameInt"] = $nameVal.$VAULT_MIG_SUFFIX;
    $newTargetNameAndIP["nameExt"] = $nameVal;
    $newTargetNameAndIP["ip"]      = getIP($newTargetNameAndIP["nameInt"]);
  }
  else {
    // try in progress
    $inProgress = getBackupsInProgress($dpu_id, $destination);
    if (count($destination) == 1) {
      $nameVal = current($destination);
      $newTargetNameAndIP["nameInt"] = $nameVal.$VAULT_MIG_SUFFIX;
      $newTargetNameAndIP["nameExt"] = $nameVal;
      $newTargetNameAndIP["ip"]      = getIP($newTargetNameAndIP["nameInt"]);
    }
    else {
      // try pending
      $pending = getBackupsPending($dpu_id, $destination);
      if (count($destination) == 1) {
        $nameVal = current($destination);
        $newTargetNameAndIP["nameInt"] = $nameVal.$VAULT_MIG_SUFFIX;
        $newTargetNameAndIP["nameExt"] = $nameVal;
        $newTargetNameAndIP["ip"]      = getIP($newTargetNameAndIP["nameInt"]);
      }
    }
  }

  // Prompt for the values if we didn't find them...
  if (count($newTargetNameAndIP) == 0 || $newTargetNameAndIP["nameInt"] == "" || $newTargetNameAndIP["ip"] == "") {
    echo "\n";
    if ($MIGRATE != true) {
      echo "Unable to determine the new Target from the replication queue.\n";
      echo "Enter the (non-VPN) IP address:";
    }
    else {
      echo "Enter the (non-VPN) IP address of the target to migrate the backups to:";
    }
    $ipVal   = trim(fgets($io));
    $nameVal = getNewTargetHostname($ipVal);

    // return the entry containing names and ip
    $newTargetNameAndIP["nameInt"] = $nameVal.$VAULT_MIG_SUFFIX;
    $newTargetNameAndIP["nameExt"] = $nameVal;
    $newTargetNameAndIP["ip"]      = $ipVal;
  }
  else {
    if ($MIGRATE === true) {
      // Found some entries, confirm...
      $name = $newTargetNameAndIP['nameExt'];
      $ip   = $newTargetNameAndIP['ip'];
      echo "Using previous migration Target ($name) and IP ($ip) found in the replication queue. Enter to continue, 'n' to supply new values.\n";
      $selVal = trim(fgets($io));
      if ($selVal == 'n') {
        $newTargetNameAndIP = getNewTargetNW();
      }
    }
  }

  logger(2,"New Target Names and IP:", $newTargetNameAndIP);

  return $newTargetNameAndIP;
}

function getNewTargetHostname($ipVal)
{
  enterFunction("getNewTargetHostname", $ipVal);
  global $io;

  // can we ping the new target?
  $retval = -1;
  $cmd = "/usr/bp/bin/cmc_replication util ping $ipVal";
  system($cmd, $retval);
  if ($retval != 0) {
    // the output of cmc_replication will appear on the screen..
    //echo "$ip is not reachable.\n";
    exit_func($io);
  }

  $hostName = "";
  $out = `/usr/bp/bin/bputil --remotecmd '$ipVal' 'hostname'`;
  // parse the return checking for SUCCESS.
  $split = explode(".", $out, 2); // SUCCESS.    true/false
  if ((count($split) == 2) && (trim($split[0]) == "SUCCESS")) {
    $hostName = trim($split[1]);
  }

  if ($hostName == "") {
    echoLog("WARNING, we were not able to obtain the new Target's hostname, enter it now:");
    $hostName = trim(fgets($io));
    logger(0,$out);
  }
  return $hostName;
}

function suspendReplication($selectedDPU)
{
  enterFunction("suspendReplication", $selectedDPU);

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // sequence
  // - get current maxconcurrency on the Source
  // - set prev max concurrency on the Source
  // - set maxconcurrency on the Source to 0
  // - restart replication  on the source

  // Get my ID
  $dpu_id = getSystemIdForDPU($selectedDPU);
  if (empty($dpu_id)) {
    logger(0,"dpu_id not found, replication suspension unchecked, for:", $selectedDPU);
    return -2; // indicates dpu not found error (bp_error() is not set!)
  }

  $currMaxConc = bp_get_ini_value('Replication', 'MaxConcurrent', $dpu_id);
  logger(0,"currMaxConc:",$currMaxConc);
  $ret = bp_set_ini_value("Replication","MaxConcurrent",0, $dpu_id);
  if ($ret === false) {
    echoLog("Unable to suspend replication. Need this set in the Sources master.ini in order to switch replication!\n");
    echoLog("error is: ".bp_error()."\n");
  }
  $ret = bp_set_ini_value("Replication","prevMaxCOn",$currMaxConc, $dpu_id);
  // check this?

  // /usr/bp/bin/cmc_replication source restart
  $out = `/usr/bp/bin/bputil --remotecmd '$selectedDPU' '/usr/bp/bin/cmc_replication source restart'`;
  if (trim($out) != "SUCCESS.") {
    echoLog("WARNING, we were not able to restart replication on the source.\n");
    echoLog($out);
  }

  return 0;
}

function resumeReplication($selectedDPU)
{
  enterFunction("resumeReplication", $selectedDPU);

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // sequence
  // - get prev max concurrency on the Source
  // - set maxconcurrency on the Source to prev
  // - restart replication on the source

  // Get my ID
  $dpu_id = getSystemIdForDPU($selectedDPU);
  if (empty($dpu_id)) {
    logger(0,"dpu_id not found, replication suspension unchecked, for:", $selectedDPU);
    return -2; // indicates dpu not found error (bp_error() is not set!)
  }

  $prevMaxCon = bp_get_ini_value('Replication', 'prevMaxCOn', $dpu_id);
  logger(0,"prevMaxCon:",$prevMaxCon);
  $ret = bp_set_ini_value("Replication","MaxConcurrent",$prevMaxCon, $dpu_id);
  if ($ret === false) {
    echoLog("Unable to resume replication for the Source. Try it directly from RRC!\n");
    echoLog("error is: ".bp_error()."\n");
  }

  // restart replication
  restartReplication(getIP($selectedDPU));

  return 0;
}

function restartReplication($remoteSysIP)
{
  enterFunction("restartReplication", $remoteSysIP);

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  if ($remoteSysIP == "") {
    $out = `/usr/bp/bin/cmc_replication source restart`;
  }
  else {
    // /usr/bp/bin/cmc_replication source restart
    $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' '/usr/bp/bin/cmc_replication source restart'`;
    if (trim($out) != "SUCCESS.") {
      echoLog("WARNING, we were not able to restart replication on the source.\n");
      echoLog($out);
    }
  }
}

function unsetOldTargetRepSettings()
{
  enterFunction("unsetOldTargetRepSettings");
  global $TRUE_VALUE_ARRAY;

  // disable replication
  /* THIS way fails...  $ret = bp_enable_replication(true); */
  // Set it directly...
  $ret = bp_set_ini_value("Replication","Enabled","no");
  if ($ret === false) {
    echoLog("disabling replication failed, need this set in master.ini in order to turn replication mirroring off to new target!\n");
    echoLog("error is: ".bp_error()."\n");
  }
}

function setOldTargetRepSettings()
{
  enterFunction("setOldTargetRepSettings");
  global $TRUE_VALUE_ARRAY;
  global $DEF_MAXCONCURRENCY;

  // enable replication
  /* THIS way fails...  $ret = bp_enable_replication(true); */
  // Set it directly...
  $ret = bp_set_ini_value("Replication","Enabled","yes");
  if ($ret === false) {
    echoLog("enable replication failed, need this set in master.ini in order to replicate to new target!\n");
    echoLog("error is: ".bp_error()."\n");
  }

  // set maxconcurrent
  $currMaxConc = bp_get_ini_value('Replication', 'MaxConcurrent');
  logger(0,"currMaxConc:",$currMaxConc);
  if ($currMaxConc < $DEF_MAXCONCURRENCY) {
    $ret = bp_set_ini_value("Replication","MaxConcurrent",$DEF_MAXCONCURRENCY);
    if ($ret === false) {
      echoLog("Unable to enable replication. Need this set in the Sources master.ini in order to start replication migration!\n");
      echoLog("error is: ".bp_error()."\n");
    }
    $ret = bp_set_ini_value("Replication","prevMaxCOn",$currMaxConc);
    // check this?
  }

  // while we are here, make sure SyncTo is not set for replication and vaulting
  $ret = bp_get_ini_value("Replication","SyncTo");
  if ($ret) {
    // check to see if null
    if (!is_null($ret)) {
      logger(0,"WARNING: master.ini [Replication].SyncTo is set to: $ret on existing target. This should be unset!");
    }
  }

  $ret = bp_get_ini_value("Securesync","SyncTo");
  if ($ret) {
    // check to see if null
    if (!is_null($ret)) {
      logger(0,"WARNING: master.ini [Securesync].SyncTo is set to: $ret on existing target. This should be unset!");
    }
  }

  // check vaulting enablement
  $ret = bp_get_ini_value("Securesync","AutoSyncEnabled");
  if ($ret) {
    $isVaultingEnabled = in_array($ret, $TRUE_VALUE_ARRAY);
    if ($isVaultingEnabled == true){
      echoLog("\nWARNING: Vaulting is currently enabled, replication migrations may be halted.\n");
    }
  }
}

function syncToSet()
{
  $syncto = bp_get_ini_value("Replication","SyncTo");
  if (is_null($syncto) || $syncto == 'vault') {
    return false;
  }
  return true;
}

function getStats($bkup_no, $dest, $migrationStartTime)
{
  global $TARGET_DB_CONN;
  global $VAULT_MIG_SUFFIX;

  $errorMessage = "";

  // only works if $destList is a single value..
  // append the suffix, as this is what the vault name contains
  $destination = $dest.$VAULT_MIG_SUFFIX;

  // COUNT = 1
  $q = "SELECT message from sds_stats where backup_no = $bkup_no and destination = '$destination' and done_epoch > $migrationStartTime ORDER by done_epoch DESC LIMIT 1";
  logger(1, "get sds_stats queue details:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    logger("An error occurred obtaining rep queue info for $destination\n");
    return $errorMessage;
  }
  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    return $errorMessage;
  }
  $row = pg_fetch_row($res);
  return $row[0];
}

function array_search_partial($arr, $keyword)
{
  $results = array();
  foreach($arr as $index => $string) {
//echo "KEYWORD: $keyword\n";
//echo "STRING: $string\n";
    if (strpos($string, $keyword) !== false) {
//echo "found a match\n";
      $results[] = $index; 
      //return $index;
    }
  }

  return $results;
}

function monitorMigrationProgress($dpu_name, $allOrLast, $newTargetNameExt="")
{
  enterFunction("monitorMigrationProgress", $dpu_name, $allOrLast, $newTargetNameExt);
  global $MONITOR_SLEEP;
  global $QUEUE_DETAILS;
  global $io;

  // configure or list to monitor and whether to stop when done
  $stopWhenDone = true;
  $srcDpus      = array();
  if ($dpu_name == "") {
    // get a list of all the Replicating Sources..
    $srcDpus = getReplicatingSources();
    // since no specific dpu passed in, let this run until cancelled.
    $stopWhenDone = false;
  }
  else {
    $srcDpus[] = $dpu_name;
  }

  // some items to help deal with and present errors
  $bkup_progress      = array();
  $migrationStartTime = time();

  $displayCounter = 0;
  while (1) {
    if ($displayCounter == 0) {
      echo "---------------------------------------------------------------------------------------------------------\n";
      echo "Source           \tNo. Bkups    No. Bkups      No. Bkups    Not         Destination\n";
      echo "                 \tCompleted    In Progress    Pending      Migrated\n";
      echo "---------------------------------------------------------------------------------------------------------\n";
    }

    // locate migration data for each dpu
    foreach ($srcDpus as $key => $dpu_name) {
      $destination = array();
      $dpu_id      = getSystemIdForDPU($dpu_name);
      if ($dpu_id == "") continue;
      $theNodes    = getNodesNameAndNoPerSystemIdOnTarget($dpu_id);
      $details     = "";
      $errors      = array();

      // if theNodes is empty, no backups were replication, so just continue
      if (count($theNodes) == 0) continue;

      // from the backups table, get list of ALL active replicated backups
      $activeBackups = getAllActiveBackups($dpu_id);

      // from the utility/backups table, get list of backups based on the request type
      $allOrLastBackups = getBackupsPerNodes($theNodes, $allOrLast);

      // from the replication queue, get all backups completed (2) for given source/nodes
      $compMigrated = getBackupsMigrated($dpu_id, $destination);

      // from replication queue, get active backups (16) and details, for given source/nodes
      $inProgress = getBackupsInProgress($dpu_id, $destination);

      // fixup the destination string, should be at most one value but you never know!
      $destList = implode(",", array_keys(array_flip($destination)));
      logger(0,"Destination:", $destList);

      // process the in progress backups for optional details, and any errors
      foreach ($inProgress as $key => $element) {
        $bkup_no   = $element["backup_no"];
        $rep_start = $element["rep_start"];
        $entry     = sprintf( "%d:%d", $bkup_no, $rep_start );

        if ($QUEUE_DETAILS) {
          if ($details == "") {
            $details = " (";
          }
          else {
            $details = $details.", ";
          }
          $details = $details.$bkup_no;
          $details = $details.":".$rep_start;
        }
   
        // Keep track of back_no and rep_start, inserting record if it doesn't exist
        if (in_array($entry, $bkup_progress) == false) {
          // Entry does not exist, remove the old ones for this bkup_no, then
          // insert the new entry.

          // Remove all the previous records that begin with backup_no, reverse order
          $entryX  = sprintf( "%d:", $bkup_no );
          $matches = array_search_partial($bkup_progress, $entryX);
          if (count($matches) > 0) {
            $index = end($matches);
            while($index)
            {
              unset($bkup_progress[$index]);
              $index = prev($matches);
            }
          }

          // add new entry
          $bkup_progress[] = $entry;
          // Get the last error info (previous bkup task) from sds_stats
          // Need the destination to limit our search
          $dest = $destList;
          if ($newTargetNameExt != "") {
            $dest = $newTargetNameExt;
          }

          // This routine returns at most one record, the last error
          // since this migration started.
          $errorMsg = getStats($bkup_no, $dest, $migrationStartTime);
          // add to error array
          if ($errorMsg != "") {
            $errors[] = "Last error at $rep_start for backup_no $bkup_no: ".$errorMsg;
          }
        }
      }

      // close out any detail messages
      if ($details != "") {
        $details = $details.")";
      }

      // from replication queue, get the rest of the backups for given source/nodes
      $pending = getBackupsPending($dpu_id, $destination);

      $newArrivals = array();
      $newArrivals = array_diff($allOrLastBackups, $compMigrated, $inProgress, $pending);

      $completed = count($compMigrated);
      logger(0,"Number of Bkups Completed:", $completed);

      $inPrg = count($inProgress);
      logger(0,"Number of Bkups In Progress:", $inPrg);

      $pnd = count($pending);
      logger(0,"Number of Bkups Pending:", $pnd);

      $newArr = count($newArrivals);
      logger(0,"Number of New Arrivals:", $newArr);

      if ($QUEUE_DETAILS) {
        $destList = $destList.$details;
      }

      // Fill in the table
      $TABS = "\t";
      $dpu_name_pad   = str_pad($dpu_name, 17);
      $completed_pad  = str_pad($completed, 13);
      $inPrg_pad      = str_pad($inPrg, 15);
      $pnd_pad        = str_pad($pnd, 13);
      $newArr_pad     = str_pad($newArr, 12);
      echo "$dpu_name_pad$TABS$completed_pad$inPrg_pad$pnd_pad$newArr_pad$destList\n";
      $displayCounter = $displayCounter + 1;

      // Process any errors captured
      foreach ($errors as $key => $errorMsg) {
        echo "$errorMsg\n";
      }

      // check to see if we done...
      if ($stopWhenDone == true) {
        if ($pnd === 0 && $inPrg === 0) {
          echo "Done, proceed to next step (-switch)!\n";
          return;
        }
      }
    }
    sleep($MONITOR_SLEEP);
    if ($displayCounter > 20) {
      $displayCounter = 0;
    }
    else if (count($srcDpus) > 1) {
      echo "---------------------------------------------------------------------------------------------------------\n";
    }
  }
}

function howManyDestinationsForActiveMigrations()
{
  enterFunction("howManyDestinationsForActiveMigrations");
  global $io;

  $srcDpus = array();
  // get a list of all the Replicating Sources..
  $srcDpus = getReplicatingSources();
  $destination = array();

  foreach ($srcDpus as $key => $dpu_name) {
    $dpu_id = getSystemIdForDPU($dpu_name);
    $theNodes = getNodesNameAndNoPerSystemIdOnTarget($dpu_id);
    // if theNodes is empty, no backups were replication, so just continue
    if (count($theNodes) == 0) continue;

    // from replication queue, get active backups (16), for given source/nodes
    $inProgress = getBackupsInProgress($dpu_id, $destination);
    if (count($inProgress) > 0) {
      continue;
    }

    // from replication queue, get the rest of the backups for given source/nodes
    $pending = getBackupsPending($dpu_id, $destination);
    if (count($pending) > 0) {
      continue;
    }
  }

  // count up the entries in destination, which should be unique
  return count($destination);
}

function setupTargetDBConn()
{
  enterFunction("setupTargetDBConn");

  global $TARGET_DB_CONN;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $TARGET_DB_CONN = pg_Connect("port=5432 dbname=bpdb user=postgres");
  if (!$TARGET_DB_CONN) {
    echoLog("A database connection error occurred.\n");
    exit_func($io);
  }
  $res = pg_query($TARGET_DB_CONN, "set search_path to bp");
  if (!$res) {
    echoLog("An error occurred setting the DB search_path to bp.\n");
    exit_func($io);
  }
}

function addBackupsToTheQueue($selectedDPU, $newTargetNameAndIP, $allOrLast)
{
  enterFunction("addBackupsToTheQueue",$selectedDPU, $newTargetNameAndIP, $allOrLast);

  global $TARGET_DB_CONN;
  global $INTR_MSG;

  // for each backup, add them to the queue using vcd util enqueue
  // we'll get the list of backups again as this may have changed 
  // since we actually got to this step.

  $backupsToMigrate = array();
  $selectedSystemId = getSystemIdForDPU($selectedDPU);

  // get all the nodes on the target associated with this system
  $theNodes = getNodesNameAndNoPerSystemIdOnTarget($selectedSystemId);
  // get the backups to migrate, sorted, not queued yet.
  $backupsToMigrate = getBackupsToMigrate($theNodes, $allOrLast);
  logger(2,"backups to migrate", $backupsToMigrate);
  if (count($backupsToMigrate) == 0) {
    echo "No backups found to migrate!\n";
    return;
  }

  // Get our internal target name
  $newTargetNameInt = $newTargetNameAndIP["nameInt"];

  // Provide some feedback if more than 25 backups
  $feedBack = 25;
  if (count($backupsToMigrate) > 25) {
    echo "Adding backups to the migration queue";
  }

  // update our signal message
  $INTR_MSG = "\nPartial migration in progress. Monitor it with --status, kill it with --cancel.\n";

  // add them to the queue 1 at a time
  $cmd = "/usr/bp/bin/vcd util enqueue ";
  foreach ($backupsToMigrate as $key => $backup_no) {
   $fullCmd = "$cmd $backup_no $newTargetNameInt\n";
   logger(0, "Enqueue CMD:", $fullCmd);
   $retVal = -1;
   system($fullCmd, $retVal);
   if ($retVal != 0) {
     echoLog("Error from enqueue...$retVal\n");
   }
   $feedBack--;
   if ($feedBack == 0) {
      echo ".";
      $feedBack = 25;
   }
  }

  if (count($backupsToMigrate) > 25) {
    echo ".done\n";
  }

  // Now update their position based on backup number to interleave backups from
  // multiple sources, concurrent replications
 
  $q = "UPDATE replication_queue SET position=backup_no where status != 2 and status != 16 and vault = '$newTargetNameInt'";
  logger(0, "interleave backups pos:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred ordering the replication queue.\n");
    echoLog($res);
  }
}

function getNewTargetLocationID($dbConn)
{
  enterFunction("getNewTargetLocationID");

  $q = "SELECT location_id from locations where name = 'Default Location'";
  logger(0, "the select of location_idt query:", $q);
  $res = pg_query($dbConn, $q);
  if (!$res) {
    echoLog("An error occurred selecting the location_id from the db, assuming value of 1.\n");
    return 1;
  }

  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    echoLog("No rows returned for location_id from the db, assuming value of 1.\n");
    return 1;
  }
  $location_id = pg_result($res, 0, "location_id");
  return $location_id;
}

function addSrcToNewTarget($newTargetIP, $selectedDPU)
{
  enterFunction("addSrcToNewTarget",$selectedDPU, $newTargetIP);

  global $TARGET_DB_CONN;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // need the old system info from current target...
  $q = "SELECT location_id, name, is_local_system, host, version,role,is_active,check_timestamp,storage_id, identity from systems where name = '$selectedDPU'";
  logger(0, "the select of src from orig target query:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred selecting our replication source from orig target db\n");
    exit_func($io);
  }

  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    return $output;
  }
  $location_id     = pg_result($res, 0, "location_id");
  $name            = pg_result($res, 0, "name");
  $is_local_system = pg_result($res, 0, "is_local_system");
  $host            = pg_result($res, 0, "host");
  $version         = pg_result($res, 0, "version");
  $role            = pg_result($res, 0, "role");
  $is_active       = pg_result($res, 0, "is_active");
  $check_timestamp = pg_result($res, 0, "check_timestamp");
  $storage_id      = pg_result($res, 0, "storage_id");
  $identity        = pg_result($res, 0, "identity");

  // issue sql command to the new target...

  $connNewTar = pg_Connect("host=$newTargetIP port=5432 dbname=bpdb user=postgres");
  // SAMPLE:
  // psql -h$newTargetIP -Upostgres bpdb -c "insert into bp.systems (location_id, name, is_local_system, host, version,role,is_active,check_timestamp,storage_id, identity)  values(1,'ueb_src_179_23','f','ueb_src_179_23','7.3','Replication Source', 'f',1384262082, 1, '5724c4f0-2dd0-11e2-bc78-21ae6fdccd29')";

  if (!$connNewTar) {
    echoLog("A database connection error occurred.\n");
    exit_func($io);
  }
  $res = pg_query($connNewTar, "set search_path to bp");
  if (!$res) {
    echoLog("An error occurred setting the DB search_path to bp.\n");
    exit_func($io);
  }

  // Obtain the new Target's default location_id
  $newLocationId = getNewTargetLocationID($connNewTar);
  // Warn if original Source location_id is not the default
  $defaultLocationId = getNewTargetLocationID($TARGET_DB_CONN);
  if ($defaultLocationId != $location_id) {
    echoLog("Warning, the Source default location within the new Target is being set to the default. Adjust after migration as needed.\n");
  }

  $q = "insert into bp.systems (location_id, name, is_local_system, host, version, role, is_active, check_timestamp, storage_id, identity)  values( $newLocationId, '$name', '$is_local_system', '$host', '$version', '$role', '$is_active', $check_timestamp, $storage_id, '$identity')";
  logger(0, "the src insert to target query", $q);


  // in case we already have an entry in their database due to a restart,etc, suppress the error...
  $saveER = error_reporting();
  error_reporting(0);

  $res = pg_query($connNewTar, $q);
  if (!$res) {
    // check if the error indicates an entry already exists...
    $okFailure = "duplicate key violates unique constraint"; 
    $lastError = pg_last_error($connNewTar);
    $pos = strpos($lastError, $okFailure);
    if ($pos === false) {
      echoLog("We were not able to update the new Target's db with our system id entry.\n");
      echoLog("The query error is: $lastError :::::\n");
      exit_func($io);
    }
    else {
      logger(0,"non-fatal error, $lastError  ...\n");
    }
  }

  error_reporting($saveER);
}
 
function removeSrcUserFromOldTarget($selectedDPU)
{
  enterFunction("removeSrcUserToNewTarget",$selectedDPU);

  global $TARGET_DB_CONN;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $q = "DELETE from users where username = '$selectedDPU'";
  logger(0, "the select of pw from orig target query:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred deleting user from orig target db.\n");
  }
}

function addSrcUserToNewTarget($selectedDPU, $newTargetIP)
{
  enterFunction("addSrcUserToNewTarget",$selectedDPU, $newTargetIP);

  global $TARGET_DB_CONN;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // need the user encrypted password from current target...
  $q = "SELECT password from users where username = '$selectedDPU'";
  logger(0, "the select of pw from orig target query:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred locating our user info from orig target db\n");
    exit_func($io);
  }

  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    logger(1, "nrows:", $nrows);
    return;
  }
  $password = pg_result($res, 0, "password");
 
  // establish connection to new target
  $connNewTar = pg_Connect("host=$newTargetIP port=5432 dbname=bpdb user=postgres");
  if (!$connNewTar) {
    echoLog("A database connection error occurred.\n");
    exit_func($io);
  }
  $res = pg_query($connNewTar, "set search_path to bp");
  if (!$res) {
    echoLog("An error occurred setting the DB search_path to bp.\n");
    exit_func($io);
  }

  // issue sql command to the new target to insert a new vault user...
  $is_vault_user = "t";
  $q = "insert into users (username, password, vault_user)  values( '$selectedDPU', '$password', '$is_vault_user')";
  logger(0, "the add user insert to target query", $q);

  // in case we already have an entry in their database due to a restart,etc, suppress the error...
  $saveER = error_reporting();
  error_reporting(0);

  $res = pg_query($connNewTar, $q);
  if (!$res) {
    // check if the error indicates an entry already exists...
    $okFailure = "duplicate key violates unique constraint"; 
    $lastError = pg_last_error($connNewTar);
    $pos = strpos($lastError, $okFailure);
    if ($pos === false) {
      echoLog("The query error is: $res :::::\n");
      echoLog("We were not able to update the new Target's db with our user entry.\n");
      exit_func($io);
    }
    else {
      logger(0,"non-fatal error, $lastError  ...\n");
    }
  }

  // issue sql command to the new target to insert a new system level 3 privilege...
  $q = "insert into system_priv (user_id, system_id, priv_level) values ((SELECT user_id from users where username='$selectedDPU'), (SELECT system_id from bp.systems where name ='$selectedDPU'), ('3'))";
  logger(0, "the add system level 3 insert to target query", $q);

  $res = pg_query($connNewTar, $q);
  if (!$res) {
    // check if the error indicates an entry already exists...
    $okFailure = "duplicate key violates unique constraint"; 
    $lastError = pg_last_error($connNewTar);
    $pos = strpos($lastError, $okFailure);
    if ($pos === false) {
      echoLog("The query error is: $res :::::\n");
      echoLog("We were not able to update the new Target's db with our user entry.\n");
      exit_func($io);
    }
    else {
      logger(0,"non-fatal error, $lastError  ...\n");
    }
  }

  // issue sql command to the new target to insert a new system level 1 privilege...
  $q = "insert into system_priv (user_id, system_id, priv_level) values ((SELECT user_id from users where username='$selectedDPU'), (SELECT system_id from bp.systems where is_local_system ='t'), ('1'))";
  logger(0, "the add system level 1 insert to target query", $q);

  $res = pg_query($connNewTar, $q);
  if (!$res) {
    // check if the error indicates an entry already exists...
    $okFailure = "duplicate key violates unique constraint"; 
    $lastError = pg_last_error($connNewTar);
    $pos = strpos($lastError, $okFailure);
    if ($pos === false) {
      echoLog("The query error is: $res :::::\n");
      echoLog("We were not able to update the new Target's db with our user entry.\n");
      exit_func($io);
    }
    else {
      logger(0,"non-fatal error, $lastError  ...\n");
    }
  }

  error_reporting($saveER);
}
 
function getHostNameLoc()
{
  enterFunction("getHostNameLoc");

  return trim(`hostname`);
}

function getHostIP()
{
  enterFunction("getHostIP");

  // find our hostname...actually our IP
  $ip = `hostname --ip-address`;
  if ($ip === "") {
   // plan B
   $ip = getIP(getHostNameLoc());
  }
  if ($ip === "") {
   // plan C
   //$hostname = `/usr/bp/bin/cmc_network hostname info`
   // more work to do here....
   echo "No hostname found!\n";
   exit_func($io);
  }
  return trim($ip);
}

function grantRemoteAccess($remoteSysIP, $addName)
{
  enterFunction("grantRemoteAccess",$remoteSysIP, $addName);

  global $LOG_NAME;
  global $io;

  // # /usr/bp/bin/cmc_grant
  // Usage: cmc_grant [grant|confirm|revoke] <hostname>
  // updates: bp.managers and pg_hba.conf, the name better be in the /etc/hosts file first

  $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' '/usr/bp/bin/cmc_grant grant $addName'`;
  // For some reason the check for SUCCESS. is not working..., so check for the success detail msg too
  if (trim($out) != "SUCCESS.") {
    $okFailure = "Granted management permission";
    $pos = strpos($out, $okFailure);
    if ($pos === false) {
      $okFailure = "already been granted management permission";
      $pos = strpos($out, $okFailure);
      if ($pos === false) {
        echoLog("We were not able to update (grant) the hba_conf file with our address.\n");
        echoLog($out);
        exit_func($io);
      }
    }
  }
}

function revokeRemoteAccess($remoteSysIP, $rmName)
{
  enterFunction("revokeRemoteAccess", $remoteSysIP, $rmName);

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // # /usr/bp/bin/cmc_grant
  // Usage: cmc_grant [grant|confirm|revoke] <hostname>
  // updates: bp.managers and pg_hba.conf, the name better be in the /etc/hosts file first

  $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' '/usr/bp/bin/cmc_grant revoke $rmName'`;
  if (trim($out) != "SUCCESS.") {
    echoLog("We were not able to update (revoke) the hba_conf file with our address.\n");
    echoLog($out);
    //if ($CONT_ON_ERROR) exit_func($io);
  }
}

function getIP($name, $remoteSysIP="")
{
  enterFunction("getIP", $name, $remoteSysIP);
  global $io;

  $ip = "";
  $subcmd = "/usr/bp/bin/cmc_network host info $name";
  if ($remoteSysIP == "") {
    $out = `$subcmd`;
  }
  else {
    $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' '$subcmd'`;
    $split = explode(".", $out, 2); // SUCCESS.    true/false
    if (count($split) < 2) {
      // cmd failed
      return $ip;
    }
    if (trim($split[0]) != "SUCCESS") {
      // cmd did not return any value
      return $ip;
    }
    $out = trim($split[1]);
  }

  // $out contains a single string with 'ip 1.1.1.1\n name system_name'
  $split = explode(chr(10), $out, 2); // chr(10) = newline
  if (count($split) < 2) {
    // cmd failed
    echoLog("WARNING, we were not able to resolve the IP address for the provided name.\n");
    echoLog($out);
    return $ip; // force the caller to deal with empty ip
  }

  if (substr($split[0],0,2) === "ip"){
    $ipSplit = explode(" ", $split[0], 2);
    $ip = $ipSplit[1];
  }
  else if (substr($split[1],0,2) === "ip"){
    $ipSplit = explode(" ", $split[1], 2);
    $ip = $ipSplit[1];
  }
  return $ip;
}

function getSourceIP($selectedDPU)
{
  enterFunction("getSourceIP", $selectedDPU);

  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $sourceIP = getIP($selectedDPU);
  if ($sourceIP === "") {
    // last resort..ask for it...
    echo "Can't figure out the IP address for the Source $selectedDPU we're migrating, please enter it:\n";
    $sourceIP = trim(fgets($io));
    if ($sourceIP === "") echo "Invalid selection. Relaunch to start over\n";
    exit_func($io);
  }

  return $sourceIP;
}

function prerequisitePhase(&$isVaultEncryptionEnabled)
{
  enterFunction("prerequisitePhase");

  // return Vault's Encryption state

  $isVaultEncryptionEnabled = false;

  echo "---------------------------------------------------------------------------------------------------------\n";
  echo "Preliminary Check  \tResult\t   Notes\n";
  echo "---------------------------------------------------------------------------------------------------------\n";
  $rc = 0;
  $rc += permissions();
  $rc += concurrencyCheck();
  //$rc += vaultRelease();
  $rc += vaultReplication();
  $rc += vaultEncryption($isVaultEncryptionEnabled);
  //$rc += vaultDR();
  $rc += vaultOtherRepActivity();
  echo "---------------------------------------------------------------------------------------------------------\n";

  return $rc;
}

function permissions()
{
  enterFunction("permissions");

  $rc = 0;

  $message = str_pad("Permissions", 24);
  if (0 != posix_getuid()) {
    $message .= str_pad("Failed", 11);
    $message .= "Must have root access privileges.\n";
    $rc = 1;
  }
  else {
    $message .= str_pad("Passed", 11);
    $message .= "User permissions ok.\n";
  }
  echoLog($message);

  return $rc;
}

function concurrencyCheck()
{
  enterFunction("concurrencyCheck");
  global $MAXNUMOFSESSIONS;

  $rc = 0;
  $message = str_pad("Concurrency", 24);

  // TODO tie this in with Hardware limits
  // # cat unitrends-release
  // Unitrends Recovery-833 - 2013-02-21T01:14:23Z
  /*
    $hardware = trim(`cat \/\/etc\/\/unitrends-release |cut -f2 -d' '`);
    // bug 16248, old style
    if ($hardware === 'DPU') {
      $hardware = trim(`cat \/\/etc\/\/unitrends-release |cut -f3 -d' '`);
    }
  */

  $myCount = 1; // this session
  // check the replication queue, counting active migrations to unique targets
  $myCount += howManyDestinationsForActiveMigrations();
  logger(2, "my concurrency count:",$myCount);
  if ($myCount > $MAXNUMOFSESSIONS) {
    $message .= str_pad("Failed", 11);
    $message .= "Maximum number of migrations (".$MAXNUMOFSESSIONS. ") have been started. ";
    $rc = 1;
  }
  else {
    $myCount--; // We're not active yet!
    $message .= str_pad("Passed", 11);
    $message .= "Active Source migrations: ".$myCount." (max ".$MAXNUMOFSESSIONS."). ";
  }

  // Check replication (mirroring) concurrency.
  // Too low will cause mirroring to take forever
  // Too high will cause mirroring to take too many resources
  // Just reporting the value!
  $currMaxConc = bp_get_ini_value('Replication', 'MaxConcurrent');
  $message .= "Concurrent backup migrations set to ".$currMaxConc.".\n";

  echoLog($message);
  return $rc;
}

function vaultReplication()
{
  enterFunction("vaultReplication");
  global $TRUE_VALUE_ARRAY;

  $rc = 0;
  $message = str_pad("Vault Replication", 24);

  // Check to see if replication is supported on this vault
  $isRepEnabled = bp_replication_supported();

  if($isRepEnabled != true){
    $message .= str_pad("Failed", 11);
    $message .= "Replication must be enabled on the Target.\n";
    $rc = 1;
  }
  else {
    $message .= str_pad("Passed", 11);
    $message .= "Target's Replication is properly configured.\n";
  }
  echoLog($message);
  return $rc;
}

function vaultEncryption(&$isVaultEncryptionEnabled)
{
  enterFunction("vaultEncryption");

  $rc = 0;
  $message = str_pad("Encryption Setting", 24);

  // TODO not convinced this is correct.
  // Can a customer ever turn encryption off
  // What is the error I'm looking for?
  // If currently enabled, we'll encrypt (any way the passphrase can't be set)
  // If currently not enabled, we won't encrypt.
  // Maybe this is a warning!

  // Return if Encryption is set
  $isVaultEncryptionEnabled = false;

  // check if the Vault has Encryption enabled

  // This is the one we flag?
  /* array(4) {
      ["active"]=>
      bool(true)
      ["enabled"]=>
      bool(false)
      ["persistent"]=>
      bool(false)
      ["passphrase_set"]=>
      bool(true)
   */

  $res = bp_get_crypt_info();

  if ($res === false) {
    logger(0,"bp_get_crypt_info:",bp_error());
    $message .= str_pad("Failed", 11);
    $message .= "Unable to determine encryption state. Please check the system state and retry." .bp_error() ."\n";
    $rc = 1;
  } else if ($res['active'] === true && $res['enabled'] === false && $res['passphrase_set'] === true) {
    $message .= str_pad("Warning", 11);
    $message .= "Encryption is configured on the Target, but the Encryption State is set to off.\n";
  }
  else if ($res['enabled'] === false) {
    $message .= str_pad("Warning", 11);
    $message .= "Encryption is not enabled.\n";
  }
  else {
    $message .= str_pad("Passed", 11);
    $message .= "Encryption is enabled.\n";
    $isVaultEncryptionEnabled = true;
  }
  echoLog($message);
  return $rc;
}

function anyActiveMigrations($targetName)
{
  enterFunction("anyActiveMigrations", $targetName);
  global $TARGET_DB_CONN;

  // Look in the replication queue on this target to determine if there are any migrations
  // in progress for the given (new) target.
  $q = "SELECT COUNT(*) from replication_queue where status != 2 and vault = '$targetName'";
  logger(1, "get active migrations per new target:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    logger(2, "An error occurred obtaining replication queue info.\n");
  }
  else if ((int)pg_result($res, 0, "count") > 0) {
    return true;
  }

  return false;
}

function vaultOtherRepActivity()
{
  enterFunction("vaultOtherRepActivity");
  global $TARGET_DB_CONN;
  global $VAULT_MIG_SUFFIX;

  $rc = 0;
  $message = str_pad("Replication Tasks", 24);

  // look in the replication queue on this target to determine if there are any active DR or CrossVault task in progress
  $q = "SELECT COUNT(*) from replication_queue where status = 16 and vault NOT LIKE '%$VAULT_MIG_SUFFIX'";
  logger(1, "get backup details:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    logger(2, "An error occurred obtaining replication info.\n");
  }
  else {
    if ((int)pg_result($res, 0, "count") > 0) {
      $message .= str_pad("Warning", 11);
      $message .= "A Disaster Recovery or CrossVault replication is in progress.\n";
      $rc = -1;
    }
    else {
      $message .= str_pad("Passed", 11);
      $message .= "No Disaster Recoveries or CrossVault replication jobs in progress.\n";
    }
  }
  echoLog($message);
  return $rc;
}

function removeSrcFromDB($selectedDPU)
{
  enterFunction("removeSrcFromDB",$selectedDPU);
  global $DBG;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $dpu_id = getSystemIdForDPU($selectedDPU);
  // leverage the api 
  // Get the nodes associated with system_id
  $theNodes = getNodesNameAndNoPerSystemIdOnTarget($dpu_id);
  foreach ($theNodes as $key => $nodeEntry) {
    // delete each node which also marks their backups for deletion
    $node_no = $nodeEntry["node_no"];
    $ret = bp_delete_client($node_no);
    if ($ret === false) {
      echoLog("error deleting node from system, ".bp_error()."\n");
    }
  }

  // Now delete the system entry
  $ret = bp_remove_system($dpu_id);
  if ($ret === false) {
    echoLog("error deleting this system from the dp systems table, ".bp_error()."\n");
  }
}

function getAllActiveBackups($dpu_id)
{
  enterFunction("getAllActiveBackups",$dpu_id);
  global $TARGET_DB_CONN;
  global $DBG;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $output = array();

  $q = "SELECT backup_no from backups where node_no in (SELECT node_no from nodes where system_id = $dpu_id) and tape_no != 4294967295 and status = 3168 ORDER by backup_no";
  logger(0, "get active backups:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred obtaining active backups.\n");
    exit_func($io);
  }
  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    logger(2, "nrows:", $nrows);
    return $output;
  }
  while ($row = pg_fetch_row($res)) {
    $output[] = (int)$row[0];
  }
  logger(2, "getAllActiveBackups", $dpu_id, $output);
 
  return $output;
}

function addUniqueNameNonMIG_SUFFIX($name, &$destination)
{
  global $VAULT_MIG_SUFFIX;
  $fixUP = basename($name, $VAULT_MIG_SUFFIX);
  if (!in_array($fixUP, $destination)) {
    $destination[] = $fixUP;
  }
}

function getBackupsMigrated($dpu_id, &$destination)
{
  enterFunction("getBackupsMigrated",$dpu_id);
  global $TARGET_DB_CONN;
  global $VAULT_MIG_SUFFIX;
  global $DBG;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $output = array();

  // Leverage the replication queue weeding out entries due to DR, or Crossvaulting
  $q = "SELECT backup_no, vault from replication_queue where backup_no in (SELECT backup_no from backups where node_no in (SELECT node_no from nodes where system_id = $dpu_id) and tape_no != 4294967295 ORDER by backup_no) and status = 2 and vault LIKE '%$VAULT_MIG_SUFFIX'";

  logger(0, "get backups migrated:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred obtaining migrated backups.\n");
    exit_func($io);
  }
  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    return $output;
  }
  while ($row = pg_fetch_row($res)) {
    $output[] = (int)$row[0];
    addUniqueNameNonMIG_SUFFIX($row[1], $destination);
  }
  logger(2,"getBackupsMigrated", $dpu_id, $destination, $output);
 
  return $output;
}

function getBackupsInProgress($dpu_id, &$destination)
{
  enterFunction("getBackupsInProgress",$dpu_id);
  global $TARGET_DB_CONN;
  global $VAULT_MIG_SUFFIX;
  global $DBG;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $output = array();

  // Leverage the replication queue weeding out entries due to DR, or Crossvaulting
  $q = "SELECT backup_no, vault, rep_start from replication_queue where backup_no in (SELECT backup_no from backups where node_no in (SELECT node_no from nodes where system_id = $dpu_id) and tape_no != 4294967295 ORDER by backup_no) and status = 16 and vault LIKE '%$VAULT_MIG_SUFFIX'";

  logger(0, "get backups in progress:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred obtaining backups in progress.\n");
    exit_func($io);
  }
  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    logger(2,"getBackupsInProgress, nrows:", $nrows);
    return $output;
  }
  while ($row = pg_fetch_row($res)) {
    $element                = array();
    $element["backup_no"]   = (int)$row[0];
    $element["destination"] = $row[1];
    $element["rep_start"]   = (int)$row[2];
    $output[]               = $element;
    addUniqueNameNonMIG_SUFFIX($row[1], $destination);
  }
  logger(2,"getBackupsInProgress", $dpu_id, $destination, $output);
 
  return $output;
}

function getBackupsPending($dpu_id, &$destination)
{
  enterFunction("getBackupsPending",$dpu_id);
  global $TARGET_DB_CONN;
  global $VAULT_MIG_SUFFIX;
  global $DBG;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $output = array();

  // Leverage the replication queue weeding out entries due to DR, or Crossvaulting
  $q = "SELECT backup_no, vault from replication_queue where backup_no in (SELECT backup_no from backups where node_no in (SELECT node_no from nodes where system_id = $dpu_id) and tape_no != 4294967295 ORDER by backup_no) and status != 2 and status != 16 and vault LIKE '%$VAULT_MIG_SUFFIX'";

  logger(0, "get backups pending:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred obtaining backups pending.\n");
    exit_func($io);
  }
  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    return $output;
  }
  while ($row = pg_fetch_row($res)) {
    $output[] = (int)$row[0];
    addUniqueNameNonMIG_SUFFIX($row[1], $destination);
  }
  logger(2,"getBackupsPending", $dpu_id, $destination, $output);
 
  return $output;
}

function displayAvailableSourcesAndStates($foundActiveMigrations)
{
  enterFunction("displayAvailableSourcesAndStates");

  global $MIGRATE;
  global $SWITCH;
  global $CLEANUP;
  global $CANCEL;
  global $CONT_ON_ERROR;

  $dpuArray = array();
  $foundActiveMigrations = false;

  // Get a list of available DPUS -- contains system_id / name pairs
  // just returning the name straight up...
  $srcDpus = getReplicatingSources();
  logger(2,"getReplicatingSources:",$srcDpus);

  if ($srcDpus === false || count($srcDpus) == 0) {
    logger(0,"No DPU sources found:",$srcDpus);
    return $dpuArray;
  }

  echo "\nListed below are the Source System(s) replicating to this Vault, and some migration information.\n";
  echo "---------------------------------------------------------------------------------------------------------\n";
  echo "# Source         \tLast Bkup        No. Bkups    No. Bkups      No. Bkups    Not         Destination\n";
  echo "                 \tGrp Completed    Completed    In Progress    Pending      Migrated\n";
  echo "---------------------------------------------------------------------------------------------------------\n";

  // for each Source, get a bunch of values to determine the migration state...
  $index = 0;
  foreach ($srcDpus as $key => $dpu_name) {
    $foundLoc = "";
    $destination = array();
    // from the backups table, get list of ALL active replicated backups
    $dpu_id = getSystemIdForDPU($dpu_name);
    if ($dpu_id == "") continue;
    $activeBackups = getAllActiveBackups($dpu_id);

    // need the nodes to get some backups
    $theNodes = getNodesNameAndNoPerSystemIdOnTarget($dpu_id);
    // if theNodes is empty, no backups were replicated, no reason to migrate
    if ((count($theNodes) == 0) && ($CONT_ON_ERROR == false)) continue;

    // from the utility/backups table, get list of LAST backup group
    $allOrLast = 0; // 0 = last
    $lastGrpBackups = getBackupsPerNodes($theNodes, $allOrLast);

    // from the utility/backups table, get list of ALL backups
    $allOrLast = 1; // 1 = all
    $allBackups = getBackupsPerNodes($theNodes, $allOrLast);

    // from the replication queue, get all backups completed (2) for given source/nodes
    $compMigrated = getBackupsMigrated($dpu_id, $destination);

    // from replication queue, get active backups (16), for given source/nodes
    $inProgress = getBackupsInProgress($dpu_id, $destination);

    // from replication queue, get the rest of the backups for given source/nodes
    $pending = getBackupsPending($dpu_id, $destination);

    // determine if any new backups have arrived that are not in the queue.
    // Note: migrations in progress are not counted here!
    // New Arrivals could be migration has never been run yet.
    $newArrivals = array();
    $newArrivals = array_diff($allBackups, $compMigrated, $inProgress, $pending);

    $completed = count($compMigrated);
    logger(0,"Number of Bkups Completed:", $completed);

    // Determine if last backup group has been migrated
    $resultLBG = array_diff($lastGrpBackups, $compMigrated);
    logger(2,"Last Backup Group Completed Diff:", $resultLBG);
    if (count($resultLBG) > 0 || $completed == 0) {
      $lastBackupGroupCompleted = "No";
    }
    else {
      $lastBackupGroupCompleted = "Yes";
    }
    logger(0,"LBG Completed:", $lastBackupGroupCompleted);

    $inPrg = count($inProgress);
    logger(0,"Number of Bkups In Progress:", $inPrg);

    $pnd = count($pending);
    logger(0,"Number of Bkups Pending:", $pnd);

    $newArr = count($newArrivals);
    logger(0,"Number of New Arrivals:", $newArr);

    // fixup the destination string, should be at most one value but you never know!
    $destList = implode(",", array_keys(array_flip($destination)));
    logger(0,"Destination:", $destList);

    // Fill in the table
    $TABS = "\t";
    $dpu_name_pad             = str_pad($dpu_name, 15);
    $lastBkupGrpCompleted_pad = str_pad($lastBackupGroupCompleted, 17);
    $completed_pad            = str_pad($completed, 13);
    $inPrg_pad                = str_pad($inPrg, 15);
    $pnd_pad                  = str_pad($pnd, 13);
    $newArr_pad               = str_pad($newArr, 12);

    if ($inPrg > 0 || $pnd > 0) {
      $foundLoc = true;
    }
    // If we're in Migrate, Switch or Cleanup mode, don't add the dpu to the returned array if there are active migrations
    if (($MIGRATE === true || $SWITCH === true || $CLEANUP === true) && ($foundLoc)) {
      echo "X $dpu_name_pad$TABS$lastBkupGrpCompleted_pad$completed_pad$inPrg_pad$pnd_pad$newArr_pad$destList\n";
      $foundActiveMigrations = true;
    }
    // If we're in Cancel mode, only add the dpu to the returned array if there are active migrations
    else if ($CANCEL === true && !$foundLoc) {
      echo "X $dpu_name_pad$TABS$lastBkupGrpCompleted_pad$completed_pad$inPrg_pad$pnd_pad$newArr_pad$destList\n";
    }
    else {
      echo "$index $dpu_name_pad$TABS$lastBkupGrpCompleted_pad$completed_pad$inPrg_pad$pnd_pad$newArr_pad$destList\n";
      // Add contents to the returned array
      $dpuArray[] = $dpu_name;
      $index++;
    }
  }
  logger(4,"dpuArray:", $dpuArray);
  return $dpuArray;
}

function displayAvailableSourcesAndStatesOrig($foundErrors)
{
  enterFunction("displayAvailableSourcesAndStates");

  // This routine builds a list of Sources to migrate
  // and gives reasons why they cannot be migrated
  // Only the migratable Sources are tagged with a # and put in the results

  $dpuArray = array();

  // Get a list of available DPUS -- contains system_id / name pairs
  // just returning the name straight up...
  $srcDpus = getReplicatingSources();
  logger(2,"getReplicatingSources:",$srcDpus);

  if ($srcDpus === false || count($srcDpus) == 0) {
    logger(0,"No DPU sources found:",$srcDpus);
    return $dpuArray;
  }

  // Parse the replication queue in order to determine migrations states
  // (in progress, completed, none). The queue is process for all sources, therefore
  // do it here and pass it to the migrationState request.
  $repQueue = parseRepQueue();

  echo "\nListed below are the Source System(s) replicating to this Vault, and some migration information.\n";
  echo "--------------------------------------------------------------------------------------\n";
  echo "# System Name\t\tStatus\t\tNotes\n";
  echo "--------------------------------------------------------------------------------------\n";

  // for each Source DPU determine their Migration state
  $index = 0;
  foreach ($srcDpus as $key => $dpu_name) {
    $state = "";
    $reason = "";
    //-----------------------------------------------------------------------
    //# System Name              Status         Notes
    //-----------------------------------------------------------------------
    //1234567890123456789012345678901234567890123456789012345
    //       T       T       T       T       T       T

    $rc = getDpuMigrationState($dpu_name, $repQueue, $state, $reason);
    $TABS = "\t";
    $dpu_namePad = str_pad($dpu_name, 15);
    $state       = str_pad($state, 15);
    if ($rc != 0 && !$CONT_ON_ERROR) {
      echo "X $dpu_namePad$TABS$state\t$reason\n";
      $foundErrors = true;
    }
    else {
      echo "$index $dpu_namePad$TABS$state\t$reason\n";
      // Add contents to the returned array
     $dpuArray[] = $dpu_name;
     $index++;
    }
  }

  logger(4,"dpuArray:", $dpuArray);
  return $dpuArray;
}

function getReplicatingSources()
{
  enterFunction("getReplicationSources");
  global $TARGET_DB_CONN;
  global $LOG_NAME;
  global $BYPASS_SRC_SEL;
  global $CONT_ON_ERROR;
  global $io;

  $output = array();

  if ($BYPASS_SRC_SEL != "") {
    logger(0, "get replicated sourcesi, bypass src selection:", $BYPASS_SRC_SEL);
    $output[] = $BYPASS_SRC_SEL;
    return $output;
  }

  $q = "SELECT name from systems where role = 'Replication Source'";
  logger(0, "get replicated sources:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred obtaining replication sources.\n");
    exit_func($io);
  }

  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    return $output;
  }
  while ($row = pg_fetch_row($res)) {
    logger(2,"System ID:", $row[0]);
    //logger(2,"Source Name:", $row[1]);
    //$pair = array();
    //$pair["system_id"] = $row[0];
    //$pair["name"] = $row[1];
    //$output[] = $pair;
    $output[] = $row[0];
  }
  return $output;
}

function getDpuMigrationState($dpu, $repQueue, &$state, &$reason)
{
  // returns:
  // = 0, continue
  // > 0, continue with warnings
  // < 0, errors, do not continue
  // Reorder things to always check pre-conditions (bug 16266)

  enterFunction("getDpuMigrationState", $dpu);

  //global $CONT_ON_ERROR2;
  global $COMPLETED;
  global $NOT_COMPLETED;

  $state   = "";
  $reason  = "";

  // loop thru the replication queue looking for backups belonging to this dpu
  // this isn't the most efficient way but the queue shouldn't be too long as it
  // is on the target and this tool should be the only one adding to it..
  // NOTE: What about cross-vaults?

  // Some defaults...
  $state   = "Not Started";
  $reason  = "Ready for Migration";

  $cat=$dpu.$COMPLETED;
  $ncat=$dpu.$NOT_COMPLETED;
  $buCompleted = 0;
  $buNotCompleted = 0;
  if(isset($repQueue[$cat])) {
    $buCompleted = (int) $repQueue[$cat];
  }
  if(isset($repQueue[$ncat])) {
    $buNotCompleted = (int) $repQueue[$ncat];
  }

  if ($buCompleted > 0 || $buNotCompleted > 0) {
    $total   = $buCompleted + $buNotCompleted;
    if ($buNotCompleted == 0) {
      $state   = "Completed";
      $reason  = "$buCompleted of $total backups have been migrated";
    } else {
      $state   = "In Progress";
      $reason  = "$buCompleted of $total backups have been migrated";
    }
  }
}

function exit_func($io)
{
  echo "Exiting...\n";
  //fclose($io); // this is causing an error: /tmp/prognam.log Bad file des...
  $res = bp_destroy_cookie();
  exit;
}

function usage()
{
echo "Usage: Utility to move a Source from one Target to another, including migrating existing backups.\n";
echo "--migrate: walk-thru the sequence to migrate replicating Sources to a new Target\n";
echo " --status: present migration status for replication Sources\n";
echo "--details: an option for --migrate and --status providing more info (backup_no, start_time) for ongoing migrations\n";
echo " --switch: walk-thru the sequence changing a Source from one Target to another after the migration\n";
echo "--cleanup: remove a migrated Source from the original Target and cleanup any temporary Target migration settings\n";
echo " --cancel: terminate a running migration\n";
echo "--source=: use the provided source to perform operations on, bypassing the utilities internal search.\n";
echo " --removeVPN: stops openvpn & removes all configuration items\n";
openvpnSequence(true);
}

function processArgs($args)
{
  global $DBG;
  global $CONT_ON_ERROR;
  global $MIGRATE;
  global $STATUS;
  global $SWITCH;
  global $CLEANUP;
  global $CANCEL;
  global $REMOVE_OPENVPN;
  global $MONITOR_SLEEP;
  global $QUEUE_DETAILS;
  global $BYPASS_SRC_SEL;
  global $io;

  $atLeastOne = 0; // migrate, status, switch, cleanup, cancel...
  while ($arg = array_shift($args)) {
    if (substr($arg, 0,9) === '--migrate') {
      $MIGRATE = true;
      $atLeastOne += 1;
    }
    if (substr($arg, 0,8) === '--status') {
      $STATUS = true;
      $atLeastOne += 1;
    }
    if (substr($arg, 0,8) === '--switch') {
      $SWITCH = true;
      $atLeastOne += 1;
    }
    if (substr($arg, 0,9) === '--cleanup') {
      $CLEANUP = true;
      $atLeastOne += 1;
    }
    if (substr($arg, 0,9) === '--cancel') {
      $CANCEL = true;
      $atLeastOne += 1;
    }
    if (substr($arg, 0,10) === '--details') {
      $QUEUE_DETAILS = true;
    }
    if (substr($arg, 0,8) === '--sleep=') {
      $split = explode("=", $arg, 2);
      if (count($split) == 2) {
        $MONITOR_SLEEP = $split[1];
      }
    }
    if (substr($arg, 0,9) === '--source=') {
      $split = explode("=", $arg, 2);
      if (count($split) == 2) {
        $BYPASS_SRC_SEL = $split[1];
      }
    }
    if (substr($arg, 0,6) === '-nodbg') {
      $DBG = false;
    }
    if (substr($arg, 0,4) === '-dbg') {
      $DBG = true;
    }
    if (substr($arg, 0,11) === '--removeVPN') {
      $REMOVE_OPENVPN = true;
      $atLeastOne += 1;
    }
    else if (substr($arg, 0,4) === '-coe') {
      $CONT_ON_ERROR = true;
    }
  }

  if ($atLeastOne != 1) {
    usage();
    exit_func($io);
  }
}

function selectSource($io, $dpuArray)
{
  enterFunction("selectSource");
  global $io;
  global $MIGRATE;
  global $SWITCH;
  global $CLEANUP;
  global $CANCEL;

  // Note the contents of dpuArray and the actually rows displayed are referenced
  // by the # presented on the display as the rows could have an X for those
  // rows of Sources not ready for migrations.

  if ($MIGRATE === true) {
    echo "\n\nSelect the system to migrate by entering the corresponding number of the entry. We'll then proceed with some additional checks before prompting to continue.\n";
  }
  else if ($SWITCH === true) {
    // Issue a generic warning about what Sources should be switched..
    echo "\n\nSelect the system to switch by entering the corresponding number of the entry. We'll then walk you through the switching procedure. When selecting the system, make sure all desired backups have been migrated.\n";
  }
  else if ($CLEANUP === true) {
    // Issue a generic warning about what Sources should be cleaned up..
    echo "\n\nSelect the system to remove from this target by entering the corresponding number of the entry. We'll then proceed in removing some items from the selected Source, the new Target (destination where the selected Source was switched to), and some items on this old Target itself. It is advisable to make sure the Source and new Target are working together (e.g. replicating), as there is no undo operation.\n";
  }
  else if ($CANCEL === true) {
    // Issue a generic warning about what Sources should be canceled..
    echo "\n\nSelect the system to cancel an active migration by entering the corresponding number of the entry. We'll then remove incomplete backup migrations from the queue.\n";
  }
  else {
    echo "\n NOT SUPPORTED YET\n";
    exit_func($io);
  }

  echo "Selection ('q' to quit): ";
  $selVal = trim(fgets($io));
  if (!is_numeric($selVal) || $selVal < 0 || $selVal >= count($dpuArray)) {
    if ($selVal != 'q') echo "Invalid selection. Relaunch to start over\n";
    exit_func($io);
  }

  // return the entry containing both user and dpu
  logger(2,"dpu selected:", $dpuArray[$selVal]);
  return $dpuArray[$selVal];
}

/////////////////////////////////////////////////////////////
///// LOGGING ROUTINES COMBINED FROM CMC and SYNCSTATS //////
/////////////////////////////////////////////////////////////
# handles logging to a specified file

/**
 * @desc - initializes log $logname (e.g. statussync), rotates files
 *         if necessary, looks up log level, and prepares log for
 *         writing.  Log will be $logname_#.log
 * @param $logname - log's base name
 * @return - true on success, error string on error
 */
function initialize_log($logname, $log_path="")
{
  global $log_max_number;
  global $log_level;
  global $log_filename;
  global $log_handle;
  global $log_is_open;

  // get log level, log path
  // master.ini stuff
  $cmd_bppath = 'eval `grep BPDIR /etc/default/bp.ini`';
  $cmd_master_ini = 'MASTER_INI=$BPDIR/bpinit/master.ini';
  $cmd_bputil = 'BPUTIL=$BPDIR/bin/bputil';
  $cmd_ini = "$cmd_bppath; $cmd_master_ini; $cmd_bputil";
  $cmd_log_level = '$BPUTIL -g "debugging" "'.$logname.'" "0" $MASTER_INI';
  $log_level = trim(shell_exec("$cmd_ini; $cmd_log_level"));
  if (!$log_path) {
    $cmd_log_path  = '$BPUTIL -g "Location Information" "Logs-Dir" "" $MASTER_INI';
    $log_path = trim(shell_exec("$cmd_ini; $cmd_log_path"));
  }
  // manage file
  $log_filename = $log_path . '/' . $logname.'_0.log';
  if ($log_is_open) {
    fclose($log_handle);
  }
  $log_is_open = false;

  // rotate files if needed
  $date_string = date('F j, Y');
  $log_preamble = 'Log for '.$logname.', created '.$date_string."\n";
  if (is_dir($log_filename)) {
    return "specified log file '$log_filename' is a directory!";
  }
  if (is_writable($log_filename)) {
    $log_handle = fopen($log_filename, 'r');
    if ($log_handle === false) {
      return "could not open logfile '$log_filename'";
    }
    if (fgets($log_handle, 1024) != $log_preamble) {
      $rotate = true;
    } else {
      $rotate = false;
    }
    fclose($log_handle);
  } else {
    $rotate = false;
  }

  if ($rotate == true) {
    $logfiles = array(); // log filenames (no path)
    $dh = @opendir($log_path);
    if ($dh === false) {
      return "could not open directory '$log_path'";
    }
    while (false !== ($filename = readdir($dh))) {
      if (preg_match('/^' . $logname . '_\d\.log$/', $filename)) { // match log name
        $lognum = (int)substr($filename, strlen($logname)+1, -4); // cut off $logname_ from start, .log from end
        $logfiles[$lognum] = $filename;
      }
    }

    closedir($dh);
    // scan for any missing files, we won't have to move them all then
    $found_missing_log = false;
    $missing_log_num = -1;
    for ($i = 0; $i < $log_max_number; $i++) {
      if (!isset($logfiles[$i])) {
        $found_missing_log = true;
        $missing_log_num = $i;
      }
    }
    if ($found_missing_log == true) {
      $lowest_delete_num = $log_max_number + 1; // move some files, so no deletions necessary
      $highest_move_num = $missing_log_num - 1; // work down from here to rename log files from app_$num.log to app_($num+1).log
    } else {
      $lowest_delete_num = $log_max_number; // moving all files, so remove the one before the max
      $highest_move_num = $log_max_number - 1; // move all files up
    }

    // handle deletions
    foreach ($logfiles as $num => $name) {
      if ($num >= $lowest_delete_num) {
        unlink($log_path . '/' . $name);
      }
    }

    // now rotate the files
    for ($i = $highest_move_num; $i >= 0; $i--) {
      if (!isset($logfiles[$i])) {
        continue; // for safety, this shouldn't happen
      }
      $oldname = $log_path . '/' . $logfiles[$i];
      $newname = $log_path . '/' . $logname . '_' . ($i+1) . '.log';
      if (false === rename($oldname, $newname)) {
        return "could not rename logfile '$oldname' to '$newname'";
      }
    }
  }

  // open file, write preamble if file is new
  $write_preamble = (is_file($log_filename) ? false : true);
  $handle = fopen($log_filename, 'a'); // open/create file for appending
  if ($handle === false) {
    return "could not open logfile '$log_filename' for writing";
  }
  if ($write_preamble) {
    fwrite($handle, $log_preamble);
  }
  $log_handle = $handle;

  // log the start of a new instance...
  logger(0, getTimestamp().'==============================================================');

  // return success
  return true;
}

/**
 * @desc - writes to the open/initialized logfile
 * @note - semantics are different for this, a newline is appended automatically
 *         because otherwise you have to double quote long strings or append a
 *         newline, causing headaches for you or for php's string parser...
 * @precondition - log has been initialized
 * @param $level - log level (0-5), 0 = always log, 5 = log only for debugging
 * @param $text - text to log
 * @param optional $newline - false to exclude a newline, default is to append one
 * @return - true on success, false on error
 */
function loggerNL($level, $text)
{
  // include a NL and a date...
  global $log_handle;
  global $log_level;
  global $DBG;

  if ($level > $log_level && !$DBG) {
    return true;
  }
  if (!$log_handle) {
    return false;
  }
  $bytes = fwrite($log_handle, getTimeStamp().$text."\n");

  // print any args supplied
  $nArguments = func_num_args();
  if ($nArguments > 0) {
    for ($i = 2; $i < $nArguments; $i ++) {
      $arg = func_get_arg($i);
      writeVariable($arg, $i, true);
    }
    fwrite($log_handle, "\n");
  }

  return ($bytes !== false);
}
function logger($level, $text)
{
  global $log_handle;
  global $log_level;
  global $DBG;

  if ($level > $log_level && !$DBG) {
    return true;
  }
  if (!$log_handle) {
    return false;
  }
  $bytes = fwrite($log_handle, $text);

  // print any args supplied
  $nArguments = func_num_args();
  if ($nArguments > 0) {
    for ($i = 2; $i < $nArguments; $i ++) {
      $arg = func_get_arg($i);
      writeVariable($arg, $i, true);
    }
    fwrite($log_handle, "\n");
  }

  return ($bytes !== false);
}
function enterFunction($stringFunction)
{
  global $log_handle;
  global $log_level;
  global $DBG;

  if (!$log_handle) {
    return false;
  }

  if ($log_level >= 2 || $DBG) {
    $message = "\n";
    $bytes = fwrite($log_handle, $message);

    $message = getTimestamp() . "Enter: " . $stringFunction;
    $nArguments = func_num_args();
    if ($nArguments == 1) {
      $message .= "(no args)\n";
    } else {
      $message .= "(" . ($nArguments - 1) . " args):\n";
    }
    $bytes = fwrite($log_handle, $message);
    //error_log($message, $this->m_type, $this->m_file);
    for ($i = 1; $i < $nArguments; $i ++) {
      $arg = func_get_arg($i);
      writeVariable($arg, $i, true);
    }
  }
}

// Logs a variable with var_dump().
function writeVariable($var, $argumentNumber = 1, $forceLog = false)
{
  global $log_handle;
  global $log_level;

  if ($log_level >= 2 || $forceLog) {
    ob_start();
    var_dump($var);
    $contents = ob_get_contents();
    ob_end_clean();
    $message = "#" . $argumentNumber . ": " . $contents;
    $bytes = fwrite($log_handle, $message);
  }
}

// Gets the current time to stamp log entries.
function getTimestamp()
{
  $startTime = time();
  $timestr = date('m/d/Y H:i:s', $startTime);
  $s = sprintf("%s: ", $timestr);
  return $s;
}

function echoLog($msg)
{
  logger(0,$msg);
  echo $msg;
}
function close_log()
{
  global $log_handle;
  if ($log_handle) {
    fclose($log_handle);
  }
}
////////////////////////  END OF LOGGER FUNCTIONS  ////////////////////
////////////////////////  END OF LOGGER FUNCTIONS  ////////////////////

function getSourceBackupsInTheQueue($srcDpu, $ipForSrc, $silent)
{
  enterFunction("getSourceBackupsInTheQueue",$srcDpu, $ipForSrc);
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // Setup some common sql to the source
  $srcConn = pg_Connect("host=$srcDpu port=5432 dbname=bpdb user=postgres");
  if (!$srcConn) {
    echoLog("A database connection error occurred to $srcDpu.\n");
    exit_func($io);
  }
  $res = pg_query($srcConn, "set search_path to bp");
  if (!$res) {
    echoLog("An error occurred setting the DB search_path to bp on $srcDpu.\n");
    exit_func($io);
  }

  // get the target as seen by the source (e.g. hosts file or bp.managers)
  $targetName = getTargetNameFromSrc($srcConn, $srcDpu, $ipForSrc);

  // check the source queue making sure nothing is active, pending
  // so when when the source replication is resumed it doesn't try
  // to replicate to the old target
  $noBackups = inTheQueue($srcConn, $srcDpu, $targetName, $silent);

  return $noBackups;
}

function inTheQueue($srcConn, $srcDpu, $targetName, $silent)
{
  enterFunction("inTheQueue",$srcDpu, $targetName, $silent);
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // This is querying the rep queue on the Source, so no Vault Name Suffix is needed!
  // Account for items suspended in the queue!
  $q = "SELECT backup_no from replication_queue where vault = '$targetName' and status != 2";
  logger(0, "get backups active in the rep queue from the original target:", $q);
  $res = pg_query($srcConn, $q);
  if (!$res) {
    echoLog("An error occurred reading the replication queue for $srcDpu.\n");
    exit_func($io);
  }
  
  $backups = "";
  $nrows = pg_num_rows($res);
  logger(0, "#ofbackupsinthequeue:", $nrows);
  if ($nrows == 0) {
    if ($silent == false) {
      echo("Currently the queue is empty.\n");
    }
    return $nrows;
  }
  if ($silent == false) {
    echoLog("The replication queue on $srcDpu contains the following backups that will not be part of a migration until they have completed:\n");
    while ($row = pg_fetch_row($res)) {
      echoLog("Backup: $row[0]\n");
    }
  }
  return $nrows;
}

function getTargetNameFromSrc($srcConn, $srcDpu, $ipForSrc)
{
  enterFunction("getTargetNameFromSrc",$srcDpu, $ipForSrc);
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // Get the Target Name as seen by the source /etc/hosts as this
  // represents the target name used for replication which could 
  // be different then the target name in the targets /etc/hosts

  $name = "";
  $subcmd = "/usr/bp/bin/cmc_network host info $ipForSrc";
  $out = `/usr/bp/bin/bputil --remotecmd '$srcDpu' '$subcmd'`;
  $split = explode(".", $out, 2); // SUCCESS.    true/false
  if (count($split) < 2) {
    // cmd failed
    echoLog("WARNING, unable to get the hostname for the provided ip ($ipForSrc) from the Source ($srcDpu) host file. Trying the DB.\n");
    return getTargetFromSrc($srcConn, $srcDpu);
  }
  if (trim($split[0]) != "SUCCESS") {
    // cmd did not return any value
    echoLog("WARNING, unable to get the hostname for the provided ip ($ipForSrc) from the Source ($srcDpu) host file. Trying the DB.\n");
    return getTargetFromSrc($srcConn, $srcDpu);
  }
  $out = trim($split[1]);

  // $out contains a single string with 'ip 1.1.1.1\n name system_name'
  $split = explode(chr(10), $out, 2); // chr(10) = newline
  if (count($split) < 2) {
    // cmd failed
    echoLog("WARNING, unable to get the hostname for the provided ip ($ipForSrc) from the Source ($srcDpu) host file. Trying the DB.\n");
    echoLog($out);
    return getTargetFromSrc($srcConn, $srcDpu);
  }

  if (substr($split[0],0,4) === "name"){
    $nameSplit = explode(" ", $split[0], 2);
    $name = $nameSplit[1];
  }
  else if (substr($split[1],0,4) === "name"){
    $nameSplit = explode(" ", $split[1], 2);
    $name = $nameSplit[1];
  }
  
  if ($name == "") {
    echoLog("WARNING, unable to get the hostname for the provided ip ($ipForSrc) from the Source ($srcDpu) host file. Trying the DB.\n");
    $name = getTargetFromSrc($srcConn, $srcDpu);
  }

  return $name;
}

function getTargetFromSrc($srcConn, $srcDpu)
{
  enterFunction("getTargetFromSrc",$srcDpu);
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // Get the Target Name as seen by the source database as this
  // represents the target name used for replication which could 
  // be different then the target name in the targets /etc/hosts

  $q = "SELECT ip, hostname from managers";
  logger(0, "get the target name as seen from the source:", $q);
  $res = pg_query($srcConn, $q);
  if (!$res || (pg_num_rows($res) == 0)) {
    echoLog("An error occurred obtaining manager (the Target) info for $srcDpu.\n");
    exit_func($io);
  }

  $targetIP   = pg_result($res, 0, "ip");
  $targetName = pg_result($res, 0, "hostname");
  logger(0, "target IP:", $targetIP);
  logger(0, "target name:", $targetName);
  #$sysIP = $_SERVER["REMOTE_ADDR"];
  #$sysIP = $_SERVER["SERVER_NAME"];
  #echo "system IP $sysIP\n";

  return $targetName;
}

function isSourceRepActive($srcDpu)
{
  // returns >0 Replication Active
  // returns 0 Replication Inactive
  // return -1, nw or other problems bp_error()
  // return -2, dpuid not found

  enterFunction("isSourceRepActive",$srcDpu);

  // Get my ID
  $dpu_id = getSystemIdForDPU($srcDpu);
  if (empty($dpu_id)) {
    logger(0,"dpu_id not found, replication unchecked, for:", $srcDpu);
    return -2; // indicates dpu not found error (bp_error() is not set!)
  }
  $maxConcurrent = bp_get_ini_value('Replication', 'MaxConcurrent', $dpu_id);
  logger(0,"MaxConcurrent:",$maxConcurrent);
  if ($maxConcurrent === false) {
    logger(0,"Could not determine REPLICATION is active: ",bp_error());
    return -1; // indicates use bp_error() for message
  }

  return $maxConcurrent;
}

function isSourceRepEnabled($srcDpu)
{
  // returns 1 Replication Enabled
  // returns 0 Replication Disabled
  // return -1, nw or other problems bp_error()
  // return -2, dpuid not found

  enterFunction("isSourceRepEnabled",$srcDpu);
  global $TRUE_VALUE_ARRAY;

  // Get my ID
  $dpu_id = getSystemIdForDPU($srcDpu);
  if (!empty($dpu_id)) {
    $isReplicationEnabled = bp_get_ini_value('Replication', 'Enabled', $dpu_id);
    logger(0,"isReplicationEnabled:",$isReplicationEnabled);
    if ($isReplicationEnabled === false) {
      logger(0,"Could not determine REPLICATION Enabled: ",bp_error());
      return -1; // indicates use bp_error() for message
    }

    $isReplicationEnabled = in_array($isReplicationEnabled, $TRUE_VALUE_ARRAY);
    if($isReplicationEnabled == true){
      return 1;
    }
  }
  else {
    logger(0,"dpu_id not found, replication unchecked, for:", $srcDpu);
    return -2; // indicates dpu not found error (bp_error() is not set!)
  }

  return 0;
}


function getSystemIdForDPU($dpu)
{
  enterFunction("getSystemIdForDPU", $dpu);

  $dpu_id = "";
  // get all the systems, then walk them to find our dpu
  $systems = bp_get_system_list();
  logger(2,"bp_get_system_list:",$systems);
  foreach ($systems as $key => $system_name) {
    if ($system_name == $dpu) {
      $dpu_id = $key;
      break;
    }
  }

  return $dpu_id;
}

function displayBackupsToMigrate($selectedDPU, &$totalAllBUsize, &$totalLastBUsize)
{
  enterFunction("displayBackupsToMigrate", $selectedDPU);
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // 5. locate the backups to be replicated per client and provide an estimate on how
  //     long migration will take
  //  a.provide the list for last group (get_all_last_backups $client)
  //  b.provide the list for all valid active backups per client (psql bp.backups)
  //  c.prompt to migrate last or all

  // get the system id for our selected DPU
  $selectedSystemId = getSystemIdForDPU($selectedDPU);

  // get all the nodes on the target associated with this system
  // theNodes is an array of arrays containing node_no / gcname pairs
  $theNodes = getNodesNameAndNoPerSystemIdOnTarget($selectedSystemId);

  echo "\nListed below are the nodes (clients) associated with Source $selectedDPU that have replicated backups on the existing Target. The total number of backups (), and their total size is displayed per their entire list of active backups and their last backup group.\n";
  echo "--------------------------------------------------------------------------------------\n";
  echo "Node       \t\tAll Backups\t\tLast Backup Group\n";
  echo "--------------------------------------------------------------------------------------\n";
  $totalEncAllBUsize  = 0;
  $totalDecAllBUsize  = 0;
  $totalEncLastBUsize = 0;
  $totalDecLastBUsize = 0;

  // for each node, locate backups to migrate
  foreach ($theNodes as $key => $nodeEntry) {
    $node_no = $nodeEntry["node_no"];
    $gcname  = $nodeEntry["gcname"];
    //-----------------------------------------------------------------------
    //# Node                     All Backups    Last Backup Group
    //-----------------------------------------------------------------------
    //1234567890123456789012345678901234567890123456789012345
    //       T       T       T       T       T       T
    
    // getMigrationBackupInfo() returns an array containing number and size based on allorLast
    // and weeds out backups already in the replication queue
    $allOrLast = 1; // 1 = all
    $migrationBackupInfo = getMigrationBackupInfo($node_no, $allOrLast); // ["number"] ["encSize"] ["decSize"]
    if ($migrationBackupInfo['number'] === 0) {
      $stringAll = "(".$migrationBackupInfo['number'].")";
    }
    else if ($migrationBackupInfo['encSize'] === 0 && $migrationBackupInfo['decSize'] === 0) {
      $stringAll = "(".$migrationBackupInfo['number'].") < 1 MB";
    }
    else {
      $size      = $migrationBackupInfo['encSize'] + $migrationBackupInfo['decSize'];
      $stringAll = "(".$migrationBackupInfo['number'].") ".$size." MB";
    }
    $totalEncAllBUsize += (int) $migrationBackupInfo['encSize'];
    $totalDecAllBUsize += (int) $migrationBackupInfo['decSize'];
    $allOrLast = 0; // 0 = last
    $migrationBackupInfo = getMigrationBackupInfo($node_no, $allOrLast);
    if ($migrationBackupInfo['number'] === 0) {
      $stringLast = "(".$migrationBackupInfo['number'].")";
    }
    else if ($migrationBackupInfo['encSize'] === 0 && $migrationBackupInfo['decSize'] === 0) {
      $stringLast = "(".$migrationBackupInfo['number'].") < 1 MB";
    }
    else {
      $size      = $migrationBackupInfo['encSize'] + $migrationBackupInfo['decSize'];
      $stringLast = "(".$migrationBackupInfo['number'].") ".$size." MB";
    }
    $gcnamePad     = str_pad($gcname, 20);
    $stringAllPad  = str_pad($stringAll,20);
    $stringLastPad = str_pad($stringLast,22);
    //echo "$gcnamePad\t$stringLastPad\t$stringAllPad\n";
    echo "$gcnamePad\t$stringAllPad\t$stringLastPad\n";
    $totalEncLastBUsize += (int) $migrationBackupInfo['encSize'];
    $totalDecLastBUsize += (int) $migrationBackupInfo['decSize'];
  }
  echo "--------------------------------------------------------------------------------------\n";

  // Now add summary data: time estimates,
  $timeEst = getTimeEst2($totalEncAllBUsize, $totalDecAllBUsize);
  settype($timeEst, 'integer');
  $HHMM = sprintf("%02d:%02d", floor($timeEst/60), $timeEst%60);
  echoLog("Duration estimate to complete ALL backup migration: ".$HHMM." (HH:MM)\n");

  $timeEst = getTimeEst2($totalEncLastBUsize, $totalDecLastBUsize);
  settype($timeEst, 'integer');
  $HHMM = sprintf("%02d:%02d", floor($timeEst/60), $timeEst%60);
  echoLog("Duration estimate to complete LAST group migration: ".$HHMM." (HH:MM)\n");
  echo "--------------------------------------------------------------------------------------\n";
  echo("Do you want to migrate all or just the last group? (Enter 1 for all, 0 for last)\n");
  echo "Selection ('q' to quit): ";
  $allOrLast = trim(fgets($io));
  if (!is_numeric($allOrLast) || $allOrLast < 0 || $allOrLast > 1) {
    if ($allOrLast != 'q') echo "Invalid selection. Relaunch to start over\n";
    exit_func($io);
  }

  return $allOrLast; //?
}

function getTimeEst($backup_size_dec, $backup_size_enc, $isVaultEncryptionEnabled)
{
  enterFunction("getTimeEst", $backup_size_dec, $backup_size_enc, $isVaultEncryptionEnabled);

  // Assume MB units supplied
  // Assume backups are encrypted if encryption is enabled
  // Assume worst case that all blocks are migrated
  // NOTE: decryption only occurs if backup is encrypted
  //       encryption only occurs if backup is encrypted and the vault has encryption enabled
  //     encrypted data: 10Mbps
  // non-encrypted data: 15Mbps

  $timeInMins = 0;

  // For now just base encrypted backups on if encryption is enabled
  // Otherwize, need to accumulate non-encrypted and encrypted to get an answer.
  if ($isVaultEncryptionEnabled) {
    $timeInMins = ($backup_size_dec/600);
  }
  else {
    $timeInMins = ($backup_size_dec/900);
  }

  if ($timeInMins < 1 && ($backup_size_dec > 0 || $backup_size_enc > 0)) {
    $timeInMins = 1;
  }
  return $timeInMins;
}

function getTimeEst2($backup_size_enc, $backup_size_dec)
{
  enterFunction("getTimeEst2", $backup_size_enc, $backup_size_dec);

  // Assume MB units supplied
  // NOTE: decryption only occurs if backup is encrypted
  //       encryption only occurs if backup is encrypted and the vault has encryption enabled
  //     encrypted data: 10Mbps
  // non-encrypted data: 15Mbps

  $timeInMins = 0;

  if ($backup_size_enc) $timeInMins += ($backup_size_enc/600);
  if ($backup_size_dec) $timeInMins += ($backup_size_dec/900);
  if ($timeInMins < 1 && ($backup_size_dec > 0 || $backup_size_enc > 0)) {
    $timeInMins = 1;
  }
  return $timeInMins;
}

function getNodesNameAndNoPerSystemIdOnTarget($selectedSystemId)
{
  enterFunction("getNodesNameAndNoPerSystemIdOnTarget", $selectedSystemId);
  global $TARGET_DB_CONN;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $output = array();

  $q = "SELECT node_no, gcname from nodes where system_id = '$selectedSystemId'";
  logger(0, "get nodes and gcname per system:", $q);
  $res = pg_query($TARGET_DB_CONN, $q);
  if (!$res) {
    echoLog("An error occurred obtaining replication nodes.\n");
    exit_func($io);
  }

  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    return $output;
  }
  while ($row = pg_fetch_row($res)) {
    logger(2,"node_no:", $row[0]);
    logger(2,"GC Name:", $row[1]);
    $pair = array();
    $pair["node_no"] = $row[0];
    $pair["gcname"] = $row[1];
    $output[] = $pair;
  }
  return $output;
}

function getBackupsPerNodes($theNodes, $allOrLast)
{
  enterFunction("getBackupsPerNodes", $theNodes, $allOrLast);
  global $TARGET_DB_CONN;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // returns an array containing the backups numbers assoc with a source (per its nodes)
  // List is not filtered by backups already migrated in or the queue to be migrated
  $output = array();
  if (count($theNodes) == 0) {
    return $output;
  }

  // all = 1, last = 0
  if ($allOrLast == 1) {
    #Technique 1: find all files not purged
    $node_q = "";
    $first = 1;
    foreach ($theNodes as $key => $nodeEntry) {
      $node_no = $nodeEntry["node_no"];
      if ($first == 1) {
        $node_q = $node_q." node_no = '$node_no' ";
        $first = 0;
      }
      else {
        $node_q = $node_q." or node_no = '$node_no' ";
      }
    }
    $q = "SELECT backup_no from backups where $node_q and tape_no != 4294967295 and status = 3168 ORDER by backup_no";
    logger(0, "get backups not purged:", $q);
    $res = pg_query($TARGET_DB_CONN, $q);
    if (!$res) {
      echoLog("An error occurred obtaining backup info for client:\n");
      logger(0,"the nodes", $theNodes);
      exit_func($io);
    }
    while ($row = pg_fetch_row($res)) {
      $output[] = $row[0];
    }
  }
  else {
    #Technique 2: find all files as part of the last backup group

    //$clientList[] = (int)$node_no;
    $systemId = 0;
    foreach ($theNodes as $key => $nodeEntry) {
      $node_no = $nodeEntry["node_no"];
      logger(2,"Calling bp_get_all_last_backups with node_no:", $node_no);
      $clientList = array();
      $clientList[] = (int)$node_no;

      $ret = bp_get_all_last_backups($clientList, $systemId);
      //$ret = bp_get_all_last_backups($theNodes, $systemId);

      if ($ret === false) {
        echoLog("error is: ".bp_error()."\n");
      } else {
        #var_dump($ret);
      }

      //$backup_ids = array();
      $needle = 'backup_ids';
      $val = false;
      $val = array_search_key( $needle, $ret );
      if ($val != false) {
        //$x = array();
        //$x[] = $val;
        //$output = array_merge($output, $x);
        $output[]=$val[0];
      }
    }
  }
  asort($output);
  logger(2,"backups per node output:", $output);
  return $output;
}

function getMigrationBackupInfo($node_no, $allOrLast)
{
  enterFunction("getMigrationBackupInfo", $node_no, $allOrLast);
  global $TARGET_DB_CONN;
  global $VAULT_MIG_SUFFIX;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $P_ENCRYPTED;
  global $io;

  // returns an array containing total_no, total_size, last_no, last_size
  // Weeds out backups already in the replication queue
  $output = array();


  // little tricky as the rep queue could have 1 of 4 cases:
  // - empty queue
  // - just DR or Cross Vault entries
  // - just migrated entries
  // - mix of migrate, DR, CV
  // In the first 2 cases, no need to look at the queue
  // In the last 2 cases, only get backups that are migrated
  // Our goal is to find all backups NOT migrated yet!

  #########################
  if ($allOrLast == 1) {
    #Technique 1: find all backups not purged and not in the replication queue
    // Leverage the replication queue weeding out entries due to DR, or Crossvaulting
    $q = "SELECT backup_no, total_megs, properties from backups where node_no='$node_no' and tape_no != 4294967295 and status = 3168 and backup_no not in (SELECT backup_no from bp.replication_queue where vault LIKE '%$VAULT_MIG_SUFFIX') ORDER by backup_no";

    logger(0, "get backup details:", $q);
    $res = pg_query($TARGET_DB_CONN, $q);
    if (!$res) {
      echoLog("An error occurred obtaining backup info for client $node_no\n");
      exit_func($io);
    }

    $nFiles   = 0; // count how many files to migrate
    $nDecSize = 0; // count the total unencrypted backup size to migrate
    $nEncSize = 0; // count the total encrypted backup size to migrate
    while ($row = pg_fetch_row($res)) {
      logger(1,"backup_no: $row[0], total_megs: $row[1]");
      # total megs for system metadata is 0!!!
      if (($row[2] & $P_ENCRYPTED) > 0) {
        $nEncSize += $row[1];
      }
      else {
        $nDecSize += $row[1];
      }

      $nFiles++;
    }

    $output["number"]  = $nFiles;
    $output["encSize"] = $nEncSize;
    $output["decSize"] = $nDecSize;
  }
  else {
    #########################
    #Technique 2: find all files as part of the last backup group

    logger(2,"Calling bp_get_all_last_backups with node_no:", $node_no);

    $clientList   = array();
    $clientList[] = (int)$node_no;
    #var_dump($clientList);
    $systemId = 0;

    $ret = bp_get_all_last_backups($clientList, $systemId);

    if ($ret === false) {
      echoLog("error is: ".bp_error()."\n");
    } else {
      #var_dump($ret);
    }

    //$backup_ids = array();
    #$backup_ids = $ret['backup_ids'];
    #$backup_ids = array_search('backup_ids', $ret);
    #var_dump($backup_ids);
    $needle = 'backup_ids';
    $results = array_search_key( $needle, $ret );
    #var_dump($results);

    $nFiles   = 0; // count how many files to migrate
    $nDecSize = 0; // count the total unencrypted backup size to migrate
    $nEncSize = 0; // count the total encrypted backup size to migrate
    
    if (is_array($results)) {
      $q = "";
      foreach ($results as $i => $value) {
        logger(1,"backup #: $results[$i]");
        // Leverage the replication queue weeding out entries due to DR, or Crossvaulting
        $q = "SELECT total_megs, properties from backups where backup_no = $results[$i] and backup_no not in (SELECT backup_no from replication_queue where vault LIKE '%$VAULT_MIG_SUFFIX')";
        logger(1, "get backup details:", $q);
        $res = pg_query($TARGET_DB_CONN, $q);
        if (!$res) {
          echoLog("An error occurred obtaining backup info for $results[$i], client $node_no\n");
        }
        else {
          $nrows = pg_num_rows($res);
          if ($nrows == 0) {
            continue;
          }
          $props = (int)pg_result($res, 0, "properties");

          if (($props & $P_ENCRYPTED) > 0) {
            $m = (int)pg_result($res, 0, "total_megs");
            logger(2," total megs:", $m);
            $nEncSize += $m;
          }
          else {
            $m = (int)pg_result($res, 0, "total_megs");
            logger(2," total megs:", $m);
            $nDecSize += $m;
          }
          $nFiles++;
        }
      }
    }
    $output["number"]  = $nFiles;
    $output["encSize"] = $nEncSize;
    $output["decSize"] = $nDecSize;
  }

  return $output;
}

function getBackupsToMigrate($theNodes, $allOrLast)
{
  enterFunction("getBackupsToMigrate", $theNodes, $allOrLast);
  global $TARGET_DB_CONN;
  global $VAULT_MIG_SUFFIX;
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  // returns an array containing a sorted list of backups to migrate per the node list
  // Weeds out backups already in the replication queue
  $output = array();
  if (count($theNodes) == 0) {
    return $output;
  }

  #########################
  if ($allOrLast == 1) {
    #Technique 1: find all files not purged and not in the replication queue
    $node_q = "";
    $first = 1;
    foreach ($theNodes as $key => $nodeEntry) {
      $node_no = $nodeEntry["node_no"];
      if ($first == 1) {
        $node_q = $node_q." node_no = '$node_no' ";
        $first = 0;
      }
      else {
        $node_q = $node_q." or node_no = '$node_no' ";
      }
    }

    // Leverage the replication queue weeding out entries due to DR, or Crossvaulting
    $q = "SELECT backup_no from backups where $node_q and tape_no != 4294967295 and status = 3168 and backup_no not in (SELECT backup_no from bp.replication_queue where vault LIKE '%$VAULT_MIG_SUFFIX') ORDER by backup_no";
    logger(0, "get backups:", $q);
    $res = pg_query($TARGET_DB_CONN, $q);
    if (!$res) {
      echoLog("An error occurred obtaining backup info for client $node_no\n");
      exit_func($io);
    }

    while ($row = pg_fetch_row($res)) {
      $output[] = $row[0];
    }
  }
  else {
    #########################
    #Technique 2: find all files as part of the last backup group
    foreach ($theNodes as $key => $nodeEntry) {
      $node_no = $nodeEntry["node_no"];

      logger(2,"Calling bp_get_all_last_backups with node_no:", $node_no);

      $clientList = array();
      $clientList[] = (int)$node_no;
      $systemId = 0;

      $ret = bp_get_all_last_backups($clientList, $systemId);

      if ($ret === false) {
        echoLog("error is: ".bp_error()."\n");
      } else {
        #var_dump($ret);
      }

      //$backup_ids = array();
      $needle = 'backup_ids';
      $results = array_search_key( $needle, $ret );

      if (!is_array($results)) {
        continue;
      }

      //$q = "";
      foreach ($results as $i => $value) {
        logger(1,"backup #: $results[$i]");
        // Leverage the replication queue weeding out entries due to DR, or Crossvaulting
        $q = "SELECT backup_no from backups where backup_no = $results[$i] and backup_no not in (SELECT backup_no from replication_queue where vault LIKE '%$VAULT_MIG_SUFFIX')";
        logger(1, "get backups:", $q);
        $res = pg_query($TARGET_DB_CONN, $q);
        if (!$res) {
          echoLog("An error occurred obtaining backup info for $results[$i], client $node_no\n");
        }
        else {
          $nrows = pg_num_rows($res);
          if ($nrows == 0) {
            continue;
          }
          // we should only have one backup per our request above...
          while ($row = pg_fetch_row($res)) {
            $output[] = $row[0];
          }
        }
      } // foreach ($results)
    } // foreach ($theNodes)

    // sort the list of backup_no
    asort($output);
  }

  logger(2,"backups per node output:", $output);
  return $output;
}

function array_search_key( $needle_key, $array )
{
  // this only finds the first occurrence.....

  foreach($array AS $key=>$value){
    if($key === $needle_key) {
      return $value;
    }
    if(is_array($value)){
      if( ($result = array_search_key($needle_key,$value)) !== false)
        return $result;
    }
  }
  return false;
} 

function getNewTargetNW()
{
  enterFunction("getNewTargetNW");

  global $io;
  global $VAULT_MIG_SUFFIX;

  // returns an array containing name, ip address
  $output = array();

  echo "\n\nEnter the name and IP address of the target to migrate the backups to.\nFirst the name:";
  $nameVal = trim(fgets($io));
  echo "Now the (non-vpn) IP address:";
  $ipVal = trim(fgets($io));

  // return the entry containing both name and ip
  logger(2,"New Target Name:", $nameVal);
  logger(2,"ip address:", $ipVal);

  $output["nameInt"] = $nameVal.$VAULT_MIG_SUFFIX;
  $output["nameExt"] = $nameVal;
  $output["ip"]      = $ipVal;

  return $output;
}

function validateNewTarget($newTargetNameAndIP)
{
  enterFunction("validateNewTarget", $newTargetNameAndIP);
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $ip      = $newTargetNameAndIP["ip"];
  $nameExt = $newTargetNameAndIP["nameExt"];
  $nameInt = $newTargetNameAndIP["nameInt"];

  // make sure name and IP are not null!
  if ($nameExt == "" || $ip == "") {
    echo "Invalid entries, must not be blank.\n";
    exit_func($io);
  }

  // make sure the name and IP is not the same as the existing target!
  $oldTargetHostname = getHostNameLoc();
  $oldTargetIP       = getIP($oldTargetHostname);
  if ($oldTargetHostname === "$nameExt" || $oldTargetIP === $ip) {
    echo "The new target must not be the same as the existing target.\n";
    exit_func($io);
  }

/* pinging during getNewTargetHostname
  // can we ping the new target?
  $retval = -1;
  $cmd = "/usr/bp/bin/cmc_replication util ping $ip";
  system($cmd, $retval);
  if ($retval != 0) {
    // the output of cmc_replication will appear on the screen..
    //echo "$ip is not reachable.\n";
    exit_func($io);
  }
*/

  // is new target replication capable?
  $out = `/usr/bp/bin/bputil --remotecmd '$ip' '/usr/bp/bin/cmc_capabilities exists replication'`;
  $split = explode(".", $out, 2); // SUCCESS.    true/false
  if (count($split) < 2) {
    // cmd failed
    echoLog("The new target could not be verified if it supports replication.\n");
    echoLog($out);
    exit_func($io);
  }
  if (trim($split[0]) != "SUCCESS") {
    // cmd failed
    echo "The new target could not be verified if it supports replication.\n";
    var_dump($out);
    exit_func($io);
  }
  if (trim($split[1]) == "false"){
    echo "The new target does not support replication.\n";
    exit_func($io);
  }
  echo "The new Target '$nameExt' has been verified for Replication support!\n";
}

function checkEncryptionOnNewTarget($newTargetNameAndIP)
{
  enterFunction("checkEncryptionOnNewTarget", $newTargetNameAndIP);
  global $LOG_NAME;
  global $CONT_ON_ERROR;
  global $io;

  $ip      = $newTargetNameAndIP["ip"];
  $nameExt = $newTargetNameAndIP["nameExt"];
  $nameInt = $newTargetNameAndIP["nameInt"];

  // is new target running with encryption?
  $out = `/usr/bp/bin/bputil --remotecmd '$ip' '/usr/bp/bin/cryptoDaemon -s'`;

  // Possible Results:
  // When cryptoDaemon is not running:
  // FAILED:
  // cryptoDaemon: no running cryptoDaemon!

  // When cryptoDaemon is running, no master key file:
  // FAILED:
  // cryptoDaemon: master key: none; connections: 0

  // For not set or enabled this is the result:
  // FAILED:
  // cryptoDaemon: master key: notset; connections: 0
 
  // When cryptoDaemon is running and enabled:
  // FAILED:
  // cryptoDaemon: master key: set; connections: 0

  // So, ignore the FAILED status, and just search for "master key: set;"

  $findme = "master key: set;";
  $pos = strpos($out,$findme);
  if ($pos === false) {
    echoLog("Encryption is not enabled on the new target. This is required to match existing target.\n");
    echoLog("Details: $out\n");
    if (!$CONT_ON_ERROR) exit_func($io);
  } else {
    logger(2, "Encryption is enabled on new target $ip");
  }
  echo "The new Target '$nameExt' has been verified for encryption support!\n";
}

function getNewTargetDefaultDevice($newTargetIP, &$defaultDevice, &$defaultDevCap)
{
  enterFunction("getNewTargetDefaultDevice", $newTargetIP);

  $rc = 0;

  // TODO, whatif...see note below...
  // In case we need more logic if the device is not found:
  // "bp/applianceAgent/backup.c" line 974 of 1392
  // TODO, if problems, allow the users to override?

  $defaultDevice = "";

  // The property DefaultTapeDevice is per legacy usage
  // located in master.ini file "DefaultTapeDevice=6  ; device number of default tape device"

  $subcmd = "/usr/bp/bin/bputil -g 'Configuration Options' 'DefaultTapeDevice' '' /usr/bp/bpinit/master.ini";
  $out = `/usr/bp/bin/bputil --remotecmd '$newTargetIP' "$subcmd"`;
  // parse the return checking for SUCCESS.
  $split = explode(".", $out, 2); // SUCCESS.    true/false
  if (count($split) < 2) {
    // cmd failed
    echoLog("WARNING, we were not able to obtain the default device for the new target.\n");
    echoLog($out);
    return -1;
  }
  if (trim($split[0]) != "SUCCESS") {
    // cmd failed
    echoLog("WARNING, we were not able to obtain the default device for the new target.\n");
    echoLog($out);
    return -1;
  }
  $defaultTapeDevice = trim($split[1]);
  //var_dump($defaultDevice);

  // Obtain the default device using SQL Query.
  $connNewTar = pg_Connect("host=$newTargetIP port=5432 dbname=bpdb user=postgres");
  if($connNewTar === FALSE) {
    $message = "Unable to connect to the new Target's database.\n";
    echoLog($message);
    return 1;
  }
  $query = "SELECT dev_rw_name, capacity, is_online FROM bp.devices WHERE device_no = '$defaultTapeDevice'";
  $res = pg_query($connNewTar, $query);
  if ($res === false) {
    $message = "Unable to get default device from the new Target's database.(".pg_last_error($connNewTar).")\n";
    echoLog($message);
    return -1;
  }

  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    $message = "There are no matching devices for our value ".$defaultTapeDevice."\n";
    echoLog($message);
    return 1;
  }

  $defaultDevice = pg_fetch_result($res, 0, "dev_rw_name");
  $defaultDevCap = pg_fetch_result($res, 0, "capacity");
  $is_online     = pg_fetch_result($res, 0, "is_online");

  //var_dump($defaultDevice);
  //var_dump($capacity);
  //var_dump($is_online);

  // make sure the device is online
  if ($is_online != 't') {
    $message = "The new Target's default device ".$defaultDevice." must be online for migration.\n";
    echoLog($message);
    return 1;
  }

  // make sure capacity is minimum of 128 GB
  $capacityGB    = $defaultDevCap/(1024*1024);
  if ($capacityGB < 128) {
    $message = "Replication minimum capacity is 128 (GB). The default device ".$defaultDevice." only has ".$capacityGB." (GB).\n";
    echoLog($message);
    return -1; // issuing a warning, but not halting it...
  }

  //$message .= $defaultDevice." capacity: ".$capacityGB." (GB)\n";
  //echoLog($message);
  return 0;
}

function getDefaultDeviceFreeSpace($newTargetIP, $defaultDevice, $defaultDevCap)
{
  enterFunction("getDefaultDeviceFreeSpace", $newTargetIP, $defaultDevice, $defaultDevCap);

  //$freeSpace = disk_free_space($vaultBackupDevice) / (1024 * 1024);
  // bug 16209: disk_free_space works at the device level...
  // leverage the db capacity value and system used value

  $subcmd = "du -s $defaultDevice";
  $out = `/usr/bp/bin/bputil --remotecmd '$newTargetIP' "$subcmd"`;
  // parse the return checking for SUCCESS.
  $split = explode(".", $out, 2); // SUCCESS.    true/false
  if (count($split) < 2) {
    // cmd failed
    echoLog("WARNING, we were not able to obtain the default device freespace for the new target.\n");
    echoLog($out);
    return -1;
  }
  if (trim($split[0]) != "SUCCESS") {
    // cmd failed
    echoLog("WARNING, we were not able to obtain the default device freespace for the new target.\n");
    echoLog($out);
    return -1;
  }
  // split it again... something like 632    /backups/D2DBackups/
  $splitAgain = explode("/", $split[1], 2); // SUCCESS.    true/false
  if (count($splitAgain) < 2) {
    // cmd failed
    echoLog("WARNING, we were not able to obtain the default device freespace for the new target.\n");
    echoLog($split[1]);
    return -1;
  }
  $usedSpace = trim($splitAgain[0]);

  $freeSpace = ($defaultDevCap - $usedSpace) / 1024;
  // return free space (MB)
  return round($freeSpace, 2);
}

//////////////////////////
//// Some openvpn routines
//////////////////////////
function isOpenVPNConfigured($remoteSysIP)
{
  enterFunction("isOpenVPNConfigured", $remoteSysIP);

  // Determine if openvpn daemon is running on the target
  $ipVal     = "";
  $OPENVPN   = "openvpn";
  $out       = "";
  $subcmd    = "ps -elf | grep -v grep |grep ".escapeshellarg($OPENVPN);

  if ($remoteSysIP == "") {
    $out = `$subcmd`;
    if ($out == "") {
      return false;
    }
  }
  else {
    $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' "$subcmd"`;
    // parse the return checking for SUCCESS.
    $split = explode(".", $out, 2); // SUCCESS.    true/false
    if ((count($split) < 1) || (trim($split[0]) != "SUCCESS")) {
      // cmd failed
      //echoLog("WARNING, we were not able to determine if a secure tunnel (openvpn) is configured on the new target.\n");
      //echoLog($out);
      return false;
    }
    if (count($split) < 2) {
      // assume not running
      return false;
    }
  
    return true; 
  }
}

function getOpenVPNIPaddr($remoteSysIP)
{
  enterFunction("getOpenVPNIPaddr", $remoteSysIP);
  global $io;

  // THIS ROUTINE IS VERY UGLY...tries to get the openvpn IP on the new target
  // by first seeing if openvpn daemon is running then parsing the results
  // of ifconfig....looking for tun0....
  // Could use SQL to get it from the bp.alerts using a like '%OpenVPN%' cmd but
  // this would require access to the new target's db from this target, which
  // isn't established yet...still ugly though....
  // OpenVPN is configured, get the open IP addr from ifconfig. If unable, prompt for it.

  $ipVal     = "";
  $INET_ADDR = "inet addr:";
  $out       = "";
  $subcmd    = "ifconfig |grep -n1 tun0|grep ".escapeshellarg($INET_ADDR);

  if ($remoteSysIP == "") {
    $out = `$subcmd`;
  }
  else {
    $out = `/usr/bp/bin/bputil --remotecmd '$remoteSysIP' "$subcmd"`;
    $split = explode(".", $out, 2); // SUCCESS.    true/false
    if (count($split) < 2) {
      // cmd failed
      echoLog("WARNING, we were not able to obtain the secure tunnel (openvpn) IP addr on the new target.\n");
      logger(2, $out);
      echo "Enter the new target openvpn IP address (e.g. 172.17.3.1):";
      $ipVal = trim(fgets($io));
      return $ipVal;
    }
    if (trim($split[0]) != "SUCCESS") {
      // cmd failed
      echoLog("WARNING, we were not able to obtain the secure tunnel (openvpn) IP addr on the new target.\n");
      logger(2, $out);
      echo "Enter the new target openvpn IP address (e.g. 172.17.3.1):";
      $ipVal = trim(fgets($io));
      return $ipVal;
    }
    $out = trim($split[1]);
  }

  // extract the ip
  // 18-   inet addr:172.17.3.1  P-t-P:172.17.3.2  Mask:255.255.255.255
  //echo "$split[1]" |`awk -F ":" {'print $2'}|awk '{print $1}'`;
  //var_dump($out);
  $split2 = explode(":", $out, 2); 
  if (count($split2) < 2) {
    // cmd failed
    echoLog("WARNING, we were not able to obtain the openvpn IP addr on the new target.\n");
    logger(2, $out);
    echo "Enter the new target openvpn IP address (e.g. 172.17.3.1):";
    $ipVal = trim(fgets($io));
    return $ipVal;
  }
  //var_dump($split2[0]);
  //var_dump($split2[1]);
  $split3 = explode(" ", $split2[1], 2); 
  if (count($split3) < 2) {
    // cmd failed
    echoLog("WARNING, we were not able to obtain the openvpn IP addr on the new target.\n");
    logger(2, $out);
    echo "Enter the new target openvpn IP address (e.g. 172.17.3.1):";
    $ipVal = trim(fgets($io));
    return $ipVal;
  }
  //var_dump($split3[0]);
  $ipVal = $split3[0];
  return $ipVal;
}

function openvpnSequence($all=false)
{
  enterFunction("openvpnSequence", $all);
  global $MIGRATE;
  global $SWITCH;
  //global $CLEANUP;
  //global $CANCEL;

  if ($all) {
    echo "\nThe OpenVPN setup procedure.\n";
  }
  else {
    echo "\nOpenVPN notes:";
  }
  if ($all) {
    echo "\nPlanning, on the new Target:\n";
    echo "  - If original Source and Target have openvpn setup, then the new Target should create a secure tunnel prior to migration. Use the RRC, Replication -> Secure Tunnel Settings, \"Create a Secure Tunnel Target\". Make sure the tunnel IP subnet (e.g. 172.17.3) is different from the original Target.\n";   // The -migrate option will check and warn.\n";
  }

  if ($all || $MIGRATE) {
    echo "\nPost migration, on the new Target:\n";
    echo "  - Before switching, a secure tunnel certificate should be signed for the Source being migrated. This is a placeholder. Use the RRC, Replication -> Secure Tunnel Settings, \"Sign the Secure Tunnel Certificate Request\". (If needed, from the Source's RRC, \"Generate a Secure Tunnel Certificate\".)\n";     // The -switch option will check and warn.\n";
    echo "\nHint: open up a browser to the Source and new Target prior to the switch procedure as you'll need to generate secure tunnel certificates at various migration stages. Be careful of refreshes as there are times during the migration where a browser startup or refresh will hang until some manual network edits are completed. Also read the Post Switch Warning below.\n";
  }

  if ($all || $SWITCH) {
    echo "\nPost switch:\n";
    echo "  - On migrated Source: remove old openvVPN (--removeVPN)\n";
    echo "  - On migrated Source: generate a NEW Source Certificate using the RRC, \"Generate a Secure Tunnel Certificate\".\n";
    echo "  - On new Target: edit the /etc/hosts file, updating the migrated source IP to match the newly generated IP address of the Source (e.g. on the Source run 'ifconfig tun0' and grab the inet addr entry).\n";
    echo "  - On new Target: (re)link Source and new Target together using the RRC, \"Sign the Secure Tunnel Certificate Request\". Make sure you use the new Source Certificate and check \"Revoke Prior Secure Tunnel Certificate\". (See Warning message below if an error is reported.)\n";
    echo "  - On migrated Source: (re)link Source and new Target together using the RRC, \"Configure the Secure Tunnel on the Source System\".\n";
    echo "  - On migrated Source: correct the /etc/hosts entry for the new Target's nonVPN address to its nonVPN IP address\n";
    echo "  - On old Target: edit the /etc/hosts file, replacing the migrated Source VPN IP with the non-VPN IP address of the Source (e.g. on the Source run 'ifconfig eth0').\n";
    echo "Warning: If the Source appears in the RRC for the new Target before all the post switch procedures are completed, you'll get an error when relinking the secure tunnel for the Source and new Target. The RRC will report 'Summary: The source system hostname is invalid. The system ... already exists.  To use a Secure Tunnel to manage this system, you must delete it, configure a Secure Tunnel, then re-add it.'. DON'T delete the system as this will wipe out all the backups migrated. Instead you must manually alter the Source's name in the db system's table, then refresh the RRC, then continue with the post switch procedure. (Use original Source name during signing.) Once procedure is complete, alter the Source's name in the db system's table back to its real value and refresh the RRC.\n";
  }
}

function remove_openvpn()
{
global $io;

$OPENVPN_DIR="/etc/openvpn";
$OPENVPN_BIN_DIR="/etc/openvpn/bin";
$EASY_RSA_DIR=$OPENVPN_DIR."/easy-rsa";

echo "\n";
echo "This script stops openvpn & removes all configuration items.\n";
echo "It will remove *everything* under the $OPENVPN_DIR directory.\n";
echo "Per this utility, it is expected this open VPN removal only occurs\n";
echo "on the Source after a migration sequence has completed.\n";
/*
echo "If removing a client configuration (DPU), you MUST also manually edit\n";
echo "/etc/hosts on both the DPU and the vault!  On the DPU, remove the line\n";
echo "for \"vpn-IP vault_hostname\" and then edit the line with the real IP\n";
echo "of the vault, changing the name from vault_hostname-nonVPN back to the\n";
echo "actual vault_hostname.  On the vault (the OpenVPN server), remove the\n";
echo "line with the vpn-IP and hostname of the DPU (and, if there is an entry\n";
echo "for the real IP of the DPU with a modified dpu_hostname-nonVPN, change\n";
echo "it back to the actual dpu_hostname).\n";
*/
echo "\nContinue with removal of OpenVPN configuration? (enter to continue, 'q' to quit.)";
$selVal = trim(fgets($io));
if ($selVal == 'q') exit_func($io);

echo "stopping openvpn service...\n";
`service openvpn stop`;

if (is_dir($EASY_RSA_DIR)) {
  echo "\nRemoving easy-rsa directory...";
  `rm -rf $EASY_RSA_DIR`;
}

if (is_file("/etc/cron.weekly/updateCRL")){
  echo "\nRemoving cron updateCRL...";
  `rm -f /etc/cron.weekly/updateCRL`;
}

if (is_dir($OPENVPN_BIN_DIR)) {
  echo "\nRemoving openvpn scripts dir...";
  `rm -rf $OPENVPN_BIN_DIR`;
}

echo "\nRemoving all files from $OPENVPN_DIR...";
`rm -rf $OPENVPN_DIR/*`;

echo "\nRemoving /usr/bp/logs.dir/openvpn* log files...";
`rm -f /usr/bp/logs.dir/openvpn*`;

echo "\nRemoving alerts related to OpenVPN configuration...";
`psql -Upostgres bpdb -c "delete from bp.alerts where alert_text like '%OpenVPN%'"`;

echo "\nPlease continue with the rest of the OpenVPN Post Switch notes!";
echo "\nDone!\n";
}

