#!/usr/bin/php
<?php


//////////////////////////////////////////////////////////////////////
// Some Globals
//////////////////////////////////////////////////////////////////////
$INSTALL_PATH="/usr/bp/bin/vaultMigration-scripts";
$MIGRATION_SCRIPT="getVaultedBackupInfo.pl migrate"; 
$CLIENT_BK_INFO_SCRIPT="getVaultedBackupInfo.pl check"; 
$DECRYPTION_EXE="/usr/bp/bin/mr71DecryptUtil";
$MG_REPORTS_FILE="vaultMigration71.rpt"; 
$LOG_NAME="vaultMigration71";
$PROGRESS_FILE="in_progress";
$PROGRESS_FILE_FP=0;
$FOLDER_NAME="migrationV71/";
$DBG = false;
$CONT_ON_ERROR  = false; // cont after all the vault checks
$CONT_ON_ERROR2 = false; // cont with all the client error checks
$TRUE_VALUE_ARRAY = array(0 => "true", 1 => "True", 2 => "TRue", 3=> "TRUe",
  4=> "TRUE", 5=>"1", 6 => "On", 7=> "ON", 7=> "on", 9=> "Yes", 10=> "YEs",
  11=> "YES", 12=> "yes", 13=> "y", 14=> "Y");
$DEMO_DATA = false;
$selectedDPU = "";
$RESERVED_SPACE = 0;
$DEFAULT_DEV_CAP = 0;

function exit_func($io) {
  echo "Exiting...\n";
  //fclose($io); // this is causing an error: /tmp/prognam.log Bad file des...
  $res = bp_destroy_cookie();
  exit;
}

/// Have to be root, otherwise all hell will break loose later...
/// We'll inform the user if they have valid permissions in the prereq phase.
if (0 != posix_getuid()) {
  echo("Invalid user permisions. Must have root access privileges.\n");
  exit;
}

//////////////////////////////////////////////////////////////////////
/* Process any args, e.g. -dbg -coe -demo */
//////////////////////////////////////////////////////////////////////
processArgs($argv);

//////////////////////////////////////////////////////////////////////
/* Logger Stuff */
//////////////////////////////////////////////////////////////////////
// 2 LOG FILES:, only one open at a time.
// The first instance below covers upfront checks before any DPU is selected (logs.dir)
// Then when the DPU is selected info logged to the migration folder per dpu
// Why 2 logs: When several instances are running concurrently, it will
//   be a mess to figure out what info is from what instance.
$rc = initialize_log($LOG_NAME);
if ($rc !== true) {
  echo "Error:".var_dump($rc)."\n";
  exit;
}

//////////////////////////////////////////////////////////////////////
/* Main program starts here */
//////////////////////////////////////////////////////////////////////

$res = bp_bypass_cookie(3);
if ($res === false) {
  echo "Bypass cookie failed\n";
  var_dump(bp_error());
  exit;
}

$io = fopen('php://stdin', 'r');

// Greeting
echo "Welcome to the Unitrends Vault Migration Utility. We're running checks on the Vault and gathering a list of systems to migrate...\n\n";

//////////// PREREQUISITE PHASE ///////////////////
// Prereq Checks
// Return Vault encryption and backup device. Needed for time and space estimates.
$isVaultEncryptionEnabled = false;
$vaultBackupDevice        = "";
if (prerequisitePhase($isVaultEncryptionEnabled, $vaultBackupDevice) != 0){ 
  echoLog("\nFound some problems, please correct, then try again.\n");
  if (!$CONT_ON_ERROR) exit_func($io);
}

// Locate Sources (DPUs) and their migration state
$foundErrors = false;
$dpuArray = displayAvailableSourcesAndStates(&$foundErrors);
if (count($dpuArray) == 0){
  if ($foundErrors === true) {
    echoLog("\nDid not identify any systems ready for migration. Please correct conditions before retrying.\n");
  }
  else {
    echoLog("\nDid not discover any systems requiring migration. Please check the Vault ensuring all configured systems are replicating.\n");
  }

  exit_func($io);
}

// Get Source Selection
$selectedDPU = selectSource($io, $dpuArray);
logger(2, "Selected DPU User/Name: ", $selectedDPU);

//Switch logs
close_log($LOG_NAME);
// Note, the call to getMigrationFolder will create it if non-existent.
$rc = initialize_log($LOG_NAME, getMigrationFolder($selectedDPU));
if ($rc !== true) {
  echo "Error:".var_dump($rc)."\n";
  exit_func($io);
}

//////////// SOURCE SELECTED PHASE ///////////////////

// Get the client list per the selected DPU
$clientList = getClientList($selectedDPU);
if (count($clientList) == 0) {
  echoLog("\nNo clients or appropriate backups found for this system.\n");
  exit_func($io);
}

// Had to move this up because the client get bk info scripts rely on this
if (extractLastDPUStateFiles($selectedDPU) != 0) {
  cleanup();
  exit_func($io);
}

// Display client list, return if any encrypted backups found
$encryptedBUsFound = false;
if (displayClientList($clientList, $selectedDPU, $isVaultEncryptionEnabled, $vaultBackupDevice, &$encryptedBUsFound) != 0) {
  cleanup();
  exit_func($io);
}

// verify we have a master key file if we have encrypted backups
if (prepareForMigration($selectedDPU, $encryptedBUsFound) != 0) {
  cleanup();
  exit_func($io);
}

echo "Enter to start migration process, or 'q' to quit.\n";
$selection = fgets($io);
if (trim($selection) === "q") {
  cleanup();
  exit_func($io);
}

declare(ticks = 1);
function sig_handler($signo)
{
  // just in case we're at the PP prompt...enable echo again.
  system('stty echo');
  echo "\nCaught signal $signo, cleaning up, exiting...\n";
  cleanup(false); // if we aborted, leave lock file (unlock) so we know we aborted
  exit;
}

// setup signal handlers, once we have some things needed cleanup
pcntl_signal(SIGTERM, "sig_handler");
pcntl_signal(SIGHUP,  "sig_handler");
pcntl_signal(SIGUSR1, "sig_handler");
pcntl_signal(SIGINT,  "sig_handler");

// Deal with encrypted backups from the Source...passphrase
$dpuEncodedPassphrase = "N/A";
if ($encryptedBUsFound === true) {
  if (obtainAndValidateSourceEncryption($selectedDPU, &$dpuEncodedPassphrase) != 0) {
    cleanup();
    exit_func($io);
  }
  // add it the file for consumption by the descryptor exe
  $set = true;
  setOrClearPassphrase($selectedDPU, $set, $dpuEncodedPassphrase);
}

//////////// DATA MIGRATION PHASE ///////////////////
$rc = startMigrations($selectedDPU, $clientList, $isVaultEncryptionEnabled);

///////////// CLEANUP PHASE  ////////////////////////
cleanup();

/////////// RESULTS PHASE ///////////////////////
presentResults($selectedDPU);

echo "\nDone, ";
exit_func($io);

///////// END OF MAIN ///////////

function cleanup($all = true)
{
  // cleanup is only called once a DPU has been selected

  global $selectedDPU; // global due to signal handler

  enterFunction("cleanup", $all);
  $syncUser = $selectedDPU['user'];

  // clear out encoded passphrase
  setOrClearPassphrase($selectedDPU);

  // clear out vault prevention
  shell_exec("rm -f /backups/$syncUser/.noSync");

  // clear out the lock file indicating we're active, not_aborted
  // we can only select DPUs from the list if it was not in-progress
  if ($all === true) {
    cleanupLockFiles($selectedDPU);
  }
}

function cleanupLockFiles($selectedDPU)
{
  global $PROGRESS_FILE;
  $migrationFolder  = getMigrationFolder($selectedDPU);
  $progress_file    = $migrationFolder.$PROGRESS_FILE;
  shell_exec("rm -f $progress_file");
}

function prerequisitePhase(&$isVaultEncryptionEnabled, &$vaultBackupDevice)
{
  enterFunction("prerequisitePhase");

  // return Vault's Encryption state
  // return Vault's Default Backup Device

  $isVaultEncryptionEnabled = false;
  $vaultBackupDevice = "";

  echo "--------------------------------------------------------------------------------------\n";
  echo "Preliminary Check  \tResult\t   Notes\n";
  echo "--------------------------------------------------------------------------------------\n";
  $rc = 0;
  $rc += permissions();
  $rc += concurrencyCheck();
  $rc += vaultSystem();
  $rc += vaultRelease();
  $rc += vaultReplication();
  $rc += vaultEncryption($isVaultEncryptionEnabled);
  $rc += vaultDefaultDevice($vaultBackupDevice);
  echo "--------------------------------------------------------------------------------------\n";

  return $rc;
}

function permissions()
{
  enterFunction("permissions");
  $rc = 0;

  $message = str_pad("Permissions", 24);
  if (0 != posix_getuid()) {
    $message .= str_pad("Failed", 11);
    $message .= "Must have root access privileges.\n";
    $rc = 1;
  }
  else {
    $message .= str_pad("Passed", 11);
    $message .= "User permissions ok.\n";
  }
  echoLog($message);

  return $rc;
}

function concurrencyCheck()
{
  enterFunction("concurrencyCheck");
  $rc = 0;
  $message = str_pad("Concurrency", 24);

  // TODO tie this in with Hardware limits
  $maxNumOfSessions = 5;

  $file = $_SERVER["SCRIPT_NAME"];
  $break = Explode('/', $file);
  $pfile = $break[count($break) - 1]; 
  logger(2, "script:", $pfile);
  exec("ps l |grep -v grep | grep -v 'vi' | grep $pfile |wc -l", $output);
  $myCount = $output[0];
  logger(2, "my concurrency count:",$myCount);
  if ($myCount > $maxNumOfSessions) {
    $message .= str_pad("Failed", 11);
    $message .= "Maximum number of migrations (".$maxNumOfSessions. ") have been started.\n";
    $rc = 1;
  }
  else {
    $message .= str_pad("Passed", 11);
    $message .= "Concurrent sessions: ".$myCount."\n";
  }
  echoLog($message);
  return $rc;
}

function vaultSystem()
{
  enterFunction("vaultSystem");
  $rc = 0;
  $message = str_pad("System", 24);

  if (bp_local_system_is_vault() === TRUE) {
    $hardware = trim(`cat \/\/etc\/\/unitrends-release |cut -f2 -d' '`);
    // bug 16248, old style
    if ($hardware === 'DPU') {
      $hardware = trim(`cat \/\/etc\/\/unitrends-release |cut -f3 -d' '`);
    }
    $message .= str_pad("Passed", 11);
    $message .= "Vault system is: ".$hardware."\n";
  }
  else {
    $message .= str_pad("Failed", 11);
    $message .= "Migration is only available for Vault configured systems.\n";
    $rc = 1;
  }
  echoLog($message);
  return $rc;
}

function vaultRelease()
{
  enterFunction("vaultRelease");
  $rc = 0;
  $message = str_pad("Software Release", 24);

  // check for version 7.1 
  // e.g. version: 7.1.0-2.2012101112.CentOS5

  // Get version for the locally installed release.
  $localID = bp_get_local_system_id();
  $sysinfo = bp_get_system_info($localID);

  #Check if version is at least 7.1
  $result = $sysinfo['version'];
  $split_version = explode('-', $result, 2);
  $after_dash = explode('.', $split_version[1], 3);
  $version = explode('.', $split_version[0], 3);
  if (($version[0] != 7) || ($version[1] < 1)) {
    $message .= str_pad("Failed", 11);
    $message .= "Found release: ".substr($result, 0,5)."..., must be at least 7.1.\n";
    $rc = 1;
  }
  else {
    $message .= str_pad("Passed", 11);
    $message .= "Installed Release: ".substr($result, 0,5)."...\n";
  }
  echoLog($message);
  return $rc;
}

function vaultReplication()
{
  global $TRUE_VALUE_ARRAY;
  enterFunction("vaultReplication");
  $rc = 0;
  $message = str_pad("Vault Replication", 24);

  // Check to see if replication is supported on this vault
  $isRepEnabled = bp_replication_supported();

  if($isRepEnabled != true){
    $message .= str_pad("Failed", 11);
    $message .= "Replication must be enabled on the Vault.\n";
    $rc = 1;
  }
  else {
    $message .= str_pad("Passed", 11);
    $message .= "Vault's Replication properly configured.\n";
  }
  echoLog($message);
  return $rc;
}

function vaultEncryption(&$isVaultEncryptionEnabled)
{
  enterFunction("vaultEncryption");
  $rc = 0;
  $message = str_pad("Encryption Setting", 24);

  // TODO not convinced this is correct.
  // Can a customer ever turn encryption off
  // What is the error I'm looking for?
  // If currently enabled, we'll encrypt (any way the passphrase can't be set)
  // If currently not enabled, we won't encrypt.
  // Maybe this is a warning!

  // Return if Encryption is set
  $isVaultEncryptionEnabled = false;

  // check if the Vault has Encryption enabled

  // This is the one we flag?
  /* array(4) {
      ["active"]=>
      bool(true)
      ["enabled"]=>
      bool(false)
      ["persistent"]=>
      bool(false)
      ["passphrase_set"]=>
      bool(true)
   */

  $res = bp_get_crypt_info();

  if ($res === false) {
    logger(0,"bp_get_crypt_info:",bp_error());
    $message .= str_pad("Failed", 11);
    $message .= "Unable to determine encryption state. Please check the system state and retry." .bp_error() ."\n";
    $rc = 1;
  } else if ($res['active'] === true && $res['enabled'] === false && $res['passphrase_set'] === true) {
    $message .= str_pad("Warning", 11);
    $message .= "Encryption is configured on the Vault, but the Encryption State is set to off.";
    $message .= " (Migration compatibility will be checked as we go.)\n";
  }
  else if ($res['enabled'] === false) {
    $message .= str_pad("Warning", 11);
    $message .= "Encryption is not enabled. (Migration compatibility will be checked as we go.)\n";
  }
  else {
    $message .= str_pad("Passed", 11);
    $message .= "Encryption is enabled.\n";
    $isVaultEncryptionEnabled = true;
  }
  echoLog($message);
  return $rc;
}

function vaultDefaultDevice(&$defaultDevice)
{
  global $DEFAULT_DEV_CAP;
  enterFunction("vaultDefaultDevice");

  $rc = 0;
  $message = str_pad("Backup Device", 24);

  // TODO, whatif...see note below...
  // In case we need more logic if the device is not found:
  // "bp/applianceAgent/backup.c" line 974 of 1392
  // TODO, if problems, allow the users to override?

  $defaultDevice = "";

  // The property DefaultTapeDevice is per legacy usage
  $defaultTapeDevice = bp_get_ini_value('Configuration Options', 'DefaultTapeDevice');

  // Obtain the default device using SQL Query.
  $dbconn_string = "port=5432 dbname=bpdb user=postgres";
  $dbconn = pg_connect($dbconn_string);
  if($dbconn === FALSE) {
    $message .= str_pad("Failed", 11);
    $message .= "Unable to connect to the Vault's database.\n";
    echoLog($message);
    return 1;
  }
  $query = "SELECT dev_rw_name, capacity, is_online FROM bp.devices WHERE device_no = '$defaultTapeDevice'";
  $res = pg_query($dbconn, $query);
  if ($res === false) {
    $message .= str_pad("Failed", 11);
    $message .= "Unable to get default device from the Vault's database.(".pg_last_error($dbconn).")\n";
    echoLog($message);
    return 1;
  }

  $nrows = pg_num_rows($res);
  if ($nrows == 0) {
    $message .= str_pad("Failed", 11);
    $message .= "There are no matching devices for our value ".$defaultTapeDevice."\n";
    echoLog($message);
    return 1;
  }

  $defaultDevice = pg_fetch_result($res, 0, "dev_rw_name");
  $capacity      = pg_fetch_result($res, 0, "capacity");
  $is_online     = pg_fetch_result($res, 0, "is_online");

  // make sure it exists
  if (!file_exists($defaultDevice)) {
    $message .= str_pad("Failed", 11);
    $message .= "Could not locate ".$defaultDevice." which is the default path for the migrated backups.\n";
    echoLog($message);
    return 1;
  }

  // make sure the device is online
  if ($is_online != 't') {
    $message .= str_pad("Failed", 11);
    $message .= "The default device ".$defaultDevice." must be online for migration.\n";
    echoLog($message);
    return 1;
  }

  // make sure capacity is minimum of 128 GB
  $capacityGB = $capacity/(1024*1024);
  $DEFAULT_DEV_CAP = $capacity;
  if ($capacityGB < 128) {
    $message .= str_pad("Warning", 11);
    $message .= "Replication minimum capacity is 128 (GB). The default device ".$defaultDevice." only has ".$capacityGB." (GB).\n";
    echoLog($message);
    return 0; // issuing a warning, but not halting it...
  }

  $message .= str_pad("Passed", 11);
  $message .= $defaultDevice." capacity: ".$capacityGB." (GB)\n";
  echoLog($message);
  return 0;
}

function getSystemIdForDPU($dpu)
{
  enterFunction("getSystemIdForDPU", $dpu);

  $dpu_id = "";
  // get all the systems, then walk them to find our dpu
  $systems = bp_get_system_list();
  logger(2,"bp_get_system_list:",$systems);
  foreach ($systems as $key => $system_name) {
    if ($system_name == $dpu) {
      $dpu_id = $key;
      break;
    }
  }

  return $dpu_id;
}

function isSourceReplicating($dpu)
{
  // returns 1 Replication Enabled
  // returns 0 Replication Disabled
  // return -1, nw or other problems bp_error()
  // return -2, dpuid not found

  global $TRUE_VALUE_ARRAY;
  enterFunction("isSourceReplicating",$dpu);

  // Get my ID
  $dpu_id = getSystemIdForDPU($dpu);
  if (!empty($dpu_id)) {
    $isReplicationEnabled = bp_get_ini_value('Replication', 'Enabled', $dpu_id);
    logger(0,"isReplicationEnabled:",$isReplicationEnabled); 
    if ($isReplicationEnabled === false) {
      logger(0,"Could not determine REPLICATION Enabled: ",bp_error());
      return -1; // indicates use bp_error() for message
    }

    $isReplicationEnabled = in_array($isReplicationEnabled, $TRUE_VALUE_ARRAY);
    if($isReplicationEnabled == true){
      return 1;
    }
  }
  else {
    logger(0,"dpu_id not found, replication unchecked, for:", $dpu);
    return -2; // indicates dpu not found error (bp_error() is not set!)
  }

  return 0;
}

function isSourceVaulting($dpu)
{
  // returns 1 Vaulting Enabled
  // returns 0 Vaulting Disabled
  // return -1, nw or other problems bp_error()
  // return -2, dpuid not found

  global $TRUE_VALUE_ARRAY;
  enterFunction("isSourceVaulting",$dpu);

  // Get my ID
  $dpu_id = getSystemIdForDPU($dpu);
  if (!empty($dpu_id)) {
    $isVaultingEnabled = bp_get_ini_value('Securesync', 'AutoSyncEnabled', $dpu_id);
    logger(0,"isVaultingEnabled:",$isVaultingEnabled); 
    if ($isVaultingEnabled === false) {
      logger(0,"Could not determine VAULTING Enabled: ", bp_error());
      return -1; // indicates use bp_error() for message
    }
    $isVaultingEnabled = in_array($isVaultingEnabled, $TRUE_VALUE_ARRAY);
    if($isVaultingEnabled == true){
      return 1;
    }
  }
  else {
    logger(0,"dpu_id not found , vaulting state unchecked for:", $dpu);
    return -2; // indicates dpu not found error (bp_error() is not set!)
  }

  return 0;
}

function isSourceReleaseCorrect($dpu, &$versionRet)
{
  // returns 1 Vaulting Enabled
  // returns 0 Vaulting Disabled
  // return -2, dpuid not found

  enterFunction("isSourceReleaseCorrect",$dpu);
  // check for version >= 7.0 
  $versionRet = "";

  // Get my ID
  $dpu_id = getSystemIdForDPU($dpu);
  if (!empty($dpu_id)) {
    // Get my info
    $dpu_info = bp_get_system_info($dpu_id);
    logger(2,"bp_get_system_info:",$dpu_info);

    // Get my version
    $result = $dpu_info['version'];
  }
  else {
    logger(0,"dpu_id not found, release unchecked for dpu:", $dpu);
    return -2; // indicates dpu not found error (bp_error() is not set!)
  }

  # Check if version is equal to 7.0 or greater
  $split_version = explode('-', $result, 2);
  $after_dash = explode('.', $split_version[1], 3);
  $version = explode('.', $split_version[0], 3);
  if ($version[0] != 7) {
    logger(0,"Unsupported version found. Must be at least 7.0", $result);
    $versionRet = $result;
    return 1;
  }

  $versionRet = $result;

  return 0;
}

function getDpuMigrationState($dpu_user, $dpu, &$state, &$reason)
{
  // returns:
  // = 0, continue
  // > 0, continue with warnings
  // < 0, errors, do not continue
  // Reorder things to always check pre-conditions (bug 16266)

  global $CONT_ON_ERROR2;
  enterFunction("getDpuMigrationState", $dpu_user, $dpu);

  $state   = "";
  $reason  = "";
  $selectedDPU = array('user' => $dpu_user,'name' => $dpu);

  // Return 1st error (unless CONT_ON_ERROR2 set)
  // Check for release
  $version = "";
  $rc = isSourceReleaseCorrect($dpu, $version);
  // This does not make a network call, so no check for network failures
  if ($rc != 0) {
    if ($rc > 0) {
      $state  = "Not Ready";
      $reason .= "Incorrect version ".substr($version, 0,5)."..., must be >= 7.0.";
    }
    else { // -2 DPU not found
      $state  = "Unknown  ";
      $reason .= "Could not determine if system is still configured for this vault.";
    }
    if (!$CONT_ON_ERROR2) return -1;
    $reason .= " , ";
  }

  // bug16207 start
  $rc = doesSourceSupportReplication($dpu);
  // returns 0 Replication Supported
  // returns 1 Replication Not Supported
  // return -1, nw or other problems bp_error()
  // return -2, dpuid not found
  if ($rc != 0) {
    if ($rc > 0) {
      $state  = "N/A";
      $reason .= "System does not support replication.";
    }
    else if ($rc == -1) {
      $state  = "Unknown  ";
      $reason .= "Replication Check: failed connecting to system (".bp_error().")";
    }
    else { // -2 DPU not found
      $state  = "Unknown  ";
      $reason .= "Could not determine if system is still configured for this vault.";
    }
    if (!$CONT_ON_ERROR2) return -1;
    $reason .= " , ";
  }
  // bug16207 end

  // check that vaulting is suspended on the Source
  // This makes a network call, so check -1 for network failures
  $rc = isSourceVaulting($dpu);
  if ($rc != 0) {
    if ($rc > 0) {
      $state  = "Not Ready";
      $reason .= "Vaulting is active and must be disabled during migration.";
    }
    else if ($rc == -1) {
      $state  = "Unknown  ";
      $reason .= "Vaulting Check: failed connecting to system (".bp_error().")";
    }
    else { // -2 DPU not found
      $state  = "Unknown  ";
      $reason .= "Could not determine if system is still configured for this vault.";
    }
    if (!$CONT_ON_ERROR2) return -1;
    $reason .= " , ";
  }

  // check for local devices
  if (anyLocalDirClients($dpu_user, $dpu) != 0) {
    $state  = str_pad("Warning", 10);
    $reason .= "The system contains a local directory on the vault. These backups will not be migrated. Please ensure this data has been archived first before continuing.";

    if (!$CONT_ON_ERROR2) return 0; // allow to continue as a warning (return 0, not 1, bug 16267)
    $reason .= " , ";
  }

  // check for legacy exchange (bug 16452, 16492)
  if (anyLegacyExchangeBackups($dpu_user, $dpu) != 0) {
    $state  = str_pad("Warning", 10);
    $reason .= "The system contains legacy exchange backups. These backups will not be migrated. Please ensure this data has been archived first before continuing.";

    if (!$CONT_ON_ERROR2) return 0; // allow to continue as a warning (return 0, not 1)
    $reason .= " , ";
  }

  // check if DR is active. (I guess so DR can't overwrite dpu_clients.rpt
  $res = bp_dr_in_progress($dpu_user);
  if ($res === false) {
    $state  = "Unknown  ";
    $reason .= "D/R Check failed:".bp_error();
    if (!$CONT_ON_ERROR2) return -1;
    $reason .= " , ";
  }
  else if ($res === "yes") {
    $state  = "Not Ready";
    $reason .= "Disaster recovery is in progress.";
    if (!$CONT_ON_ERROR2) return -1;
    $reason .= " , ";
  }

  // Check for zombies (running pl scripts or migrations in progress)
  // This can occur when this script is killed -9
  $zReason = "";
  $rc = isSourceZombie($dpu_user, $dpu, $zReason);
  // 2 Cases:
  //  rc=0: No Zombies, continue with other checks
  //  rc=-1 Failed, no other checks needed, use returned reason
  if ($rc < 0) {
    $state  = "Aborted  ";
    $reason .= $zReason;
    if (!$CONT_ON_ERROR2) return -1;
    $reason .= " , ";
  }

  // Check for aborted or migrations in progress)
  // Aborted tasks can occur when this script is cntrl-c
  $zReason = "";
  $rc = isSourceAbortedOrInProgress($selectedDPU, $zReason);
  // 3 Cases:
  //  rc=0: Not in progress, continue with other checks
  //  rc=1: Aborted, allow this session to restart it. 
  //  rc=-1 In Progress, no other checks needed, use returned reason
  if ($rc == 0) {
    //$state = "Not Started";
    // allow other checks to be made...
  } else if ($rc > 0) {
    $state  = "Aborted  ";
    $reason .= $zReason;
    return 0;
  }
  else { // -1 In progress, don't let 2 run concurrently for the same dpu
    $state  = "In Progress";
    $reason .= $zReason;
    return -1;
  }

  // Check for completed (no failed backups, process completed)
  $zReason = "";
  $rc = isSourceMigrationCompleted($selectedDPU, $zReason);
  // 3 Cases:
  //  rc=0: No migration attempts, continue with other checks
  //  rc=-1 Failed, no other checks needed, use returned reason
  //  rc=1  Success, no other checks needed, use returned reason
  if ($rc == 0) {
    //$state = "Not Started";
    // allow other checks to be made...
  } else if ($rc > 0) {
    $state  = "Success  ";
    return 0;
  }
  else { // -1,  some failures
    $state  = "Failed   ";
    $reason .= $zReason;
    if (!$CONT_ON_ERROR2) return 0; // make it selectable for retries, bug 16235
    $reason .= " , ";
  }

  // if CONT_ON_ERROR2, the error code isn't checked..
  if (!$CONT_ON_ERROR2) {
    $state   = "Not Started";
    $reason  = "Ready for Migration";
  }

  return 0;
}

function displayAvailableSourcesAndStates($foundErrors)
{
  global $CONT_ON_ERROR;
  global $DEMO_DATA;

  enterFunction("displayAvailableSourcesAndStates");

  // This routine builds a list of Sources to migrate
  // and gives reasons why they cannot be migrated
  // Only the migratable Sources are tagged with a # and put in the results

  $dpuArray = array();

  // Get a list of available DPUS (users)
  $dpuUsers = my_bp_get_vaulted_dpus_users(); # returns the UEB's user
  logger(2,"bp_get_vaulted_dpus:",$dpuUsers);

  // NOTE: replicated dpus do not show up in this list, therefore, if all replicating, nothing to show

  if ($dpuUsers === false || count($dpuUsers) == 0) {
    logger(0,"No DPU Users found:",$dpuUsers);
    return $dpuArray;
  }

  echo "\nListed below are the System(s) configured for this Vault, and some migration information. Systems already replicating are omitted.\n";
  echo "--------------------------------------------------------------------------------------\n";
  echo "# System Name\t\tStatus\t\tNotes\n";
  echo "--------------------------------------------------------------------------------------\n";

  // for each DPU User, locate their DPU and determine their Migration state
  // As we go thru the list, it will contain valid and invalid Sources.
  // Add valid ones to dpuArray
  // Valid is also complete, in case the user wants to do one again.
  $index = 0;
  foreach ($dpuUsers as $key => $dpu_user) {
    $dpu_name = bp_get_dpu_name($dpu_user);
    if ($dpu_name === false) {
      $state="Not Ready";
      $reason="System not found, possible no backups initiated between system and Vault.";
      echo "X $dpuUsers\t$state\t$reason";
      $foundErrors = true;
      continue;
    }
    logger(2, "DPU Name:", $dpu_name);
    $state = "";
    $reason = "";
    //-----------------------------------------------------------------------
    //# System Name              Status         Notes
    //-----------------------------------------------------------------------
    //1234567890123456789012345678901234567890123456789012345
    //       T       T       T       T       T       T

    $rc = getDpuMigrationState($dpu_user, $dpu_name, $state, $reason);
    $TABS = "\t";
    $dpu_namePad = str_pad($dpu_name, 15);
    $state       = str_pad($state, 15);
    if ($rc != 0 && !$CONT_ON_ERROR) {
      echo "X $dpu_namePad$TABS$state\t$reason\n";
      $foundErrors = true;
    }
    else {
      echo "$index $dpu_namePad$TABS$state\t$reason\n";
      // Add contents to the returned array
     $dpuArray[] = array('user' => $dpu_user,'name' => $dpu_name);
     $index++;
    }
  }

  if ($DEMO_DATA == true) {
    $dpu_user = "Recovery-411";
    $dpu_name = "Recovery-411";
    $state  = "Not Started ";
    $reason = "Ready for Migration [demo]";
    $rc = getDpuMigrationState($dpu_user, $dpu_name, $state, $reason);
    $TABS = "\t";
    $demoReason = "[demo]".$reason;
    $dpu_namePad = str_pad($dpu_name, 15);
    echo "$index $dpu_namePad$TABS$state\t$demoReason\n";
    $dpuArray[] = array('user' => $dpu_user,'name' => $dpu_name);

    $index++;
    $dpu_user = "Recovery-812";
    $dpu_name = "Recovery-812";
    $state  = "Not Started ";
    $reason = "Ready for Migration [demo-sql]";
    $rc = getDpuMigrationState($dpu_user, $dpu_name, $state, $reason);
    $TABS = "\t";
    $demoReason = "[demo]".$reason;
    $dpu_namePad = str_pad($dpu_name, 15);
    echo "$index $dpu_namePad$TABS$state\t$demoReason\n";
    $dpuArray[] = array('user' => $dpu_user,'name' => $dpu_name);
  }

  // check for orphan systems
  $orphanDPUs = locateOrphanDPUs();
  if (is_array($orphanDPUs)) {
    //var_dump($arrayDiff);
    $state=str_pad("Orphan",15);
    $reason="Found a system that is no longer configured for this Vault";
    foreach ($orphanDPUs as $key => $orphanDPU) {
      $orphanDPUPad = str_pad($orphanDPU, 15);
      echo "X $orphanDPUPad$TABS$state\t$reason\n";
      $foundErrors = true;
    }
  }
  logger(4,"dpuArray:", $dpuArray);
  return $dpuArray;
}

function selectSource($io, $dpuArray)
{
  global $io;
  enterFunction("selectSource");

  // Note the contents of dpuArray and the actually rows displayed are referenced
  // by the # presented on the display as the rows could have an X for those
  // rows of Sources not ready for migrations.

  echo "\n\nSelect the system to migrate by entering the corresponding number of the entry. We'll then proceed with some additional checks before prompting to continue.\n";
  echo "Selection ('q' to quit): ";
  $selVal = trim(fgets($io));
  if (!is_numeric($selVal) || $selVal < 0 || $selVal >= count($dpuArray)) {
    if ($selVal != 'q') echo "Invalid selection. Relaunch to start over\n";
    exit_func($io);
  }

  // return the entry containing both user and dpu
  logger(2,"user and name:", $dpuArray[$selVal]);
  return $dpuArray[$selVal];
}

function anyLocalDirClients($dpu_user, $dpu)
{
  // check if any clients configured have a local directory configuration
  // and report it as this is an unsupported condition.
  // TODO, this routine could be combined with displayClientList

  global $io;
  enterFunction("anyLocalDirClients",$dpu_user, $dpu);

  $clientList = bp_get_dpu_clients($dpu_user, $dpu);
  if ($clientList === false) {
    logger(0,"bp_get_dpu_clients, false rtn:", bp_error());
    return 0;
  }
  if (count($clientList) == 0) {
    logger(0,"No clients were found for the selected system.", $dpu_user, $dpu);
    return 0;
  }

  foreach($clientList as $key => $client) {
    if (trim($client) === "LocalDir") {
      logger(0,"Found a LocalDir Client:", $client);
      return 1;
    }
  }

  return 0;
}

function anyLegacyExchangeBackups($dpu_user, $dpu)
{
  // check if any legacy exchange backups are present
  // and report it as this is an unsupported condition.
  // TODO, this routine could be combined with displayClientList

  global $io;
  enterFunction("anyLegacyExchangeBackups",$dpu_user, $dpu);

  $clientList = bp_get_dpu_clients($dpu_user, $dpu);
  if ($clientList === false) {
    logger(0,"bp_get_dpu_clients, false rtn:", bp_error());
    return 0;
  }
  if (count($clientList) == 0) {
    logger(0,"No clients were found for the selected system.", $dpu_user, $dpu);
    return 0;
  }

  foreach($clientList as $key => $client) {
    if (trim($client) === "lastExchange") {
      logger(0,"Found a legacy Exchange backup entry:", $client);
      return 1;
    }
  }

  return 0;
}

function getClientList($selectedDPU)
{
  global $io;
  enterFunction("getClientList", $selectedDPU);

  $syncUser = $selectedDPU['user'];
  $dpu      = $selectedDPU['name'];
  $clientList = bp_get_dpu_clients($syncUser, $dpu);
  // this calls actually extracts dpu_clients.rpt from lastDPUState and puts it here: /backups/<user>/syncd/<dpu>/dpu_restore
  logger(2,"clientList:",$clientList);
  if ($clientList === false) {
    var_dump(bp_error());
    $filepath = "/backups/$syncUser/syncd/$dpu/archives/lastDPUstate";
    echoLog("This error could be due to missing: $filepath\n");
    echo "The file may automatically appear within 5 minutes, if not, ensure backups have been vaulted.\n";
    exit_func($io);
  }
  if (count($clientList) == 0) {
    logger(0,"No clients (or appropriate backups) were found for the selected system", bp_error());
  }

  return $clientList;
}

function displayClientList($clientList, $selectedDPU, $isVaultEncryptionEnabled, $vaultBackupDevice, &$encryptedBUsFound)
{
  global $io;
  global $CONT_ON_ERROR;
  global $RESERVED_SPACE;
  global $log_filename; // bug 16223

  enterFunction("displayClientList", $clientList, $selectedDPU);

  // Assumes the clientList contains valid clients

  // initialize the reference encryptedBUsFound
  $encryptedBUsFound = false;

  echo "\nListed below are the Client(s) and their backup information targeted for migration.\n";
  echo "--------------------------------------------------------------------------------------\n";
  echo "Client Name\t\t# of Backups\tBackup Size (GB)\tEncrypted\n";
  echo "--------------------------------------------------------------------------------------\n";

  $backup_size_dec_dpu_tot = 0;
  $backup_size_enc_dpu_tot = 0;
  $backup_count_dpu_tot    = 0;
  $fatal = ""; // could be null
  $messages = ""; // could be null
  foreach($clientList as $key => $client) {
    if (trim($client) === "LocalDir" || trim($client) === "lastExchange" ) {
      // NOTE, skip any local dir or legacy exchange hits as they were warned during the dpu checks earlier.
      logger(2,"Found a Local Dir or legacy exchange for the selected DPU:\n", $selectedDPU);
      continue;
    }

    $backupInfo = getBackupInfoPerClient($client, $selectedDPU, $isVaultEncryptionEnabled);
    if (!count($backupInfo)) {
      echo "$client\n";
    }
    else {
      // sizes are in MB, convert for display
      $backup_count    = $backupInfo[0]['BackupDecryptedCount'];
      $backup_count   += $backupInfo[0]['BackupEncryptedCount'];
      $backup_size_dec = $backupInfo[0]['BackupDecrypted'];
      $backup_size_enc = $backupInfo[0]['BackupEncrypted'];
      $fatal          .= $backupInfo[0]['Fatal'];
      $messages       .= $backupInfo[0]['Messages'];

      $backup_enc_msgPad = str_pad("N/A",6);
      if ($backup_count > 0) {
        $backup_enc_msg = "";
        if ($backup_size_enc > 0){
          $backup_enc_msg = "Yes";
          if ($backup_size_dec > 0) $backup_enc_msg .= "/No";
        }
        else {
          $backup_enc_msg = "No";
        }
        $backup_enc_msgPad = str_pad($backup_enc_msg,6);
      }

      $backup_size_tot_gb = round((($backup_size_dec + $backup_size_enc) / 1024),2);
      $TABS      = "";
      $clientPad = str_pad($client, 24);
      $bksizePad = str_pad($backup_size_tot_gb, 17);
      echo "$clientPad$TABS$backup_count\t\t$bksizePad\t$backup_enc_msgPad\n";
      
      // Tally (MB) DPU values for all their clients. Needed for time est., and/or RPT values
      $backup_size_dec_dpu_tot += $backup_size_dec;
      $backup_size_enc_dpu_tot += $backup_size_enc;
      $backup_count_dpu_tot    += $backup_count;

      // Set the encryption flag which will cause the caller to ask for a passphrase
      if ($backup_size_enc > 0) {
        $encryptedBUsFound = true;
      }
    }
  }
  echo "--------------------------------------------------------------------------------------\n";

  // Special Case: If we found any encrypted files and the Vault encryption is not enabled,
  // flag this and move on
  if (!$isVaultEncryptionEnabled && $encryptedBUsFound) {
    echo "Errors:";
    echoLog("\nFound encrypted backups to migrate but encryption is not enabled on the Vault.\nPlease enable encryption before retrying.\n");
    if (!$CONT_ON_ERROR) {
      echo "--------------------------------------------------------------------------------------\n";
      return 1;
    }
  }

  // Add any fatal errors and warning messages
  if (!empty($fatal)) {
    // need to add newlines between each fatal msg, assuming each one starts with [client]...
    $fatal = str_replace("[","\n[",$fatal);
    echo "Errors:";
    echoLog("$fatal\n");
    echo "--------------------------------------------------------------------------------------\n";
  }
  if (!empty($messages)) {
    // bug 16223, add logfile to messages too long, end at complete entry
    // trim messages if too long...point the user to the log file...
    if (strlen($messages) > 400) {
      $messages = substr($messages,0,400);
      // cut it off at complete entries
      $pos = strrpos($messages,"[");
      if ($pos > 0) {
        $messages = substr($messages,0,$pos);
      }
      $messages .= "\nMore messages: see <".$log_filename."> for complete list.";
    }
    // need to add newlines between each message, assuming each one starts with [client]...
    $messages = str_replace("[","\n[",$messages);
    echo "Warnings:";
    echoLog("$messages\n");
    echo "--------------------------------------------------------------------------------------\n";
  }

  // any orphan clients?
  $orphanClients = locateOrphanClients($selectedDPU);
  $syncUser      = $selectedDPU['user'];
  $dpu           = $selectedDPU['name'];
  $foundOrphan   = 0;
  if (is_array($orphanClients)) {
    //var_dump($arrayDiff);
    foreach ($orphanClients as $key => $dpuClient) {
      // weed out space, ourself, migrationV71
      if ($dpuClient == $dpu || $dpuClient == '' || $dpuClient == 'migrationV71') { continue; }
      echo "Warning: Orphan client found: $dpuClient\n";
      $foundOrphan = 1;
    }
  }
  if (!empty($fatal) || !empty($messages) || $foundOrphan) {
    echo "Please address the above errors and/or warnings appropriately before continuing.\n";
    echo "--------------------------------------------------------------------------------------\n";
  }

  // Add in total size needed, total available, warn if we don't have enough.
  $freeSpace     = getDefaultDeviceFreeSpace($vaultBackupDevice);
  $backup_size_dpu_tot = $backup_size_dec_dpu_tot + $backup_size_enc_dpu_tot;
  if ($backup_size_dpu_tot > $freeSpace) {
    echoLog("Insufficient Space!! Need: ".round(($backup_size_dpu_tot/1024),2)." (GB)  Available: " .round(($freeSpace/1024),2)." (GB)\n");
    if (!$CONT_ON_ERROR) return 1;
  }
  echoLog("Total Space Needed: ".round(($backup_size_dpu_tot/1024),2)." (GB)   Total Space Available: ".round((($freeSpace-$RESERVED_SPACE)/1024),2)." (GB)");
  if ($RESERVED_SPACE > 0) {
    echoLog("(includes ".round(($RESERVED_SPACE/1024),2)." (GB) for migrations in progress.)\n");
  }
  else {
    echo "\n";
  }

  // Now add summary data: time estimates, 
  $timeEst = getTimeEst($backup_size_dec_dpu_tot, $backup_size_enc_dpu_tot, $isVaultEncryptionEnabled);
  settype($timeEst, 'integer');
  $HHMM = sprintf("%02d:%02d", floor($timeEst/60), $timeEst%60);
  echoLog("Duration estimate to complete this migration: ".$HHMM." (HH:MM)\n");
  echo "--------------------------------------------------------------------------------------\n";

  // Add the count and space calculation to the rpt file so it can be used while
  //  others instances are checking for available space and if we're done. These
  //  values are adjusted as migrations are completed.
  setRPTValues($selectedDPU, $backup_count_dpu_tot, $backup_size_dpu_tot);

  return 0;
}

function extractLastDPUStateFiles($selectedDPU)
{
  global $io;
  enterFunction("extractLastDPUStateFiles");

  $rc = 0;

  // selectedDPU is an array with 1 entry, syncUser, and dpu name
  $syncUser = $selectedDPU['user'];
  $dpu      = $selectedDPU['name'];

  // extract lastDPU Content
  /* e.g. /backups/VMware_UEB_User/syncd/VMware_UEB/archives/lastDPUstate */

  $file= "/backups/$syncUser/syncd/$dpu/archives/lastDPUstate";
  logger(2,"looking for file:", $file);
  if (file_exists($file)) {
    $localfiles[] = $file;
    logger(2,"found lastDPUstate file");
  }

  // Make sure...
  if (!count($localfiles)) {
    $filepath = "/backups/$syncUser/syncd/$dpu/archives/lastDPUstate";
    echoLog("No lastDPUstat file available ($filepath), nothing to migrate.\n\n");
    echo "The file may automatically appear within 5 minutes, if not, ensure backups have been vaulted.\n";
    exit_func($io);
  }

  //////////////////////////////////////////////////////////////////////
  /* Extract the dbdumps.sql and dpu_clients.rpt */
  /*
     e.g. /usr/bp/bin/bkup -x
       -f /backups/VMware_UEB_User/syncd/VMware_UEB/archives/lastDPUstate
       -zFLAT -zWHERE=/backups/VMware_UEB_User/syncd/migrationV_71
         /usr/bp/reports.dir/dpu_clients.rpt
         /usr/bp/reports.dir/dbdump.sql
         /var/lib/misc/cryptoDaemonMasterKeys
  */
  //////////////////////////////////////////////////////////////////////

  // build the command
  $bkup_exe         = "/usr/bp/bin/bkup -x ";
  $lastDPUstateFile = "-f /backups/$syncUser/syncd/$dpu/archives/lastDPUstate ";
  $zFLAT            = " -zFLAT ";
  $migrationFolder  = getMigrationFolder($selectedDPU);
  $zWHERE           = " -zWHERE=".$migrationFolder;
  $targetFiles      = " /usr/bp/reports.dir/dpu_clients.rpt /usr/bp/reports.dir/dbdump.sql";
  $targetFilesEncKeys = " /var/lib/misc/cryptoDaemonMasterKeys";
  $extractFileCMD = "$bkup_exe $lastDPUstateFile $zFLAT $zWHERE $targetFiles $targetFilesEncKeys 2> /dev/null";
  logger(0,"extraction cmd:", $extractFileCMD);

  $some_value = exec($extractFileCMD);

  if (file_exists("$migrationFolder"."dpu_clients.rpt")) {
     logger(0,"extracted dpu_clients.rpt");
  }
  if (file_exists("$migrationFolder"."dbdump.sql")) {
    logger(0,"extracted dbdump.sql");
  }
  // TODO validate this file is optional, and we won't die if it is not there
  if (file_exists("$migrationFolder"."cryptoDaemonMasterKeys")) {
    logger(0,"extracted cryptoDaemonMasterKeys");
  }

  return 0;
}

function prepareForMigration($selectedDPU, $encyptedBUsFound)
{
  enterFunction("prepareForMigration");
  $migrationFolder = getMigrationFolder($selectedDPU);

  if ($encyptedBUsFound) {
    if (!file_exists("$migrationFolder"."cryptoDaemonMasterKeys")) {
      // oops! we have encrypted backps but no master key file....error!
      $filepath = "$migrationFolder"."cryptoDaemonMasterKeys";
      echoLog("We are unable to proceed as the encryption master key file ($filepath) needed for decrypting the backups is not present.\n");
      return 1;
    }
  }

  // lock the file indicating we're active
  // Once we lock the file, which means we created it, we should clean it up
  // under normal exits on our behalf.
  obtainProgressLockOrExit($selectedDPU);

  return 0;
}

function startMigrations($selectedDPU, $clientList, $isVaultEncryptionEnabled)
{
  global $DBG;
  global $INSTALL_PATH;
  global $MIGRATION_SCRIPT;
  global $log_filename;
  enterFunction("startMigrations");
  $startTime = time();
  $syncUser  = $selectedDPU['user'];
  $dpu       = $selectedDPU['name'];

  // lock down vaulted per the dpu (user). This gets cleaned up upon exit.
  shell_exec("touch /backups/$syncUser/.noSync");
 
  // Clear out any failures from past migration tries
  // Leave successes in place, keeps a history of progress and prevents 
  // duplicating migrations.

  clearOutFailures($selectedDPU);

  $migrationFolder = getMigrationFolder($selectedDPU);
  $dbg             = ($DBG ? '1' : '0');
  $vaultEncrypted  = ($isVaultEncryptionEnabled ? '1' : '0');

  // this is ugly, but because only STDERR gets displayed 'real-time' from the perl script,
  // we'll swap STDOUT and STDERR as the perl script logically has the output we want tagged
  // for STDOUT, and the errors going to STDERR if -dbg is set. Then we also want the errors
  // to be appended to our log_file (2>> $log_filename).
  $cmd = "/usr/bin/perl $INSTALL_PATH/$MIGRATION_SCRIPT $migrationFolder $log_filename $dbg $syncUser $dpu $vaultEncrypted ". implode(" ", $clientList)." 3>&1 1>&2 2>&3 2>> $log_filename";
  logger(0,"migration cmd:", $cmd);
  $retVal = "";
  exec($cmd);

  // dump out actual time it took so we can adjust our estimates...
  $t = time() - $startTime;
  $s = sprintf("%02d%s%02d%s%02d", floor($t/3600), ":", ($t/60)%60, ":", $t%60);
  logger(0,"migration done, elapse time:", $s);
  return $retVal;
}

function getMigrationFolder($selectedDPU)
{
  global $FOLDER_NAME;

  // TODO is it always /backups?
  // Note, don't log here because it may not be open yet. 

  $syncUser = $selectedDPU['user'];
  $migrationFolder = "/backups/$syncUser/syncd/$FOLDER_NAME";// e.g. migrationV71/
  if (!is_dir($migrationFolder)) {
    $ret = mkdir($migrationFolder);
    if ($ret !== true) {
      return "";
    }
  }
  return $migrationFolder;

/* recursive needed????
// recursively calls createPath..
if (is_dir($path)) return true;
    $prev_path = substr($path, 0, strrpos($path, '/', -2) + 1 );
    $return = createPath($prev_path);
    return ($return && is_writable($prev_path)) ? mkdir($path) : false;
*/
}

function getBackupInfoPerClient($client, $selectedDPU, $isVaultEncryptionEnabled)
{
  global $DBG;
  global $INSTALL_PATH;
  global $CLIENT_BK_INFO_SCRIPT;
  global $log_filename;
  enterFunction("getBackupInfoPerClient", $client, $selectedDPU);

  //Returns an array with BackupDecryptedCount BackupEncryptedCount
  //   BackupSizeEncrypted BackupSizeOther";

  $syncUser        = $selectedDPU['user'];
  $dpu             = $selectedDPU['name'];
  $migrationFolder = getMigrationFolder($selectedDPU);
  $dbg             = ($DBG ? '1' : '0');
  $vaultEncrypted  = ($isVaultEncryptionEnabled ? '1' : '0');

  // Note, we use STDOUT to get the client data. STDERR is logged to the file
  $cmd = "/usr/bin/perl $INSTALL_PATH/$CLIENT_BK_INFO_SCRIPT $migrationFolder $log_filename $dbg $syncUser $dpu $vaultEncrypted $client 2>> $log_filename";

  logger(0,"$CLIENT_BK_INFO_SCRIPT:", $cmd);
  $output = array();
  exec($cmd, $output,$retval);
  // if no data, then the caller will display a list of clients with empty fields, NO backups, NO data...
  if (count($output) == 0) {
    return array();
  }

  logger(0, "retval:",$retval);
  logger(0, "output:",$output);
  // returns something like:
  // ENCRYPTED_SIZE:0:ENCRYPTED_BKUPS:0:DECRYPTED_SIZE:8095:DECRYPTED_BKUPS:2:
  // Values are in MB, let the presentation convert to GB

  // make sure we have a valid string to parse
  $countIt = substr_count($output[0],":");
  if ($countIt != 11) {
    $clientInfo[] = array(
      'BackupDecryptedCount' => 0,
      'BackupEncryptedCount' => 0,
      'BackupDecrypted' => 0,
      'BackupEncrypted' => 0,
      'Fatal' => $output[0],
      'Messages' => "");
  }
  else {
    // ugly parse...but from array to string to tokens...
    list($E_SIZE_TXT, $E_SIZE_V, $E_CNT_TXT, $E_CNT_V,
       $D_SIZE_TXT, $D_SIZE_V, $D_CNT_TXT, $D_CNT_V,
       $FATAL_TXT, $FATAL_V, $MSG_TXT, $MSG_V) = explode(":", $output[0]);
    $clientInfo[] = array(
      'BackupDecryptedCount' => $D_CNT_V,
      'BackupEncryptedCount' => $E_CNT_V,
      'BackupDecrypted' => $D_SIZE_V,
      'BackupEncrypted' => $E_SIZE_V,
      'Fatal' => $FATAL_V,
      'Messages' => $MSG_V);
  }
  return $clientInfo;
}

function getTimeEst($backup_size_dec, $backup_size_enc, $isVaultEncryptionEnabled)
{
  // Assume MB units supplied
  // NOTE: decryption only occurs if backup is encrypted
  //       encryption only occurs if backup is encrypted and the vault has encryption enabled
  //     encrypted data: 10Mbps
  // non-encrypted data: 20Mbps
  
  $timeInMins = 0;

  if ($backup_size_dec) $timeInMins += ($backup_size_dec/1200);
  if ($backup_size_enc) $timeInMins += ($backup_size_enc/600);
  return $timeInMins;
}

function getDefaultDeviceFreeSpace($vaultBackupDevice)
{
  global $DEFAULT_DEV_CAP;
  enterFunction("getDefaultDeviceFreeSpace", $vaultBackupDevice, $DEFAULT_DEV_CAP);

  //$freeSpace = disk_free_space($vaultBackupDevice) / (1024 * 1024);
  // bug 16209: disk_free_space works at the device level...
  // leverage the db capacity value and system used value

  $usedSpace = `du -s $vaultBackupDevice`;
  $freeSpace = ($DEFAULT_DEV_CAP - $usedSpace) / 1024;
  // return free space (MB)
  return round($freeSpace, 2);
}

function getReservedSpace($selectedDPU)
{
  global $MG_REPORTS_FILE;
  enterFunction("getReservedFreeSpace", $selectedDPU);

  // assumes the caller has checked in_progress state as this value
  // is only meaningful if a migration is in progress.

  $migrationFolder = getMigrationFolder($selectedDPU);
  $mg_rpt_file     = $migrationFolder.$MG_REPORTS_FILE;
  
  // get Total_Space_Needed
  // NOTES: info is logged to logs.dir/vaultMigration...
  $reserved_space = 0;
  $cmd = "/usr/bp/bin/bputil -g \"Backups\" \"Total_Space_Needed\" 0 $mg_rpt_file ";
  logger(2,"CMD Total_Space_Needed:", $cmd);
  $shell_ret = shell_exec("$cmd");
  logger(2,"Total_Space_Needed:", $shell_ret);
  $output = explode(chr(10),rtrim($shell_ret)); // chr(10) = /n
  if (is_array($output)) {
    $reserved_space = $output[0];
  }
  return $reserved_space;
}

function processArgs($args)
{
  global $DBG;
  global $CONT_ON_ERROR;
  global $CONT_ON_ERROR2;
  global $DEMO_DATA;

// TODO add optional arg to specific a specific DPU (SOURCE) to process
// TODO this could be a global, really need DPU_USER and DPU

  //array_shift($args);
  while ($arg = array_shift($args)) {
    if (substr($arg, 0,6) === '-nodbg') {
      $DBG = false;
    }
    if (substr($arg, 0,4) === '-dbg') {
      $DBG = true;
    }
    else if (substr($arg, 0,5) === '-coe2') {
      $CONT_ON_ERROR2 = true;
    }
    else if (substr($arg, 0,4) === '-coe') {
      $CONT_ON_ERROR = true;
    }
    else if (substr($arg, 0,5) === '-demo') {
      $DEMO_DATA = true;
    }
    else if (substr($arg, 0,8) === '-collect') {
      collectInfo();
      exit(0);
    }
  }
}

function setRPTValues($selectedDPU, $backup_count_dpu_tot, $backup_size_dpu_tot)
{
  global $MG_REPORTS_FILE;
  enterFunction("setRPTValues", $selectedDPU, $backup_size_dpu_tot);

  $migrationFolder = getMigrationFolder($selectedDPU);
  $mg_rpt_file     = $migrationFolder.$MG_REPORTS_FILE;

  // Set the total number of migrations needed
  // As backups are completed (success) and posted to .rpt file, there
  // backup count is subtracted from Total_Remaining so other sessions can monitor
  // if the dpu is completed or not.
  $cmd = "/usr/bp/bin/bputil -p \"Backups\" \"Total_Remaining\" $backup_count_dpu_tot $mg_rpt_file ";
  logger(0,"cmd:",$cmd);
  $shell_ret = shell_exec("$cmd");

  // Set the total space needed
  // As backups are completed (success or failure) and posted to mg rpt file, their
  // backup size is subtracted from Total_Needed so other sessions can monitor
  // how much is "reserved"
  $cmd = "/usr/bp/bin/bputil -p \"Backups\" \"Total_Space_Needed\" $backup_size_dpu_tot $mg_rpt_file ";
  logger(0,"cmd:",$cmd);
  $shell_ret = shell_exec("$cmd");
}

function isSourceMigrationCompleted($selectedDPU, &$reason)
{
  global $MG_REPORTS_FILE;

  // NOTE: items below are logged to /usr/bp/logs.dir/vaultMigration71_*.log
  // how to check if really done:
  // - no failures posted
  // - what about no backups needed?
  // - what if kept track of # of backups remaining?

  enterFunction("isSourceMigrationCompleted", $selectedDPU);

  $migrationFolder = getMigrationFolder($selectedDPU);
  $mg_rpt_file     = $migrationFolder.$MG_REPORTS_FILE;
  
  // Count completed Backups
  $success_cnt = 0;
  $NONE="NONE_FOUND";
  $foundAtLeastOneResult = 0;
  $cmd = "/usr/bp/bin/bputil -g \"Backups Success\" \"NULL\" $NONE $mg_rpt_file |grep -v \"\[Backups Success\]\"|cut -f1 -d=";
  logger(2,"CMD BU Success:", $cmd);

  $shell_ret = shell_exec("$cmd");
  logger(2,"Success:", $shell_ret);
  $output = explode(chr(10),rtrim($shell_ret)); // chr(10) = /n
  if (is_array($output) && $output[0] != $NONE) {
    $success_cnt = count($output);
  }

  // Find failures 
  $failure_cnt = 0;
  $cmd = "/usr/bp/bin/bputil -g \"Backups Failed\" \"NULL\" $NONE $mg_rpt_file |grep -v \"\[Backups Failed\]\"";
  logger(2,"CMD BU Failed:", $cmd);

  $shell_ret = shell_exec("$cmd");
  logger(2,"Failures:", $shell_ret);
  $output = explode(chr(10),rtrim($shell_ret));
  if (is_array($output) && $output[0] != $NONE) {
    $failure_cnt = count($output);
  }

  // Find remaining 
  $remaining_cnt = 0;
  $cmd = "/usr/bp/bin/bputil -g \"Backups\" \"Total_Remaining\" 0 $mg_rpt_file ";
  logger(2,"CMD BU Remaining:", $cmd);

  $shell_ret = shell_exec("$cmd");
  logger(2,"Remaining:", $shell_ret);
  $remaining_cnt = rtrim($shell_ret); // bug 16274

  // 3 Cases:
  //  rc=0: No migration attempts
  //  rc=1  Success, no other checks needed, use returned reason
  //  rc=-1 Failed, no other checks needed, use returned reason

  $rc = 0;
  // Not attempted yet?
  if ($success_cnt == 0 && $failure_cnt == 0){
    $reason = ""; // let the caller fill this in with other checks
    return $rc;
  }

  // Successes!
  if ($success_cnt > 0 ) {
    $reason = "Successful migrations: $success_cnt. ";
    $rc = 1;
  }

  // Failures?
  if ($failure_cnt > 0) {
    $reason .= "Failed migrations: $failure_cnt. ";
    $rc = -1; 
  }

  // Not attempted yet?
  if ($remaining_cnt > 0) {
    $reason .= "Remaining migrations: $remaining_cnt. ";
    $rc = -1; 
  }

  return $rc; 
}

function isSourceZombie($dpu_user, $dpu, &$reason)
{
  global $DBG;
  global $MIGRATION_SCRIPT;

  enterFunction("isSourceZombie", $dpu_user, $dpu);

  $output=array();
  $cmd="ps l | grep -v grep |grep ".escapeshellarg($MIGRATION_SCRIPT)." |grep ".escapeshellarg($dpu_user)." | awk {'print $4'}";
  logger(2,"zombie command:", $cmd);
  exec($cmd, $output);
  if (is_array($output)) {
    foreach( $output as $ppid ) {
      if ($ppid == '1') {
        $selectedDPU     = array('user' => $dpu_user,'name' => $dpu);
        $migrationFolder = getMigrationFolder($selectedDPU);
        $logfile         = $migrationFolder."vaultMigration71_*.log";
        $reason = "A previous migration for this system is still active, and migrations may still be in progress. Monitor activity in the debug log $logfile. If no action, kill the Zombie process affiliated with $dpu_user before continuing.";
        return -1;
      }
    }
  }
  return 0;
}

function obtainProgressLockOrExit($selectedDPU)
{
  global $DBG;
  global $io;
  global $MIGRATION_SCRIPT;
  global $PROGRESS_FILE;
  global $PROGRESS_FILE_FP;

  //  rc=0: Not in progress, continue with other checks
  //  rc=1: Aborted, allow this session to restart it. 
  //  rc=-1 In Progress, no other checks needed, use returned reason

  enterFunction("obtainProgressLockOrExit", $selectedDPU);
  $rc = 0;

  $migrationFolder = getMigrationFolder($selectedDPU);
  $progress_file   = $migrationFolder.$PROGRESS_FILE;

  // if the file exists, means we were aborted, so now obtain ownership
  // if file does not exist, fresh start or restart

  if (!file_exists($progress_file)) {
    //echo "file does not exist, creating it, and grabbing the lock\n";
    shell_exec("touch $progress_file");
  }

  $PROGRESS_FILE_FP = fopen("$progress_file", "r+");
  if (flock($PROGRESS_FILE_FP, LOCK_EX |LOCK_NB)) {  // acquire an exclusive lock
    logger(2, "Got the lock, owner of the migration!", $selectedDPU);
  } else {
    echoLog("Couldn't get the migration lock for selected system, task must be in progress!\n", $selectedDPU);
    exit_func($io);
  }
}

function isSourceAbortedOrInProgress($selectedDPU, &$reason)
{
  global $DBG;
  global $MIGRATION_SCRIPT;
  global $PROGRESS_FILE;
  global $PROGRESS_FILE_FP;
  global $RESERVED_SPACE;

  //  rc=0: Not in progress, continue with other checks
  //  rc=1: Aborted, allow this session to restart it. 
  //  rc=-1 In Progress, no other checks needed, use returned reason (obtains reserved space)

  enterFunction("isSourceAbortedOrInProgress", $selectedDPU);
  $rc = 0;

  $migrationFolder = getMigrationFolder($selectedDPU);
  $progress_file   = $migrationFolder.$PROGRESS_FILE;

  // source is aborted if the PROGRESS FILES exist and we can get the lock
  if (file_exists($progress_file)) {
    $PROGRESS_FILE_FP = fopen("$progress_file", "r+");

    if (flock($PROGRESS_FILE_FP, LOCK_EX |LOCK_NB)) {  // acquire an exclusive lock
      $reason = "A previous migration task was aborted, verify state of the system before continuing!";
      flock($PROGRESS_FILE_FP, LOCK_UN); // unlock it now, we'll obtain it later
      $rc = 1;
    } else {
      $reason = "A migration is in progress for this system.";
      // obtain the space estimated for the in progress migration
      // store it globally, since it is needed several functions further down
      $RESERVED_SPACE += getReservedSpace($selectedDPU);
      $rc = -1;
    }
  }

  return $rc;
}

function presentResults($selectedDPU)
{
  global $MG_REPORTS_FILE;
  enterFunction("presentResults", $selectedDPU);

// TODO, do .ini line entries need the trailing ;? and if so, why doesn't -p add it?
  $migrationFolder = getMigrationFolder($selectedDPU);
  $mg_rpt_file     = $migrationFolder.$MG_REPORTS_FILE;
  
  // List Backups Completed Successful
  $NONE="NONE_FOUND";
  $numOfSuccesses = 0;
  $numOfFailures  = 0;
  $cmd = "/usr/bp/bin/bputil -g \"Backups Success\" \"NULL\" $NONE $mg_rpt_file |grep -v \"\[Backups Success\]\"|cut -f1 -d=";

  $shell_ret = shell_exec("$cmd");
  logger(2,"Success:", $shell_ret);
  $output = explode(chr(10),rtrim($shell_ret)); // chr(10) = /n
  if (is_array($output) && $output[0] != $NONE) {
    $onetime = true;
    foreach( $output as $value ) {
      $numOfSuccesses++;
      if ($onetime) {
        echo "SUCCESS....\n";
        $onetime = false;
      }
      echo trim($value)."\n";
    }
  }

  // Find failures and their message
  $cmd = "/usr/bp/bin/bputil -g \"Backups Failed\" \"NULL\" $NONE $mg_rpt_file |grep -v \"\[Backups Failed\]\"";

  $shell_ret = shell_exec("$cmd");
  logger(2,"Failures:", $shell_ret);
  $output = explode(chr(10),rtrim($shell_ret));
  if (is_array($output) && $output[0] != $NONE) {
    $onetime = true;
    foreach( $output as $value ) {
      $numOfFailures++;
      if ($onetime) {
        echo "FAILURES....\n";
        $onetime = false;
      }
      //echo trim($value)."\n";
      $expstr = explode("=", $value);
      echo trim($expstr[0]);
      echo "  Reason: ". trim($expstr[1]). "\n";
    }
  }

  if ($numOfSuccesses === 0 && $numOfFailures === 0) {
    echoLog("No Results Posted\n");
  }

  define("GIGABYTE", 1073741824);
  if ($numOfSuccesses > 0 && $numOfFailures === 0) {
    $syncUser   = $selectedDPU['user'];
    $dpu        = $selectedDPU['name'];
    $result     = getVaultSpaceUsed($syncUser);
    $VaultBytes = $result['VaultBytes'];
    $GBStr      = round(($VaultBytes / GIGABYTE), 2);
    //$TempBytes= $result['TempBytes'];
    echoLog ("\nOnce replication is enabled on $dpu system, this will free up $GBStr (GB)\n");
  }
}

function clearOutFailures($selectedDPU)
{
  global $MG_REPORTS_FILE;
  // Find failures

  // TODO, is there a better way?
  // Poor Mans approach...read file, rename Backups Failures to Prevous Failures

  $migrationFolder = getMigrationFolder($selectedDPU);
  $mg_rpt_file     = $migrationFolder.$MG_REPORTS_FILE;

  $fhandle = fopen($mg_rpt_file,"r");
  $content = fread($fhandle,filesize($mg_rpt_file));
  $content = str_replace("Backups Failed", "Previous Failures", $content);
 
  $fhandle = fopen($mg_rpt_file,"w");
  fwrite($fhandle,$content);
  fclose($fhandle);
}

/////////////////////////////////////////////////////////////
///// LOGGING ROUTINES COMBINED FROM CMC and SYNCSTATS //////
/////////////////////////////////////////////////////////////
# handles logging to a specified file

// TODO is 4 ENOUGH
$log_max_number = 4; // {app}_0.log through {app}_#.log
global $log_level;
global $log_filename;
global $log_handle;
global $log_is_open;

/**
 * @desc - initializes log $logname (e.g. statussync), rotates files
 *         if necessary, looks up log level, and prepares log for
 *         writing.  Log will be $logname_#.log
 * @param $logname - log's base name
 * @return - true on success, error string on error
 */
function initialize_log($logname, $log_path="")
{
  global $log_max_number;
  global $log_level;
  global $log_filename;
  global $log_handle;
  global $log_is_open;

  // get log level, log path
  // master.ini stuff
  $cmd_bppath = 'eval `grep BPDIR /etc/default/bp.ini`';
  $cmd_master_ini = 'MASTER_INI=$BPDIR/bpinit/master.ini';
  $cmd_bputil = 'BPUTIL=$BPDIR/bin/bputil';
  $cmd_ini = "$cmd_bppath; $cmd_master_ini; $cmd_bputil";
  $cmd_log_level = '$BPUTIL -g "debugging" "'.$logname.'" "0" $MASTER_INI';
  $log_level = trim(shell_exec("$cmd_ini; $cmd_log_level"));
  if (!$log_path) {
    $cmd_log_path  = '$BPUTIL -g "Location Information" "Logs-Dir" "" $MASTER_INI';
    $log_path = trim(shell_exec("$cmd_ini; $cmd_log_path"));
  }

  // manage file
  $log_filename = $log_path . '/' . $logname.'_0.log';
  if ($log_is_open) {
    fclose($log_handle);
  }
  $log_is_open = false;

  // rotate files if needed
  $date_string = date('F j, Y');
  $log_preamble = 'Log for '.$logname.', created '.$date_string."\n";
  if (is_dir($log_filename)) {
    return "specified log file '$log_filename' is a directory!";
  }
  if (is_writable($log_filename)) {
    $log_handle = fopen($log_filename, 'r');
    if ($log_handle === false) {
      return "could not open logfile '$log_filename'";
    }
    if (fgets($log_handle, 1024) != $log_preamble) {
      $rotate = true;
    } else {
      $rotate = false;
    }
    fclose($log_handle);
  } else {
    $rotate = false;
  }

  if ($rotate == true) {
    $logfiles = array(); // log filenames (no path)
    $dh = @opendir($log_path);
    if ($dh === false) {
      return "could not open directory '$log_path'";
    }
    while (false !== ($filename = readdir($dh))) {
      if (preg_match('/^' . $logname . '_\d\.log$/', $filename)) { // match log name
        $lognum = (int)substr($filename, strlen($logname)+1, -4); // cut off $logname_ from start, .log from end
        $logfiles[$lognum] = $filename;
      }
    }
    closedir($dh);
    // scan for any missing files, we won't have to move them all then
    $found_missing_log = false;
    $missing_log_num = -1;
    for ($i = 0; $i < $log_max_number; $i++) {
      if (!isset($logfiles[$i])) {
        $found_missing_log = true;
        $missing_log_num = $i;
      }
    }
    if ($found_missing_log == true) {
      $lowest_delete_num = $log_max_number + 1; // move some files, so no deleteions necessary
      $highest_move_num = $missing_log_num - 1; // work down from here to rename log files from app_$num.log to app_($num+1).log
    } else {
      $lowest_delete_num = $log_max_number; // moving all files, so remove the one before the max
      $highest_move_num = $log_max_number - 1; // move all files up
    }

    // handle deletions
    foreach ($logfiles as $num => $name) {
      if ($num >= $lowest_delete_num) {
        unlink($log_path . '/' . $name);
      }
    }

    // now rotate the files
    for ($i = $highest_move_num; $i >= 0; $i--) {
      if (!isset($logfiles[$i])) {
        continue; // for safety, this shouldn't happen
      }
      $oldname = $log_path . '/' . $logfiles[$i];
      $newname = $log_path . '/' . $logname . '_' . ($i+1) . '.log';
      if (false === rename($oldname, $newname)) {
        return "could not rename logfile '$oldname' to '$newname'";
      }
    }
  }

  // open file, write preamble if file is new
  $write_preamble = (is_file($log_filename) ? false : true);
  $handle = fopen($log_filename, 'a'); // open/create file for appending
  if ($handle === false) {
    return "could not open logfile '$log_filename' for writing";
  }
  if ($write_preamble) {
    fwrite($handle, $log_preamble);
  }
  $log_handle = $handle;

// log the start of a new instance...
logger(0, getTimestamp().'==============================================================');
  // return success
  return true;
}

/**
 * @desc - writes to the open/initialized logfile
 * @note - semantics are different for this, a newline is appended automatically
 *         because otherwise you have to double quote long strings or append a
 *         newline, causing headaches for you or for php's string parser...
 * @precondition - log has been initialized
 * @param $level - log level (0-5), 0 = always log, 5 = log only for debugging
 * @param $text - text to log
 * @param optional $newline - false to exclude a newline, default is to append one
 * @return - true on success, false on error
 */
function loggerNL($level, $text)
{
  // include a NL and a date...
  global $log_handle;
  global $log_level;
  global $DBG;

  if ($level > $log_level && !$DBG) {
    return true;
  }
  if (!$log_handle) {
    return false;
  }
  $bytes = fwrite($log_handle, getTimeStamp().$text."\n");

  // print any args supplied
  $nArguments = func_num_args();
  if ($nArguments > 0) {
    for ($i = 2; $i < $nArguments; $i ++) {
      $arg = func_get_arg($i);
      writeVariable($arg, $i, true);
    }
    fwrite($log_handle, "\n");
  }

  return ($bytes !== false);
}

function logger($level, $text)
{
  global $log_handle;
  global $log_level;
  global $DBG;

  if ($level > $log_level && !$DBG) {
    return true;
  }
  if (!$log_handle) {
    return false;
  }
  $bytes = fwrite($log_handle, $text);

  // print any args supplied
  $nArguments = func_num_args();
  if ($nArguments > 0) {
    for ($i = 2; $i < $nArguments; $i ++) {
      $arg = func_get_arg($i);
      writeVariable($arg, $i, true);
    }
    fwrite($log_handle, "\n");
  }

  return ($bytes !== false);
}

function enterFunction($stringFunction) {
  global $log_handle;
  global $log_level;
  global $DBG;

  if (!$log_handle) {
    return false;
  }

  if ($log_level >= 2 || $DBG) {
    $message = "\n";
    $bytes = fwrite($log_handle, $message);

    $message = getTimestamp() . "Enter: " . $stringFunction;
    $nArguments = func_num_args();
    if ($nArguments == 1) {
      $message .= "(no args)\n";
    } else {
      $message .= "(" . ($nArguments - 1) . " args):\n";
    }
    $bytes = fwrite($log_handle, $message);
    //error_log($message, $this->m_type, $this->m_file);
    for ($i = 1; $i < $nArguments; $i ++) {
      $arg = func_get_arg($i);
      writeVariable($arg, $i, true);
    }
  }
}

// Logs a variable with var_dump().
function writeVariable($var, $argumentNumber = 1, $forceLog = false)
{
  global $log_handle;
  global $log_level;

  if ($log_level >= 2 || $forceLog) {
    ob_start();
    var_dump($var);
    $contents = ob_get_contents();
    ob_end_clean();
    $message = "#" . $argumentNumber . ": " . $contents;
    $bytes = fwrite($log_handle, $message);
  }
}

// Gets the current time to stamp log entries.
function getTimestamp() {
  $startTime = time();
  $timestr = date('m/d/Y H:i:s', $startTime);
  $s = sprintf("%s: ", $timestr);
  return $s;
}

function echoLog($msg)
{
  logger(0,$msg);
  echo $msg;
}
function close_log()
{
  global $log_handle;
  if ($log_handle) {
    fclose($log_handle);
  }
}
////////////////////////  END OF LOGGER FUNCTIONS  ////////////////////
////////////////////////  END OF LOGGER FUNCTIONS  ////////////////////

function obtainAndValidateSourceEncryption($selectedDPU, $dpuEncodedPassphrase)
{
  enterFunction("obtainAndValidateSourceEncryption", $selectedDPU);

  $rc = 0;
  echo "Because this system has encrypted backups, we'll need its Passphrase.";
  $not_valid = true;
  while($not_valid) {
    echo "\nValue:";
    system('stty -echo');
    $passphrase = trim(fgets(STDIN));
    system('stty echo');
    // validate the passphrase
    $rc = validateAndEncodePassPhrase($selectedDPU, &$passphrase);
    if ($rc == 0) {
      // add a new line since the users CR didn't echo
      echo "\nValidated\n";
      $not_valid = false;
      // The returned passphrase from the validate routine may be
      // different to accommadate urlencoded, so use the one returned.
      $dpuEncodedPassphrase=$passphrase;
    }
    else {
      echo "\nInvalid, try again.";
    }
  }
  return $rc;
}

function validateAndEncodePassPhrase($selectedDPU, $dpuPassphrase)
{
  global $INSTALL_PATH;
  global $DECRYPTION_EXE;
  enterFunction("validateAndEncodePassPhrase", $selectedDPU);

  $migrationFolder = getMigrationFolder($selectedDPU);
  $filepath        = "$migrationFolder"."cryptoDaemonMasterKeys";

  // For validation, best to just pass in the pp asis cl arg instead of messing
  //  with the file. We'll add the value to the file during migration start...
  // bpcrypt checks for -p or -r, path to the rpt file 

  $cmd = "$DECRYPTION_EXE -x -m $filepath -p ".escapeshellarg($dpuPassphrase)." 2> /dev/null";
  $retval = -1;
  $cmdStripped = str_replace($dpuPassphrase, "********", $cmd);
  logger(0,"cmd_pp:",$cmdStripped);
  system($cmd, $retval);
  if ($retval == 0) {
    // success, return encoded passphrase
    // first encode it before it goes to the exe 
    $encodedPassphrase = "";
    for ($i=0; $i < strlen($dpuPassphrase); $i++) {
      $encodedPassphrase .= dechex(ord($dpuPassphrase[$i]));
    }
    $dpuPassphrase = $encodedPassphrase;
    return 0;
  }

  // We failed, try the urlencoded version of the passphrase
  // and if it works, return it...
  $urlPassphrase = urlencode($dpuPassphrase);
  $cmd = "$DECRYPTION_EXE -x -m $filepath -p ".escapeshellarg($urlPassphrase)." 2> /dev/null";
  $cmdStripped = str_replace($urlPassphrase, "*******E", $cmd);
  logger(0,"cmd_enc:",$cmdStripped);
  system($cmd, $retval);
  if ($retval == 0) {
    // success, return encoded passphrase
    $encodedPassphrase = "";
    for ($i=0; $i < strlen($urlPassphrase); $i++) {
      $encodedPassphrase .= dechex(ord($urlPassphrase[$i]));
    }
    $dpuPassphrase = $encodedPassphrase;
    return 0;
  }
  logger(2,"cmd result:", $retval);
  return -1;
}

function setOrClearPassphrase($selectedDPU, $addTrue=false, $dpuEncodedPassphrase='')
{
  global $MG_REPORTS_FILE;
  enterFunction("setOrClearPassphrase", $addTrue);

  $migrationFolder = getMigrationFolder($selectedDPU);
  $mg_rpt_file     = $migrationFolder.$MG_REPORTS_FILE;

  $passphrase="";
  if ($addTrue === true) {
    // but first encode it to hex
    //for ($i=0; $i < strlen($dpuEncodedPassphrase); $i++) {
    //  $passphrase .= dechex(ord($dpuEncodedPassphrase[$i]));
    //}
    $passphrase = $dpuEncodedPassphrase;
  }
  $cmd = "/usr/bp/bin/bputil -p \"Backups\" \"Timing\" \"$passphrase\" $mg_rpt_file ";
  $cmdStripped = str_replace($passphrase, "*******H", $cmd);
  logger(0,"cmd_hex:",$cmdStripped);
  $shell_ret = shell_exec("$cmd");

  return;
}

//
// This function returns the amount of space used on the vault for a given DPU, by
// running the 'du' command; the size used is in bytes.  The vaulted data is kept
// on the /backups directory, with the subdirectory of SyncUser and the 'syncd' directory
// under that.
//
// TAKEN FROM: /var/www/html/recoveryconsole/bpl/dpvlicense.php
function getVaultSpaceUsed($dpuUser, $homeDirectory = NULL)
{
  if ($homeDirectory == NULL) {
    $path = "/backups/" . $dpuUser . "/syncd/";
  } else {
    $path = $homeDirectory . "/syncd/";
  }
  $TotalBytes = -1;
  $VaultBytes = -1;
  $TempBytes = -1;

  // Get bytes used in the syncd directory for the DPU.
  $command = sprintf("sudo -S /usr/bin/du -s -b %s", $path);
  $handle = popen($command, 'r');
  if ($handle) {
    $commandReturn = fread($handle, 1024);
    pclose($handle);
  }
  // The value comes back as <size><TAB><directory name>
  $a = explode("\t", $commandReturn);
  if (is_array($a) && count($a) > 0) {
    $TotalBytes = $a[0];
  }

  // If we have data used, exclude temp space to get true vaulted space.
  if ($TotalBytes > 0) {
    $command = sprintf("sudo -S /usr/bin/du -s -b --exclude='new-*' --exclude='*.resume
' %s", $path);
    $handle = popen($command, 'r');
    if ($handle) {
      $commandReturn = fread($handle, 1024);
      pclose($handle);
    }
    // The value comes back as <size><TAB><directory name>
    $a = explode("\t", $commandReturn);
    if (is_array($a) && count($a) > 0) {
      $VaultBytes = $a[0];
    }
  }

  if ($TotalBytes > 0 && $VaultBytes > 0) {
    $TempBytes = $TotalBytes - $VaultBytes;
  }

  $result = array("VaultBytes" => $VaultBytes, "TempBytes" => $TempBytes);
  return $result;
}

function locateOrphanDPUs()
{
  enterFunction("locateOrphanDPUs");

  // bug 16493, redirect garbage
  $dpus = `find /backups -name syncd -print 2>/dev/null | cut -f3 -d'/'`;
  $output = explode(chr(10),rtrim($dpus)); // chr(10) = /n
  logger(3, "find /backups -name syncd -print: ", $output);

  $dpuUsers = "";
  $dpuUsers = my_bp_get_vaulted_dpus_users(); # returns the UEB's user

  $arrayDiff = array_diff($output, $dpuUsers);
  logger(3, "orphans dpus: ", $arrayDiff);
  return $arrayDiff;
}

function locateOrphanClients($selectedDPU)
{
  enterFunction("locateOrphanClients", $selectedDPU);

  $syncUser   = $selectedDPU['user'];
  $dpu        = $selectedDPU['name'];
  $clientList = getClientList($selectedDPU);

  // bug 16493, redirect garbage
  $cmd = "find /backups/$syncUser/syncd -print 2>/dev/null|cut -f5 -d'/'|sort -u";
  $dpuClients = `$cmd`;
  $output = explode(chr(10),rtrim($dpuClients)); // chr(10) = /n
  logger(3, "find /backups/$syncUser/syncd -print: ", $output);

  $arrayDiff = array_diff($output, $clientList);
  logger(3, "orphans clients: ", $arrayDiff);
  return $arrayDiff;
}

function my_bp_get_vaulted_dpus_users()
{
  enterFunction("my_bp_get_vaulted_dpus_users");
  // local version because the bp_get_vaulted_dpus() prints out an ugly message: 
  // sh: /usr/bp/bin/: is a directory, because of a leading ; in their cat cmd

  $output = array();
  $syncUsers = `cat /etc/passwd | grep /backups | cut -d: -f1`;
  $output = explode(chr(10),rtrim($syncUsers)); // chr(10) = /n
  logger(3, "dpu users: ", $output);
  return $output;
}

function collectInfo()
{
  global $DBG;
  global $FOLDER_NAME;

  enterFunction("collectInfo");

  // get a tmp folder
  $tmpPath = '/tmp';
  $tmpdir = tempdir($tmpPath);
  //echo "$tmpdir\n";
  $slash = '/';

  // get a list of vaulted DPUs
  $syncDirs = array();
  $syncUsers = `cat /etc/passwd | grep /backups | cut -d: -f1`;
  $syncDirs = explode(chr(10),rtrim($syncUsers)); // chr(10) = /n

  // ls listing of Vaulted DPUs
  if (is_array($syncDirs)) {
    $ext = "_ls_alr";
    foreach ($syncDirs as $key => $syncDir) {
      $dirIn = "/backups/".str_pad($syncDir, 15);
      `ls -alR $dirIn > $tmpdir$slash$syncDir$ext`;
    }
  }

  // space on D2DBackups
  $space_info_file="space_info_file";
  // need to bypass cookies to obtain backup device (space) info
  $res = bp_bypass_cookie(3);
  if ($res === false) {
    $f = fopen("$tmpdir$slash$space_info_file", "w"); 
    fwrite($f, "Unable to obtain default device data due to bypass cookie failure:".bp_error()."\n"); 
    fclose($f); 
  }
  else {
    ob_start();
    $vaultBackupDevice="";
    vaultDefaultDevice($vaultBackupDevice); 
    $myStr = ob_get_contents();
    ob_end_clean();
    $freeSpace = round((getDefaultDeviceFreeSpace($vaultBackupDevice) / 1024), 2);
    $f = fopen("$tmpdir$slash$space_info_file", "w"); 
    fwrite($f, $myStr); 
    fwrite($f, "Freespace: ".$freeSpace." (GB)\n"); 
    fclose($f); 
  }
  // Append the system info
  `df -h >> $tmpdir$slash$space_info_file`;

  // get a list of all migrationV71 directories
  $migrationV71List="";
  if (is_array($syncDirs)) {
    foreach ($syncDirs as $key => $syncUser) {
      $v71Dir = "/backups/$syncUser/syncd/$FOLDER_NAME"; 
      if (file_exists($v71Dir)){ 
        $migrationV71List .= " ".$v71Dir;
      }
    }
  }
  //echo $migrationV71List;

  // get a list of vault migration log.dirs files
  $logList = `ls /usr/bp/logs.dir/vaultM*log`;
  $logList = rtrim($logList);
  $logList = str_replace(chr(10)," ",$logList);

  // get a list of updatedb log.dirs files
  $updatedbLogList = `ls /usr/bp/logs.dir/updatedb*log`;
  $updatedbLogList = rtrim($updatedbLogList);
  $updatedbLogList = str_replace(chr(10)," ",$updatedbLogList);

  // now zip it all

  $collectionFile = "/tmp/vm71_".getmypid().".tgz";
  $cmd = "tar cvzf $collectionFile $tmpdir $migrationV71List $logList $updatedbLogList";
  logger(2,"collection cmd:", $cmd);
  `$cmd 2>/dev/null`;

  `rm -rf $tmpdir`;
  echo "Collected data is located here: $collectionFile\n";
  if (!$DBG) {
    echo "(To collect more info, rerun the migration cmd with -dbg, then collect data again.)\n";
  }
}

function tempdir($dir) {
    $tempfile=tempnam($dir,'');
    if (file_exists($tempfile)) { unlink($tempfile); }
    mkdir($tempfile);
    if (is_dir($tempfile)) { return $tempfile; }
}

// bug 16207 start
function doesSourceSupportReplication($dpu)
{
  // returns 0 Replication Supported
  // returns 1 Replication Not Supported
  // return -1, nw or other problems bp_error()
  // return -2, dpuid not found

  global $TRUE_VALUE_ARRAY;

  enterFunction("doesSourceSupportReplication", $dpu);

  $dpu_id = getSystemIdForDPU($dpu);
  if (empty($dpu_id)) {
    logger(0,"dpu_id not found, replication support unchecked, for:", $dpu);
    return -2; // indicates dpu not found error (bp_error() is not set!)
  }
  $isReplicationSupported = bp_replication_supported($dpu_id);
  logger(0,"isReplicationSupported:",$isReplicationSupported); 
  if ($isReplicationSupported === -1) {
    logger(0,"Could not determine REPLICATION supported: ",bp_error());
    return -1; // indicates use bp_error() for message
  }

  $isReplicationSupported = in_array($isReplicationSupported, $TRUE_VALUE_ARRAY);
  if ($isReplicationSupported == true){
    return 0;
  }

  return 1; // not enabled
}
// bug 16207 end
