<?php

class JobsActive
{
    private $BP;
    const DEFAULT_NIC = "eth0";

    public function __construct($BP, $sid, $Functions)
    {
        $this->BP = $BP;
        $this->now = time();
		$this->sid = $sid;
	    $this->itemCount = 0;

        $this->functions = $Functions;

        require_once('jobs.functions.php');
        $this->jobsFunctions = new JobsFunctions($this->BP);
    }


    public function get_active_jobs($which, $filter, $sid, $systems)
    {
        $allJobs = array();

        $specialJob = explode(".", $which);

        foreach($systems as $sid => $sysName){
            //this handles <app_id>.ir or flr jobs types
            if (count($specialJob) > 1){
                $allJobs = $this->getSpecialJob($specialJob, $which, $sid);
            } else {
                if ($filter === "ir") {
                    $IRJobs = $this->getIRJobs($sid);
                } else {
                    $IRJobs = array();
                }
                if ($filter === "flr") {
                    $FLRJobs = $this->getFLRJobs($sid);
                } else {
                    $FLRJobs = array();
                }
                if ($filter === "" || $filter === "replication") {
                    $replicationJobs = array(); // TODO $this->activeReplicationJobs();
                } else {
                    $replicationJobs = array();
                }

                if ($filter == "" || $filter === "backup" || $filter === "restore" || $filter === "archive") {
                    $normalJobs = array();
                    $jobs = $this->BP->get_job_list($sid);

                    if ($jobs !== false and $jobs !== null) {
                        foreach ($jobs as $id => $name) {
                            if (is_numeric($which) && $which > 0 and $id != $which) {
                                continue;
                            }
                            if ($which) {
                                $jobInfo = $this->BP->get_job_info($id, $sid);
                                if (isset($jobInfo['client_id'])) {
                                    $clientInfo = $this->BP->get_client_info($jobInfo['client_id'], $sid);
                                    $clientName = $clientInfo['name'];
                                }

                                $data = $this->buildActiveJobsOutput($jobInfo, $sysName, $sid, $clientName);
                                $filtered = $this->buildReturn($data, $filter);

                                if ($filtered != NULL) {
                                    $normalJobs[] = $filtered;
                                }
                            }
                        }
                    }
                } else {
                    $normalJobs = array();
                }

                // merge all for final result.
                $allJobs = array_merge($allJobs, $IRJobs, $FLRJobs, $replicationJobs, $normalJobs);
            }
        }

        return ($allJobs);
    }

    function buildActiveJobsOutput($jobData, $sysName, $sid, $clientName)
    {
        $data = array();

        $name = isset($jobData['name']) ? $jobData['name'] : null;
        $type = isset($jobData['type']) ? $this->jobsFunctions->getJobType($jobData['type']) : null;
        //$mode = isset($jobData['type']) ? $jobData['type'] : null;
        $mode = isset($jobData['type']) ? $this->functions->getBackupTypeString($jobData['type']) : null;
        $startDate = isset($jobData['start_time']) ? $this->functions->formatDateTime($jobData['start_time']) : null;

        $status = null;
        if (isset($jobData['status'])) {
            $status = $jobData['status'];
            if (is_int($status)) {
                // Backup Status: 0 = Successful, 1 = Warning, 2 = Failed.
                $status = ($status === 0) ? 'Successful' : (($status === 1) ? 'Warning' : 'Failed');
            }
        }

        $suspendable = isset($jobData['status']) && $jobData['status'] == 'QUEUED' ? true : false;
        $resumable = isset($jobData['status']) && $jobData['status'] == '*ON HOLD*' ? true : false;
        $instanceName = isset($jobData['instance_short_name'])  ? $jobData['instance_short_name'] : null;
        $assetName = (!strcasecmp($instanceName,'file-level')) ? $clientName : (!strcasecmp($instanceName, 'archive')) ? "Backup Copy" : $instanceName;
        $scheduleName = isset($jobData['schedule_name']) ? $jobData['schedule_name'] : null;
        $jobName = isset($scheduleName) ? $scheduleName : 'Backup of ' + $assetName;
        // Do this as the core is not returning duration.  Format duration as a string.
        $elapsedTime = isset($jobData['elapsed_time']) ? true : false;
        $duration = isset($jobData['start_time']) && $elapsedTime == false ? $this->functions->formatTimeDelta($this->now - $jobData['start_time']) : "";
        $comment = isset($jobData['comment']) ? $jobData['comment'] : null;
        $percentComplete = isset($jobData['percent_complete']) ? $jobData['percent_complete'] : null;
        $id = isset($jobData['id']) ? $jobData['id'] : null;
        $appName = "";
        if (isset($jobData['instance_id'])) {
            $instanceID = $jobData['instance_id'];
            $instanceInfo = $this->BP->get_appinst_info($instanceID, $sid);
            if($instanceInfo != false) {
                $appName = $instanceInfo[$instanceID]['app_type'];
            }
        }

            $data = array(
                'id' => $id,
                'name' => $jobName,
                'status' => $this->mapActiveJobStatus($status),
                'comment' => $comment,
                'percent_complete' => $percentComplete,
                'start_date' => $startDate,
                'mode' => $mode,
                'type' => $type,
                'application' => $appName == "Archive" ? "Backup Copy" : $appName,
                'client_name' => $clientName,
                'instance_name' => $instanceName  == "Archive" ? "Backup Copy" : $instanceName,
                'asset_name' => $assetName,
                'duration' => $duration,
                'suspendable' => $suspendable,
                'resumable' => $resumable,
                'system_id' => $sid,
                'system_name' => $sysName
            );

        return $data;
    }

    function activeReplicationJobs(){
        require_once('replication.php');
        $replication = new Replication($this->BP);
        return $replication->get("queue", "active", $this->sid);
    }

    function getSpecialJob($specialJob, $which, $sid){
        if($specialJob[1] == "ir") {
            $tempJobs = $this->getIRJobs($sid);
        } elseif($specialJob[1] == "flr") {
            $tempJobs = $this->getFLRJobs($sid);
        }
        foreach($tempJobs as $job){
            if($job['id'] == $which){
                $allJobs = $job;
            }
        }

        return $allJobs;
    }
    function buildIROutput($IRJobs, $sid, $type){

        $id = isset($IRJobs['id']) ? $IRJobs['id'] : null;
        $status = isset($IRJobs['status']) ? $IRJobs['status'] : null;
        $detail = isset($IRJobs['detail']) ? $IRJobs['detail'] : null;
        $percentage = ($status == 'error' || $status == 'dead') ? 0 : ($status == 'available' ? 100 : isset($IRJobs['percentage']) ? $IRJobs['percentage'] : null);
        $time = isset($IRJobs['time']) ? $this->functions->formatDateTime($IRJobs['time']) : null;
        $duration = isset($IRJobs['time']) ? $this->functions->formatTimeDelta($this->now - $IRJobs['time']) : null;
        $audit = isset($IRJobs['audit']) ? $IRJobs['audit'] == 1 ? "true" : "false": "n/a";

        $instanceName = isset($IRJobs['vm_name']) ? $IRJobs['vm_name'] : null;
        $client = isset($IRJobs['vm_server']) ? $IRJobs['vm_server'] : null;
        $appInst = $this->BP->get_appinst_info($id, $sid);
        if ($appInst !== false and $id !== null){
            foreach($appInst as $appInfo) {
                $type = isset($appInfo['app_type']) ? $appInfo['app_type'] : "n/a";
            }
        }
        $data = array(
            'id' => $id . ".ir",
            'name' => "IR of " . $instanceName,
            'status' => $status,
            'comment' => $detail,
            'percent_complete' => $percentage,
            'start_date' => $time,
            'mode' => "Instant Recovery",
            'type' => "restore",
            'application' => $type,
            'client_name' => $client,
            'instance_name' => $instanceName,
            'asset_name' => $instanceName,
            'duration' => $duration,
            'audit' => $audit,
            'unique' => $audit,
            'suspendable' => false,
            'resumable' => false,
            'system_id' => $sid,
            'system_name' => $this->functions->getSystemNameFromID($sid)
        );

        return $data;
    }

    function getIRJobs($sid){
        $allJobs = array();
        $vmIRStatus = $this->BP->get_vm_ir_status($sid);
        $hvIRStatus = $this->BP->get_hyperv_ir_status($sid);
        if ($vmIRStatus !== false) {
            foreach ($vmIRStatus as $IR) {
                $type = "VMware";
                $IRJobs = $this->buildIROutput($IR, $sid, $type);
                if(!empty($IRJobs)) {
                    $allJobs[] = $IRJobs;
                }
            }
        }
        if ($hvIRStatus !== false) {
            $type = "Hyper-V";
            foreach ($hvIRStatus as $IR) {
                $IRJobs = $this->buildIROutput($IR, $sid, $type);
                if(!empty($IRJobs)) {
                    $allJobs[] = $IRJobs;
                }
            }
        }
        return $allJobs;
    }

    function getFLRJobs($sid){
        $allJobs = array();
        $flrStatus = $this->BP->get_disk_image_status($sid);
        if ($flrStatus !== false){
            foreach ($flrStatus as $FLR) {
                $FLRJobs = $this->buildFLROutput($FLR, $sid);
                $allJobs[] = $FLRJobs;
            }
        }
        return $allJobs;
    }

    function buildFLROutput($FLRjobs, $sid) {
	
        $id = isset($FLRjobs['id']) ? $FLRjobs['id'] : null;
        $status = isset($FLRjobs['status']) ? $FLRjobs['status'] : null;
        $time = isset($FLRjobs['available_since']) ? $this->functions->formatDateTime($FLRjobs['available_since']) : null;
        $duration = isset($FLRjobs['available_since']) ? $this->functions->formatTimeDelta($this->now - $FLRjobs['available_since']) : null;
        $availableTargets = isset($FLRjobs['available_targets']) ? $FLRjobs['available_targets'] : null;
        $message = isset($FLRjobs['message']) ? $FLRjobs['message'] : null;
        $percentage = null;
        if (strcasecmp($status, 'available') == 0) {
            $percentage = 100;
        } else if (strcasecmp($status, 'error') == 0) {
            $percentage = 0;
        }
        $shareDetails = null;
        if (strstr($availableTargets, 'smb')) {
            $network = $this->BP->get_network_info(JobsActive::DEFAULT_NIC, $sid);
            $shareHost = ($network !== false) ? $network['ip'] : '{this_appliance_ip}';
            $shareDetails = "\\\\" . $shareHost . "\\" . 'flr' . $id;
        }


        $appArray = $this->BP->get_application_clients($FLRjobs['id'], $sid);
        $appName = $appArray !== false ? $appArray['app_name'] : null;

        $appInst = $this->BP->get_appinst_info($id, $sid);
        if ($appInst !== false and $id !== null){
            foreach($appInst as $appInfo) {
                $type = isset($appInfo['app_type']) ? $appInfo['app_type'] : "n/a";
                $client = $type == "Hyper-V" ? $appInfo['client_name'] : $appInfo['primary_name'];
                $instanceName = $type == "Hyper-V" ? $appInfo['primary_name'] : $appInfo['secondary_name'];
            }
        }
        $data = array(
            'id' => $id . ".flr",
            'name' => "FLR of " . $instanceName,
            'status' => $status,
            'comment' => $message,
            'percent_complete' => $percentage,
            'start_date' => $time,
            'mode' => "File-Level Recovery",
            'type' => "Restore",
            'application' => $appName,
            'client_name' => $client,
            'instance_name' => $instanceName,
            'asset_name' => $instanceName,
            'duration' => $duration,
            'available_targets' => $availableTargets,
            'share_details' => $shareDetails,
            'unique' => $availableTargets,
            'suspendable' => false,
            'resumable' => false,
            'system_id' => $sid,
            'system_name' => $this->functions->getSystemNameFromID($sid)
        );

        return $data;
    }
    function buildReturn($data, $filter){
        $skip = false;

        if ($filter != "") {
            switch ($filter) {

                // get them all from jobs list.
                case 'active':
                    break;
                // see if type matches user selection.  If not, skip it.
                default:
                    if ($data['type'] != $filter) {
                        $skip = true;
                    }
                    break;
            }
        }
        if ($skip) {
            $data = NULL;
        }

        return $data;
    }

    function processActiveJobs($which, $filter, $sid, $sysName)
    {
        $allJobs = array();
        $jobs = $this->BP->get_job_list($sid);

        foreach ($jobs as $id => $name) {
            if (is_numeric($which) && $which > 0 and $id != $which) {
                continue;
            }
            if ($which) {
                $jobInfo = $this->BP->get_job_info($id, $sid);
                if (isset($jobInfo['client_id'])) {
                    $clientInfo = $this->BP->get_client_info($jobInfo['client_id'], $sid);
                    $clientName = $clientInfo['name'];
                }

                $data = $this->buildOutput($jobInfo, $sysName, $sid, $clientName, $which);
                $filtered = $this->buildReturn($data, $filter);

                if ($filtered != NULL) {
                    $allJobs['data'][] = $filtered;
                }
            }
        }
        return $allJobs;
    }

    function mapActiveJobStatus($status) {
        $status_map = array('*ON HOLD*' => 'Paused',
                            'QUEUED' => 'Queued',
                            'CONNECTING' => 'Connecting',
                            ' *ACTIVE*' => 'Active',
                            'ACTIVE' => 'Active',
                            'DB. UPDATING' => 'DB Update',
                            'CANCELLED' => 'Cancelled',
                            'SUCCESS*' => 'Warning',
                            'SUCCESSFUL' => 'Successful',
                            ' FAILED' => 'Error',
                            'COMPLETE' => 'Complete',
                            'PROCESS TERMINATED' => 'Terminated',
                            'UNKNOWN' => 'Unknown',
                            'INCOMPLETE/Cancelled' => 'Incomplete/Cancelled');

        return ($status_map[$status]);

    }

} //end JobsActive class


