<?php

$dir = '/api/includes/reports';

class ArchiveHistoryReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_archive_history_report($systems, $data)
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        $specified_archive_set_id = false;
        if ( array_key_exists('archive_set_id', $data) )
        {
            $specified_archive_set_id = (int)$data['archive_set_id'];
        }

        $specified_target_media = false;
        if ( array_key_exists('target', $data) )
        {
            $specified_target_media = $data['target'];
        }

        $start_time = false;
        if ( array_key_exists('start_date', $data) )
        {
            $start_time = strtotime($data['start_date']);
        }
        $end_time = false;
        if ( array_key_exists('end_date', $data) )
        {
            $end_time = $this->functions->formatEndDate($data['end_date']);
        }


        foreach ($systems as $system_id => $system_name)
        {
            if ( $specified_archive_set_id === false )
            {
                $archive_sets_filter = array();
                if ( $start_time !== false )
                {
                    $archive_sets_filter['job_start_time'] = $start_time;
                }
                if ( $end_time !== false )
                {
                    $archive_sets_filter['job_end_time'] = $end_time;
                }
                if ( $specified_target_media !== false )
                {
                    $archive_sets_filter['target'] = $specified_target_media;
                }

                $archive_sets_list = $this->BP->get_archive_sets($archive_sets_filter, $system_id);
            }
            else
            {
                $archive_sets_list = array( array('archive_set_id' => $specified_archive_set_id) );
            }

            if ( $archive_sets_list !== false and count($archive_sets_list) > 0 )
            {
                // Because the user can pass in an archive_set_id, $archive_set may only have an 'archive_set_id' and not any of the other fields
                foreach ( $archive_sets_list as $archive_set )
                {
                    $archive_set_info = $this->BP->get_archive_set_info($archive_set['archive_set_id'], $system_id);

                    if ( $archive_set_info !== false and count($archive_set_info) > 0 )
                    {

                        $temp_archive_set = array();
                        $temp_archive_set['archive_set_id'] = $archive_set['archive_set_id'];
                        $temp_archive_set['available'] = $archive_set_info['available'];
                        $temp_archive_set['date'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $archive_set['timestamp']); //TBD
                        $temp_archive_set['description'] = $archive_set_info['description'];
                        $temp_archive_set['is_imported'] = $archive_set_info['is_imported'];
                        switch ($archive_set_info['status'])
                        {
                            case Constants::ARCHIVE_STATUS_ARCHIVE_IN_PROGRESS:
                                $temp_archive_set['status'] = Constants::STATUS_ARCHIVE_IN_PROGRESS;
                                break;
                            case Constants::ARCHIVE_STATUS_IMPORT_IN_PROGRESS:
                                $temp_archive_set['status'] = Constants::STATUS_IMPORT_IN_PROGRESS;
                                break;
                            case Constants::ARCHIVE_STATUS_ARCHIVE_FAILED:
                            case Constants::ARCHIVE_STATUS_IMPORT_FAILED:
                                $temp_archive_set['status'] = Constants::STATUS_FAILURE;
                                break;
                            case Constants::ARCHIVE_STATUS_ARCHIVE_SUCCESS:
                            case Constants::ARCHIVE_STATUS_IMPORT_SUCCESS:
                            default:
                                $temp_archive_set['status'] = Constants::STATUS_SUCCESS;
                                break;
                        }
                        $temp_archive_set['system_name'] = $system_name;
                        $temp_archive_set['target'] = $archive_set_info['profile']['target'];
                        $temp_archive_set['archives'] = array();

                        $archives_list = $this->BP->get_archive_status( array('archive_set_id' => $archive_set['archive_set_id']), $system_id);
                        if ( $archives_list !== false and count($archives_list) > 0 )
                        {
                            foreach ( $archives_list as $archive )
                            {
                                $temp_archive = array();
                                $temp_archive['app_name'] = (array_key_exists('app_name', $archive) ? $archive['app_name'] :'');
                                $temp_archive['archive_id'] = $archive['archive_id'];
                                $temp_archive['archive_time'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $archive['archive_time']);
                                $temp_archive['backup_time'] = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $archive['backup_time']);
                                $temp_archive['client_name'] = $archive['client_name'];
                                $temp_archive['compressed'] = $archive['compressed'];
                                $temp_archive['deduped'] = $archive['deduped'];
                                $temp_archive['elapsed_time'] = gmdate(Constants::DATE_FORMAT_HRS_MINS_SECS, $archive['elapsed_secs']);
                                $temp_archive['encrypted'] = $archive['encrypted'];
                                $temp_archive['fastseed'] = $archive['fastseed'];
                                $temp_archive['files'] = (array_key_exists('files', $archive) ? $archive['files'] :'N/A');
                                $temp_archive['instance_description'] = $archive['instance_description'];
                                $temp_archive['is_imported'] = $archive['is_imported'];
                                $temp_archive['orig_backup_id'] = $archive['orig_backup_id'];
                                $temp_archive['os_type'] = (array_key_exists('os_type', $archive) ? $archive['os_type'] :'');
                                $temp_archive['size'] = $archive['size'];
                                $temp_archive['success'] = $archive['success'];
                                $temp_archive['type'] = $this->functions->getBackupTypeString($archive['type']);

                                $temp_archive_set['archives'][] = $temp_archive;

                            }
                        }

                        $report_data[] = $temp_archive_set;
                        $report_count += 1;  // Counts the number of archive sets, not the individual archives
                    }
                }
            }
        }

        //Temporary until more data is available
        $report_total = $report_count;

        $report['count'] = $report_count;
        $report['total'] = $report_total;
        $report['start'] = $report_start;
        $report['data'] = $report_data;
        return $report;
    }

} 