<?php

$dir = '/api/includes/reports';

class ProtectionSummaryReport
{
    private $BP;

    public function __construct( $BP, $functionsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
    }

    public function get_protection_summary_report($systems, $data)
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        $report_data['clients'] = array();
        $report_data['clients']['protected'] = array();
        $report_data['clients']['unprotected'] = array();

        $report_data['vms'] = array();
        $report_data['vms']['protected'] = array();
        $report_data['vms']['unprotected'] = array();

        $report_data['backups'] = array();
        $report_data['backups']['successes'] = 0;
        $report_data['backups']['warnings'] = 0;
        $report_data['backups']['failures'] = 0;
        $report_data['backups']['clients'] = array();
        $report_data['backups']['vms'] = array();

        $report_data['replicated_backups'] = array();
        $report_data['replicated_backups']['successes'] = 0;
        $report_data['replicated_backups']['warnings'] = 0;
        $report_data['replicated_backups']['failures'] = 0;
        $report_data['replicated_backups']['clients'] = array();
        $report_data['replicated_backups']['vms'] = array();

        $report_data['archives'] = array();
        $report_data['archives']['successes'] = 0;
        $report_data['archives']['warnings'] = 0;
        $report_data['archives']['failures'] = 0;
        $report_data['archives']['clients'] = array();
        $report_data['archives']['vms'] = array();

        $start_time = false;
        if ( array_key_exists('start_date', $data) )
        {
            $start_time = strtotime($data['start_date']);
        }
        $end_time = false;
        if ( array_key_exists('end_date', $data) )
        {
            $end_time = $this->functions->formatEndDate($data['end_date']);
        }

        foreach ($systems as $system_id => $system_name)
        {
            $possibly_protected_clients_by_file_level_instance_id = array();
            $possibly_protected_vms_by_instance_id = array();

            // If a $possibly_replicating_clients_by_client_id[] is true, then a replicated backup has been found for it.
            // If a $possibly_replicating_clients_by_client_id[] is false, then the client is setup to replicate, but a replicated backup has not been found for it yet.
            // If a $possibly_replicating_clients_by_client_id[] is null, then the client is not setup to replicate
            $possibly_replicating_clients_by_client_id = array();
            // Each $possibly_replicating_vms_by_client_id_then_instance_id[][] behaves the same as $possibly_replicating_clients_by_client_id[]
            $possibly_replicating_vms_by_client_id_then_instance_id = array();

            $client_file_level_instance_id_by_client_id = array();

            $backup_summary_filter_array = array();
            $backup_summary_filter_array['sync_types'] = false;
            $backup_summary_filter_array['grandclients'] = false;
            $backup_summary_filter_array['sys_metadata'] = false;
            if ( $start_time !== false )
            {
                if ( $end_time !== false )
                {
                    $backup_summary_filter_array['intervals'] = array( array( 'start_time' => $start_time, 'end_time' => $end_time ) );
                }
                else
                {
                    // If only a start_time is given, give the current time as the end
                    $backup_summary_filter_array['intervals'] = array( array( 'start_time' => $start_time, 'end_time' => time() ) );
                }
            }
            elseif ( $end_time !== false )
            {
                // If only an end_time is given, give 0 Unix time as the beginning, this will get everything up to that point
                $backup_summary_filter_array['intervals'] = array( array( 'start_time' => 0, 'end_time' => $end_time ) );
            }
            $backup_summary_by_time = $this->BP->get_backup_summary( $backup_summary_filter_array, $system_id );

            if ( $backup_summary_by_time !== false and count($backup_summary_by_time) > 0 )
            {
                foreach ( $backup_summary_by_time as $backup_summary_by_clients )
                {
                    foreach ( $backup_summary_by_clients as $backup_summary_by_client )
                    {
                        $client_info = $this->BP->get_client_info($backup_summary_by_client['client_id'], $system_id);

                        if ( $backup_summary_by_client['client_id'] !== 1 or $client_info['name'] !== $system_name )
                        {

                            $report_data['backups']['successes'] += $backup_summary_by_client['successes'];
                            $report_data['backups']['warnings'] += ( array_key_exists('warnings', $backup_summary_by_client) ? $backup_summary_by_client['warnings'] : 0 );
                            $report_data['backups']['failures'] += $backup_summary_by_client['failures'];

                            // If a $possibly_replicating_clients_by_client_id[] is false, then the client is setup to replicate, but a replicated backup has not been found for it yet.
                            // If a $possibly_replicating_clients_by_client_id[] is null, then the client is not setup to replicate
                            $possibly_replicating_clients_by_client_id[$backup_summary_by_client['client_id']] = ($client_info['is_synchable'] === true ? false : null);

                            $possibly_replicating_vms_by_client_id_then_instance_id[$backup_summary_by_client['client_id']] = false;

                            // If there are no successful backups then check to see if warnings are returned and there are any warnings
                            // If there are no successes or warnings, then add the client to the unprotected array
                            if ( $backup_summary_by_client['successes'] === 0
                                and ( !(array_key_exists('warnings', $backup_summary_by_client))
                                    or $backup_summary_by_client['warnings'] === 0) )
                            {
                                if ( $client_info['name'] !== Constants::CLIENT_NAME_VCENTER_RRC )
                                {
                                    $report_data['clients']['unprotected'][] = array( 'system_name' => $system_name, 'client_name' => $client_info['name'],
                                        'instance_name' => '', 'database_name' => '' );
                                }

                                // Put the client name here since we don't have a file_level_instance_id and we may not need one
                                $client_file_level_instance_id_by_client_id[$backup_summary_by_client['client_id']] = $client_info['name'];

                                foreach ($client_info['applications'] as $applicationID => $applicationInfo)
                                {
                                    switch ($applicationInfo['type'])
                                    {
                                        case Constants::APPLICATION_TYPE_NAME_VMWARE:
                                            foreach ( $applicationInfo['servers'] as $esxServerUUID => $esxServerName )
                                            {
                                                $vmList = $this->BP->get_vm_info($esxServerUUID, NULL, true, false, $system_id);
                                                if ( $vmList !== false and count($vmList) > 0 )
                                                {
                                                    $possibly_replicating_vms_by_client_id_then_instance_id[$backup_summary_by_client['client_id']] = array();
                                                    foreach ($vmList as $vm)
                                                    {
                                                        $report_data['vms']['unprotected'][] = array( 'system_name' => $system_name, 'client_name' => $esxServerName,
                                                            'instance_name' => $vm['name'], 'database_name' => '' );
                                                        // If a $possibly_replicating_vms_by_client_id_then_instance_id[][] is false, then the vm is setup to replicate, but a replicated backup has not been found for it yet.
                                                        // If a $possibly_replicating_vms_by_client_id_then_instance_id[][] is null, then the vm is not setup to replicate
                                                        $possibly_replicating_vms_by_client_id_then_instance_id[$backup_summary_by_client['client_id']][$vm['instance_id']] =
                                                            ($vm['is_synchable'] === true ? false : null);
                                                    }
                                                }
                                            }
                                            break;
                                        case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                                            $hyperVList = $this->BP->get_hyperv_info($backup_summary_by_client['client_id'], true, $system_id);
                                            if ( $hyperVList !== false and count($hyperVList) > 0 )
                                            {
                                                $possibly_replicating_vms_by_client_id_then_instance_id[$backup_summary_by_client['client_id']] = array();
                                                foreach ($hyperVList as $hyperVVM)
                                                {
                                                    $report_data['vms']['unprotected'][] = array('system_name' => $system_name, 'client_name' => $client_info['name'],
                                                        'instance_name' => $hyperVVM['name'], 'database_name' => '');
                                                    // If a $possibly_replicating_vms_by_client_id_then_instance_id[][] is false, then the vm is setup to replicate, but a replicated backup has not been found for it yet.
                                                    // If a $possibly_replicating_vms_by_client_id_then_instance_id[][] is null, then the vm is not setup to replicate
                                                    $possibly_replicating_vms_by_client_id_then_instance_id[$backup_summary_by_client['client_id']][$hyperVVM['instance_id']] =
                                                        ($hyperVVM['is_synchable'] === true ? false : null);
                                                }
                                            }
                                            break;
                                    }
                                }
                            }
                            else // Some part of the client is protected
                            {
                                if ( $client_info['name'] !== Constants::CLIENT_NAME_VCENTER_RRC )
                                {
                                    $file_level_instance_id = $this->BP->get_file_level_info($backup_summary_by_client['client_id'], $system_id);
                                    $possibly_protected_clients_by_file_level_instance_id[$file_level_instance_id] = array(
                                        'system_name' => $system_name,
                                        'client_name' => $client_info['name'],
                                        'instance_name' => '',
                                        'database_name' => '' );
                                    $client_file_level_instance_id_by_client_id[$backup_summary_by_client['client_id']] = $file_level_instance_id;
                                }
                                else
                                {
                                    // Put the client name here for the vCenter-RRC so that it doesn't cause errors later
                                    $client_file_level_instance_id_by_client_id[$backup_summary_by_client['client_id']] = $client_info['name'];
                                }

                                foreach ($client_info['applications'] as $applicationID => $applicationInfo)
                                {
                                    switch ($applicationInfo['type'])
                                    {
                                        case Constants::APPLICATION_TYPE_NAME_VMWARE:
                                            foreach ( $applicationInfo['servers'] as $esxServerUUID => $esxServerName )
                                            {
                                                $vmList = $this->BP->get_vm_info($esxServerUUID, NULL, true, false, $system_id);
                                                if ( $vmList !== false and count($vmList) > 0 )
                                                {
                                                    $possibly_replicating_vms_by_client_id_then_instance_id[$backup_summary_by_client['client_id']] = array();
                                                    foreach ($vmList as $vm)
                                                    {
                                                        $possibly_protected_vms_by_instance_id[$vm['instance_id']] = array(
                                                            'system_name' => $system_name,
                                                            'client_name' => $esxServerName,
                                                            'instance_name' => $vm['name'],
                                                            'database_name' => '' );
                                                        // If a $possibly_replicating_vms_by_client_id_then_instance_id[][] is false, then the vm is setup to replicate, but a replicated backup has not been found for it yet.
                                                        // If a $possibly_replicating_vms_by_client_id_then_instance_id[][] is null, then the vm is not setup to replicate
                                                        $possibly_replicating_vms_by_client_id_then_instance_id[$backup_summary_by_client['client_id']][$vm['instance_id']] =
                                                            ($vm['is_synchable'] === true ? false : null);
                                                    }
                                                }
                                            }
                                            break;
                                        case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                                            $hyperVList = $this->BP->get_hyperv_info($backup_summary_by_client['client_id'], true, $system_id);
                                            if ( $hyperVList !== false and count($hyperVList) > 0 )
                                            {
                                                $possibly_replicating_vms_by_client_id_then_instance_id[$backup_summary_by_client['client_id']] = array();
                                                foreach ($hyperVList as $hyperVVM)
                                                {
                                                    $possibly_protected_vms_by_instance_id[$hyperVVM['instance_id']] = array(
                                                        'system_name' => $system_name,
                                                        'client_name' => $client_info['name'],
                                                        'instance_name' => $hyperVVM['name'],
                                                        'database_name' => '');
                                                    // If a $possibly_replicating_vms_by_client_id_then_instance_id[][] is false, then the vm is setup to replicate, but a replicated backup has not been found for it yet.
                                                    // If a $possibly_replicating_vms_by_client_id_then_instance_id[][] is null, then the vm is not setup to replicate
                                                    $possibly_replicating_vms_by_client_id_then_instance_id[$backup_summary_by_client['client_id']][$hyperVVM['instance_id']] =
                                                        ($hyperVVM['is_synchable'] === true ? false : null);
                                                }
                                            }
                                            break;
                                    }
                                }
                            }
                        }
                    }
                }
            }

            $clientLastBackups = $this->BP->get_last_backup_per_instance( array_keys($possibly_protected_clients_by_file_level_instance_id), $system_id );
            $clientInstanceIDsByBackupID = array();
            if ( $clientLastBackups !== false and count($clientLastBackups) > 0 )
            {
                foreach ($clientLastBackups as $clientLastBackupAndInstanceArray)
                {
                    $clientInstanceIDsByBackupID[$clientLastBackupAndInstanceArray['backup_id']] = $clientLastBackupAndInstanceArray['instance_id'];
                }
            }

            $backup_status_filter_array = array( 'system_id' => $system_id, 'backup_ids' => array_keys($clientInstanceIDsByBackupID) );
            $client_backup_status_result_array = $this->BP->get_backup_status( $backup_status_filter_array );

            if ( $client_backup_status_result_array !== false and count($client_backup_status_result_array) > 0 )
            {
                foreach ( $client_backup_status_result_array as $latest_backup_status )
                {
                    $backup_status = null;

                    //  If the backup returned from get_last_backup_per_instance is outside of the time frame given, then search for one within the time frame
                    // This is optimized to only search if the end_time is out of date, because is the start_time is wrong and it is the latest backup, then no later backup should exist.
                    if ( ( $end_time !== false and $latest_backup_status['start_time'] > $end_time ) )
                    {
                        $single_client_backup_status_filter_array = array();
                        $single_client_backup_status_filter_array['system_id'] = $system_id;
                        $single_client_backup_status_filter_array['client_id'] = $latest_backup_status['client_id'];
                        $single_client_backup_status_filter_array['type'] = array(Constants::BACKUP_TYPE_MASTER,
                            Constants::BACKUP_TYPE_DIFFERENTIAL,
                            Constants::BACKUP_TYPE_INCREMENTAL,
                            Constants::BACKUP_TYPE_BAREMETAL,
                            Constants::BACKUP_TYPE_SELECTIVE );
                        if ( $start_time !== false )
                        {
                            $single_client_backup_status_filter_array['start_time'] = $start_time;
                        }
                        if ( $end_time !== false )
                        {
                            $single_client_backup_status_filter_array['end_time'] = $end_time;
                        }
                        $single_client_backup_status_result_array = $this->BP->get_backup_status( $single_client_backup_status_filter_array );

                        if ( $single_client_backup_status_result_array !== false and count($single_client_backup_status_result_array) > 0 )
                        {
                            $temp_backup_status = array( 'start_time' => 0 );
                            foreach ( $single_client_backup_status_result_array as $single_client_backup_status )
                            {
                                if ( $temp_backup_status['start_time'] < $single_client_backup_status['start_time'] )
                                {
                                    $temp_backup_status = $single_client_backup_status;
                                }
                            }
                            $backup_status = $temp_backup_status;
                        }
                    }
                    // This is optimized to only search if the end_time is out of date, because is the start_time is wrong and it is the latest backup, then no later backup should exist.
                    // If the start_time is not wrong, then this is a valid backup and can be used
                    elseif ( !($start_time !== false and $latest_backup_status['start_time'] < $start_time ) )
                    {
                        $backup_status = $latest_backup_status;
                    }

                    if ( $backup_status !== null )
                    {
                        $last_backup_time = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $backup_status['start_time']);
                        $client_instance_id = $clientInstanceIDsByBackupID[$backup_status['id']];
                        if ( $this->functions->isBackupTypeSecuresync($backup_status['type']) )
                        {
                            //replicated backup
                            $report_data['replicated_backups']['clients'][] = array(
                                'system_name' => $possibly_protected_clients_by_file_level_instance_id[$client_instance_id]['system_name'],
                                'client_name' => $possibly_protected_clients_by_file_level_instance_id[$client_instance_id]['client_name'],
                                'instance_name' => '',
                                'database_name' => '',
                                'last_replication' => $last_backup_time,
                                'target_name' => (array_key_exists('destination', $backup_status) ? $backup_status['destination'] : '') );
                            $possibly_protected_clients_by_file_level_instance_id[$client_instance_id]['last_replication'] = $last_backup_time;
                            $possibly_replicating_clients_by_client_id[$backup_status['id']] = true;
                        }
                        else
                        {
                            //regular backup
                            $report_data['backups']['clients'][] = array(
                                'system_name' => $possibly_protected_clients_by_file_level_instance_id[$client_instance_id]['system_name'],
                                'client_name' => $possibly_protected_clients_by_file_level_instance_id[$client_instance_id]['client_name'],
                                'instance_name' => '',
                                'database_name' => '',
                                'last_backup' => $last_backup_time,
                                'types' => array( $this->functions->getBackupTypeDisplayName($backup_status['type']) ) );  // In future outs, have more than one type here
                            $possibly_protected_clients_by_file_level_instance_id[$client_instance_id]['last_backup'] = $last_backup_time;
                        }
                    }
                }
            }

            $vmLastBackups = $this->BP->get_last_backup_per_instance( array_keys($possibly_protected_vms_by_instance_id), $system_id );
            $vmInstanceIDsByBackupID = array();
            if ( $vmLastBackups !== false and count($vmLastBackups) > 0 )
            {
                foreach ($vmLastBackups as $vmLastBackupAndInstanceArray)
                {
                    $vmInstanceIDsByBackupID[$vmLastBackupAndInstanceArray['backup_id']] = $vmLastBackupAndInstanceArray['instance_id'];
                }
            }

            $backup_status_filter_array['backup_ids'] = array_keys($vmInstanceIDsByBackupID);
            $vm_backup_status_result_array = $this->BP->get_backup_status( $backup_status_filter_array );

            if ( $vm_backup_status_result_array !== false and count($vm_backup_status_result_array) > 0 )
            {
                $backup_statuses_by_client_then_backups = array();
                foreach ( $vm_backup_status_result_array as $latest_backup_status )
                {
                    $vm_instance_id = $vmInstanceIDsByBackupID[$latest_backup_status['id']];
                    $backup_status = null;

                    //  If the backup returned from get_last_backup_per_instance is outside of the time frame given, then search for one within the time frame
                    // This is optimized to only search if the end_time is out of date, because is the start_time is wrong and it is the latest backup, then no later backup should exist.
                    if ( ( $end_time !== false and $latest_backup_status['start_time'] > $end_time ) )
                    {
                        if ( array_key_exists($latest_backup_status['client_id'], $backup_statuses_by_client_then_backups) )
                        {
                            $single_client_backup_status_result_array = $backup_statuses_by_client_then_backups[ $latest_backup_status['client_id'] ];
                        }
                        else
                        {
                            $single_client_backup_status_filter_array = array();
                            $single_client_backup_status_filter_array['system_id'] = $system_id;
                            $single_client_backup_status_filter_array['client_id'] = $latest_backup_status['client_id'];
                            $single_client_backup_status_filter_array['type'] = array(Constants::BACKUP_TYPE_VMWARE_DIFFERENTIAL,
                                Constants::BACKUP_TYPE_VMWARE_FULL,
                                Constants::BACKUP_TYPE_VMWARE_INCREMENTAL,
                                Constants::BACKUP_TYPE_HYPER_V_DIFFERENTIAL,
                                Constants::BACKUP_TYPE_HYPER_V_FULL,
                                Constants::BACKUP_TYPE_HYPER_V_INCREMENTAL, );
                            if ( $start_time !== false )
                            {
                                $single_client_backup_status_filter_array['start_time'] = $start_time;
                            }
                            if ( $end_time !== false )
                            {
                                $single_client_backup_status_filter_array['end_time'] = $end_time;
                            }
                            $single_client_backup_status_result_array = $this->BP->get_backup_status( $single_client_backup_status_filter_array );
                            $backup_statuses_by_client_then_backups[ $latest_backup_status['client_id'] ] = $single_client_backup_status_result_array;
                        }

                        if ( $single_client_backup_status_result_array !== false and count($single_client_backup_status_result_array) > 0 )
                        {
                            $temp_backup_status = array( 'start_time' => 0 );
                            foreach ( $single_client_backup_status_result_array as $single_client_backup_status )
                            {
                                if ( $vm_instance_id === $single_client_backup_status['instance_id'] and $temp_backup_status['start_time'] < $single_client_backup_status['start_time'] )
                                {
                                    $temp_backup_status = $single_client_backup_status;
                                }
                            }
                            $backup_status = $temp_backup_status;
                        }
                    }
                    // This is optimized to only search if the end_time is out of date, because is the start_time is wrong and it is the latest backup, then no later backup should exist.
                    // If the start_time is not wrong, then this is a valid backup and can be used
                    elseif ( !($start_time !== false and $latest_backup_status['start_time'] < $start_time ) )
                    {
                        $backup_status = $latest_backup_status;
                    }

                    if ( $backup_status !== null )
                    {
                        $last_backup_time = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $backup_status['start_time']);
                        if ( $this->functions->isBackupTypeSecuresync($backup_status['type']) )
                        {
                            //replicated backup
                            $report_data['replicated_backups']['vms'][] = array(
                                'system_name' => $possibly_protected_vms_by_instance_id[$vm_instance_id]['system_name'],
                                'client_name' => $possibly_protected_vms_by_instance_id[$vm_instance_id]['client_name'],
                                'instance_name' => $possibly_protected_vms_by_instance_id[$vm_instance_id]['instance_name'],
                                'database_name' => $possibly_protected_vms_by_instance_id[$vm_instance_id]['database_name'],
                                'last_replication' => $last_backup_time,
                                'target_name' => (array_key_exists('destination', $backup_status) ? $backup_status['destination'] : '') );
                            $possibly_protected_vms_by_instance_id[$vm_instance_id]['last_replication'] = $last_backup_time;
                            $possibly_replicating_vms_by_client_id_then_instance_id[$backup_status['client_id']][$vm_instance_id] = true;
                        }
                        else
                        {
                            //regular backup
                            $report_data['backups']['vms'][] = array(
                                'system_name' => $possibly_protected_vms_by_instance_id[$vm_instance_id]['system_name'],
                                'client_name' => $possibly_protected_vms_by_instance_id[$vm_instance_id]['client_name'],
                                'instance_name' => $possibly_protected_vms_by_instance_id[$vm_instance_id]['instance_name'],
                                'database_name' => $possibly_protected_vms_by_instance_id[$vm_instance_id]['database_name'],
                                'last_backup' => $last_backup_time,
                                'types' => array( $this->functions->getBackupTypeDisplayName($backup_status['type']) ) );  // In future outs, have more than one type here
                            $possibly_protected_vms_by_instance_id[$vm_instance_id]['last_backup'] = $last_backup_time;
                        }
                    }
                }
            }

            // Unset the $backup_status_filter_array['backup_ids'] so that the filter can be reused
            unset($backup_status_filter_array['backup_ids']);
            // Above, we wanted to know if a particular backup wasn't in the time range, now we don't want those results.
            if ( $start_time !== false )
            {
                $backup_status_filter_array['start_time'] = $start_time;
            }
            if ( $end_time !== false )
            {
                $backup_status_filter_array['end_time'] = $end_time;
            }

            $backup_summary_filter_array['sync_types'] = true;
            $replicated_backup_summary_by_time = $this->BP->get_backup_summary( $backup_summary_filter_array, $system_id );

            if ( $replicated_backup_summary_by_time !== false and count($replicated_backup_summary_by_time) > 0 )
            {
                foreach ( $replicated_backup_summary_by_time as $replicated_backup_summary_by_clients )
                {
                    foreach ( $replicated_backup_summary_by_clients as $replicated_backup_summary_by_client )
                    {
                        $report_data['replicated_backups']['successes'] += $replicated_backup_summary_by_client['successes'];
                        $report_data['replicated_backups']['warnings'] += ( array_key_exists('warnings', $replicated_backup_summary_by_client) ? $replicated_backup_summary_by_client['warnings'] : 0 );
                        $report_data['replicated_backups']['failures'] += $replicated_backup_summary_by_client['failures'];

                        // Check to see if there are successful replicated backups for this client
                        // This is to fill in the case that there are replicated backups that aren't for the most recent backup
                        if ( $replicated_backup_summary_by_client['successes'] > 0
                            or ( (array_key_exists('warnings', $replicated_backup_summary_by_client))
                                and $replicated_backup_summary_by_client['warnings'] > 0) )
                        {
                            // If the client has successful replications and we haven't found one yet, then search for one
                            // If it only has applications with replicated backups, then this could be a false positive
                            // If in the future we want only to search for clients that are currently replicating, then change !== true to === false
                            if ( $possibly_replicating_clients_by_client_id[$replicated_backup_summary_by_client['client_id']] !== true )
                            {
                                $backup_status_filter_array['client_id'] = $replicated_backup_summary_by_client['client_id'];
                                $backup_status_filter_array['type'] = array(Constants::BACKUP_TYPE_SECURESYNC_MASTER,
                                    Constants::BACKUP_TYPE_SECURESYNC_DIFFERENTIAL,
                                    Constants::BACKUP_TYPE_SECURESYNC_INCREMENTAL,
                                    Constants::BACKUP_TYPE_SECURESYNC_BAREMETAL );
                                $replicated_file_level_backups = $this->BP->get_backup_status( $backup_status_filter_array );
                                if ( $replicated_file_level_backups !== false and count($replicated_file_level_backups) > 0 )
                                {
                                    $last_replication_time = 0;
                                    $last_replication_target = '';
                                    foreach ( $replicated_file_level_backups as $replicated_file_level_backup )
                                    {
                                        if ( $replicated_file_level_backup['start_time'] > $last_replication_time )
                                        {
                                            $last_replication_time = $replicated_file_level_backup['start_time'];
                                            $last_replication_target = (array_key_exists('destination', $replicated_file_level_backup) ? $replicated_file_level_backup['destination'] : '');
                                        }
                                    }
                                    if ( $last_replication_time !== 0 )
                                    {
                                        $last_replication_time = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $last_replication_time);
                                        $temp_file_level_instance_id = $client_file_level_instance_id_by_client_id[$replicated_backup_summary_by_client['client_id']];
                                        if ( is_string($temp_file_level_instance_id) )
                                        {
                                            $temp_client_name = $temp_file_level_instance_id;
                                            $temp_file_level_instance_id = $this->BP->get_file_level_info($replicated_backup_summary_by_client['client_id'], $system_id);
                                            $client_file_level_instance_id_by_client_id[$replicated_backup_summary_by_client['client_id']] = $temp_file_level_instance_id;
                                            $possibly_protected_clients_by_file_level_instance_id[ $temp_file_level_instance_id ] = array(
                                                'system_name' => $system_name,
                                                'client_name' => $temp_client_name,
                                                'instance_name' => '',
                                                'database_name' => '' );
                                        }
                                        //replicated backup
                                        $report_data['replicated_backups']['clients'][] = array(
                                            'system_name' => $possibly_protected_clients_by_file_level_instance_id[$temp_file_level_instance_id]['system_name'],
                                            'client_name' => $possibly_protected_clients_by_file_level_instance_id[$temp_file_level_instance_id]['client_name'],
                                            'instance_name' => '',
                                            'database_name' => '',
                                            'last_replication' => $last_replication_time,
                                            'target_name' => $last_replication_target );
                                        $possibly_protected_clients_by_file_level_instance_id[$temp_file_level_instance_id]['last_replication'] = $last_replication_time;
                                        $possibly_replicating_clients_by_client_id[$replicated_backup_summary_by_client['client_id']] = true;
                                    }
                                }
                            }

                            // In the future, it might be nice to put a check here to see if the $possibly_replicating_vms_by_instance_id has any that might apply for this client
                            // If the client has successful replications and we haven't found one yet, then search for one
                            // If it only has applications with replicated backups, then this could be a false positive
                            // If in the future we want only to search for clients that are currently replicating, then change !== true to === false
                            if ( $possibly_replicating_vms_by_client_id_then_instance_id[$replicated_backup_summary_by_client['client_id']] !== false )
                            {
                                $client_has_vms_missing_replications = false;
                                $last_replication_time_per_instance = array();
                                foreach ( $possibly_replicating_vms_by_client_id_then_instance_id[$replicated_backup_summary_by_client['client_id']]
                                          as $possibly_replicating_vm_instance_id => $possibly_replicating_vm_status )
                                {
                                    // Checks to see if a VM is setup to replicate, but we haven't found a replication for it yet
                                    // For VMs, don't worry about the case that it used to be setup to replicate and no longer is setup to replicate
                                    if ( $possibly_replicating_vm_status === false )
                                    {
                                        $client_has_vms_missing_replications = true;
                                    }
                                    // Add all instances to make sure that an entry exists to prevent errors later
                                    $last_replication_time_per_instance[$possibly_replicating_vm_instance_id] = 0;
                                }
                                if ( $client_has_vms_missing_replications === true )
                                {
                                    $backup_status_filter_array['client_id'] = $replicated_backup_summary_by_client['client_id'];
                                    $backup_status_filter_array['type'] = array( Constants::BACKUP_TYPE_SECURESYNC_VMWARE_FULL,
                                        Constants::BACKUP_TYPE_SECURESYNC_VMWARE_DIFFERENTIAL,
                                        Constants::BACKUP_TYPE_SECURESYNC_VMWARE_INCREMENTAL,
                                        Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_FULL,
                                        Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_INCREMENTAL,
                                        Constants::BACKUP_TYPE_SECURESYNC_HYPER_V_DIFFERENTIAL );
                                    $replicated_vm_backups = $this->BP->get_backup_status( $backup_status_filter_array );
                                    if ( $replicated_vm_backups !== false and count($replicated_vm_backups) > 0 )
                                    {
                                        $last_replication_time_per_instance = array();
                                        $last_replication_target_per_instance = array();
                                        foreach ( $replicated_vm_backups as $replicated_vm_backup )
                                        {
                                            // Check to see if we already have the latest replication, if not then process this backup
                                            if ( $possibly_replicating_vms_by_client_id_then_instance_id[$replicated_backup_summary_by_client['client_id']][$replicated_vm_backup['instance_id']] !== true )
                                            {
                                                if ( $replicated_vm_backup['start_time'] > $last_replication_time_per_instance[$replicated_vm_backup['instance_id']] )
                                                {
                                                    $last_replication_time_per_instance[$replicated_vm_backup['instance_id']] = $replicated_vm_backup['start_time'];
                                                    $last_replication_target_per_instance[$replicated_vm_backup['instance_id']] =
                                                        (array_key_exists('destination', $replicated_vm_backup) ? $replicated_vm_backup['destination'] : '');
                                                }
                                            }
                                        }
                                        if ( count($last_replication_target_per_instance) > 0 )
                                        {
                                            foreach ( $last_replication_target_per_instance as $replicating_vm_instance_id => $last_replication_target )
                                            {
                                                $last_replication_time = date(Constants::DATE_FORMAT_DATE_THEN_12HR_TIME, $last_replication_time_per_instance[$replicating_vm_instance_id]);
                                                // If regular backups weren't found for this vm but a replicated backup was, then add it to protected vms
                                                if ( !array_key_exists($replicating_vm_instance_id, $possibly_protected_vms_by_instance_id) )
                                                {
                                                    $appinst_info_array = $this->BP->get_appinst_info( array( $replicating_vm_instance_id ), $system_id );
                                                    if ( $appinst_info_array !== false and count($appinst_info_array) > 0 )
                                                    {
                                                        foreach ( $appinst_info_array as $appinst_info )
                                                        {
                                                            if ( $appinst_info['app_type'] === Constants::APPLICATION_TYPE_NAME_HYPER_V)
                                                            {
                                                                $possibly_protected_vms_by_instance_id[$replicating_vm_instance_id] = array(
                                                                    'system_name' => $system_name,
                                                                    'client_name' => $appinst_info['client_name'],
                                                                    'instance_name' => $appinst_info['primary_name'],
                                                                    'database_name' => '');
                                                            }
                                                            elseif ( $appinst_info['app_type'] === Constants::APPLICATION_TYPE_NAME_VMWARE)
                                                            {
                                                                $possibly_protected_vms_by_instance_id[$replicating_vm_instance_id] = array(
                                                                    'system_name' => $system_name,
                                                                    'client_name' => $appinst_info['primary_name'],
                                                                    'instance_name' => $appinst_info['secondary_name'],
                                                                    'database_name' => '');
                                                            }
                                                        }
                                                    }
                                                }
                                                $report_data['replicated_backups']['vms'][] = array(
                                                    'system_name' => $possibly_protected_vms_by_instance_id[$replicating_vm_instance_id]['system_name'],
                                                    'client_name' => $possibly_protected_vms_by_instance_id[$replicating_vm_instance_id]['client_name'],
                                                    'instance_name' => $possibly_protected_vms_by_instance_id[$replicating_vm_instance_id]['instance_name'],
                                                    'database_name' => $possibly_protected_vms_by_instance_id[$replicating_vm_instance_id]['database_name'],
                                                    'last_replication' => $last_replication_time,
                                                    'target_name' => $last_replication_target );
                                                $possibly_protected_vms_by_instance_id[$replicating_vm_instance_id]['last_replication'] = $last_replication_time;
                                                $possibly_replicating_vms_by_client_id_then_instance_id[$replicated_backup_summary_by_client['client_id']][$replicating_vm_instance_id] = true;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            $archive_sets_filter = array();
            if ( $start_time !== false )
            {
                $archive_sets_filter['job_start_time'] = $start_time;
            }
            if ( $end_time !== false )
            {
                $archive_sets_filter['job_end_time'] = $end_time;
            }
            $archive_sets_list = $this->BP->get_archive_sets($archive_sets_filter, $system_id);
            if ( $archive_sets_list !== false and count($archive_sets_list) > 0 )
            {
                foreach ( $archive_sets_list as $archive_set )
                {
                    switch ($archive_set['status'])
                    {
                        case Constants::ARCHIVE_STATUS_ARCHIVE_IN_PROGRESS:
                        case Constants::ARCHIVE_STATUS_IMPORT_IN_PROGRESS:
                            $report_data['archives']['warnings'] += 1;
                            break;
                        case Constants::ARCHIVE_STATUS_ARCHIVE_FAILED:
                        case Constants::ARCHIVE_STATUS_IMPORT_FAILED:
                            $report_data['archives']['failures'] += 1;
                            break;
                        case Constants::ARCHIVE_STATUS_ARCHIVE_SUCCESS:
                        case Constants::ARCHIVE_STATUS_IMPORT_SUCCESS:
                        default:
                            $report_data['archives']['successes'] += 1;
                            break;
                    }
                }
            }

            $archive_schedule_list = $this->BP->get_archive_schedule_list($system_id);
            if ( $archive_schedule_list !== false and count($archive_schedule_list) > 0 )
            {
                foreach ( $archive_schedule_list as $archive_schedule )
                {
                    if ( $archive_schedule['enabled'] === 1 )
                    {
                        $archive_schedule_info = $this->BP->get_archive_schedule_info($archive_schedule['id'], $system_id);
                        if ( $archive_schedule_info !== false and count($archive_schedule_info['profile']) > 0 )
                        {
                            $appinst_info_array = $this->BP->get_appinst_info( implode( ', ', $archive_schedule_info['profile']['instances']), $system_id);
                            if ( $appinst_info_array !== false and count($appinst_info_array) > 0 )
                            {
                                $archive_schedule_types = array();
                                foreach ( $archive_schedule_info['profile']['types'] as $backup_type)
                                {
                                    // To avoid duplicates, put the type in the key instead of the value
                                    $archive_schedule_types[$this->functions->getBackupTypeDisplayName($backup_type)] = true;
                                }
                                foreach ( $appinst_info_array as $app_instance_id => $appinst_info )
                                {
                                    switch ( $appinst_info['app_type'] )
                                    {
                                        case Constants::APPLICATION_TYPE_NAME_FILE_LEVEL:
                                            if ( is_string($client_file_level_instance_id_by_client_id[$appinst_info['client_id']]) )
                                            {
                                                $client_file_level_instance_id_by_client_id[$appinst_info['client_id']] = $app_instance_id;
                                                $possibly_protected_clients_by_file_level_instance_id[ $app_instance_id ] = array(
                                                    'system_name' => $system_name,
                                                    'client_name' => $appinst_info['client_name'],
                                                    'instance_name' => '',
                                                    'database_name' => '' );
                                            }
                                            //replicated backup
                                            $report_data['archives']['clients'][] = array(
                                                'system_name' => $possibly_protected_clients_by_file_level_instance_id[$app_instance_id]['system_name'],
                                                'client_name' => $possibly_protected_clients_by_file_level_instance_id[$app_instance_id]['client_name'],
                                                'instance_name' => '',
                                                'database_name' => '',
                                                'next_archive_job' => $archive_schedule['name'],
                                                'types' => $archive_schedule_types,
                                                'media' => $archive_schedule_info['profile']['target'] );
                                            $possibly_protected_clients_by_file_level_instance_id[$app_instance_id]['next_archive_job'] = $archive_schedule['name'];
                                            break;
                                        case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                                            if ( !array_key_exists($app_instance_id, $possibly_protected_vms_by_instance_id) )
                                            {
                                                $possibly_protected_vms_by_instance_id[ $app_instance_id ] = array(
                                                    'system_name' => $system_name,
                                                    'client_name' => $appinst_info['client_name'],
                                                    'instance_name' => $appinst_info['primary_name'],
                                                    'database_name' => '' );
                                            }
                                            //replicated backup
                                            $report_data['archives']['vms'][] = array(
                                                'system_name' => $possibly_protected_vms_by_instance_id[$app_instance_id]['system_name'],
                                                'client_name' => $possibly_protected_vms_by_instance_id[$app_instance_id]['client_name'],
                                                'instance_name' => $possibly_protected_vms_by_instance_id[$app_instance_id]['instance_name'],
                                                'database_name' => '',
                                                'next_archive_job' => $archive_schedule['name'],
                                                'types' => $archive_schedule_types,
                                                'media' => $archive_schedule_info['profile']['target'] );
                                            $possibly_protected_vms_by_instance_id[$app_instance_id]['next_archive_job'] = $archive_schedule['name'];
                                            break;
                                        case Constants::APPLICATION_TYPE_NAME_VMWARE:
                                            if ( !array_key_exists($app_instance_id, $possibly_protected_vms_by_instance_id) )
                                            {
                                                $possibly_protected_vms_by_instance_id[ $app_instance_id ] = array(
                                                    'system_name' => $system_name,
                                                    'client_name' => $appinst_info['primary_name'],
                                                    'instance_name' => $appinst_info['secondary_name'],
                                                    'database_name' => '' );
                                            }
                                            //replicated backup
                                            $report_data['archives']['vms'][] = array(
                                                'system_name' => $possibly_protected_vms_by_instance_id[$app_instance_id]['system_name'],
                                                'client_name' => $possibly_protected_vms_by_instance_id[$app_instance_id]['client_name'],
                                                'instance_name' => $possibly_protected_vms_by_instance_id[$app_instance_id]['instance_name'],
                                                'database_name' => '',
                                                'next_archive_job' => $archive_schedule['name'],
                                                'types' => $archive_schedule_types,
                                                'media' => $archive_schedule_info['profile']['target'] );
                                            $possibly_protected_vms_by_instance_id[$app_instance_id]['next_archive_job'] = $archive_schedule['name'];
                                            break;
                                    }
                                }
                            }
                        }
                    }
                }
            }

            foreach ( $possibly_protected_clients_by_file_level_instance_id as $possibly_protected_client )
            {
                if ( array_key_exists('last_backup' , $possibly_protected_client ) )
                {
                    if ( !array_key_exists('last_replication' , $possibly_protected_client ) )
                    {
                        $possibly_protected_client['last_replication'] = '';
                    }
                    if ( !array_key_exists('next_archive_job' , $possibly_protected_client ) )
                    {
                        $possibly_protected_client['next_archive_job'] = '';
                    }
                    $report_data['clients']['protected'][] = $possibly_protected_client;
                }
                elseif ( array_key_exists('last_replication' , $possibly_protected_client ) )
                {
                    $possibly_protected_client['last_backup'] = '';
                    if ( !array_key_exists('next_archive_job' , $possibly_protected_client ) )
                    {
                        $possibly_protected_client['next_archive_job'] = '';
                    }
                    $report_data['clients']['protected'][] = $possibly_protected_client;
                }
                elseif ( array_key_exists('next_archive_job' , $possibly_protected_client ) )
                {
                    $possibly_protected_client['last_backup'] = '';
                    $possibly_protected_client['last_replication'] = '';
                    $report_data['clients']['protected'][] = $possibly_protected_client;
                }
                else
                {
                    $report_data['clients']['unprotected'][] = $possibly_protected_client;
                }
            }

            foreach ( $possibly_protected_vms_by_instance_id as $possibly_protected_vm )
            {
                if ( array_key_exists('last_backup' , $possibly_protected_vm ) )
                {
                    if ( !array_key_exists('last_replication' , $possibly_protected_vm ) )
                    {
                        $possibly_protected_vm['last_replication'] = '';
                    }
                    if ( !array_key_exists('next_archive_job' , $possibly_protected_vm ) )
                    {
                        $possibly_protected_vm['next_archive_job'] = '';
                    }
                    $report_data['vms']['protected'][] = $possibly_protected_vm;
                }
                elseif ( array_key_exists('last_replication' , $possibly_protected_vm ) )
                {
                    $possibly_protected_vm['last_backup'] = '';
                    if ( !array_key_exists('next_archive_job' , $possibly_protected_vm ) )
                    {
                        $possibly_protected_vm['next_archive_job'] = '';
                    }
                    $report_data['vms']['protected'][] = $possibly_protected_vm;
                }
                elseif ( array_key_exists('next_archive_job' , $possibly_protected_vm ) )
                {
                    $possibly_protected_vm['last_backup'] = '';
                    $possibly_protected_vm['last_replication'] = '';
                    $report_data['vms']['protected'][] = $possibly_protected_vm;
                }
                else
                {
                    $report_data['vms']['unprotected'][] = $possibly_protected_vm;
                }
            }
        }
        $report_total = $report_count;  //Temporary

        $report['count'] = $report_count;
        $report['total'] = $report_total;
        $report['start'] = $report_start;
        $report['data'] = $report_data;
        return $report;
    }

}