<?php

$dir = '/api/includes/reports';

class StorageStatusReport
{
    private $BP;

    public function __construct( $BP, $functionsClass, $alertsClass )
    {
        $this->BP = $BP;
        $this->functions = $functionsClass;
        $this->alerts = $alertsClass;
    }

    public function get_storage_status_report($systems, $data)
    {
        $report = array();
        $report_count = 0;
        //Temporary until more data is available
        $report_total = $report_count;
        $report_start = 1;
        $report_data = array();

        foreach ($systems as $system_id => $system_name)
        {
            $storage_list = $this->BP->get_storage_list($system_id);
            if ( $storage_list !== false and count($storage_list) > 0 )
            {
                foreach ($storage_list as $storage_ID => $storage_name)
                {
                    $storage_info = $this->BP->rest_get_storage_info($storage_ID, $system_id);

                    if ( $storage_info !== false )
                    {
                        $temp_storage_info = array();
                        $temp_storage_info['system_id'] = $system_id;
                        $temp_storage_info['system_name'] = $system_name;
                        $temp_storage_info['id'] = $storage_info['id'];
                        $temp_storage_info['name'] = $storage_info['name'];
                        $temp_storage_info['is_default'] = $storage_info['is_default'];
                        $temp_storage_info['type'] = '';  //Filled in below
                        $temp_storage_info['protocol'] = ''; //Filled in below, if applicable
                        $temp_storage_info['usage'] = $storage_info['usage'];
                        $temp_storage_info['online'] = ($storage_info['status'] === 'online');
                        $temp_storage_info['status'] = $storage_info['status'];
                        $temp_storage_info['mb_size'] = $storage_info['mb_size'];
                        $temp_storage_info['mb_free'] = $storage_info['mb_free'];
                        $temp_storage_info['mb_used'] = $storage_info['mb_used'];
                        /*$temp_storage_info['average_write_speed'] = 'CoreWorkNeeded'; //TBD
                        $temp_storage_info['average_written_daily'] = 'CoreWorkNeeded'; //TBD
                        $temp_storage_info['daily_change_rate'] = 'CoreWorkNeeded'; //TBD*/
                        $temp_storage_info['daily_growth_rate'] = $storage_info['daily_growth_rate'];
                        $temp_storage_info['dedup'] = $storage_info['dedup']; //TBD
                        //$temp_storage_info['effective_size'] = 'CoreWorkNeeded'; //TBD
                        $temp_storage_info['alerts'] = array(); //Filled in below, if applicable
                        $temp_storage_info['size_history'] = array(); //Filled in below

                        switch ( $storage_info['type'] )
                        {
                            case Constants::STORAGE_TYPE_NAME_INTERNAL_DAS_NOT_MODIFIABLE:
                                $temp_storage_info['type'] = Constants::STORAGE_TYPE_DISPLAY_NAME_INTERNAL_DAS_NOT_MODIFIABLE;
                                break;
                            case Constants::STORAGE_TYPE_NAME_ISCI_SAN:
                                $temp_storage_info['type'] = Constants::STORAGE_TYPE_DISPLAY_NAME_ISCI_SAN;
                                break;
                            case Constants::STORAGE_TYPE_NAME_FC_SAN:
                                $temp_storage_info['type'] = Constants::STORAGE_TYPE_DISPLAY_NAME_FC_SAN;
                                break;
                            case Constants::STORAGE_TYPE_NAME_AOE_SAN:
                                $temp_storage_info['type'] = Constants::STORAGE_TYPE_DISPLAY_NAME_AOE_SAN;
                                break;
                            case Constants::STORAGE_TYPE_NAME_NAS:
                                $temp_storage_info['protocol'] = $storage_info['properties']['protocol'];
                                $temp_storage_info['type'] = Constants::STORAGE_TYPE_DISPLAY_NAME_NAS;
                                break;
                            case Constants::STORAGE_TYPE_NAME_ADDED_INTERNAL_DAS_MODIFIABLE:
                                $temp_storage_info['type'] = Constants::STORAGE_TYPE_DISPLAY_NAME_ADDED_INTERNAL_DAS_MODIFIABLE;
                                break;
                        }

                        if ( $storage_info['has_alert'] === true and count($storage_info['alerts']) > 0 )
                        {
                            $temp_storage_info['alerts'] = $this->alerts->processAlerts( $storage_info['alerts'], $system_name, $system_id );
                        }

                        if ( count($storage_info['storage_history']) > 0 )
                        {
                            foreach ( $storage_info['storage_history'] as $size_history_day )
                            {
                                $temp_storage_info['size_history'][] = array( 'day' => $size_history_day['date'],
                                                                              'mb_size' => $size_history_day['mb_size'],
                                                                              'mb_used' => $size_history_day['mb_used'],
                                                                              'mb_free' => $size_history_day['mb_free'] );
                            }
                        }

                        $report_data[] = $temp_storage_info;
                        $report_count += 1;
                    }
                }
            }
        }
        $report_total = $report_count;  //Temporary

        $report['count'] = $report_count;
        $report['total'] = $report_total;
        $report['start'] = $report_start;
        $report['data'] = $report_data;
        return $report;

    }

}