<?php

class Restores
{
    private $BP;

    public function __construct($BP)
    {
        $this->BP = $BP;
    }

    public function get($which, $data, $sid, $systems)
    {
        if (is_string($which[0]))
        {
            switch ($which[0])
            {
                case 'targets':
                    return $this->get_restore_targets($data, $sid);
                    break;
            }
        }
    }

    public function get_restore_targets($data, $sid)
    {
        $targets = array();
        if ( $sid === false )
        {
            $sid = $this->BP->get_local_system_id();
        }

        $application_type = false;
        $instance_id = false;
        $return_all_servers = false;
        // If both an instance_id and backup_id are passed in, ignore the backup_id
        if ( array_key_exists('instance_id', $data) )
        {
            $instance_id = (int)$data['instance_id'];
            if ( $instance_id === 0 )
            {
                $return_all_servers = true;
            }
            else
            {
                $appinst_info_array = $this->BP->get_appinst_info( $instance_id, $sid );
                if ( $appinst_info_array !== false and count($appinst_info_array) > 0 )
                {
                    foreach ( $appinst_info_array as $appinst_info )
                    {
                        $application_type = $appinst_info['app_type'];

                        //  There should only be one instance id, so no reason to go through a loop.  Throw out additional results from the array.
                        break;
                    }
                }
            }
        }
        else if ( array_key_exists('backup_id', $data) )
        {
            $backup_types = $this->getBackupType( array( (int)$data['backup_id'] ), $sid );
            if ( $backup_types !== false and count($backup_types) > 0 )
            {
                foreach ( $backup_types as $backup_type_array )
                {
                    require_once('function.lib.php');
                    $function = new Functions($this->BP);

                    $application_type = $function->getApplicationTypeFromApplictionID( $backup_type_array['app_id'] );
                    $instance_id = $backup_type_array['instance_id'];

                    //  There should only be one backup id, so no reason to go through a loop.  Throw out additional results from the array.
                    break;

                }
            }
        }
        else
        {
            $return_all_servers = true;
        }

        if ( $return_all_servers === true )
        {
            $hyper_v_recovery_clients = array();

            // For Hyper-V, get all of the servers using bp_get_hyperv_servers_for_wir, then pass in those results to get_Hyper_V_restore_targets_array
            // Using DOS as a fill in
            $hyper_v_servers = $this->BP->get_hyperv_servers_for_wir( Constants::OS_DOS, $sid );

            if ( $hyper_v_servers !== false and count($hyper_v_servers) > 0 )
            {
                foreach ( $hyper_v_servers as $hyper_v_server )
                {
                    $hyper_v_recovery_clients[$hyper_v_server['client_id']] = $hyper_v_server['name'];
                }
            }

            //Get the Hyper-V targets
            $targets = $this->get_Hyper_V_restore_targets_array( $sid, false, $hyper_v_recovery_clients, $hyper_v_servers );

            // Now merge in the VMware results
            $targets = array_merge( $targets, $this->get_VM_restore_targets_array( 0, $sid ) );
        }
        elseif ( $application_type !== false and $instance_id !== false )
        {
            switch ( $application_type )
            {
                case Constants::APPLICATION_TYPE_NAME_HYPER_V:
                    $targets = $this->get_Hyper_V_restore_targets_array( $sid, $instance_id, false, false, $data );
                    break;
                case Constants::APPLICATION_TYPE_NAME_VMWARE:
                    $targets = $this->get_VM_restore_targets_array( $instance_id, $sid );
                    break;
            }
        }
        else
        {
            $targets = false;
        }


        return ( $targets !== false ? array( 'targets' => $targets) : false );
    }

    private function get_Hyper_V_restore_targets_array( $sid, $instance_id = false, $hyper_v_recovery_clients = false, $hyper_v_servers = false, $data = null )
    {
        $hyper_v_targets_array = array();

        // If $instance_id is false, then $hyper_v_recovery_clients must not be equal to false
        if ( $instance_id !== false )
        {
            // this returns the original client on which we are trying to restore
            $hyper_v_application_clients = array();
            $temp_application_clients = $this->BP->get_application_clients( $instance_id, $sid );
            if ($temp_application_clients !== false) {
                if (array_key_exists('clients', $temp_application_clients)) {
                    $temp_clients = $temp_application_clients['clients'];
                    foreach ($temp_clients as $temp_client_id => $temp_client_name) {
                        $hyper_v_application_clients[$temp_client_id] = $temp_client_name;
                    }
                }
            }
            $recovery_clients = $this->BP->get_hyperv_recovery_clients( $instance_id, $sid ) ;
            $hyper_v_recovery_clients =  $hyper_v_application_clients + $recovery_clients;
            if (array_key_exists('backup_id', $data) && array_key_exists('target', $data)) {
                if ($data['target'] === 'original') {
                    $hyper_v_recovery_clients = array();
                    $clientID = 0;
                    $clients = $this->getClientIDFromBackup(array((int)$data['backup_id']), $sid);
                    if ($clients !== false) {
                        foreach ($clients as $client) {
                            $clientID = $client;
                            break;
                        }
                    }
                    foreach ($hyper_v_application_clients as $client_id => $client_name) {
                        if ($client_id == $clientID) {
                            $hyper_v_recovery_clients[$client_id] = $client_name;
                        }
                    }
                }
            } elseif (array_key_exists('type', $data)) {
                // Return only the servers that are eligible for Instant Recovery
                if ($data['type'] === 'ir') {
                    $hyper_v_recovery_clients = array();
                    $hyper_v_ir_clients = $this->BP->get_hyperv_servers_for_ir($instance_id, $sid);
                    if ($hyper_v_ir_clients !== false) {
                        foreach ($hyper_v_ir_clients as $hyper_v_ir_client) {
                            $hyper_v_recovery_clients[$hyper_v_ir_client['client_id']] = $hyper_v_ir_client['name'];
                        }
                    }
                } elseif ($data['type'] === 'restore') {
                    $hyper_v_recovery_clients = array();
                    $hyper_v_original_client = array();
                    $clientID = 0;
                    $clients = $this->getClientIDFromBackup(array((int)$data['backup_id']), $sid);
                    if ($clients !== false) {
                        foreach ($clients as $client) {
                            $clientID = $client;
                            break;
                        }
                    }
                    foreach ($hyper_v_application_clients as $client_id => $client_name) {
                        if ($client_id == $clientID) {
                            $hyper_v_original_client[$client_id] = $client_name;
                        }
                    }
                    $hyper_v_recovery_clients =  $hyper_v_original_client + $recovery_clients;
                }
            }
        }
        // This function call might have already been made, so no need to make it twice
        if ( $hyper_v_servers === false )
        {
            // Using DOS as a fill in
            $hyper_v_servers = $this->BP->get_hyperv_servers_for_wir( Constants::OS_DOS, $sid );
        }

        if ( $hyper_v_recovery_clients !== false and count($hyper_v_recovery_clients) > 0 )
        {
            $hyper_v_servers_info_array = array();

            if ( $hyper_v_servers !== false and count($hyper_v_servers) > 0 )
            {
                foreach ( $hyper_v_servers as $hyper_v_server )
                {
                    $hyper_v_servers_info_array[$hyper_v_server['client_id']] = $hyper_v_server['capabilities'];
                }
            }

            foreach ( $hyper_v_recovery_clients as $hyper_v_client_id => $hyper_v_client_name )
            {
                $temp_restore_target = array();
                $temp_restore_target['id'] = $hyper_v_client_id;
                $temp_restore_target['name'] = $hyper_v_client_name;

                $hyper_v_storage = $this->BP->get_hyperv_storage( $hyper_v_client_id, $sid );
                if ( $hyper_v_storage !== false and count($hyper_v_storage) > 0 )
                {
                    $temp_restore_target['datastores'] = $hyper_v_storage;
                }

                if ( array_key_exists( $hyper_v_client_id, $hyper_v_servers_info_array ) )
                {
                    $temp_restore_target['capabilities'] = $hyper_v_servers_info_array[$hyper_v_client_id];
                }

                $hyper_v_targets_array[] = $temp_restore_target;
            }
        }
        return $hyper_v_targets_array;
    }

    private function get_VM_restore_targets_array( $instance_id, $sid )
    {
        $vm_targets_array = array();
        $vm_restore_targets = $this->BP->get_vm_restore_targets( $instance_id, $sid );
        if ( $vm_restore_targets !== false and count($vm_restore_targets) > 0 )
        {
            $vm_servers = $vm_restore_targets['servers'];
            foreach ( $vm_servers as $vm_restore_target )
            {
                $temp_restore_target = array();
                $temp_restore_target['id'] = $vm_restore_target['uuid'];
                $temp_restore_target['name'] = $vm_restore_target['name'];
                $temp_restore_target['datastores'] = $vm_restore_target['datastores'];  /*array();
                            if ( array_key_exists('datastores', $vm_restore_target) and count($vm_restore_target['datastores']) > 0 )
                            {
                                foreach ( $vm_restore_target['datastores'] as $vm_data_store )
                                {
                                    $temp_restore_target['datastores'][] = $vm_data_store;
                                }
                            }*/
                $temp_restore_target['groups'] = $this->get_vm_groups( $vm_restore_target['uuid'], $vm_restore_target['name'], $sid);
                $temp_restore_target['capabilities'] = $vm_restore_target['capabilities'];
                $vm_targets_array[] = $temp_restore_target;
            }
        }
        return $vm_targets_array;
    }

    private function get_vm_groups( $uuid, $name, $sid )
    {
        $groups_info = array();
        $resourcePools = $this->BP->get_resource_pool_info( $uuid, $sid );
        $vapps         = $this->BP->get_vApp_info( $uuid, $sid );

        $include_vGroups = ( $resourcePools!== false || $vapps!== false  );

        if ( $include_vGroups ) {
            $groups_info['Name'] = $name;
        }

        if ( $resourcePools!== false && $resourcePools != NULL ) {
            foreach( $resourcePools as $rp ){
                $rpKey = $rp['key'];
                $rpName = $rp['name'];
                $rpParentType = $rp['parentType'];
                $rpParentKey = $rp['parentKey'];
                $temp_groups_info = array( 'Key' => $rpKey,
                    'Name' => $rpName,
                    'ParentType' => $rpParentType,
                    'ParentKey' => $rpParentKey,
                    'Type'=>"0" );
                $groups_info['VG'][] = $temp_groups_info;
            }
        }

        if ( $vapps!== false && $vapps != NULL ) {
            foreach( $vapps as $va ){
                $vaKey = $va['key'];
                $vaName = $va['name'];
                $vaParentType = $va['parentType'];
                $vaParentKey = $va['parentKey'];
                $temp_groups_info = array( 'Key' => $vaKey,
                    'Name' => $vaName,
                    'ParentType' => $vaParentType,
                    'ParentKey' => $vaParentKey,
                    'Type'=> "1" );
                $groups_info['VG'][] = $temp_groups_info;
            }
        }

        return $groups_info;
    }

    public function post($which, $data, $sid)
    {
        require_once('function.lib.php');
        $function = new Functions($this->BP);

        $sid = $sid !== false ? $sid : $this->BP->get_local_system_id();
        $result = array();
        if (is_string($which[0])) {
            switch ($which[0]) {
                case 'instant':
                    $backups = $this->getRestoreGroup($data['backup_id'], $sid);
                    if (array_key_exists('error', $backups)) {
                        $result = $backups['error'];
                    } else {
                        $backupArrStr = explode(",", $backups);
                        $backupArray = array();
                        $i = 0;
                        foreach ($backupArrStr as $backup) {
                            $backupArray[$i++] = (int)$backup;
                        }
                        $backupType = $this->getBackupType($backupArray, $sid);
                        if (array_key_exists('target', $data)) {
                            $target = $data['target'];
                            $host = $target['host'];
                            if (array_key_exists('datastore', $target)) {
                                $dataStore = $target['datastore'];
                            }
                            if (array_key_exists('directory', $target)) {
                                $directory = $target['directory'];
                            }
                            $name = $target['name'];
                            if (array_key_exists('group', $target)) {
                                $group = $target['group'] == "" ? NULL : $target['group'];
                            }
                            if (array_key_exists('switch', $target)) {
                                $switch = $target['switch'] == "" ? NULL : $target['switch'];
                            }
                        }
                        $address = $data['address'];
                        $audit = (int)$data['audit'];
                        $poweron = isset($data['poweron']) ? $data['poweron'] : 1;
                        if ($function->isBackupVMWare($backupType[0]['type'])) {
                            $result = $this->BP->vmware_ir_start($host, $name, $dataStore, $address, $audit,
                                $backups, $poweron, $group, $sid);
                        } elseif ($function->isBackupHyperV($backupType[0]['type'])) {
                            $result = $this->BP->hyperv_ir_start($host, $name, $directory, $address, $audit,
                                $backups, $poweron, $switch, $sid);

                        } else {
                            $result = "Backup id " .$data['backup_id']. " is not supported.";
                        }
                    }
                    break;

                case 'full':
                    $backups = $this->getRestoreGroup($data['backup_id'], $sid);
                    if (array_key_exists('error', $backups)) {
                        $result = $backups['error'];
                    } else {
                        $backupArrStr = explode(",", $backups);
                        $backupArray = array();
                        $i = 0;
                        foreach ($backupArrStr as $backup) {
                            $backupArray[$i++] = (int)$backup;
                        }
                        $backupType = $this->getBackupType($backupArray, $sid);

                        if (array_key_exists('client_id', $data)) {
                            $client_id = (int)$data['client_id'];
                        }
                        $includes = $data['includes'];
                        $includesArrStr = explode(",", $includes);
                        $includesArray = array();
                        if ($includes !== "") {
                            $i = 0;
                            foreach ($includesArrStr as $include) {
                                $includesArray[$i++] = (int)$include;
                            }
                        }
                        $excludes = $data['excludes'];
                        $excludesArrStr = explode(",", $excludes);
                        $excludesArray = array();
                        if ($excludes !== "") {
                            $j = 0;
                            foreach ($excludesArrStr as $exclude) {
                                $excludesArray[$j++] = (int)$exclude;
                            }
                        }
                        $target = $data['target'];
                        $database = "";
                        $instance = "";
                        $host = "";
                        $datastore = "";
                        $name = "";
                        $group = "";
                        $metadata = "";
                        $template = "";
                        if (array_key_exists('flat', $target)) {
                            $flat = $target['flat'];
                        }
                        if (array_key_exists('non_destructive', $target)) {
                            $non_destructive = $target['non_destructive'];
                        }
                        if (array_key_exists('newer', $target)) {
                            $newer = $target['newer'];
                        }
                        if (array_key_exists('today', $target)) {
                            $today = $target['today'];
                        }
                        if (array_key_exists('unix', $target)) {
                            $unix = $target['unix'];
                        }
                        if (array_key_exists('database', $target)) {
                            $database = $target['database'];
                        }
                        if (array_key_exists('instance', $target)) {
                            $instance = $target['instance'];
                        }
                        if (array_key_exists('point_in_time', $target)) {
                            $point_in_time = $target['point_in_time'];
                        }
                        if (array_key_exists('host', $target)) {
                            $host = $target['host'];
                        }
                        if (array_key_exists('datastore', $target)) {
                            $datastore = $target['datastore'];
                        }
                        if (array_key_exists('name', $target)) {
                            $name = $target['name'];
                        }
                        if (array_key_exists('group', $target)) {
                            $group = $target['group'];
                        }
                        if (array_key_exists('metadata', $target)) {
                            $metadata = $target['metadata'];
                        }
                        if (array_key_exists('template', $target)) {
                            $template = $target['template'];
                        }
                        if (array_key_exists('directory', $target)) {
                            $directory = $target['directory'];
                        }

                        $bcmd = $data['before_cmd'];
                        $acmd = $data['after_cmd'];

                        if (isset($flat) && isset($non_destructive) && isset($newer) && isset($today) && isset($unix)) {
                            $backup = (int)$backups;
                            $options = array();
                            if (isset($client_id)) {
                                $options['client_id'] = $client_id;
                            }
                            if (isset($directory)) {
                                $options['directory'] = $directory;
                            }
                            $options['flat'] = $flat;
                            if ($bcmd !== "") {
                                $options['before_command'] = $bcmd;
                            }
                            if ($acmd !== "") {
                                $options['after_command'] = $acmd;
                            }
                            $options['non_destructive'] = $non_destructive;
                            $options['newer'] = $newer;
                            $options['today'] = $today;
                            $options['unix'] = $unix;

                            $result = $this->BP->restore_files($backup, $includesArray, $excludesArray, $options, $sid);
                        } else {
                            if ($function->isBackupHyperV($backupType[0]['type']) || $function->isBackupVMWare($backupType[0]['type']) ||
                                $function->isBackupExchange($backupType[0]['type']) || $function->isBackupSQL($backupType[0]['type'])
                            ) {
                                $destination = "";
                                if ($function->isBackupExchange($backupType[0]['type'])) {
                                    $destination = $database;
                                } elseif ($function->isBackupSQL($backupType[0]['type'])) {
                                    $destination = $instance . '|' . $database;
                                } elseif ($function->isBackupVMWare($backupType[0]['type'])) {
                                    $destination = $host . '|' . $datastore . '|' . $name . '|' . $group . '|' . $metadata . '|' . $template;
                                }
                                $options = array();
                                if (isset($client_id)) {
                                    $options['client_id'] = $client_id;
                                }
                                if (isset($target)) {
                                    $options['target'] = $destination;
                                }
                                if (isset($directory)) {
                                    $options['directory'] = $directory;
                                }
                                if (isset($point_in_time)) {
                                    $options['point_in_time'] = $point_in_time;
                                }
                                if ($bcmd !== "") {
                                    $options['before_command'] = $bcmd;
                                }
                                if ($acmd !== "") {
                                    $options['after_command'] = $acmd;
                                }
                                $resultArray = $this->BP->restore_application($backups, $options, $sid);
                                if ($resultArray !== false) {
                                    $tempRes = "";
                                    $delim = "";
                                    foreach ($resultArray as $resArr) {
                                        $tempRes = $tempRes . $delim . $resArr['job_id'];
                                        $delim = ",";
                                    }
                                    $result['id'] = $tempRes;
                                } else {
                                    $result = $resultArray;
                                }
                            } elseif ($this->isBackupSharePoint($backupType[0]['type']) || $this->isBackupOracle($backupType[0]['type'])) {
                                $options = array();
                                if (isset($client_id)) {
                                    $options['client_id'] = $client_id;
                                }
                                $resultArray = $this->BP->rae_restore_application($backups, $options, $sid);
                                if ($resultArray !== false) {
                                    $tempRes = "";
                                    $delim = "";
                                    foreach ($resultArray as $resArr) {
                                        $tempRes = $tempRes . $delim . $resArr['job_id'];
                                        $delim = ",";
                                    }
                                    $result['id'] = $tempRes;
                                } else {
                                    $result = $resultArray;
                                }
                            } else {
                                $result = "Backup id " .$data['backup_id']. " is not supported.";
                            }
                        }
                    }
                    break;

                case 'files':
                    if (isset($data['related'])) {
                        $related = $data['related'];
                    } else {
                        $related = 1;
                    }
                    if ($related === 1) {
                        $backups = $this->getRestoreGroup($data['backup_id'], $sid);
                    } else {
                        $backups = $data['backup_id'];
                    }
                    if (array_key_exists('error', $backups)) {
                        $result = $backups['error'];
                    } else {
                        $backupArrStr = explode(",", $backups);
                        $backupArray = array();
                        $i = 0;
                        foreach ($backupArrStr as $backup) {
                            $backupArray[$i++] = (int)$backup;
                        }
                        $backupType = $this->getBackupType($backupArray, $sid);
                        $path = "";
                        if (array_key_exists('path', $data)) {
                            $path = $data['path'];
                        }
                        if ($function->isBackupExchange($backupType[0]['type'])) {
                            $result = $this->BP->backup_mount($backups, $sid);
                        } elseif ($function->isBackupVMWare($backupType[0]['type']) || $function->isBackupHyperV($backupType[0]['type'])) {
                            $result = $this->BP->create_disk_image($backups, $sid);
                        } elseif ($this->isBackupOracle($backupType[0]['type']) || $this->isBackupSharePoint($backupType[0]['type'])) {
                            $result = $this->BP->create_application_share($backups, $path, $sid);
                        } else {
                            $result = "Backup id " .$data['backup_id']. " is not supported.";
                        }
                    }
                    break;
            }
        }
        return $result;
    }

    /*
     * Passing a string of backupIDs will return an array of backup IDs and type of backups
     */
    private function getBackupType($backupIDs, $sid)
    {
        $backupType = array();
        $result_format = array();
        $result_format['system_id'] = $sid;
        $result_format['backup_ids'] = $backupIDs;

        $resultArray = $this->BP->get_backup_status($result_format);
        if ($resultArray !== false) {
            foreach ($resultArray as $result) {
                $backupType[] = array( 'id' => $result['id'],
                                    'type' => $result['type'],
                                    'app_id' => ( array_key_exists('app_id', $result) ? $result['app_id'] : false ),
                                    'instance_id' => ( array_key_exists('instance_id', $result) ? $result['instance_id'] : false ));
            }
        }
        return $backupType;
    }

    private function getRestoreGroup($backupID, $sid)
    {
        $backups = "";
        $delim = "";
        $groupArray = $this->BP->get_restore_group($backupID, $sid);
        if ($groupArray !== false) {
            foreach ($groupArray as $group) {
                $backups = $backups . $delim . $group['id'];
                $delim = ",";
            }
        } else {
            $backups['error'] = $groupArray;
        }
        return $backups;
    }

    private function isBackupSharePoint($backupType)
    {
        $isSharePoint = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_SHAREPOINT_FULL:
            case Constants::BACKUP_TYPE_SHAREPOINT_DIFF:

                $isSharePoint = true;
                break;
        }
        return $isSharePoint;
    }

    private function isBackupOracle($backupType)
    {
        $isOracle = false;

        switch($backupType){
            case Constants::BACKUP_TYPE_ORACLE_FULL:
            case Constants::BACKUP_TYPE_ORACLE_INCR:

                $isOracle = true;
                break;
        }
        return $isOracle;
    }

    private function getClientIDFromBackup($backupID, $sid)
    {
        $clientID = array();
        $result_format = array();
        $result_format['system_id'] = $sid;
        $result_format['backup_ids'] = $backupID;

        $resultArray = $this->BP->get_backup_status($result_format);
        if ($resultArray !== false) {
            foreach ($resultArray as $result) {
                $clientID[] = $result['client_id'];
            }
        }
        return $clientID;
    }
}













?>
