<?php

class VirtualClients
{
    private $BP;

    const WIR_FILTER        = "wir";
    const VMWARE_IR_FILTER  = "vm_ir";
    const HYPERV_IR_FILTER  = "hv_ir";

    const STATE_NEW         = "new";
    const STATE_HALTED      = "halted";

    public function __construct($BP)
    {
        $this->BP = $BP;
    }

    public function get($which, $data, $sid, $systems)
    {
        $virtualClients = array();

        if ($which == -1) {
            //GET /api/virtual_clients/?sid={sid}&type={type}
            $virtualClients = $this->get_virtual_clients($systems, $data);
        }
        else {
            switch ($which[0]) {
                case 'supported':
                    // GET /api/virtual_clients/supported/?sid={sid}
                    $virtualClients = $this->get_virtual_clients_supported($systems);
                    break;
                case 'candidates':
                    // GET /api/virtual_clients/candidates/?sid={sid}
                    $virtualClients = $this->get_virtual_clients_candidates($systems);
                    break;
                default:
                    // GET /api/virtual_clients/{id}
                    $virtualID = (int)$which[0];
                    $type = $data['type'];
                    if (isset($which[1]) && is_string($which[1]) ) {
                        switch($which[1]) {
                            case 'details':
                                // GET /api/virtual_clients/{id}/details/?type={type}
                                $virtualClients = $this->get_virtual_clients_details($virtualID, $type, $sid);
                                break;
                            case 'backup_history':
                                // GET /api/virtual_clients/{id}/backup_history/?sid={sid}
                                $virtualClients = $this->get_virtual_clients_backup_history($virtualID, $sid);
                                break;
                            case 'state':
                                //GET /api/virtual_clients/{id}/state
                                $virtualClients = $this->get_virtual_clients_state($virtualID, $sid);
                                break;
                        }
                    }
                    break;
            }
        }

        return $virtualClients;

    }

    private function get_virtual_clients_supported($systems)
    {
        $supported = array();
        foreach ($systems as $systemID => $systemName) {
            $virtualClientsSupported = $this->BP->virtual_clients_supported($systemID);
            if ($virtualClientsSupported !== -1) {
                $temp_supported_array = array();
                $temp_supported_array['system_id'] = $systemID;
                $temp_supported_array['system_name'] = $systemName;
                $temp_supported_array['supported'] = $virtualClientsSupported;

                $supported['wir'][] = $temp_supported_array;
            }
        }
        return $supported;
    }

    private function get_virtual_clients_candidates($systems)
    {
        $virtualCandidates = array();
        foreach ($systems as $systemID => $systemName) {
            $support = $this->BP->virtual_clients_supported($systemID);
            if ($support !== -1) {
                if ($support !== false) {
                    $candidates = $this->BP->get_virtual_candidates($systemID);
                    if ($candidates !== false) {
                        $temp_candidates_array = array();
                        $temp_candidates_array['system_id'] = $systemID;
                        $temp_candidates_array['system_name'] = $systemName;
                        foreach ($candidates as $candidate) {
                            $temp_candidates = array();
                            $temp_candidates['ClientID'] = $candidate['client_id'];
                            $temp_candidates['ClientName'] = $candidate['client_name'];
                            $temp_candidates['ClientOS'] = $candidate['os_id'];
                            if (isset($candidate['virtual_id'])) {
                                $temp_candidates['VirtualID'] = $candidate['virtual_id'];
                            }
                            if (isset($candidate['backup_id'])) {
                                $temp_candidates['BackupID'] = $candidate['backup_id'];
                            }
                            if (isset($candidate['grandclient'])) {
                                $temp_candidates['GrandClient'] = $candidate['grandclient'];
                            }
                            if (isset($candidate['system_id'])) {
                                $temp_candidates['SystemID'] = $candidate['system_id'];
                            }
                            if (isset($candidate['system_name'])) {
                                $temp_candidates['SystemName'] = $candidate['system_name'];
                            }
                            $temp_candidates_array['Candidates'][] = $temp_candidates;
                        }
                        $virtualCandidates['data'][] = $temp_candidates_array;
                    }
                } else {
                    $virtualCandidates = "Windows Failover Virtualization is not supported.";
                }
            }
        }
        return $virtualCandidates;
    }

    private function get_virtual_clients($systems, $data)
    {
        $virtual_clients = array();

        foreach ($systems as $systemID => $systemName) {
            $temp_virtual_array = array();
            $temp_virtual_array['system_id'] = $systemID;
            $temp_virtual_array['system_name'] = $systemName;

            if (array_key_exists('type', $data)) {
                switch($data['type']) {
                    case VirtualClients::WIR_FILTER:
                        $temp_virtual_array['wir'] = $this->get_wir_clients($systemID);
                        break;
                    case VirtualClients::VMWARE_IR_FILTER:
                        $temp_virtual_array['vm_ir'] = $this->get_vm_ir_clients($systemID);
                        break;
                    case VirtualClients::HYPERV_IR_FILTER:
                        $temp_virtual_array['hv_ir'] = $this->get_hv_ir_clients($systemID);
                        break;
                }
            }
            else {
                $temp_virtual_array['wir'] = $this->get_wir_clients($systemID);
                $temp_virtual_array['vm_ir'] = $this->get_vm_ir_clients($systemID);
                $temp_virtual_array['hv_ir'] = $this->get_hv_ir_clients($systemID);
            }
            $virtual_clients['data'][] = $temp_virtual_array;
        }

        return $virtual_clients;
    }

    private function get_wir_clients($systemID)
    {
        $wir_clients = array();

        $support = $this->BP->virtual_clients_supported($systemID);
        if ($support !== -1) {
            if ($support !== false) {
                $clients = $this->BP->get_virtual_client_list($systemID);
                if ($clients !== false) {
                    foreach ($clients as $client) {
                        $temp_wir_clients_array = array();
                        $temp_wir_clients_array['virtual_id'] = $client['virtual_id'];
                        $temp_wir_clients_array['type'] = VirtualClients::WIR_FILTER;

                        $clientDetail = $this->BP->get_virtual_client($client['virtual_id'], $systemID);
                        if (isset($clientDetail['hypervisor_type'])) {
                            $temp_wir_clients_array['hypervisor_type'] = $clientDetail['hypervisor_type'];
                        }
                        if (isset($clientDetail['vm_name'])) {
                            $temp_wir_clients_array['vm_name'] = $clientDetail['vm_name'];
                        }
                        if (isset($clientDetail['hypervisor_name'])) {
                            $temp_wir_clients_array['ServerName'] = $clientDetail['hypervisor_name'];
                        }
                        $temp_wir_clients_array['Mode'] = $client['current_state'];
                        /*
                         * To be implemented in phase 2

                        $temp_wir_clients_array['Status'] = 'NeedsCoreWork';
                        $temp_wir_clients_array['Created'] = 'NeedsCoreWork';
                        $temp_wir_clients_array['Duration'] = 'NeedsCoreWork';
                        $temp_wir_clients_array['Resources'] = 'NeedsCoreWork';

                        */

                        $wir_clients['data'][] = $temp_wir_clients_array;
                    }
                }
            } else {
                $wir_clients = "Windows Failover Virtualization is not supported.";
            }
        }
        return $wir_clients;
    }

    private function get_vm_ir_clients($systemID)
    {
        require_once('function.lib.php');
        $functions = new Functions($this->BP);
        $vmIR_clients = array();

        $vmIRStatus = $this->BP->get_vm_ir_status($systemID);

        if ($vmIRStatus !== false) {
            foreach ($vmIRStatus as $vm_status_info) {
                if (isset($vm_status_info['id'])) {
                    $temp_vmIR_clients_array = array();
                    $temp_vmIR_clients_array['virtual_id'] = $vm_status_info['id'];
                    $temp_vmIR_clients_array['type'] = VirtualClients::VMWARE_IR_FILTER;
                }
                if (isset($vm_status_info['vm_name'])) {
                    $temp_vmIR_clients_array['vm_name'] = $vm_status_info['vm_name'];
                }
                if (isset($vm_status_info['vm_server'])) {
                    $temp_vmIR_clients_array['ServerName'] = $vm_status_info['vm_server'];
                }
                if (isset($vm_status_info['status'])) {
                    $temp_vmIR_clients_array['Status'] = $vm_status_info['status'];
                }
                if(isset($vm_status_info['audit'])) {
                    $temp_vmIR_clients_array['Mode'] = $vm_status_info['audit'] == 1 ? 'Audit' : 'Instant Recovery';
                }
                if (isset($vm_status_info['time'])) {
                    $temp_vmIR_clients_array['Created'] = date($functions::DATE_TIME_FORMAT_US, $vm_status_info['time']);
                    $temp_vmIR_clients_array['Duration'] = $this->get_duration($vm_status_info['time']);
                }
                /*
                 * To be implemented in phase 2
                 $temp_vmIR_clients_array['Resources'] = 'NeedsCoreWork';
                */

                $vmIR_clients['data'][] = $temp_vmIR_clients_array;
            }
        }
        return $vmIR_clients;
    }

    private function get_hv_ir_clients($systemID)
    {
        require_once('function.lib.php');
        $functions = new Functions($this->BP);
        $hvIR_clients = array();

        $hvIRStatus = $this->BP->get_hyperv_ir_status($systemID);

        if ($hvIRStatus !== false) {
            foreach ($hvIRStatus as $hv_status_info) {
                if (isset($hv_status_info['id'])) {
                    $temp_hvIR_clients_array = array();
                    $temp_hvIR_clients_array['virtual_id'] = $hv_status_info['id'];
                    $temp_hvIR_clients_array['type'] = VirtualClients::HYPERV_IR_FILTER;
                }
                if (isset($hv_status_info['vm_name'])) {
                    $temp_hvIR_clients_array['vm_name'] = $hv_status_info['vm_name'];
                }
                if (isset($hv_status_info['vm_server'])) {
                    $temp_hvIR_clients_array['ServerName'] = $hv_status_info['vm_server'];
                }
                if (isset($hv_status_info['status'])) {
                    $temp_hvIR_clients_array['Status'] = $hv_status_info['status'];
                }
                if (isset($hv_status_info['audit'])) {
                    $temp_hvIR_clients_array['Mode'] = $hv_status_info['audit'] == 1 ? 'Audit' : 'Instant Recovery';
                }
                if (isset($hv_status_info['time'])) {
                    $temp_hvIR_clients_array['Created'] = date($functions::DATE_TIME_FORMAT_US, $hv_status_info['time']);
                    $temp_hvIR_clients_array['Duration'] = $this->get_duration($hv_status_info['time']);
                }
                /*
                 * To be implemented in phase 2
                 $temp_hvIR_clients_array['Resources'] = 'NeedsCoreWork';
                */

                $hvIR_clients['data'][] = $temp_hvIR_clients_array;
            }
        }
        return $hvIR_clients;
    }

    private function get_virtual_clients_details($virtualID, $type, $sid)
    {
        $details = array();

        switch ($type) {
            case VirtualClients::WIR_FILTER:
                $details = $this->get_wir_clients_details($virtualID, $sid);
                break;
            case VirtualClients::VMWARE_IR_FILTER:
                $temp_vm_details = $this->get_vm_ir_clients($sid);
                $temp_vm_data = $temp_vm_details['data'];
                foreach ($temp_vm_data as $vmwareDetails) {
                    if ($vmwareDetails['virtual_id'] === $virtualID) {
                        $details = $vmwareDetails;
                    }
                }
                break;
            case VirtualClients::HYPERV_IR_FILTER:
                $temp_hv_details = $this->get_hv_ir_clients($sid);
                $temp_hv_data = $temp_hv_details['data'];
                foreach ($temp_hv_data as $hvDetails) {
                    if ($hvDetails['virtual_id'] === $virtualID) {
                        $details = $hvDetails;
                    }
                }
                break;
        }

        return $details;
    }

    private function get_wir_clients_details($virtualID, $sid)
    {
        $wir_details = array();

        $client = $this->BP->get_virtual_client($virtualID, $sid);
        if ($client !== false) {
            if (isset($client['real_client_id'])) {
                $clientID = $client['real_client_id'];
                $clientInfo = $this->BP->get_client_info($clientID, $sid);
                if ($clientInfo !== false) {
                    $clientName = $clientInfo['name'];
                }
            } else {
                $clientID = -1;
                $clientName = 'dummy';
            }
            $temp_real_client_details = array();
            $temp_real_client_details['real_client_id'] = $clientID;
            $temp_real_client_details['real_client_name'] = $clientName;
            $temp_real_client_details['real_processors'] = $client['processors'];
            $temp_real_client_details['real_memory'] = $client['memory'];

            if (count($client['disks']) > 0 || isset($client['boot_disk'])) {
                $disks = $client['disks'];
                $i = 0;
                $s = '';
                if (isset($client['boot_disk']) && $client['boot_disk'] != 0) {
                    $s .= 'Boot Disk ' . $i . ': ' . $client['boot_disk'] . ' GB';
                    $sep = ', ';
                    $i++;
                } else {
                    $s = '';
                    $i = 0;
                    $sep = '';
                }
                foreach ($disks as $disk) {
                    $s .= $sep . 'Disk ' . $i . ': ' . $disk . ' GB';
                    $sep = ', ';
                    $i++;
                }
                $temp_real_client_details['real_disks'] = $s;
            }

            $wir_details['real_client'][] = $temp_real_client_details;

            $temp_virtual_client_details = array();
            $temp_virtual_client_details['virtual_id'] = $virtualID;
            if (isset($client['vm_name'])) {
                $temp_virtual_client_details['VMName'] = $client['vm_name'];
            }
            if (isset($client['processors'])) {
                $temp_virtual_client_details['Processors'] = $client['processors'];
            }
            if (isset($client['memory'])) {
                $temp_virtual_client_details['Memory'] = $client['memory'];
            }
            $temp_virtual_client_details['Included_Volumes'] = $client['incl_vols'];
            if (isset($client['excl_vols'])) {
                $temp_virtual_client_details['Excluded_Volumes'] = $client['excl_vols'];
            }
            $wir_details['virtual_client'] = $temp_virtual_client_details;
        }

        return $wir_details;
    }

    private function get_virtual_clients_backup_history($virtualID, $sid)
    {
        $backup_history = array();

        $result = $this->BP->get_last_virtual_restore($virtualID, $sid);
        if ($result !== false) {
            if (count($result) > 0) {
                foreach ($result as $backup) {
                    $last_backup = $this->displayBackupItem($backup);
                    $backup_history['Last'][] = $last_backup;
                }
            }
            $result = $this->BP->get_virtual_restore_backlog($virtualID, $sid);
            if ($result !== false) {
                foreach ($result as $backup) {
                    $pending_backup = $this->displayBackupItem($backup);
                    $backup_history['Pending'][] = $pending_backup;
                }
            }
        }
        return $backup_history;
    }

    private function get_virtual_clients_state($virtualID, $sid)
    {
        $state = array();
        $state = $this->BP->get_virtual_client_state($virtualID, $sid);
        return $state;
    }

    private function displayBackupItem($backup)
    {
        require_once('function.lib.php');
        $functions = new Functions($this->BP);

        $backup_item = array();

        $day = date($functions::DATE_FORMAT_US, $backup['start_time']);
        $time = date($functions::TIME_FORMAT_12H, $backup['start_time']);

        // Check to see if type is set, because if the backup is already purged, it will not be.
        if (isset($backup['type'])) {
            $backupType	= $functions->getBackupTypeString($backup['type']);
        } else {
            $backupType = "unknown";
        }

        $backup_item['Type'] = $backupType;
        $backup_item['Identification'] = $backup['id'];
        $backup_item['ClientName'] = $backup['cname'];
        $backup_item['Date'] = $day;
        $backup_item['Time'] = $time;

        if (isset($backup['size'])) {
            $backup_item['Size'] = $backup['size'] . ' MB';
        }

        // If the backup has file count, note this.
        // if type is 'verify', show "N/A" rather than 0
        if (isset($backup['files'])) {
            if($backup['type'] == 'verify'){
                $backup_item['Files'] =  "N/A";
            } else {
                $backup_item['Files'] = $backup['files'];
            }
        }

        // See if backup is currently being restored (for backlog API).
        if (isset($backup['currently_running'])) {
            $backup_item['Running'] = $backup['currently_running'] == true ? '1' : '0';
        }

        return $backup_item;
    }

    private function get_duration($createdTime)
    {
        $duration = '';
        $elapsedTime = time() - $createdTime;

        // get the days
        $days = intval(intval($elapsedTime) / (3600*24));
        if($days> 0) {
            $duration .= $days. 'd ';
        }

        // get the hours
        $hours = (intval($elapsedTime) / 3600) % 24;
        if($hours > 0) {
            $duration .= $hours . 'h ';
        }

        // get the minutes
        $minutes = (intval($elapsedTime) / 60) % 60;
        if($minutes > 0) {
            $duration .= $minutes . 'm';
        }

        //less than a minute
        if ( !($days > 0 || $hours > 0 || $minutes > 0)) {
            $duration .= '<1m';
        }
        return $duration;
    }

    public function post($data, $sid)
    {
        $result = array();
        $clientInfo = array();
        if (array_key_exists('client', $data)) {
            $clientData = $data['client'];
            $clientInfo['real_client_id'] = intval($clientData['ClientID']);
            $clientInfo['processors'] = intval($clientData['Processors']);
            $clientInfo['memory'] = intval($clientData['Memory']);
            $clientInfo['efi'] = $clientData['efi'];
            $clientInfo['disks'] = $this->getIDs($clientData['Disks']);
            $clientInfo['boot_disk'] = intval($clientData['Boot']);
            $clientInfo['incl_vols'] = explode(",", $clientData['Include']);
            if (array_key_exists('Exclude', $clientData)) {
                $clientInfo['excl_vols'] = explode(",", $clientData['Exclude']);
            }
            if (array_key_exists('instances', $clientData)) {
                $clientInfo['instances'] = $this->getIDs($clientData['instances']);
            }
            if (array_key_exists('AuditVerify', $clientData)) {
                $clientInfo['audit_verify'] = intval($clientData['AuditVerify']);
            }
            $clientInfo['hypervisor_info'] = $clientData['hypervisor_info'];
            $hypervisorInfo = $clientData['hypervisor_info'];
            if (array_key_exists('hyperv_host', $hypervisorInfo)) {
                $clientInfo['hypervisor_info']['hyperv_host'] = intval($hypervisorInfo['hyperv_host']);
            }
            $clientInfo['network_info'] = $clientData['network_info'];
        }
        $support = $this->BP->virtual_clients_supported($sid);
        if ($support !== -1) {
            if ($support !== false) {
                $virtualID = false;
                if (array_key_exists('hyperv_host', $clientInfo['hypervisor_info'])) {
                    if (in_array('Hyper-V host', $support)) {
                        $virtualID = $this->BP->save_virtual_client($clientInfo, $sid);
                    } else {
                        $error = "Windows Failover Virtualization is not supported on Hyper-V host.";
                    }
                } elseif (array_key_exists('esx_host', $clientInfo['hypervisor_info'])) {
                    if (in_array('VMware host', $support)) {
                        $virtualID = $this->BP->save_virtual_client($clientInfo, $sid);
                    } else {
                        $error = "Windows Failover Virtualization is not supported on VMware host.";
                    }
                }
                if ($virtualID !== false) {
                    $result['result'][]['id'] = $virtualID;
                } else {
                    $result = $virtualID;
                }
            } else {
                $result = "Windows Failover Virtualization is not supported.";
            }
        }
        return (isset($error) ? $error : $result);
    }

    private function getIDs($ids)
    {
        $idArray = array();
        $stringsArray = explode(",", $ids);
        $i = 0;
        foreach ($stringsArray as &$id) {
            $idArray[$i++] = intval($id);
        }
        return $idArray;
    }

    public function put($which, $data, $sid)
    {
        $result = array();
        $clientInfo = array();
        if ($which == -1) {
            $result = "You must provide a virtual client ID.";
        } else {
            if (is_string($which[0])) {
                switch ($which[0]) {
                    case 'audit':
                        if (isset($which[1]) && is_string($which[1]) ) {
                            switch($which[1]) {
                                case 'start':
                                    if (isset($which[2])) {
                                        $virtualID = intval($which[2]);
                                        if ($this->checkValidState($virtualID, $sid)) {
                                            $result = $this->BP->audit_virtual_client($virtualID, true);
                                        }
                                    }
                                    break;
                                case 'stop':
                                    if (isset($which[2])) {
                                        $virtualID = intval($which[2]);
                                        if ($this->checkValidState($virtualID, $sid)) {
                                            $result = $this->BP->audit_virtual_client($virtualID, false);
                                        }
                                    }
                                    break;
                            }
                        }
                        break;
                    case 'live':
                        if (isset($which[1]) && is_string($which[1]) ) {
                            switch($which[1]) {
                                case 'start':
                                    if (isset($which[2])) {
                                        $virtualID = intval($which[2]);
                                        if ($this->checkValidState($virtualID, $sid)) {
                                            $result = $this->BP->run_virtual_client($virtualID, true);
                                        }
                                    }
                                    break;
                                case 'stop':
                                    if (isset($which[2])) {
                                        $virtualID = intval($which[2]);
                                        if ($this->checkValidState($virtualID, $sid)) {
                                            $result = $this->BP->run_virtual_client($virtualID, false);
                                        }
                                    }
                                    break;
                            }
                        }
                        break;
                    case 'restores':
                        if (isset($which[1]) && is_string($which[1]) ) {
                            switch($which[1]) {
                                case 'start':
                                    if (isset($which[2])) {
                                        $virtualID = intval($which[2]);
                                        if ($this->isValid($virtualID, $sid)) {
                                            $result = $this->BP->disable_virtual_restores($virtualID, false, $sid);
                                        }
                                    }
                                    break;
                                case 'stop':
                                    if (isset($which[2])) {
                                        $virtualID = intval($which[2]);
                                        $result = $this->BP->disable_virtual_restores($virtualID, true, $sid);
                                    }
                                    break;
                            }
                        }
                        break;
                }
            } else {
                if (array_key_exists('client', $data)) {
                    $clientData = $data['client'];
                    $clientInfo['id'] = intval($which[0]);
                    $clientInfo['processors'] = intval($clientData['Processors']);
                    $clientInfo['memory'] = intval($clientData['Memory']);
                    if (array_key_exists('AuditVerify', $clientData)) {
                        $clientInfo['audit_verify'] = intval($clientData['AuditVerify']);
                    }
                }
                $result = $this->BP->save_virtual_client($clientInfo, $sid);
            }
        }
        return $result;
    }

    private function checkValidState($virtualID, $sid)
    {
        $valid = true;
        $state = $this->get_virtual_clients_state($virtualID, $sid);
        $currentState = $state['current_state'];
        if ($currentState === VirtualClients::STATE_NEW) {
            $valid = false;
        } elseif ($currentState === VirtualClients::STATE_HALTED) {
            $valid = false;
        } else {
            $valid = true;
        }
        return $valid;
    }

    private function isValid($virtualID, $sid)
    {
        $state = $this->get_virtual_clients_state($virtualID, $sid);
        $valid = $state['valid'];
        return $valid;
    }

    public function delete($which, $data, $sid)
    {
        $result = array();
        if ($which == null) {
            $result = "You must provide a virtual client ID.";
        } else {
            $id = intval($which);
            if (array_key_exists('type', $data)) {
                switch ($data['type']) {
                    case 'wir':
                        $delete_from_hypervisor = intval($data['deleteFromHypervisor']);
                        $result = $this->BP->delete_virtual_client($id, $delete_from_hypervisor, $sid);
                        break;
                    case 'vm_ir':
                        $force = intval($data['force']);
                        $result = $this->BP->vmware_ir_destroy($id, $force, $sid);
                        break;
                    case 'hv_ir':
                        $force = intval($data['force']);
                        $result = $this->BP->hyperv_ir_destroy($id, $force, $sid);
                        break;
                }
            } else {
                $result = "Specify the type.";
            }
        }
        return $result;
    }
}
?>