<?php
// defines BP, xml, action, cookie
require 'header.php';

// defines functions for URL parsing.
require('urloptions.php');

// defines functions for backup status processing.
require('basebackupstatus.php');

define('SYNC_XML_TAG', 'SecureSyncStatus');
define('BACKUP_XML_TAG', 'Backup');

switch ($action) {
		/*
				Report Types match the following CMC strings.
				
				public static const	INFO:String		= "info";
				public static const FAILURE:String	= "failure";
				public static const BACKUP:String	= "backup";
				public static const DEVICE:String	= "device";
				public static const SQL:String		= "sql";
				public static const SYNC:String		= "sync";
		*/
	case "failure":
	case "backup":
	case "sql":
	case "sync":	
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
			$cid = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;
			
			//
			// Default to retrieve all backup types.
			//
			$backupTypes = $GLOBAL_BACKUP_REPORT_TYPES;
			
			$result_format = array();
			$attributes = getTimeArguments();
			
			// Set the XML tag
			$tag = BACKUP_XML_TAG;
					
			// 
			//   Override the default backup types for SQL and securesync reports.
			//
			if ($action == 'sql') {
				$backupTypes = $GLOBAL_SQL_TYPES;
			} else if ($action == 'sync') {
				//$backupTypes = $GLOBAL_SS_TYPES;
				//'sync' is passed in by Restore From Vault tool.  Only need master and differential backups.
				$backupTypes = array("securesync master", "securesync differential");
				$tag = SYNC_XML_TAG;
			}
			
			$result_format = array_merge($result_format, $attributes, array('type' => $backupTypes));
						
			//
			// Loop through all systems.  Will be only one in the list for a DPU, multiple systems on a Vault.
			//
			$systems = $BP->get_system_list();
		
			$xml->push('BackupInformation');
			$originalFormat = $result_format;
			$localID = $BP->get_local_system_id();
			$bVault = $BP->local_system_is_vault();
			
			foreach ($systems as $systemID => $systemName) {
				if ($systemID == $dpuID) { // only return the clients for this system, removed this --> || $dpuID == $localID) {
					//
					// If no client passed in, loop through all clients on the system.
					//
					if ($cid == -1) {
						$clients = $BP->get_client_list($systemID);
					} else {
						$clientName = getClientName($BP, $cid, $systemID);
						$clients = array($cid => $clientName);
					}
					
					// If this system is a vault and we are looking at this system, set clients and devices to
					// NONE and continue to next system.
					if ($bVault && ($systemID == $localID)) {
						$clients = array();
						$deviceList = array();
					} else {
						//
						// Build the device list once per system and pass to the XML construction function.
						//
						$deviceList = $BP->get_device_list($systemID);
					}
						
					foreach ($clients as $clientID => $clientName) {
						$result_format = array_merge($originalFormat, array('system_id' => $systemID, 'client_id' => $clientID));
						$backupStatusArray = $BP->get_backup_status($result_format);
						if ($backupStatusArray !== false && count($backupStatusArray) > 0) {
							$resultArray = array();
							$lastMaster = 0;
							$lastIncremental = 0;
							foreach ($backupStatusArray as $backupStatus) {
								if ($backupStatus['status'] == STATUS_SUCCESS || $backupStatus['status'] == STATUS_WARNING) {
									if ($backupStatus['type'] == 'securesync master' &&
										$backupStatus['start_time'] > $lastMaster) {
										$resultArray['lastmaster'] = $backupStatus;
										$lastMaster = $backupStatus['start_time'];
									} else if ($backupStatus['type'] == 'securesync differential' &&
										$backupStatus['start_time'] > $lastIncremental) {
										$resultArray['lastIncremental'] = $backupStatus;
										$lastIncremental = $backupStatus['start_time'];
									}
								}
							}
							
							//
							// If the last Master is later than the last Incremental, only report the last Master.
							//
							if (array_key_exists('lastIncremental', $resultArray)) {
								if ($lastMaster > $lastIncremental) {
									unset($resultArray['lastIncremental']);
								}
							}
								
							foreach ($resultArray as $result) {
								createBaseBackupStatusXML($xml, $result, $deviceList,
															  $systemID, $systemName,
															  $clientID, $clientName, $tag);
							}
						}
					}
				}
			}
			$xml->pop(); // BackupInformation	
			echo($xml->getXml());
			break;
			
		default:
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
			$result_format = array('system_id' => $dpuID);
			$backupType = NULL;
			if (isset($_GET['bsync'])) {
				$backupType = getTypeForDetails(true, $action);
				$result_format['sync_msg'] = 1;
			} else {
				$backupType = getTypeForDetails(false, $action);
			}
			if ($backupType != NULL) {
				$result_format['type'] = $backupType;
			}
			if (isset($_GET['bid'])) {
				$backupID = (int)$_GET['bid'];
				$backupIDs = array($backupID);
				$attributes = array('backup_ids' => $backupIDs);
			} else {
				$attributes = getTimeArguments();
			}
			$result_format = array_merge($result_format, $attributes);
			$backupStatusArray = $BP->get_backup_status($result_format);
			if ($backupStatusArray !== false) {
				if (count($backupStatusArray) > 0) {
					$result = $backupStatusArray[0];
					
					if (isset($_GET['bid'])) {
						$backupID = (int)$_GET['bid'];
						$backupIDs = array($backupID);
						$attributes = array('backup_ids' => $backupIDs, 'system_id' => $dpuID);
						$result_format = $attributes;
						$backupArray = $BP->get_backup_info($result_format);
						if ($backupArray !== false && count($backupArray) > 0) {
							$result['output'] = $backupArray[0]['output'];
							$result['command'] = $backupArray[0]['command'];
							$result['purgeable'] = $backupArray[0]['purgeable'];
						}
					}	
					
					$xml->push('root');
					createBaseBackupStatusXML($xml, $result, NULL,
											  $dpuID, "unknown",
											  $result['client_id'], "unknown");
					$xml->pop();
				}
			} else {
				$BP->buildResult($xml, false);
			}
			echo($xml->getXml());
			break;
}

$BP->destroyCookie();
?>