<?php
// defines BP, xml, action, cookie
require 'header.php';

switch ($action) {				
	case "list":
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
			
			//
			// Loop through all systems.  Will be only one in the list for a DPU, multiple systems on a Vault.
			//
			$systems = $BP->get_system_list();
			
			$xml->push('root');
			$localID = $BP->get_local_system_id();
			$bVault = $BP->local_system_is_vault();
			
			$bManager = $BP->local_system_is_mgr();
			$bMDPU = isset($_GET['mdpu']) ? true : false;
			
			foreach ($systems as $systemID => $systemName) {
				if ($systemID == $dpuID || $dpuID == $localID) {
					// If this system is a vault and we are looking at this system, skip it and go to the next.
					if ($bVault && ($systemID == $localID)) {
						;
					// If this system is a manager and we are not running on the MDPU node nor are
					// not looking at the selected system, skip it.
					} else if (!$bVault && $bManager && !$bMDPU && ($systemID != $dpuID)) {
						;
					} else {						
						$retentionInfo = $BP->get_retention_info($systemID);
						// If there is only system in the list, error out. Otherwise, continue and put the
						// error in the grid.
						if ($retentionInfo === false) {
							createErrorXML($xml, $BP, $systemName);
						} else {
							// Get the max_backup value if available, using a cmc_command.  If available, it
							// will be returned as 'nnn GB'.  Use split to tokenize and get the number (nnn).
							// If the count of tokens returned by the command is not 2 (nnn GB), then don't set
							// maxBackups -- it is not a valid format and could be an informational
							// message returned to the user by the cmc_command script.
							$maxBackups = -1;
							$output = $BP->run_command("Maximum Backups", "", $systemID);
							if ($output !== false) {
								$a = split(' ', $output);
								if (count($a) == 2) {
									$maxBackups = $a[0];
								}
							}
							$vfSpaceInfo = $BP->get_virtual_failover($systemID);		
							buildRetentionXML($xml, $retentionInfo, $vfSpaceInfo, $systemName, $maxBackups);
						}
					}
				}
			}	
			$xml->pop(); //root
			echo($xml->getXml());
			break;

}

//
// This function walks through the retention information and outputs the XML structure.
//
function buildRetentionXML($xml, $retentionInfo, $vfSpaceInfo, $systemName, $maxBackups)
{
	// Check to see if user passed in the available capacity, i.e., it is not -1.
	// Use it if set; otherwise use the 'available' field in the $retentionInfo array.
	$availableCapacity = toNumberString($maxBackups);
	if ($availableCapacity == -1) {
		$availableCapacity = toNumberString($retentionInfo['available']);
	}
	
	$xml->push("Retention", array("Name" => $systemName));
	$xml->element('Available', $availableCapacity);
	if (array_key_exists('used', $retentionInfo)) {
		$xml->element('Used', toNumberString($retentionInfo['used']));
	}
	if (array_key_exists('allowed', $vfSpaceInfo)) {
		$xml->element('VFUsed', $vfSpaceInfo['allowed']);
	}
	if (array_key_exists('message', $retentionInfo)) {
		$xml->element('Error', $retentionInfo['message']);
	}
	if (array_key_exists('clients', $retentionInfo)) {
		$clientList = $retentionInfo['clients'];
		
		foreach ($clientList as $client) {
			$name = array_key_exists('name', $client) ? $client['name'] : UNKNOWN_ELEMENT;
			$xml->push("Client", array("Name" => $name));
			if (array_key_exists('last_master', $client)) {
				$xml->element('LastMaster', toNumberString($client['last_master']));
			}
			if (array_key_exists('last_exchange', $client)) {
				$xml->element('LastExchange', toNumberString($client['last_exchange']));
			}
			if (array_key_exists('last_baremetal', $client)) {
				$xml->element('LastBareMetal', toNumberString($client['last_baremetal']));
			}
			if (array_key_exists('num_masters', $client)) {
				$xml->element('NumMasters', $client['num_masters']);
			}
			if (array_key_exists('num_incrementals', $client)) {
				$xml->element('NumIncrementals', $client['num_incrementals']);
			}
			if (array_key_exists('num_baremetals', $client)) {
				$xml->element('NumBareMetals', $client['num_baremetals']);
			}
			if (array_key_exists('sql', $client)) {
				$sqlInfo = $client['sql'];
				foreach ($sqlInfo as $sql) {
					$instanceName = array_key_exists('instance_name', $sql) ? $sql['instance_name'] : UNKNOWN_ELEMENT;
					$databaseName = array_key_exists('database_name', $sql) ? $sql['database_name'] : UNKNOWN_ELEMENT;
					$xml->push("SQL", array("Instance" => $instanceName, "Database" => $databaseName));
					$xml->element('LastFull', toNumberString($sql['last_full']));
					$xml->pop(); // SQL
				}
			}	
			if (array_key_exists('vmware', $client)) {
				$vmInfo = $client['vmware'];
				foreach ($vmInfo as $vm) {
					$serverName = array_key_exists('esx_name', $vm) ? $vm['esx_name'] : UNKNOWN_ELEMENT;
					$vmName = array_key_exists('vm_name', $vm) ? $vm['vm_name'] : UNKNOWN_ELEMENT;
					$xml->push("VMware", array("Server" => $serverName, "VM" => $vmName));
					$xml->element('LastFull', toNumberString($vm['last_full']));
					$xml->pop(); // VMware
				}
			}
			if (array_key_exists('hyperv', $client)) {
				$hypervInfo = $client['hyperv'];
				foreach ($hypervInfo as $hyperv) {
					$vmName = array_key_exists('vm_name', $hyperv) ? $hyperv['vm_name'] : UNKNOWN_ELEMENT;
					$guid = array_key_exists('guid', $hyperv) ? $hyperv['guid'] : UNKNOWN_ELEMENT;
					$xml->push("HyperV", array("VM" => $vmName, "guid" => $guid));
					$xml->element('LastFull', toNumberString($hyperv['last_full']));
					$xml->pop(); // HyperV
				}
			}
			if (array_key_exists('oracle', $client)) {
				$info = $client['oracle'];
				foreach ($info as $item) {
					$databaseName = array_key_exists('instance_name', $item) ? $item['instance_name'] : UNKNOWN_ELEMENT;
					$xml->push("Oracle", array("Database" => $databaseName));
					$xml->element('LastFull', toNumberString($item['last_full']));
					$xml->pop(); // Oracle
				}
			}
			if (array_key_exists('sharepoint', $client)) {
				$info = $client['sharepoint'];
				foreach ($info as $item) {
					$instanceName = array_key_exists('instance_name', $item) ? $item['instance_name'] : UNKNOWN_ELEMENT;
					$xml->push("SharePoint", array("Instance" => $instanceName));
					$xml->element('LastFull', toNumberString($item['last_full']));
					$xml->pop(); // SharePoint
				}
			}
			if (array_key_exists('ucssp', $client)) {
				$info = $client['ucssp'];
				foreach ($info as $item) {
					$instanceName = array_key_exists('instance_name', $item) ? $item['instance_name'] : UNKNOWN_ELEMENT;
					$xml->push("UCS_Service_Profile", array("Instance" => $instanceName));
					$formattedString = is_int($item['last_full']) ? $item['last_full'] : sprintf("%.4f", $item['last_full']);
					$xml->element('LastFull', $formattedString);
					$xml->pop(); // ServiceProfile
				}
			}
			if (array_key_exists('ndmp', $client)) {
				$info = $client['ndmp'];
				foreach ($info as $item) {
					$volumeName = array_key_exists('instance_name', $item) ? $item['instance_name'] : UNKNOWN_ELEMENT;
					$xml->push("Volume", array("Volume" => $volumeName));
					$xml->element('LastFull', toNumberString($item['last_full']));
					$xml->pop(); //Volume
				}
			}

			$xml->pop(); // client
		}
	}	
	$xml->pop(); // Retention
}

//
// This function logs an error if we cannot retrieve retention info.
function createErrorXML($xml, $BP, $systemName)
{

	global $Log;
	$message = " Could not get retention information for system: " . $systemName;
	$message = $BP->getError() . ":" . $message;
	$Log->writeError($message, true);
	
	$xml->push("Retention", array("Name" => $systemName));						
	$xml->element('Error', $BP->getError());
	$xml->pop(); // Retention
}

$BP->destroyCookie();
?>