<?php
// defines BP, xml, action, cookie
require 'header.php';

define("MANAGED_DPU", "Managed DPU");
define("VAULTING_DPU", "DPU Configured for Vaulting");
define("REPLICATING_DPU", "Replication Source");

$localID = $BP->get_local_system_id();
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $localID;

// initialize host options array.
$hostOptions = array();

switch ($action) {
	case "list":	
			$bValid = true;
			if (!isset($_GET['sid']) || $dpuID == $BP->get_local_system_id()) {
				$hostInfo = $BP->get_hostname();
			} else {
				$hostInfo = $BP->get_hostname($dpuID);
			}
			if ($hostInfo === false) {
				$bValid = false;
			} else {
				$xml->push("root");
				$xml->push("Host", array("Name" => $hostInfo['name']));
				if (array_key_exists('long_name', $hostInfo))
					$xml->element('Longname', $hostInfo['long_name']);
				$xml->pop(); // hostname
				$xml->pop(); //root
			}
			if (!$bValid) {
				$xml->clear(false);
				$BP->buildResult($xml, false);
			}
			echo($xml->getXml());
			break;
					
	case "change":
			// if changing, specify oldname and new name.		
			$newName = $_GET['newname'];
			$options = array ('long_name' => 'longname');
			$bOptions = array('keep_alias' => 'alias');
			$hostOptions = buildOptions($hostOptions, $options, $bOptions);
			
			// see if replicating and stop here if so.
			$checkReplicationMessage = checkReplication($BP, $dpuID);
			if ($checkReplicationMessage === NULL) {
					
				// Retrieve the existing hostname, name field should be set on the return.
				$originalName = NULL;
				$errorString = NULL;
				$result = $BP->get_hostname($dpuID);
				if ($result !== false) {
					$hostnameArray = $result;
					if (array_key_exists('name', $hostnameArray)) {
						$originalName = $hostnameArray['name'];
					}
					
					// change hostname.
					if ($result !== false) {
						$result = $BP->change_hostname($newName, $hostOptions, $dpuID);
					}
					
					// if hostname changed, change client name.  Do this after hostname change or it will fail.
					if ($result !== false) {
						$resultClient = updateClientName($BP, $dpuID, $originalName, $newName);
						
						// change of client name successful, continue, otherwise, fall back.
						if ($resultClient !== false) {
							// update system name.
							if (isset($_GET['sysname']) && ($_GET['sysname'] == '1')) {
								$result = updateSystemName($BP, $dpuID, $originalName, $newName);
							}
		
							// If modifying a remote entry ($dpuID != local system ID), change host file and system.host.
							// This code will not be executed as this functionality is not allowed by the RRC.
							if ($result !== false) {
								$localID = $BP->get_local_system_id();
								if ($BP->local_system_is_vault() && $dpuID != $localID) {
									$result = updateManagedDPU($BP, $dpuID, $originalName, $newName);
								}
							}
						} else {
							
							$result = false;
							if ($errorString == NULL) {
								$errorString = $BP->getError();
								global $Log;
								$Log->writeVariable("Error when saving client info for hostname change: $errorString");
							}
								
							// change longname back if changed earlier.
							$hostOptions = array();
							if (isset($hostnameArray['long_name'])) {
								$hostOptions['long_name'] = $hostnameArray['long_name'];
							}
							$hostResult = $BP->change_hostname($originalName, $hostOptions, $dpuID);
							$messageAssign = "Check your network to ensure your IP is configured, " .
											 "and make sure you have a registered client that matches your hostname.";
							if ($hostResult !== false) {
								$errorString = 	"Error changing registered client to $newName.  $messageAssign\n\n" .
											   	"Hostname will remain $originalName.\n";
							} else {
								$errorString = 	"Error changing registered client to $newName.  $messageAssign\n\n" .
												"Error reverting hostname back to $originalName: " . $BP->getError();
							}
						}
					}
					
					// report defined error if passed in, othwerwise get bpl error.
					if ($errorString == NULL) {
						$BP->buildResult($xml, $result);
					} else {
						$BP->buildResult($xml, $result, $errorString);
					}
					
				} else {
					$BP->buildResult($xml, $result);
				}
			} else {
				$BP->buildResult($xml, false, $checkReplicationMessage);
			}

			echo($xml->getXml());
			break;
}

//
// Check to see if replicating or vaulting, and return message if so.
//
function checkReplication($BP, $dpuID) {
	global $localID;
	$message = NULL;
	
	$result = ($dpuID == $localID) ? $BP->local_system_is_replicating() : $BP->is_replication_configured($dpuID);
	if (is_string($result) || $result === true) {
		$message = "This system is replicating, so its hostname cannot be changed.";
	} else {
		$result = ($dpuID == $localID) ? $BP->local_system_is_vaulting() : $BP->is_vaulting_configured($dpuID);
		if (is_string($result) || $result === true) {
			$message = "This system is vaulting, so its hostname cannot be changed.";
		}
	}
	
	return $message;
}

//
// Updates the client name to the new name.
//
function updateClientName($BP, $dpuID, $originalName, $newName) {
	
	$result = false;
	$clientID = getClientID($BP, $originalName, $dpuID);
	if ($clientID != -1) {
		$clientInfo = $BP->get_client_info($clientID, $dpuID);
		$clientUpdate = array();
		if ($clientInfo !== false) {
			$clientUpdate['id'] = $clientInfo['id'];
			$clientUpdate['name'] = $newName;
			if ( $clientInfo['os_type_id'] == '40' && isset($clientInfo['generic_property']) ) {
				$result = $BP->save_rae_client_info($clientUpdate, $dpuID);
			} else {
				$result = $BP->save_client_info($clientUpdate, $dpuID);
			}
		}
	} else {
		global $errorString;
		$errorString = "No client $originalName found.  Please create client with name $originalName prior to changing hostname.";
	}
	return $result;
}

//
// Update the name in the navigational tree.
//
function updateSystemName($BP, $dpuID, $originalName, $newName) {
	$result = false;
	
	$systemUpdate = array();
	$systemInfo = $BP->get_system_info($dpuID);
	if ($systemInfo !== false) {
		$systemUpdate['id'] = $systemInfo['id'];
		$systemUpdate['name'] = $newName;
		$result = $BP->save_system_info($systemUpdate);
	}

	// If on a cross-vault, look for the crossvault DPU by name or role.
	$crossVaultDPU = $originalName . '.dpu';
	$systems = $BP->get_system_list();
	if ($systems !== false) {
		foreach ($systems as $systemID => $systemName) {
			$systemInfo = $BP->get_system_info($systemID);
			if ($systemInfo !== false) {
				$role = $systemInfo['role'];
				$hostname = $systemInfo['host'];
				// A cross-vault DPU will have hostname .dpu AND
				//      it be named '.dpu' or have a role that it is being managed or is vaulting.
				// should be only 1 (if 1).  Find it, change it and leave.
				if ($hostname === 'localhost') {
					if ($systemName == $crossVaultDPU || $role === MANAGED_DPU || $role === VAULTING_DPU || $role === REPLICATING_DPU) {
						$systemUpdate = array();
						$systemUpdate['id'] = $systemInfo['id'];
						$systemUpdate['name'] = str_replace($originalName, $newName, $systemName);
						$result = $BP->save_system_info($systemUpdate);
						break;
					}
				}
			}
		}
	}
	return $result;
}

//
// Update the host and system name of remote systems.
//
function updateManagedDPU($BP, $dpuID, $originalName, $newName) {
	
	$result = true;
	$hosts = $BP->get_host_list();
	if ($hosts !== false) {
		foreach ($hosts as $name) {
			if ($name == $originalName) {
				$hostInfo = $BP->get_host_info($name);
				if ($hostInfo !== false) {
					$hostInfo['name'] = $newName;
					$result = $BP->save_host_info($hostInfo);
				} else {
					$result = false;
				}
				break;
			}
		}
		// update systems.host for remote system name.
		if ($result !== false) {
			$systemInfo = $BP->get_system_info($dpuID);
			if ($systemInfo !== false) {
				$systemInfo['host'] = $newName;
				$result = $BP->save_system_info($systemInfo);
			}
		}
	} else {
		$result = false;
	}
	return $result;
}

$BP->destroyCookie();
?>
