<?php
// defines BP, xml, action, cookie
require 'header.php';

// general backup list operations.
require 'listsbase.php';
$ListBase = new ListBase();

// Initialize variables to be used in copy and save.
$optionInfo = array();
$bValid = true;
$bCopy = false;

switch ($action) {

	case "list":
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
			$ListBase->buildOptionsList($BP, $xml, $dpuID);
			echo($xml->getXml());
			break;
					
	case "load":
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();						
			if (($id = $ListBase->getID($BP, 'get_option_lists', $dpuID)) == -1) {
				$BP->buildResult($xml, false, "Error: cannot find option list for loading.");
			} else {
				$optionList = $BP->get_options($id, $dpuID);
				if ($optionList === false) {
					$BP->buildResult($xml, false);
				} else {
					$xml->push("root");
					createOptionXML($xml, $optionList);					
					$xml->pop(); // root
				}
			}
			echo($xml->getXml());
			break;
			
	case "copy":
			$bValid = false;
			$bCopy = true;
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
			$originalID = (int)$_GET['oid'];	
			$optionList = $BP->get_option_lists($dpuID);			
			if ($optionList === false) {
				$BP->buildResult($xml, false);
			} else {
				foreach ($optionList as $options) {
					if ($options['id'] == $originalID) {
						$optionInfo = $options;
						unset($optionInfo['id']);
						$optionDefinitions = $BP->get_options($options['id'], $dpuID);
						$optionInfo['options'] = $optionDefinitions;
						$bValid = true;
						break;
					}
				}
			}		
			// Fall through to save the constructed options copy.
					
	case "save":
			if ($bValid) {
				$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
				$id = $ListBase->getID($BP, 'get_option_lists', $dpuID);
				if ($id == -1) {
					if (isset($_GET['name'])) {
						$optionInfo = array_merge($optionInfo, array("name" => $_GET['name']));
					} else {
						$BP->buildResult($xml, false, "Error: must specify either name or id of options to save.");
						$bValid = false;
					}
				} else {
					$optionInfo = array("id" => $id);
					if (isset($_GET['name'])) {
						$optionInfo = array_merge($optionInfo, array("name" => $_GET['name']));
					}
				}
			}
			if ($bValid) {
				if ($bCopy) {
					unset($optionInfo['id']);
					$optionInfo['read_only'] = false;
				} else {											
					$options = array ('description' => 'desc',
									  'type' => 'stype',
									  'family' => 'family');
					$bOptions = array('read_only' => 'ro');		
					$optionInfo = buildOptions($optionInfo, $options, $bOptions);
					
					// specific options
					$optionList = array();
					$options = array(	'dev_name' => 'dev',
										'read_locking' => 'lk',
										'comment' => 'co',
										'raw_device' => 'raw',
										'raw_device_command' => 'rcmd',
										'before_command' => 'bcmd',
										'after_command' => 'acmd');
					$bOptions = array(	'speed_option' => 'speed',
										'make_catalog' => 'cat');
					$optionList = buildOptions($optionList, $options, $bOptions);
					
					$iOptions = array(	'depth' => 'depth',
										'verify_level' => 'vl');									
					$optionList = addIntegerOptions($optionList, $iOptions);
					
					$optionInfo['options'] = $optionList;
					
				}
				$ListBase->buildListResult($xml, $BP->save_options($optionInfo, $dpuID));
			}
			echo($xml->getXml());			
			break;
				
	case "delete":
			$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
			if (($id = $ListBase->getID($BP, 'get_option_lists', $dpuID)) == -1) {
				$BP->buildResult($xml, false, "Error: cannot find identifier for deleting options list.");
			} else {
				$BP->buildResult($xml, $BP->delete_options($id, $dpuID));
			}
			echo($xml->getXml());
			break;
}

//
// This function walks the optionList returned from the load and creates the XML for each option.
//
function createOptionXML($xml, $optionList)
{
	if (array_key_exists('dev_name', $optionList)) {
		$xml->element("Device", $optionList['dev_name']);
	}

	$readLocking = array_key_exists('read_locking', $optionList) ? $optionList['read_locking'] : "none";
	$xml->element("ReadLocking", $readLocking);
	
	$speedOption = "0";
	if (array_key_exists('speed_option', $optionList)) {
		$speedOption = $optionList['speed_option'] == true ? "1" : "0";
	}
	$xml->element("SpeedOption", $speedOption);
	
	$verifyLevel = array_key_exists('verify_level', $optionList) ? $optionList['verify_level'] : 0;
	$xml->element("VerifyLevel", $verifyLevel);
	
	$depth = array_key_exists('depth', $optionList) ? $optionList['depth'] : "0";
	$xml->element("Depth", $depth);
	
	if (array_key_exists('comment', $optionList)) {
		$xml->element("Comment", $optionList['comment']);
	}
	
	if (array_key_exists('raw_device', $optionList)) {
		// space-separated to multiple xml nodes
		$rawDevices = explode(' ', $optionList['raw_device']);
		for ($i = 0; $i < count($rawDevices); $i++) {
			$xml->element("RawDevice", $rawDevices[$i]);
		}
	}
	
	if (array_key_exists('raw_device_command', $optionList)) {
		$xml->element("RawDeviceCommand", $optionList['raw_device_command']);
	}
	
	if (array_key_exists('before_command', $optionList)) {
		$xml->element("BeforeCommand", $optionList['before_command']);
	}
	
	if (array_key_exists('after_command', $optionList)) {
		$xml->element("AfterCommand", $optionList['after_command']);
	}
	
	$catalog = "1";
	if (array_key_exists('make_catalog', $optionList)) {
		$catalog = $optionList['make_catalog'] == true ? "1" : "0";
	}
	$xml->element("MakeCatalog", $catalog);

}

$BP->destroyCookie();
?>