<?php
// defines BP, xml, action, cookie
require '../header.php';

// defines functions for master.ini processing
require $rootdir . 'bpl/baseini.php';

// parameters passed to all API's
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
//$syncType = isset($_GET['synctype']) ? $_GET['synctype'] : "SecureSync";

switch ($action) {
	case "list":
		// true - configured, false - not configured, -1 - error.
		$result = $BP->is_replication_configured($dpuID);	
		if ($result !== -1) {
			
			// true - enabled, false - not enabled, -1 - error.
			$result = $BP->is_replication_enabled($dpuID);
			if ($result !== -1) {
				$xml->push('root');
				buildReplicationProcessXML($xml, $BP, "Replication");
				$xml->element('Replicating', $result == true ? '1' : '0');
				buildReplicationConnectionXML($xml, $BP, "CMC");
				$bIsSupported = $BP->is_appconfig_supported($dpuID);
				$xml->element('ItemVaulting', $bIsSupported ? '1' : '0');
				$xml->pop();
			} else {
				$BP->buildResult($xml, false);
			}
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
						
	case "update":
		$SyncTo = $BP->get_ini_value("Replication", "SyncTo", $dpuID);
		$settings = array('SyncTo' => $SyncTo, 'VaultName' => $SyncTo);
		// Only update values passed in by the user, with no defaults (bug 17872).
		if (isset($_GET['timedelay'])) {
			$settings['TimeDelay'] = $_GET['timedelay'];
		}
		if (isset($_GET['maxconcurrent'])) {
			$settings['MaxConcurrent'] = $_GET['maxconcurrent'];
		}
		if (isset($_GET['retry'])) {
			$settings['RetryCount'] = $_GET['retry'];
		}
		if (isset($_GET['checkinterval'])) {
			$settings['CheckInterval'] = $_GET['checkinterval'];
		}
		if (isset($_GET['mailto'])) {
			$settings['ReportMailTo'] = $_GET['mailto'];
		}	
		// Block-out could be very long, pass via POST so need to decode the string.
		if (isset($_REQUEST['block'])) {
			$settings['BlockOutPeriods'] = urldecode($_REQUEST['block']);
		}
		// Do not set the following parameters directly in master.ini.
		// Each have their own APIs.
		$QueueScheme = isset($_GET['queuescheme']) ? $_GET['queuescheme'] : -1;
		$mailTime = isset($_GET['mailtime']) ? $_GET['mailtime'] : -1;
	
		$result = saveReplicationAttributes($settings, $BP, $dpuID);
		// If no error, also update replication startegy if passed in.
		if ($result != false && $QueueScheme != -1) {
			$result = changeReplicationStrategy($QueueScheme, $BP, $dpuID);
		}
		// If no error, also update master.ini with connection settings.
		if ($result !== false) {
			$result = saveIniValues($BP, $dpuID);
		}
		// If no error, also update the time that replication report is emailed out
		if ($result != false && $mailTime != -1) {
			$result = saveReplicationReportTime($mailTime, $BP, $dpuID);
		}
		
		$BP->buildResult($xml, $result);
		echo($xml->getXml());
		break;
					
	case "enable":
		$BP->buildResult($xml, $BP->enable_securesync(true, $dpuID));
		echo($xml->getXml());
		break;
		
	case "disable":
		$BP->buildResult($xml, $BP->enable_securesync(false, $dpuID));
		echo($xml->getXml());
		break;
		
	case "stop":
		// Disable future vaulting and stop current vaulting by restarting securesync.
		$result = $BP->enable_securesync(false, $dpuID);
		if ($result !== false) {
			$BP->buildResult($xml, $BP->restart_securesync($dpuID));
		} else {
			$BP->buildResult($xml, false);
		}
		echo($xml->getXml());
		break;
		
	case "restart":
		$BP->buildResult($xml, $BP->restart_securesync($dpuID));
		echo($xml->getXml());
		break;
		
	case "enabled":
		$BP->buildResult($xml, $BP->is_securesync_enabled($dpuID));
		echo($xml->getXml());
		break;
}

//
// This function constructs the XML returned for replication settings.
//
function buildReplicationProcessXML($xml, $BP, $sectionName ) {
	//displaySection($xml, $BP, $sectionName);
	global $dpuID;
	$sectionValues = $BP->get_ini_section($sectionName, $dpuID);
	
	$xml->push("Section", array("Name" => $sectionName));
	foreach ($sectionValues as $setting) {
		$name = $setting['field'];
		$description = array_key_exists('description', $setting) ? $setting['description'] : "";
		$value = $setting['value'];
		if ($name === 'Enabled') {
			$concurrent = $BP->get_ini_value('Replication', 'MaxConcurrent', $dpuID);
			$value = (int)$concurrent > 0 ? 'Yes' : 'No';
		}
		$xml->emptyelement("Field", array("Name" => $name, "Description" => $description, "Value" => $value));
	}
	$xml->pop(); // Section
}


//
// This function constructs the XML returned for securesync settings.
//
function buildReplicationConnectionXML($xml, $BP, $sectionName) {
	displaySection($xml, $BP, $sectionName);
}

function saveIniValues($BP, $dpuID) {

	$options = array ('VaultConnection' => 'vc',
					  'VaultUpstreamSpeed' => 'vu',
					  'VaultDownstreamSpeed' => 'vd',
					  'VaultBandwidth' => 'vb');				  
	$bOptions = array();		
	$connections = array();		
	$connections = buildOptions($connections, $options, $bOptions);
	
	$result = true;
	foreach ($connections as $field => $value) {
		$result = $BP->set_ini_value("CMC", $field, $value, $dpuID);
		if ($result !== true) {
			break;
		}
	}
	
	return $result;
}

function saveReplicationAttributes($settings, $BP, $dpuID) {
	
	foreach ($settings as $field => $value) {
		$result = $BP->set_ini_value("Replication", $field, $value, $dpuID);
		if ($result !== true) {
			break;
		}
	}
	
	return $result;

}

function saveReplicationReportTime($reportTime, $BP, $dpuID) {
	$result = true;
	// retrieve the original time
	$origReportTime = $BP->get_ini_value("Replication", "ReportTime", $dpuID);
	$origReportTimeSplit = explode(":", $origReportTime);
	$origReportTimeHours = isset($origReportTimeSplit[0]) ? $origReportTimeSplit[0] : -1;
	// remove leading zeros from the hour so it can be compared
	$origReportTimeHours = ltrim($origReportTimeHours, '0');
	$origReportTimeMinutes = isset($origReportTimeSplit[1]) ? $origReportTimeSplit[1] : "00";

	$reportTimeSplit = explode(":", $reportTime);
	$newReportTimeHours = isset($reportTimeSplit[0]) ? $reportTimeSplit[0] : -1;
	$newReportTimeMinutes = isset($reportTimeSplit[1]) ? $reportTimeSplit[1] : "00";
		
	// save the time only if it has been changed	
	if ($origReportTimeHours!=$newReportTimeHours || $origReportTimeMinutes!=$newReportTimeMinutes &&
			$newReportTimeHours!=-1) {
		$result = $BP->save_replication_report_time($newReportTimeHours, $newReportTimeMinutes, $dpuID);
	}
	
	return $result;
}

function changeReplicationStrategy($strategy, $BP, $dpuID) {
	$strategyName = getReplicationStrategyName($strategy);
	$result = $BP->set_replication_queue_strategy($strategyName, $dpuID);
	return $result;
}

// This function returns the name of the replication strategy
function getReplicationStrategyName($field) {
	$name = $field;		
	switch ($field) {
		case "0":
				$name = "Manual";
				break;
		case "1":
				$name = "Maximize Retention";
				break;
		case "2":
				$name = "Recency";
				break;
	}
	return $name;
}


$BP->destroyCookie();
?>
