<?php

$outputPDFFilename = "ReplicationReport.pdf";
$outputFilename = "replication_report.html";
$reportsDir = "/var/www/html/recoveryconsole/reports.dir/";

if (php_sapi_name() === 'cli') {
	$options = getopt("e:s:i:mf");
	$endTime = isset($options['e']) ? floatval($options['e']) : time();
	$startTime = isset($options['s']) ? floatval($options['s']) : $endTime - ( 60 * 60 * 24 );

	$dpuID = isset($options['i']) ? (int)$options['i'] : 1;

	$skipEmail = isset($options['m']) ? true : false;

	$createFile = isset($options['f']) ? true : false;
	$outputFile = $reportsDir . $outputFilename;
} else {
	$endTime = isset($_GET['end_date']) ? floatval( $_GET['end_date'] ): time();
	$startTime = isset($_GET['start_date']) ? floatval( $_GET['start_date'] ) : $endTime - ( 60 * 60 * 24 );

	$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : 1;

	$skipEmail = isset($_GET['skip_mail']) && (int)$_GET['skip_mail'] == 1 ? TRUE: FALSE;

	$createFile = isset($_GET['file']) && (int)$_GET['file'] == 1 ? true : false;
	$rootdir = $_SERVER['DOCUMENT_ROOT'] . "/recoveryconsole/";
	$outputFile = $rootdir . "reports.dir/" . $outputFilename;
}

require "ReportView.php";

$history  = curl_init("localhost/recoveryconsole/bpl/syncstatus.php?type=replicate&arguments=start:$startTime,end:$endTime&sid=$dpuID&auth=1:");
$active   = curl_init("localhost/recoveryconsole/bpl/replication/service.php?type=list&start_pos=-1&max=50&sid=$dpuID&test=-1&auth=1:");
$sysInfo  = curl_init("localhost/recoveryconsole/bpl/getsysteminfo.php?sid=$dpuID&auth=1:");
$iniInfo  = curl_init("localhost/recoveryconsole/bpl/ini.php?type=list&sid=$dpuID&section=Replication&field=ReportMailTo&auth=1:");
$hostInfo = curl_init("localhost/recoveryconsole/bpl/hostname.php?type=list&sid=$dpuID&auth=1:");

$options = array
(
    CURLOPT_HEADER=>false,
    CURLOPT_RETURNTRANSFER=>true,
    CURLOPT_FOLLOWLOCATION=>true
);
curl_setopt_array($history,$options);
$history_output = curl_exec($history);
curl_close($history);

curl_setopt_array($active,$options);
$active_output = curl_exec($active);
curl_close($active);

curl_setopt_array($sysInfo,$options);
$sysInfo_output = curl_exec($sysInfo);
curl_close($sysInfo);

curl_setopt_array($iniInfo,$options);
$iniInfo_output = curl_exec($iniInfo);
curl_close($iniInfo);

curl_setopt_array($hostInfo,$options);
$hostInfo_output = curl_exec($hostInfo);
curl_close($hostInfo);

$reportView = new ReportView;

function writeTrace( $traceString ) {	
	echo( $traceString."\n" );
}

$counter = 0;

if ( $history_output || $active_output || $sysInfo_output ) {
	$historySummary = array();
	$queueSummary   = array();
	$activeSummary  = array();
	
	$historyTable = "";
	$activeTable  = "";
	$queueTable   = "";
	
	if ( $history_output ) {
		$xml = new SimpleXMLElement( $history_output );
		
		$startDate = date( "F j, Y, g:i a T", $startTime);
		$endDate   = date( "F j, Y, g:i a T", $endTime);
		
		$numRows = count( $xml->children() );
		
		$historyRows = "";
		
		foreach( $xml->SecureSyncStatus as $secureSync ) {
			$completeDate = $secureSync->Date;
			$completeTime = $secureSync->Time;
			
			
			$dateAndTime = "$completeDate $completeTime";	
			
			$size = intval( $secureSync->SyncSize );
			$size = $size <= 0 ? 1 : $size;
			$replicationFailed = (strlen(stristr($secureSync->SyncMessage, "failed"))>0) ? true : false;
			
			
			$numFIles = intval( $secureSync->Files );
			$numFIles = $numFIles <= 0 ? 1 : $numFIles;
			
			$sourceSystem = $secureSync[ 'DPUName' ];
			
			if ( !isset( $historySummary[ "$sourceSystem" ] )) {
				$subArray = array( "source" => $sourceSystem,
								   "sourceCount" => 1,
								   "sizeCount" => $size,
								   "fileCount" => $numFIles );
				
				$historySummary[ "$sourceSystem" ] = $subArray;
				
			} else {
				$historySummary[ "$sourceSystem" ][ "sourceCount" ]++;
				$historySummary[ "$sourceSystem" ][ "sizeCount" ]  += $size;
				$historySummary[ "$sourceSystem" ][ "fileCount" ] += $numFIles;
			}
			
			$historyRows.= $reportView->getHistoryRow( $secureSync[ 'DPUName' ],
													   $secureSync[ 'ClientName' ],
													   processInstanceName( $secureSync->InstanceName, $secureSync[ 'Type' ] ),
													   $secureSync->TargetName,
													   $secureSync[ 'Type' ],
													   $dateAndTime,
													   gmdate("H:i:s", floatval( $secureSync->ElapsedTime)),
													   ( $size <= 0 ? 1 : $size ),
													   ( $numFIles <= 0 ? 1 : $numFIles ),
													   getYesOrNoForTrueOrFalse( $secureSync->Encrypted ),													   
													   $replicationFailed,
													   $secureSync->SyncMessage, 
													   $counter >= ( $numRows - 1 ) );
			$counter++;
		}
		
		if ( $historyRows != "" ) {
			$summary = $reportView->getSummary( $historySummary );
			$historyTable = $reportView->getHistoryTable( $summary[ "rows" ], 
														  $summary[ "totalBackups" ], 
														  $summary[ "totalFiles" ],
														  $summary[ "totalSize" ],
														  $historyRows );
		}
	}
	
	$numRows = 0;
	$activeCounter = 0;
	$queueCounter = 0;
	
	if ( $active_output ) {
		$xml = new SimpleXMLElement( $active_output );
		
		$replicationCount = intval( $xml->Queue[0][ "TotalCount"] );
		$systemCount      = intval( $xml->Queue[0][ "SystemCount"] );
		
		$activeRows = "";
		$queueRows = "";
		
		$numActiveRows = 0;
		$numQueueRows = 0;
		
		foreach( $xml->Replication as $queueItem ) {
			$intStatus = intval( $queueItem->Status );
			
			if ( $intStatus == 16 ) {
				$numActiveRows++;
			} else {
				$numQueueRows++;
			}
		}
		
		foreach( $xml->Replication as $queueItem ) {
			$startDate = $queueItem->StartDate;
			$startTime = $queueItem->StartTime;
			
			$dateAndTime = "$startDate $startTime";
			
			if ( $startDate == "" || $startTime == "" ) continue;		
			
			$size = intval( $queueItem->FinalSize );
			$size = $size <= 0 ? 1 : $size;
			
			$sourceSystem = $queueItem->SystemName;
			
			$intStatus = intval( $queueItem->Status );
			
			$activeReplication = $intStatus == 16;
			
			if ( $activeReplication ) {
				if ( !isset( $activeSummary[ "$sourceSystem" ] )) {
					$subArray = array( "source" => $sourceSystem,
									   "sourceCount" => 1,
									   "sizeCount" => $size );
					
					$activeSummary[ "$sourceSystem" ] = $subArray;
					
				} else {
					$activeSummary[ "$sourceSystem" ][ "sourceCount" ]++;
					$activeSummary[ "$sourceSystem" ][ "sizeCount" ]  += $size;
				}
				
				
				$activeRows.= $reportView->getActiveRow( $sourceSystem, 
														 $queueItem->ClientName,
														 processInstanceName( $queueItem->InstanceName, $queueItem->Type ),
														 $queueItem->TargetName,
														 $queueItem->Type,
														 $queueItem->OperationName,
														 $dateAndTime,
														 gmdate("H:i:s", floatval( $queueItem->ElapsedTime)),
														 "$size",
														 getYesOrNoForTrueOrFalse( $queueItem->Encrypted ),
														 $activeCounter >= ( $numActiveRows - 1 ));
														 
				$activeCounter++;
														 
				
			} else {
				if ( !isset( $queueSummary[ "$sourceSystem" ] )) {
					$subArray = array( "source" => $sourceSystem,
									   "sourceCount" => $replicationCount,
									   "sizeCount" => $size );
					
					$queueSummary[ "$sourceSystem" ] = $subArray;
					
				} else {
					$queueSummary[ "$sourceSystem" ][ "sourceCount" ] = $replicationCount;
					$queueSummary[ "$sourceSystem" ][ "sizeCount" ]  += $size;
				}
				
				$queueRows.= $reportView->getQueueRow( $sourceSystem, 
													   $queueItem->ClientName,
													   processInstanceName( $queueItem->InstanceName, $queueItem->Type ),
													   $queueItem->TargetName,
													   $queueItem->Type,
													   $dateAndTime,
													   gmdate("H:i:s", floatval( $queueItem->ElapsedTime)),
													   "$size",
													   getYesOrNoForTrueOrFalse( $queueItem->Encrypted ),
													   ( $intStatus == 4 || $intStatus == 8 || $intStatus == 32 ),
													   $queueItem->SyncMessage, 
													   $queueCounter >= ( $numQueueRows - 1 ));
				$queueCounter++;							   
			}
			
		}
		
		if ( $activeRows != "" ) {
			$summary = $reportView->getSummary( $activeSummary, false );
			$activeTable = $reportView->getActiveTable( $summary[ "totalBackups" ], 
														$summary[ "totalSources" ],
														$summary[ "totalBackups" ],
														$summary[ "totalSize" ],
														$summary[ "rows" ],
														$activeRows );
		}
		
		if ( $queueRows != "" ) {
			$summary = $reportView->getSummary( $queueSummary, false );
			$queueTable = $reportView->getQueueTable( $summary[ "totalBackups" ], 
													  $summary[ "totalSources" ],
													  $summary[ "totalBackups" ],
													  $summary[ "totalSize" ],
													  $summary[ "rows" ],
													  $queueRows );
		}
	}
	
	$assetTag = "";
	$runDate = "";
	$version = "";
	$systemName = "";
	
	if ( $sysInfo_output ) {
		$xml = new SimpleXMLElement( $sysInfo_output );
		
		$systemName = $xml->SystemInformation[0]->Name;;
		$systemVersion = $xml->SystemInformation[0]->Version;
		$assetTag = $xml->SystemInformation[0]->AssetTag;
		$runDate  = date( "F j, Y, g:i a T", $endTime);
		$version  = $xml->SystemInformation[0]->Version;
	}
	
	$reportData = $reportView->getCompoundPage( $systemName,
												$systemVersion,
												$historyTable, 
												$activeTable, 
												$queueTable, 
												$assetTag,
												$runDate, 
												$version);
												// $history_output."\n\n".$active_output."\n\n".$sysInfo_output
	
	
	if ($createFile) {
		$result = file_put_contents( $outputFile, $reportData );
	} else {
		$result = TRUE;
	}
	
	GLOBAL $outputPDFFilename;
	$pdfFilename = $systemName . "-" . $outputPDFFilename;
	$pdfResult = createPDF($dpuID, $outputFile, $pdfFilename);
	
	if ( !$skipEmail && $iniInfo_output && $reportData && $hostInfo_output ) {
		$xml = new SimpleXMLElement( $iniInfo_output );
		$mailList = $xml->Section[0]->Field[0]["Value"];
		
		$hostInfoXML = new SimpleXMLElement( $hostInfo_output );
		$longHostName = $hostInfoXML->Host[0]->Longname[0];
		
		$list = "$mailList";
		$fromAddress = "reports@$longHostName";
		
		// have to at least have form a@b.c
		if ( $list ) {
			$mailingAddresses = preg_split("/[\s,]+/",  $mailList );
			
			$reportData = preg_replace("#(?<!\r)\n#si", "\r\n", $reportData);
			
		
			$currentSystemTime = date( "F j, Y, g:i a T", $endTime);
			foreach ( $mailingAddresses as $key => $address ) {
				$mailAndHeadersResult = createMailAndHeaders( "Replication.Report@$longHostName", $fromAddress, $address, "Replication Report for $currentSystemTime", $reportData, $pdfResult );
			}																
		}	
	}

	if (file_exists($pdfResult)) {
		unlink($pdfResult);
	}	

	if ( $result === FALSE ) {
		echo( "failed to write to disk" );
	} else {
		echo( $reportData );
	}
	
} else {
	echo( "This did not work"); 
}

function processInstanceName( $instanceNameRaw, $type ) {
	
	if ( $instanceNameRaw === NULL || $instanceNameRaw == "" ) {
		$instanceNameRaw = "Target Instance";
	}
	
	if( strpos( $instanceNameRaw, "|" ) !== false ) {
		$instanceParts = explode( "|", $instanceNameRaw );
		
		if ( strpos( $type, "VMware" ) !== false || strpos( $type, "SQL" ) !== false ) {
			$instanceParts = array_reverse( $instanceParts );
		}
		return $instanceParts[0];
	}
	
	return $instanceNameRaw;	
}

function createMailAndHeaders( $reportTitle, $fromEmailAddress, $sendTo, $subject, $message, $pdfResult) {
	if ($pdfResult === false || !file_exists($pdfResult)) {
		createMailAndHeadersNoAttachment( $reportTitle, $fromEmailAddress, $sendTo, $subject, $message);
	}
	else {
		createMailAndHeadersPDF( $reportTitle, $fromEmailAddress, $sendTo, $subject, $message, $pdfResult);
	}
}

function createMailAndHeadersNoAttachment( $reportTitle, $fromEmailAddress, $sendTo, $subject, $message) {
	$headers   = array();
	$headers[] = "Content-type: text/html; charset=utf-8";
	$headers[] = "From:  $reportTitle <$fromEmailAddress>";
	$headers[] = "Reply-To:  $reportTitle <$fromEmailAddress>";
	$headers[] = "$reportTitle: {$subject}";
	$headers[] = "X-Mailer: PHP/".phpversion();
	
	$headersComplex = implode("\r\n", $headers );
	
	$result = mail( $sendTo, $subject, $message, $headersComplex );
	return $result;
}

function createMailAndHeadersPDF( $reportTitle, $fromEmailAddress, $sendTo, $subject, $message, $pdfResult) {	
	$boundary = md5(uniqid(time()));
	$eol = "\r\n";
	
		echo( "attach pdf $pdfResult" );
	// encode data (puts attachment in proper format (RFC 2045)
    $attachment = chunk_split(base64_encode(file_get_contents($pdfResult)));
    $pdfFilename = basename($pdfResult);
    
	// main header (multipart mandatory)	
	$headers  = "From: $reportTitle <$fromEmailAddress>" . $eol;
	$headers .= "Reply-To: $reportTitle <$fromEmailAddress>" . $eol;
	$headers .= 'MIME-Version: 1.0' . $eol;
	$headers .= "Content-Type: multipart/mixed; boundary=\"" . $boundary . "\"" . $eol . $eol; 
	//$headers .= 'Content-Transfer-Encoding: 7bit' . $eol;
	$headers .= 'This is a MIME encoded message.' . $eol;
	
	// message
	$headers .= '--' . $boundary . $eol;
	$headers .= 'Content-Type: text/HTML; charset=iso-8859-1' . $eol;
	$headers .= 'Content-Transfer-Encoding: 8bit' . $eol . $eol;
	$headers .= $message . $eol . $eol;
	
	// attachment
	$headers .= '--' . $boundary . $eol;
	$headers .= "Content-Type: application/pdf; name=\"" . $pdfFilename . "\"" . $eol; 
	$headers .= "Content-Transfer-Encoding: base64" . $eol;
	$headers .= "Content-Disposition: attachment; filename=\"" . $pdfFilename . "\"" . $eol . $eol;
	$headers .= $attachment . $eol . $eol;
	$headers .= '--' . $boundary . '--';
	
	// send message
	$result = mail( $sendTo, $subject, "", $headers);
	
	return $result;
}

function getYesOrNoForTrueOrFalse( $value ) {
	if ( $value == 1 || $value == "true" || $value === "true" || $value === "1" ) {
		return "Yes";
	} else {
		return "No";
	}
}

// Returns full path to PDF file upon success
// Returns false upon failure or if PDF is not to be created
function createPDF($systemID, $htmlFilePath, $pdfFilename) {
	// load pdf conversion class
	$rootdir = "/var/www/html/recoveryconsole";
	$pdfScript = $rootdir . '/system/pdf_conversion.php';
	// if this method is called more than one time - it will fail without require_once
	require_once $pdfScript;
	$pdfConversion = new PDFConversion();
	
	// create PDF file if necessary
	// needs to create a cookie
	$result = $pdfConversion->createPDF($systemID, $htmlFilePath, $pdfFilename, true);
	return $result;
}

?>
