<?php
// defines BP, xml, action, cookie
require 'header.php';

// defines functions for URL parsing.
require('urloptions.php');

// defines functions for backup status processing.
require('basebackupstatus.php');

// set the action
$action = "sync";

//
// See if user passed in system ID and client ID.
//
$dpuID = isset($_GET['sid']) ? (int)$_GET['sid'] : $BP->get_local_system_id();
$cid = isset($_GET['cid']) ? (int)$_GET['cid'] : -1;

//
// Set backup types to sync backups.
//
$result_format = array();
$attributes = getTimeArguments();
$backupTypes = $GLOBAL_SS_TYPES;		
$result_format = array_merge($result_format, $attributes, array('type' => $backupTypes));
			
//
// Loop through all systems.  Will be only one in the list for a DPU, multiple systems on a Vault.
//
$systems = $BP->get_system_list();

$xml->push('VaultingInformation');
$originalFormat = $result_format;
$localID = $BP->get_local_system_id();
$bVault = $BP->local_system_is_vault();
$bManager = $BP->local_system_is_mgr();
$bMDPU = isset($_GET['mdpu']) ? true : false;

if ($bVault) {
	$vaultOperationList = $BP->get_operation_list("", $localID);
	// filter out any non-sync operations
	$vaultOperationList = filterVaultingOperations($vaultOperationList);
}

foreach ($systems as $systemID => $systemName) {
	if ($systemID == $dpuID || $dpuID == $localID) {
		// If no client passed in, loop through all clients on the system.
		if ($cid == -1) {
			$clients = $BP->get_client_list($systemID);
		} else {
			$clientName = getClientName($BP, $cid, $systemID);
			$clients = array($cid => $clientName);
		}
		
		// If this system is a vault and we are looking at this system, set clients and devices to
		// NONE and continue to next system.
		if ($bVault && ($systemID == $localID)) {
			$clients = array();
			$deviceList = array();
		// If this system is a manager and we are not running on the MDPU node nor are
		// not looking at the selected system, skip it.
		} else if (!$bVault && $bManager && !$bMDPU && ($systemID != $dpuID)) {
			$clients = array();
			$deviceList = array();
		} else {
			// Build the device and operation lists once per system.  If on a vault, add
			// the vaulting operations to the list.
			$deviceList = $BP->get_device_list($systemID);
			$operationList = $BP->get_operation_list("", $systemID);
			// filter out any non-sync operations
			$operationList = filterVaultingOperations($operationList);
			if ($bVault) {
				$operationList = array_merge($operationList, $vaultOperationList);
			}
		}
			
		foreach ($clients as $clientID => $clientName) {
			$result_format = array_merge($originalFormat, array('system_id' => $systemID, 'client_id' => $clientID));
			$backupStatusArray = $BP->get_backup_status($result_format);
			
			#if ($backupStatusArray !== false && count($backupStatusArray) > 0) {
			# To get the first sync ever, just check for false, not false and count.
			if ($backupStatusArray !== false) {
				$last = 0;
				$lastMaster = 0;
				$lastIncremental = 0;
				$lastBareMetal = 0;
				$lastSQL = 0;
				$lastExchange = 0;
				$lastLocalDir = 0;
				$lastExchangeFull = array();
				$lastExchangeDiff = array();
				$lastExchangeIncr = array();
				$lastSQLFull = array();
				$lastSQLDiff = array();
				$lastSQLTrans = array();
				$lastVMFull = array();
				$lastVMDiff = array();
				$lastVMIncr = array();	
				$lastHyperVFull = array();
				$lastHyperVIncr = array();	
				$lastHyperVDiff = array();		
				$resultArray = array();
				
				// Get the operations for this client.  See if any syncs are in progress.
				foreach ($operationList as $operation) {
					$operationName = trim($operation['name']);
					if (isSyncOperation($operationName)) {
						$for = trim($operation['on_behalf_of']);
						$operationSource = trim($operation['source']);
						
						if ($for == $clientName && $operationSource == $systemName) {
							if (array_key_exists('backup_id', $operation)) {
								$inProgressResult['id'] = $operation['backup_id'];
								$inProgressResult['type'] = trim($operation['type']);
								$inProgressResult['start_time'] = $operation['start_time'];
								$inProgressResult['complete'] = false;
								$inProgressResult['elapsed_time'] = $operation['current_time'] - $operation['start_time'];
								$inProgressResult['phase'] = $operationName;
								$inProgressResult['current_size'] = toNumberString($operation['current_size']);
								$inProgressResult['final_size'] = toNumberString($operation['final_size']);
								$inProgressResult['size'] = $inProgressResult['current_size'];
							}
						}
					}
				}

				// Loop through the vaulted results, and save latest of each type.  If a backup is not complete,
				// see if we can find an associated operation with it.
				foreach ($backupStatusArray as $backupStatus) {
					if ($backupStatus['status'] == STATUS_SUCCESS || $backupStatus['status'] == STATUS_WARNING) {							
						if ($backupStatus['type'] == 'securesync master' &&
							$backupStatus['start_time'] > $lastMaster) {
							$resultArray['master'] = $backupStatus;
							$lastMaster = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync differential' &&
								   $backupStatus['start_time'] > $lastIncremental) {
							$resultArray['incremental'] = $backupStatus;
							$lastIncremental = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync baremetal' &&
								   $backupStatus['start_time'] > $lastBareMetal) {
							$resultArray['baremetal'] = $backupStatus;
							$lastBareMetal = $backupStatus['start_time'];								
						} else if ($backupStatus['type'] == 'securesync localdir' &&
								   $backupStatus['start_time'] > $lastLocalDir) {
							$resultArray['localdir'] = $backupStatus;
							$lastLocalDir = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync SQL' &&
								   $backupStatus['start_time'] > $lastSQL) {
							$resultArray['sql'] = $backupStatus;
							$lastSQL = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync msexch full' &&
								  (!isset($lastExchangeFull[$backupStatus['database_name']]) ||
								    isset($lastExchangeFull[$backupStatus['database_name']]) &&
									$backupStatus['start_time'] > $lastExchangeFull[$backupStatus['database_name']])) {
							$resultIndex = 'exchange full.' . $backupStatus['database_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastExchangeFull[$backupStatus['database_name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync msexch diff' &&
								  (!isset($lastExchangeDiff[$backupStatus['database_name']]) ||
								    isset($lastExchangeDiff[$backupStatus['database_name']]) &&
									$backupStatus['start_time'] > $lastExchangeDiff[$backupStatus['database_name']])) {
							$resultIndex = 'exchange diff.' . $backupStatus['database_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastExchangeDiff[$backupStatus['database_name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync msexch incr' &&
								  (!isset($lastExchangeIncr[$backupStatus['database_name']]) ||
								    isset($lastExchangeIncr[$backupStatus['database_name']]) &&
									$backupStatus['start_time'] > $lastExchangeIncr[$backupStatus['database_name']])) {
							$resultIndex = 'exchange incr.' . $backupStatus['database_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastExchangeIncr[$backupStatus['database_name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync exchange' &&
								   $backupStatus['start_time'] > $lastExchange) {
							$resultArray['exchange'] = $backupStatus;
							$lastExchange = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync mssql full' &&
								  (!isset($lastSQLFull[$backupStatus['database_name']]) ||
								    isset($lastSQLFull[$backupStatus['database_name']]) &&
									$backupStatus['start_time'] > $lastSQLFull[$backupStatus['database_name']])) {
							$resultIndex = 'sql full.' . $backupStatus['database_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastSQLFull[$backupStatus['database_name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync mssql diff' &&
								  (!isset($lastSQLDiff[$backupStatus['database_name']]) ||
								    isset($lastSQLDiff[$backupStatus['database_name']]) &&
									$backupStatus['start_time'] > $lastSQLDiff[$backupStatus['database_name']])) {
							$resultIndex = 'sql diff.' . $backupStatus['database_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastSQLDiff[$backupStatus['database_name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync mssql trans' &&
								  (!isset($lastSQLTrans[$backupStatus['database_name']]) ||
								    isset($lastSQLTrans[$backupStatus['database_name']]) &&
									$backupStatus['start_time'] > $lastSQLTrans[$backupStatus['database_name']])) {
							$resultIndex = 'sql trans.' . $backupStatus['database_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastSQLTrans[$backupStatus['database_name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync vmware full' &&
								  (!isset($lastVMFull[$backupStatus['database_name']]) ||
								    isset($lastVMFull[$backupStatus['database_name']]) &&
									$backupStatus['start_time'] > $lastVMFull[$backupStatus['database_name']])) {
							$resultIndex = 'vmware full.' . $backupStatus['database_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastVMFull[$backupStatus['database_name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync vmware diff' &&
								  (!isset($lastVMDiff[$backupStatus['database_name']]) ||
								    isset($lastVMDiff[$backupStatus['database_name']]) &&
									$backupStatus['start_time'] > $lastVMDiff[$backupStatus['database_name']])) {
							$resultIndex = 'vmware diff.' . $backupStatus['database_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastVMDiff[$backupStatus['database_name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync vmware incr' &&
								  (!isset($lastVMIncr[$backupStatus['database_name']]) ||
								    isset($lastVMIncr[$backupStatus['database_name']]) &&
									$backupStatus['start_time'] > $lastVMIncr[$backupStatus['database_name']])) {
							$resultIndex = 'vwmare incr.' . $backupStatus['database_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastVMIncr[$backupStatus['database_name']] = $backupStatus['start_time'];
						}
						else if ($backupStatus['type'] == 'securesync hyperv incr' &&
								  (!isset($lastHyperVIncr[$backupStatus['vm_name']]) ||
								    isset($lastHyperVIncr[$backupStatus['vm_name']]) &&
									$backupStatus['start_time'] > $lastHyperVIncr[$backupStatus['vm_name']])) {
							$resultIndex = 'hyperv incr.' . $backupStatus['vm-name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastHyperVIncr[$backupStatus['vm-name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync hyperv full' &&
								  (!isset($lastHyperVFull[$backupStatus['vm_name']]) ||
								    isset($lastHyperVFull[$backupStatus['vm_name']]) &&
									$backupStatus['start_time'] > $lastHyperVFull[$backupStatus['vm_name']])) {
							$resultIndex = 'hyperv full.' . $backupStatus['vm_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastHyperVFull[$backupStatus['vm_name']] = $backupStatus['start_time'];
						} else if ($backupStatus['type'] == 'securesync hyperv diff' &&
								  (!isset($lastHyperVDiff[$backupStatus['vm_name']]) ||
								    isset($lastHyperVDiff[$backupStatus['vm_name']]) &&
									$backupStatus['start_time'] > $lastHyperVDiff[$backupStatus['vm_name']])) {
							$resultIndex = 'hyperv diff.' . $backupStatus['vm_name'];
							$resultArray[$resultIndex] = $backupStatus;
							$lastHyperVDiff[$backupStatus['vm_name']] = $backupStatus['start_time'];
						}
					}
				}
				
				// If we found an in-progress operation for the backup, use it and apply the relevant fields from the backup to it.
				$resultFields = array('size', 'sync_size');
				if (isset($inProgressResult)) {
					$syncBackupType = $inProgressResult['type'];
					switch ($syncBackupType) {
						case 'master':
						case 'incremental':
						case 'differential':
						case 'baremetal':
						case 'localdir':
						case 'sql':
						case 'exchange':
						case 'hyperv':
							$resultArray[$syncBackupType] = $inProgressResult;
							break;

						// Convert vaulting types to known names.
						case 'MSExch Full':
						case 'MSExch Diff':
						case 'MSExch Incr':
						case 'MSSQL Full':
						case 'MSSQL Diff':
						case 'MSSQL Trans':
							$syncType = getBackupTypeString('securesync ' . strtolower($syncBackupType));
							$inProgressResult['type'] = $syncType;
							$resultArray[$syncBackupType] = $inProgressResult;
							break;

						case 'ESX VM Full':
						case 'ESX VM Diff':
						case 'ESX VM Incr':
							$syncType = getBackupTypeString($syncBackupType);
							$inProgressResult['type'] = $syncType;
							$resultArray[$syncBackupType] = $inProgressResult;
							break;
							
						case 'hyperv full':
						case 'hyperv incr':
						case 'hyperv diff':
							$syncType = getBackupTypeString($syncBackupType);
							$inProgressResult['type'] = $syncType;
							$resultArray[$syncBackupType] = $inProgressResult;
							break;
					}
				}
				
				// We could have 0-3 results for each client.  Build the XML for each.
				if (count($resultArray) > 0) {
					foreach ($resultArray as $result) {
						createVaultingXML($xml, $result, $deviceList,
										  $systemID, $systemName,
										  $clientID, $clientName);
					}
				}
				
				unset($inProgressResult);	

			}

		}
	}
}

$xml->pop(); // VaultingInformation	
echo($xml->getXml());

function isSyncOperation($name)
{
	$result = false;
	
	switch ($name) {
	
	case 'patch':
	case 'delta':
	case 'copy':
	case 'signature':
		$result = true;
		break;
	}
	
	return $result;
}

//
// This function builds an expanded status which is a combination of values returned by
// bp_get_backup_status and bp_get_backup_info.
//
function createVaultingXML($xml, $backup, $deviceList, $dpuID, $dpuName, $clientID, $clientName)
{
	// convert timestamp to local time/day
	$day = date('m/d/Y', $backup['start_time']);
	$time = date(TIME_FORMAT_12HR, $backup['start_time']);
		
	$backupType	= getBackupTypeString($backup['type']);
			
	global $action;		
	$Tag = ($action == 'sync') ? "SecureSyncStatus" : "Backup";
	if (!isset($backup['id'])) {
		$backup['id'] = "N/A";
	}
	$xml->push($Tag, array('Type' => $backupType, "Identification" => $backup['id'], "DPUName" => $dpuName, "ClientName" => $clientName));
	$xml->element('Date', $day);
	$xml->element('Time', $time);
	$xml->element('ElapsedTime', $backup['elapsed_time']);
	$xml->element('Complete', $backup['complete'] == true ? 1 : 0);
	
	if ($backup['complete']) {
		// If the backup has completed, note this.
		if (array_key_exists('status', $backup)) {
			switch ($backup['status']) {
				case STATUS_SUCCESS:		$backupStatus = "Success"; break;
				case STATUS_WARNING:		$backupStatus = "Warning"; break;
				case STATUS_FAILURE:		$backupStatus = "Failure"; break;
			}
			$xml->element('Result', $backupStatus);
		}
	} else {
		$xml->element('Result', "Active");
	}

	// Output size
	if (array_key_exists('size', $backup)) {
		$xml->element('Size', $backup['size']);
	}
	
	// Output sync size
	if (array_key_exists('sync_size', $backup)) {
		$xml->element('SyncSize', $backup['sync_size']);
	}
	
	// If the backup has file count, note this.
	if (array_key_exists('files', $backup)) {
		$xml->element('Files', $backup['files']);
	}
	
	// If the backup was encrypted, note this.
	if (array_key_exists('encrypted', $backup)) {
		$xml->element('Encrypted', $backup['encrypted'] == true ? 1 : 0);
	}
	
	// Output sync status
	if (array_key_exists('sync_status', $backup)) {
		$backupStatus = createSyncStatusString($backup['sync_status']);
		$xml->element('SyncStatus', $backupStatus);
	}
	
	// Extras for operations
	if (array_key_exists('phase', $backup)) {
		$xml->element('Phase', $backup['phase']);
	}
	if (array_key_exists('final_size', $backup)) {
		$xml->element('FinalSize', $backup['final_size']);
	}
	if (array_key_exists('current_size', $backup)) {
		$xml->element('CurrentSize', $backup['current_size']);
	}
	
	if (array_key_exists('database_name', $backup)) {
		$xml->element('Database', $backup['database_name']);
	}
	// If an application, get instance ID.
	if (array_key_exists('instance_id', $backup)) {
		$xml->element('InstanceID', $backup['instance_id']);
	}
	// If an application, get name and ID.
	if (array_key_exists('app_name', $backup)) {
		$xml->emptyelement('Application', array('Name' => $backup['app_name'])); //, 'ID' => $backup['app_id']));
	}
			
	$xml->pop(); // Backup
}

function toPhaseName($operationName)
{
	$s = "";
	switch ($operationName) {
		case "delta":
			$s = "Delta";
			break;
		case "patch":
			$s = "Patch";
			break;
		case "signature":
			$s = "Signature";
			break;
		case "copy":
			$s = "Copy";
			break;
	}
	return $s;
}

// takes return from bp_operation_progress and 
// filters out all but vaulting operations
function filterVaultingOperations($operationList) {
	$newOperationList = array();
	foreach ($operationList as $operation) {
		$operationName = trim($operation['name']);
		if (isSyncOperation($operationName)) {
			$newOperationList[] = $operation;
		}
	}
		
	return $newOperationList;
}

$BP->destroyCookie();
?>