<?php

require_once 'logger.php';
initialize_log('statussync');
$date_string = date('r');
logger(4, '=============================================================='."\n".'==============================================================');
logger(1, 'StatusSync run starting at '.time().': '.$date_string);
$err = 'Error processing request, please see the StatusSync log for more details';


// Retrieves xml feeds from feeder scripts and updates the local database
//
// From the web, set a POST or GET variable named 'systems' to a comma-delimited
// list of system ids to sync.  From the command line, enter them as separate arguments:
//
// http://site/aggregator.php?systems=1,2,3,4
// [system]# php -f aggregator.php 1 2 3 4


// settings
// feeds will be $feed_host_prefix . $host . $feed_relative_url
$secure_connection = true; // true for secure connection
$feed_host_prefix = 'ssl://';     // secure, usually "ssl://"
$feed_host_suffix = ':443';       // secure, usually ":443" for https
//$i_feed_host_prefix = 'tcp://';   // insecure, usually "tcp://"
//$i_feed_host_suffix = ':80';      // insecure, usually ":80" for http
$feed_relative_url = '/statussync/feed.php';
$default_timeout = 15; // timeout, in seconds, this is overridden by master.ini
$default_retention_days = 31; // days of rentention, this is overridden by master.ini
$quiet = false; // true to suppress output
$debug = false; // true for extra status
$connection = null; // database connection

// master.ini stuff
$cmd_bppath = 'eval `grep BPDIR /etc/default/bp.ini`';
$cmd_master_ini = 'MASTER_INI=$BPDIR/bpinit/master.ini';
$cmd_bputil = 'BPUTIL=$BPDIR/bin/bputil';
$cmd_ini = "$cmd_bppath; $cmd_master_ini; $cmd_bputil";

// check for someone trying to access via the web
if (isset($_SERVER['REQUEST_URI'])) {
	logger(0, 'Attempted invocation of StatusSync via web browser:');
	$keys = array('HTTP_USER_AGENT', 'SERVER_NAME', 'SERVER_ADDR', 'SERVER_PORT', 'REMOTE_ADDR', 'REMOTE_PORT', 'REQUEST_METHOD', 'QUERY_STRING', 'REQUEST_URI', 'REQUEST_TIME');
	$stats = array();
	foreach ($keys as $key => $val) {
		$stats[$key] = $_SERVER[$key];
	}
	logger(1, 'attempt statistics:' . print_r($stats, true));
	logger(1, 'HTTP GET array: ' . print_r($_GET, true));
	logger(1, 'HTTP POST array: ' . print_r($_POST, true));
	logger(1, 'COOKIE array: ' . print_r($_COOKIE, true));
	echo "Page is unavailable.<br><br>";
	die();
}

// setup
$retval = bp_bypass_cookie(3);
if ($retval === false) {
	exit_error('Could not bypass cookie system');
}
register_shutdown_function("bp_destroy_cookie");

// first make sure this is a vault
if (bp_local_system_is_vault() != true) {
	exit_error('This system must be a Vault for StatusSync to aggregate data');
}

// next make sure that input is enabled, otherwise get nothing
$cmd_enabled = '$BPUTIL -g "StatusSync" "EnableInput" "no" $MASTER_INI';
$enabled_output = trim(shell_exec("$cmd_ini; $cmd_enabled"));
$enabled = false;
if (strtolower($enabled_output{0}) == 'y' ||
    strtolower($enabled_output{0}) == 't' ||
    $enabled_output == 1) {
	$enabled = true;
}
if ($enabled == false) {
	exit_error('StatusSync aggregation is currently disabled');
}

// set retention days
$cmd_days = '$BPUTIL -g "StatusSync" "RetentionDays" "' . $default_retention_days . '" $MASTER_INI';
$retention_days_output = trim(shell_exec("$cmd_ini; $cmd_days"));
if (is_numeric($retention_days_output)) {
	$retention_days = intval($retention_days_output);
} else {
	logger(5, 'Invalid or unset retention days value in configuration file');
	$retention_days = $default_retention_days;
}
logger(3, "Retention days set to $retention_days");

// set connection timeout
$cmd_timeout = '$BPUTIL -g "StatusSync" "CheckTimeoutSeconds" "' . $default_timeout . '" $MASTER_INI';
$timeout_output = trim(shell_exec("$cmd_ini; $cmd_timeout"));
if (is_numeric($timeout_output)) {
	$timeout = intval($timeout_output);
} else {
	logger(5, 'Invalid or unset timeout value in configuration file');
	$timeout = $default_timeout;
}
logger(3, "Timeout seconds set to $timeout");

// next check secure transfer settings
/*
$cmd_allow_insecure = '$BPUTIL -g "StatusSync" "AllowInsecureTransfer" "no" $MASTER_INI';
$allow_insecure_output = trim(shell_exec("$cmd_ini; $cmd_allow_insecure"));
if (strtolower($allow_insecure_output{0}) == 'y' ||
strtolower($allow_insecure_output{0}) == 't' ||
$allow_insecure_output == 1) {
	$allow_insecure = true;
} else {
	$allow_insecure = false;
}
*/

process_argument_switches(); // sets quiet, debug, timeout, ...
$feeds = array();
open_db_conn() or exit_error('error opening database connection');
get_feed_list($feeds) or exit_error('could not retrieve feed list');
get_feed_responses($feeds);  // returns false for errors, but we may still have updates to do
update_database($feeds);
$date_string = date('r');
logger(1, 'StatusSync run finished at '.time().': '.$date_string);
exit (0);


/**
 * @desc - exits, logging and conditionally printing an error string
 * @param optional $err_str - error string
 */
function exit_error($err_str = '(no error string supplied)')
{
	global $err;
	if ($debug) {
		echo $err_str . "\n";
	} elseif (!$quiet) {
		echo $err . "\n";
	}
	logger(0, $err_str);
	exit(0);
}

/**
 * @desc - returns false, logging and conditionally printing an error string
 * @param optional $err_str - error string
 * @return - false, so that you can exit from a function using:
 *              return exit_error('your error string');
 */
function print_error($err_str = '(no error string supplied)')
{
	global $err;
	if ($debug) {
		echo $err_str . "\n";
	} elseif (!$quiet) {
		echo $err . "\n";
	}
	logger(1, $err_str);
	return false;
}


/**
 * @desc - processes command-line switches (but not dpu ids):
 *         --debug - run in debug mode (lots of output)
 *         --help - show these switches
 *         --timeout - set connection timeout, in seconds
 *         --quiet - run quietly
 * @postcondition - global variables are appropriately set
 */
function process_argument_switches()
{
	global $timeout;
	global $quiet;
	global $debug;
	global $argc;
	global $argv;

	logger(4, 'Processing argument switches');
	logger(5, "$argc arguments passed in: ".print_r($argv, true));

	if (isset($argc) and ($argc > 1)) {
		$error = false;
		for ($index = 1; $index < $argc; $index++) {
			$debug_set = false;
			$quiet_set = false;
			switch(strtolower($argv[$index])) {
				case '--': // to handle end of php arg list
					unset($argv[$index]);
					break;
				case '--debug':
					if ($quiet_set == true) {
						logger(2, 'both quiet and debug flags attempted, failing');
						$error = true;
						break;
					}
					$debug = true;
					$quiet = false;
					unset($argv[$index]);
					$debug_set = true;
					break;
				case '--help':
					display_help();
					exit;
				case '--timeout':
					if (isset($argv[$index + 1])) {
						$temp_timeout = $argv[$index + 1];
						if (is_numeric($temp_timeout)) {
							$timeout = $temp_timeout;
						} else {
							logger(2, 'non-numeric timeout value given');
							$error = true;
						}
						unset($argv[$index + 1]);
					} else {
						logger(2, 'no timeout value found');
						$error = true;
					}
					unset($argv[$index]);
					$index++;
					break;
				case '--quiet':
					if ($debug_set == true) {
						logger(2, 'both quiet and debug flags attempted, failing');
						$error = true;
						break;
					}
					$debug = false;
					$quiet = true;
					unset($argv[$index]);
					$quiet_set = true;
					break;
				default:
					// nothing
			}
		}
		if ($error == true) {
			display_help();
			exit;
		}

		/* taking advantage of the writable nature of these global variables... */
		$argv = array_values($argv);
		$argc = count($argv);
	}
	logger(5, 'Values set:');
	logger(5, 'debug '.($debug == true ? 'true' : 'false'));
	logger(5, 'timeout '.($timeout == true ? 'true' : 'false'));
	logger(5, 'quiet '.($quiet == true ? 'true' : 'false'));
	return true;
}


/**
 * @desc - displays usage information
 */
function display_help()
{
	global $argv;
	logger(4, 'displaying help screen');

	echo <<<EOHELP
Usage: {$argv[0]} [options] [optional dpu id list]
       php -f {$argv[0]} -- [options] [optional dpu id list]

  (note that -- is used to delimit php options from script options)

  --debug        run in debug mode (lots of output)
  --help         display this output
  --timeout <s>  set connection timeout to s (in seconds)
  --quiet        run quietly (cannot be used with --debug)


EOHELP;
}



/**
 * @desc - opens a database connection
 */
function open_db_conn()
{
	global $connection;
	logger(4, 'connecting to database... ', false);
	$connection = pg_connect("host=localhost dbname=bpdb user=postgres options='-c search_path=statussync,public'");
	if ($connection === false) {
		logger(2, "database error: ".pg_last_error());
		exit_error('could not connect to database');
	}
	logger(4, 'connected');
	return true;
}



/**
 * @desc - fills in the $feeds array with parameters for every feed that
 *         needs to be retrieved
 */
function get_feed_list(&$feeds)
{
	global $feed_host_prefix;
	global $feed_host_suffix;
//	global $i_feed_host_prefix;
//	global $i_feed_host_suffix;
	global $feed_relative_url;
	global $timeout;
	global $quiet;
	global $debug;
	global $connection;
	global $argc;
	global $argv;

	logger(4, 'getting feed list');

// debugging
//$feeds[] = array('system_name' => 'test', 'host' => '127.0.0.1', 'url' => "{$feed_host_prefix}127.0.0.1{$feed_host_suffix}{$feed_relative_url}");
//$feeds[] = array('system_name' => 'devl-testing', 'host' => '192.168.20.132', 'url' => "{$feed_host_prefix}192.168.20.132{$feed_host_suffix}{$feed_relative_url}");

	// set id list, be sure to check for web requests and command-line requests
	$id_list = array();
	if (isset($_REQUEST['systems'])) {
		$slist = 'Web-supplied system list detected';
		if ($debug) echo $slist . "\n";
		logger(4, $slist);
		$id_list = explode(',', $_REQUEST['systems']);
	} elseif ((isset($argc)) and ($argc > 1)) {
		$slist = 'Command-line supplied system list detected';
		if ($debug) echo $slist . "\n";
		logger(4, $slist);
		$id_list = array_splice($argv, 1);
	} else {
		$slist = 'No supplied system list detected, default to database lookup';
		if ($debug) echo $slist . "\n";
		logger(4, $slist);

		$system_list = bp_get_system_list();
		if ($system_list !== false) {
			$id_list = array_keys($system_list);
		} else {
			return print_error('Error getting system list: ' . bp_error());
		}
		$local_id = bp_get_local_system_id();
		if ($local_id !== false) {
			unset($id_list[array_search($local_id, $id_list)]);
		} else {
			return print_error('Error getting local system id: ' . bp_get_local_system_id());
		}

	}
	$str = 'ID list is:'."\n".print_r($id_list, true);
	if ($debug) echo $str;
	logger(4, $str);

	// get system info
	$feeds = array();
	foreach ($id_list as $id) {
		logger(5, 'Processing ID '.$id);
		$res = bp_get_system_info($id);
		if ($res === false) {
			return print_error('Error getting system info for id '.$id.':'. bp_error());
		}

		$latest_time = get_last_update_time($id);
		if (!is_numeric($latest_time)) {
			logger(3, 'latest_time was not numeric, defaulting to 0');
			$latest_time = 0;
		}
		$request_vars = '?latest_time='.$latest_time;
		if (isset($res) and $res !== false and $res !== null) {
			$thisfeed['system_name'] = $res['name'];
			$thisfeed['host'] = $res['host'];
			$thisfeed['url'] = $feed_host_prefix . $res['host'] . $feed_host_suffix . $feed_relative_url . $request_vars;
	//		$thisfeed['insecure_url'] = $i_feed_host_prefix . $res['host'] . $i_feed_host_suffix . $feed_relative_url . $request_vars;
			$thisfeed['request_vars'] = $request_vars;
			$feeds[$res['id']] = $thisfeed;
			logger(5, 'Feed is: '.print_r($thisfeed, true));
		}
	}
	// feeds array is keyed by id, with $thisfeed arrays for each value
	return true;
}



/**
 * @desc - connects to feed servers simultaneously and fills out $feeds
 *         array with responses.
 * @return - true if all feeds successful, false otherwise (but note that
 *           there may still be some successful feeds, so keep processing
 */
function get_feed_responses(&$feeds) {
	global $feed_host_prefix;
	global $feed_host_suffix;
	global $feed_relative_url;
	global $timeout;
	global $quiet;
	global $debug;
	global $connection;
	global $secure_connection;

	logger(3, 'Getting feed responses');

	/* The following method for asynchronous connections was shown in */
	/* the blog of Wez Furlong, a php internals developer */
	/* It has been somewhat modified... */
	$status_strings = array();
	$status_codes = array(); // 0 = success, 1 = new connection, 2 = waiting for response, 3 = response received, 4 = failed
	$socket_data = array();
	$sockets = array();
	$written = array();
	$request_strings = array();

	/* Initiate connections to all the hosts simultaneously */
	foreach ($feeds as $id => $feed) {
		logger(4, 'Initializing feed connections');
		logger(5, 'initializing connection to system '.$id, false);
		$host = $feed['host'];
		if ($debug) echo "host id $id is: $host\n";
		$s = stream_socket_client($feed_host_prefix . $host . $feed_host_suffix, $errno, $errstr, $timeout, 
		     STREAM_CLIENT_ASYNC_CONNECT|STREAM_CLIENT_CONNECT);
		if ($s) {
			$sockets[$id] = $s;
			$status_strings[$id] = "new connection";
			$status_codes[$id] = 1;
			logger(5, 'success');
		} else {
			$status_strings[$id] = "failed, $errno $errstr";
			$status_codes[$id] = 4;
			logger(5, 'failed: '.$status_strings[$id]);
		}
		if ($debug) echo "status is ".$status_codes[$id].", ".$status_strings[$id]."\n";
	}

	/* Build query strings */
	logger(4, 'Building query strings');
	if (isset($_SERVER['SERVER_ADDR'])) {
		$query_from = 'status sync: '.$_SERVER['SERVER_ADDR'];
	} else {
		$query_from = 'status sync: command-line';
	}
	foreach ($feeds as $id => $feed) {
		logger(5, 'Creating query string for id '.$id.':');
		$host = $feed['host'];
		$request_vars = $feed['request_vars'];
		$request_string =  "GET {$feed_relative_url}{$request_vars} HTTP/1.0\r\n" .
		                   "Host: $host\r\n" .
		                   "From: $query_from\r\n" .
		                   "User-Agent: BP Status Sync\r\n" .
		                   "\r\n"; 
		$request_strings[$id] = $request_string;
		logger(5, $request_string);
	}

	/* Now, wait for the results to come back in */
	$str = 'Reading from streams';
	if ($debug) echo $str."\n";
	logger(4, $str);
	$log_state = array(); // keeps track of previous log messages during stream loop to prevent duplicate entries
	while (count($sockets) > 0) {
		$read = $write = $sockets;
		$n = stream_select($read, $write, $e = null, $timeout);

		if ($n > 0) {
			/* writeable sockets can accept an HTTP request */
			foreach ($write as $w) {
				logger(5, 'writable stream '.$n);
				$id = array_search($w, $sockets);

				/* tasks for sockets that have not yet been written to */
				if (!isset($written[$id])) {
					logger(5, 'stream is new/uninitialized');
					if ($secure_connection == true) {
						logger(2, 'enabling cryptography on stream '.$id.'... ', false);
                                                $crypto_success = @stream_socket_enable_crypto($w, true, STREAM_CRYPTO_METHOD_SSLv23_CLIENT);
                                                if ($crypto_success == false) {
                                                        logger(2, 'failed');
                                                } else {
                                                        logger(2, 'successful');
                                                }
					}
					$written[$id] = 0;
					$feeds[$id]['request'] = '';
				}
				if ($written[$id] === true) {
					continue;
				}

				/* write request */
				$request_part = substr($request_strings[$id], $written[$id]);
				$str = 'sending request, id '.$id.': '."\n".$request_part;
				if ($debug) echo $str;
				logger(5, $str);
				$bytes = @fwrite($w, $request_part);
				if ($bytes !== false and $bytes !== null) {
					$written[$id] += $bytes;
					if (!isset($log_state[$id]) or $log_state[$id] != 'w1') {
						logger(5, 'wrote '.$bytes.' bytes, '.$written[$id].' total for stream');
					}
					$log_state[$id] = 'w1';
					$feeds[$id]['request'] .= $request_part;
				} else {
					$written[$id] = true;
					unset($sockets[$id]);
					if (!isset($log_state[$id]) or $log_state[$id] != 'w2') {
						logger(2, 'error writing to connection socket for id '.$id);
					}
					$log_state[$id] = 'w2';
					$status_strings[$id] = 'error writing to connection socket';
					$status_codes[$id] = 4;
				}
				if (strlen($request_strings[$id]) == $written[$id]) {
					if (!isset($log_state[$id]) or $log_state[$id] != 'w3') {
						logger(4, 'request for id '.$id.' fully written to socket');
					}
					$log_state[$id] = 'w3';
					$written[$id] = true;
				}
				$status_strings[$id] = 'waiting for response'; 
				$status_codes[$id] = 2; 
			}


			/* readable sockets either have data for us, or are failed connection attempts */
			foreach ($read as $r) {
				logger(5, 'readable stream '.$n);
				$id = array_search($r, $sockets);
				$data = fread($r, 8192);
				$str = 'data, id '.$id.': '.$data;
				if ($debug) echo $str . "\n";
				logger(5, $str);
				if (strlen($data) == 0) {
					/* no data received, error -or- data has already come in */
					if ($status_codes[$id] != 3) { // response received
						if (!isset($log_state[$id]) or $log_state[$id] != 'r1') {
							logger(2, 'no data received on stream for id '.$id);
						}
						$log_state[$id] = 'r1';
						$status_strings[$id] = 'no data received';
						$status_codes[$id] = 4;
					}
					fclose($r);
					unset($sockets[$id]);
				} else {
					/* store data */
					if (!isset($socket_data[$id])) {
						$socket_data[$id] = '';
					}
					$socket_data[$id] .= $data;
					if (!isset($log_state[$id]) or $log_state[$id] != 'r2') {
						logger(4, 'response received on stream for id '.$id);
					}
					$log_state[$id] = 'r2';
					$status_strings[$id] = 'response received';
					$status_codes[$id] = 3;
				}
			}
			
			/* add a slight delay if we still have open sockets, just so we don't busy wait */
			if (count($sockets) > 0) {
				usleep(1000);
			}
		} else {
			/* timed out waiting; assume that all hosts associated with $sockets are faulty */
			foreach ($sockets as $id => $s) {
				logger(2, 'stream for id '.$id.' timed out');
				$status_strings[$id] = "timed out " . $status_strings[$id];
				$status_codes[$id] = 4;
			}
			break;
		}
	}

	/* extract data */
	logger(4, 'Extracting data');
	$error = true;  //set to false on successful data save
	foreach ($feeds as $id => $feed) {
		logger(5, 'Extracting for feed id '.$id);
		/* be sure to set original array contents after modifications */
		$feed['data'] = null;
		if (isset($status_codes[$id])) {
			$code = $status_codes[$id];
			$string = $status_strings[$id];
		} else {
			$code = 4;
			$string = 'no connection';
		}
		$feed['status_code'] = $code;
		$feed['status_string'] = $string;
		logger(4, 'Status code is '.$code.': '.$string);
		if ($code == 3) { // response received, there will be data for us
			$response = $socket_data[$id];
			$response_nl = "\r\n";

			/* save http head */
			$divider = $response_nl . $response_nl;
			$divpt = strpos($response, $divider);
			if ($debug) echo "head/content dividing point is $divpt\n";
			$feed['head'] = substr($response, 0, $divpt);

			/* only save http data if the response was 200 (OK) */
			$http_tokenized = explode(' ', substr($response, 0, strpos($response, $response_nl)));
			if (is_array($http_tokenized) and isset($http_tokenized[1])) {
				$http_code = $http_tokenized[1];
			} else {
				$http_code = null;
			}
			$feed['http_response_code'] = $http_code;
			if ($http_code == '200') {
				$response_data = substr($response, $divpt + strlen($divider));
				$feed['data'] = $response_data;
				if (ensure_valid_response($response_data) == true) {
					$feed['status_string'] = 'valid xml response received';
					$feed['status_code'] = 0;
					$error = false;
				} else {
					$feed['status_string'] = 'xml response not well-formed';
					$feed['status_code'] = 4;
				}
			} else {
				$feed['status_string'] = 'http response code not 200';
				$feed['status_code'] = 4;
			}
			$str = print_r($feed, true);
			if ($debug) echo $str;
			logger(5, 'feed is: ');
			logger(5, $str);
		}
		$feeds[$id] = $feed; /* set contents, because foreach with a reference doesn't seem to work... */
	}
	//if ($debug) echo "\nFEEDS:\n";
	//if ($debug) print_r($feeds);

	// diagnostic printouts
	foreach ($feeds as $id => $feed) {
		$host = $feed['host'];
		$status = $feed['status_string'];
		$host_str = 'Host: '.$host;
		$status_str = 'Status: '.$status;
		logger(1, $host_str);
		logger(1, $status_str);
		if ($quiet == false) {
			echo $host_str."\n";
			echo $status_str."\n\n";
		} 
	}
	if ($error) {
		logger(2, 'Errors detected, returning false');
		return false;
	} else {
		logger(4, 'No errors, returning true');
		return true;
	}
}


/**
 * @desc - checks response text xml for validity
 * @return - true if valid, false otherwise
 */
function ensure_valid_response($response_text) {
	global $debug;

	logger(4, 'checking received response for validity');

	$valid = true;
	// very basic, but for now we trust the feed generator to create something well-formed
	if (substr($response_text, 0, 6) != '<?xml ') {
		$valid = false;
	}
	if ($debug) echo 'response text valid? ' . ($valid == true ? 'true' : 'false') . "\n";
	if ($valid == true) {
		logger(4, 'response seems valid');
	} else {
		logger(1, 'invalid xml response recieved');
	}
	return $valid;
}



/**
 * @desc - retrieves the last update time for a specified dpu id, as 
 *         returned from the dpu itself
 * @return - varies with result:
 *           false if the query fails
 *           0 for a successful query with no results
 *           last sync time for a query with results
 */
function get_last_update_time($dpu_id)
{
	global $debug;

	logger(4, 'Getting most recent update time for system '.$dpu_id);

	$query = "SELECT last_sync_remotetime FROM sync_metadata WHERE system_id = '" . pg_escape_string($dpu_id) ."';";
	$pg_result = @pg_query($query);
	if ($pg_result !== false and $pg_result !== null) {
		$last_sync = 0; // default for successful queries with no results returned
		$last_sync_row = @pg_fetch_assoc($pg_result);
		if (is_array($last_sync_row) and isset($last_sync_row['last_sync_remotetime'])) {
			$last_sync = $last_sync_row['last_sync_remotetime'];
			$str = 'last sync for dpu '.$dpu_id.' was at '.$last_sync;
			if ($debug) echo $str."\n";
			logger(4, $str);
		}
		return $last_sync;
	} else {
		logger(2, 'No previous sync time for system '.$dpu_id.' found');
		return false;
	}
}



/**
 * @desc - updates the local sync database with information from the feeds
 */
function update_database($feeds) {
	global $feed_host_prefix;
	global $feed_host_suffix;
	global $feed_relative_url;
	global $timeout;
	global $quiet;
	global $debug;
	global $connection;
	global $retention_days;

	logger(3, 'Updating database records with feed data');

	/* purge old data - do it first to speed up later queries */
	logger(5, 'Purging old data');
	$purge_tables = array(
		array('table' => 'sync_alerts', 'time_key' => 'updated'),
		array('table' => 'sync_backups', 'time_key' => 'start_time'),
		array('table' => 'sync_sds', 'time_key' => 'done_epoch')
	);
	foreach ($purge_tables as $table) {
		$query = "DELETE FROM {$table['table']} WHERE {$table['time_key']} < " . (time() - ($retention_days * 86400)) . ";";
		$res = pg_query($query);
		if ($res === false) {
			logger(1, 'Error purging data: '.pg_last_error());
			logger(1, 'Query was: '.$query);
		} else {
			logger(5, 'Query was: '.$query);
		}
	}
	logger(5, 'Adding necessary systems to sync_metadata');
	$query = "INSERT INTO sync_metadata SELECT system_id, 0 AS last_sync_localtime, 0 AS last_sync_remotetime FROM bp.systems WHERE system_id NOT IN (SELECT system_id FROM sync_metadata);";
	$res = pg_query($query);
	if ($res === false) {
		logger(1, 'Error adding new systems to sync_metadata: '.pg_last_error());
		logger(1, 'Query was: '.$query);
	} else {
		logger(5, 'Query was: '.$query);
	}


	/* Loop through each xml feed, extract data, and insert into the database */
	/* as a transaction.  As a policy, each and every element in the xml feed */
	/* is escaped using htmlspecialchars before being encoded into the stream */
	/* on the client side, so we have to unescape every element (even all the */
	/* ids) before using the data.                                            */
	foreach ($feeds as $id => $feed) {
		logger(4, 'Extracting data for feed id '.$id);
		$system_id = pg_escape_string($id);
		// feed is selected and system_id is set
		if ($feed['status_code'] != 0) {
			logger(4, 'Status code non-zero, no extraction for this feed will take place');
			continue;
		}
		$sxml = simplexml_load_string($feed['data']);

		$query = "BEGIN;\n";

		/* save metadata */
		$time = pg_escape_string(htmlspecialchars_decode($sxml->metadata->row->time));
		$query .= "UPDATE sync_metadata SET last_sync_remotetime = '{$time}', last_sync_localtime = '" . time() . "' WHERE system_id = '{$system_id}';\n";


		/* process clients */
		$TEMPINSERT = '';
		$UPDATE = '';
		foreach ($sxml->nodes->row as $row_obj) {
			if (count($row_obj) > 0) {
				$row = decode_and_pg_escape_array($row_obj);
				$row['system_id'] = $system_id;
				$node_no = $row['node_no'];
				$strings = pg_make_strings($row);
				$TEMPINSERT .= "INSERT INTO nodes_temp {$strings['key_string']} VALUES {$strings['val_string']};\n";
				$UPDATE .= "UPDATE sync_nodes SET {$strings['update_string']} WHERE system_id = '{$system_id}' AND node_no = '{$node_no}';\n";
			}
		}
		$query .= "CREATE TEMPORARY TABLE nodes_temp (LIKE sync_nodes INCLUDING DEFAULTS) ON COMMIT DROP;\n";
		$query .= $TEMPINSERT;
		$query .= "DELETE FROM sync_nodes WHERE system_id = '{$system_id}' AND node_no NOT IN (SELECT node_no FROM nodes_temp);\n";
		$query .= "INSERT INTO sync_nodes SELECT * FROM nodes_temp WHERE node_no NOT IN (SELECT node_no FROM sync_nodes WHERE system_id = '{$system_id}');\n";
		$query .= $UPDATE;
		$query .= "DROP TABLE nodes_temp;\n";
		

		/* process alerts */
		// deletions
		$IDs = '';
		foreach ($sxml->alert_ids->row as $row_obj) {
			$IDs .= (int)pg_escape_string(htmlspecialchars_decode($row_obj->alert_id)) . ',';
		}
		if (empty($IDs)) {
			$DELETE = "DELETE FROM sync_alerts WHERE system_id = '{$system_id}';\n";
		} else {
			$DELETE = "DELETE FROM sync_alerts WHERE system_id = '{$system_id}' AND alert_id NOT IN (" . substr($IDs, 0, -1) . ");\n";
		}
		// additions or updates
		$TEMPINSERT = '';
		$UPDATE = '';
		foreach ($sxml->alerts->row as $row_obj) {
			if (count($row_obj) > 0) {
				$row = decode_and_pg_escape_array($row_obj);
				$row['system_id'] = $system_id;
				$alert_id = $row['alert_id'];
				$strings = pg_make_strings($row);
				$TEMPINSERT .= "INSERT INTO alerts_temp {$strings['key_string']} VALUES {$strings['val_string']};\n";
				$UPDATE .= "UPDATE sync_alerts SET {$strings['update_string']} WHERE system_id = '{$system_id}' AND alert_id = '{$alert_id}';\n";
			}
		}
		$query .= "CREATE TEMPORARY TABLE alerts_temp (LIKE sync_alerts INCLUDING DEFAULTS) ON COMMIT DROP;\n";
		$query .= $TEMPINSERT;
		$query .= $DELETE;
		$query .= "INSERT INTO sync_alerts SELECT * FROM alerts_temp WHERE alert_id NOT IN (SELECT alert_id FROM sync_alerts WHERE system_id = '{$system_id}');\n";
		$query .= $UPDATE;
		$query .= "DROP TABLE alerts_temp;\n";


		/* process backups */
		// deletions
		$IDs = '';
		foreach ($sxml->backup_ids->row as $row_obj) {
			$IDs .= (int)pg_escape_string(htmlspecialchars_decode($row_obj->backup_no)) . ',';
		}
		if (empty($IDs)) {
			$DELETE = "DELETE FROM sync_backups WHERE system_id = '{$system_id}';\n";
		} else {
			$DELETE = "DELETE FROM sync_backups WHERE system_id = '{$system_id}' AND backup_no NOT IN (" . substr($IDs, 0, -1) . ");\n";
		}
		// additions or updates
		$TEMPINSERT = '';
		$UPDATE = '';
		foreach ($sxml->backups->row as $row_obj) {
			if (count($row_obj) > 0) {
				$row = decode_and_pg_escape_array($row_obj);
				$row['system_id'] = $system_id;
				$backup_no = $row['backup_no'];
				$strings = pg_make_strings($row);
				$TEMPINSERT .= "INSERT INTO backups_temp {$strings['key_string']} VALUES {$strings['val_string']};\n";
				$UPDATE .= "UPDATE sync_backups SET {$strings['update_string']} WHERE system_id = '{$system_id}' AND backup_no = '{$backup_no}';\n";
			}
		}
		$query .= "CREATE TEMPORARY TABLE backups_temp (LIKE sync_backups INCLUDING DEFAULTS) ON COMMIT DROP;\n";
		$query .= $TEMPINSERT;
		$query .= $DELETE;
		$query .= "INSERT INTO sync_backups SELECT * FROM backups_temp WHERE backup_no NOT IN (SELECT backup_no FROM sync_backups WHERE system_id = '{$system_id}');\n";
		$query .= $UPDATE;
		$query .= "DROP TABLE backups_temp;\n";


		/* process sds */
		// deletions
		$DELETE = "DELETE FROM sync_sds WHERE system_id = '{$system_id}';\n";
		// additions or updates
		$INSERT = '';
		foreach ($sxml->sds->row as $row_obj) {
			if (count($row_obj) > 0) {
				$row = decode_and_pg_escape_array($row_obj);
				$row['system_id'] = $system_id;
				$backup_no = $row['backup_no'];
				$strings = pg_make_strings($row);
				$INSERT .= "INSERT INTO sync_sds {$strings['key_string']} VALUES {$strings['val_string']};\n";
			}
		}
		// delete sds stats with no associated backup info, do it here to keep app out of having to know referential integrity rules
		$query .= $DELETE;
		$query .= $INSERT;
		$query .= "DELETE FROM sync_sds WHERE system_id = '{$system_id}' AND backup_no NOT IN (SELECT backup_no FROM sync_backups WHERE system_id = '{$system_id}');\n";


		/* process sql backups */
		// deletions
		$DELETE = "DELETE FROM sync_sql_backups WHERE system_id = '{$system_id}' AND sql_ref NOT IN (SELECT backup_no FROM sync_backups WHERE system_id = '{$system_id}');\n";
		// additions or updates
		$TEMPINSERT = '';
		$UPDATE = '';
		foreach ($sxml->sql->row as $row_obj) {
			if (count($row_obj) > 0) {
				$row = decode_and_pg_escape_array($row_obj);
				$row['system_id'] = $system_id;
				$sql_ref = $row['sql_ref'];
				$strings = pg_make_strings($row);
				$TEMPINSERT .= "INSERT INTO sql_backups_temp {$strings['key_string']} VALUES {$strings['val_string']};\n";
				$UPDATE .= "UPDATE sync_sql_backups SET {$strings['update_string']} WHERE system_id = '{$system_id}' AND sql_ref = '{$sql_ref}';\n";
			}
		}
		$query .= "CREATE TEMPORARY TABLE sql_backups_temp (LIKE sync_sql_backups INCLUDING DEFAULTS) ON COMMIT DROP;\n";
		$query .= $TEMPINSERT;
		$query .= $DELETE;
		$query .= "INSERT INTO sync_sql_backups SELECT * FROM sql_backups_temp WHERE sql_ref NOT IN (SELECT sql_ref FROM sync_sql_backups WHERE system_id = '{$system_id}');\n";
		$query .= $UPDATE;
		$query .= "DROP TABLE sql_backups_temp;\n";


		/* commit all changes for this system */
		$query .= "COMMIT;\n";
		logger(5, 'About to commit query, query is:'."\n".'==============================================================='."\n");
		logger(5, $query);
		logger(5, "\n".'==============================================================='."\n");
		if ($debug) {
			echo "\n$query\n";
		}
		$res = pg_query($query);
		if ($res === false) {
			logger(1, 'Error committing query for id '.$id.', rolling back: '.pg_last_error());
			$rollback_res = pg_query("ROLLBACK;\n");
			if ($rollback_res === false) {
				logger(1, 'Rollback failed: '.pg_last_error());
			}
		}
		$str = 'done with system with id ' . $system_id . ', status was: ' . ($res === false ? 'failure' : 'success');
		logger(2, $str);
		if (!$quiet) {
			echo $str."\n";
		}
	}
	logger(3, 'Done extracting data');
}


/**
 * @desc - htmlspecialchars_decode's and pg_escape_string's all keys/values 
 *         of an xml-representation of a row of sql results
 */
function decode_and_pg_escape_array($arr)
{
	$new = array();
	foreach ($arr as $key => $value) {
		if ((!is_array($value)) and count($value) <= 1) { // prove it's not an array or other iterable object
			if (pg_valid_key($key)) {
				$new[$key] = pg_escape_string(htmlspecialchars_decode($value));
			}
		}
	}
	return $new;
}

/**
 * @desc - returns whether a sql identifier has only valid characters present
 *         (alphanumeric, _)
 */
function pg_valid_key($key)
{
	$match = preg_match('/\w/', $key);
	return ($match == 1);
}


/**
 * @desc - returns strings usable to insert or update content for an
 *         entire row of SQL results
 * @precondition - row has a count > 0
 */
function pg_make_strings($row)
{
	$key_string = '';
	$val_string = '';
	$update_string = '';
	foreach ($row as $key => $value) {
		$key_string .= "$key, ";
		$val_string .= "'$value', ";
		$update_string .= "$key = '$value', ";
	}
	$key_string = '(' . substr($key_string, 0, -2) . ')';
	$val_string = '(' . substr($val_string, 0, -2) . ')';
	$update_string = substr($update_string, 0, -2);
	return array('key_string' => $key_string,
	             'val_string' => $val_string,
		     'update_string' => $update_string);
}
